@@ -30,33 +30,18 @@
 static HWND parent_wnd, child1_wnd, child2_wnd;
 static INT notify_format;
 static BOOL notify_query_received;
-static WCHAR test_w[] = {'t', 'e', 's', 't', 0};
-static CHAR test_a[] = {'t', 'e', 's', 't', 0};
+static const CHAR test_a[] = "test";
+static const WCHAR test_w[] = L"test";
 /* Double zero so that it's safe to cast it to WCHAR * */
-static CHAR te_a[] = {'t', 'e', 0, 0};
-static WCHAR empty_w[] = {0};
-static CHAR empty_a[] = {0};
-static CHAR large_a[] = "You should have received a copy of the GNU Lesser General Public License along with this ...";
-static WCHAR large_w[] =
-{
-    'Y', 'o', 'u', ' ', 's', 'h', 'o', 'u', 'l', 'd', ' ', 'h', 'a', 'v', 'e', ' ', 'r', 'e', 'c', 'e', 'i', 'v', 'e',
-    'd', ' ', 'a', ' ', 'c', 'o', 'p', 'y', ' ', 'o', 'f', ' ', 't', 'h', 'e', ' ', 'G', 'N', 'U', ' ', 'L', 'e', 's',
-    's', 'e', 'r', ' ', 'G', 'e', 'n', 'e', 'r', 'a', 'l', ' ', 'P', 'u', 'b', 'l', 'i', 'c', ' ', 'L', 'i', 'c', 'e',
-    'n', 's', 'e', ' ', 'a', 'l', 'o', 'n', 'g', ' ', 'w', 'i', 't', 'h', ' ', 't', 'h', 'i', 's', ' ', '.', '.', '.', 0
-};
-static WCHAR large_truncated_65_w[65] =
-{
-    'Y', 'o', 'u', ' ', 's', 'h', 'o', 'u', 'l', 'd', ' ', 'h', 'a', 'v', 'e', ' ', 'r', 'e', 'c', 'e', 'i', 'v',
-    'e', 'd', ' ', 'a', ' ', 'c', 'o', 'p', 'y', ' ', 'o', 'f', ' ', 't', 'h', 'e', ' ', 'G', 'N', 'U', ' ', 'L',
-    'e', 's', 's', 'e', 'r', ' ', 'G', 'e', 'n', 'e', 'r', 'a', 'l', ' ', 'P', 'u', 'b', 'l', 'i', 'c', 0
-};
-static WCHAR large_truncated_80_w[80] =
-{
-    'Y', 'o', 'u', ' ', 's', 'h', 'o', 'u', 'l', 'd', ' ', 'h', 'a', 'v', 'e', ' ', 'r', 'e', 'c', 'e',
-    'i', 'v', 'e', 'd', ' ', 'a', ' ', 'c', 'o', 'p', 'y', ' ', 'o', 'f', ' ', 't', 'h', 'e', ' ', 'G',
-    'N', 'U', ' ', 'L', 'e', 's', 's', 'e', 'r', ' ', 'G', 'e', 'n', 'e', 'r', 'a', 'l', ' ', 'P', 'u',
-    'b', 'l', 'i', 'c', ' ', 'L', 'i', 'c', 'e', 'n', 's', 'e', ' ', 'a', 'l', 'o', 'n', 'g', ' ', 'w'
-};
+static const CHAR te_a[] = {'t', 'e', 0, 0};
+static const CHAR large_a[] =
+    "You should have received a copy of the GNU Lesser General Public License along with this ...";
+static const WCHAR large_w[] =
+    L"You should have received a copy of the GNU Lesser General Public License along with this ...";
+static const WCHAR large_truncated_65_w[65] =
+    L"You should have received a copy of the GNU Lesser General Public";
+static const WCHAR large_truncated_80_w[80] =
+    L"You should have received a copy of the GNU Lesser General Public License along w";
 static WCHAR buffer[64];
 
 /* Text field conversion test behavior flags. */
@@ -96,26 +81,26 @@ static struct notify_test_info
 struct notify_test_send
 {
     /* Data sent to pager */
-    WCHAR *send_text;
+    const WCHAR *send_text;
     INT send_text_size;
     INT send_text_max;
     /* Data expected by parent of pager */
-    void *expect_text;
+    const void *expect_text;
 };
 
 struct notify_test_receive
 {
     /* Data sent to pager */
-    WCHAR *send_text;
+    const WCHAR *send_text;
     INT send_text_size;
     INT send_text_max;
     /* Data for parent to write */
-    CHAR *write_pointer;
-    CHAR *write_text;
+    const CHAR *write_pointer;
+    const CHAR *write_text;
     INT write_text_size;
     INT write_text_max;
     /* Data when message returned */
-    void *return_text;
+    const void *return_text;
     INT return_text_max;
 };
 
@@ -145,69 +130,69 @@ static const struct notify_test_send test_dont_convert_send_data[] =
 
 static const struct notify_test_receive test_convert_receive_data[] =
 {
-    {empty_w, sizeof(empty_w), ARRAY_SIZE(buffer), NULL, test_a, sizeof(test_a), -1, test_w, ARRAY_SIZE(buffer)},
-    {empty_w, sizeof(empty_w), ARRAY_SIZE(buffer), test_a, NULL, 0, -1, test_w, ARRAY_SIZE(buffer)},
-    {NULL, sizeof(empty_w), ARRAY_SIZE(buffer), test_a, NULL, 0, -1, NULL, ARRAY_SIZE(buffer)},
-    {empty_w, sizeof(empty_w), ARRAY_SIZE(buffer), large_a, NULL, 0, -1, large_truncated_65_w, ARRAY_SIZE(buffer)},
-    {empty_w, sizeof(empty_w), ARRAY_SIZE(buffer), empty_a, 0, 0, 1, empty_w, 1},
+    {L"", sizeof(L""), ARRAY_SIZE(buffer), NULL, test_a, sizeof(test_a), -1, test_w, ARRAY_SIZE(buffer)},
+    {L"", sizeof(L""), ARRAY_SIZE(buffer), test_a, NULL, 0, -1, test_w, ARRAY_SIZE(buffer)},
+    {NULL, sizeof(L""), ARRAY_SIZE(buffer), test_a, NULL, 0, -1, NULL, ARRAY_SIZE(buffer)},
+    {L"", sizeof(L""), ARRAY_SIZE(buffer), large_a, NULL, 0, -1, large_truncated_65_w, ARRAY_SIZE(buffer)},
+    {L"", sizeof(L""), ARRAY_SIZE(buffer), "", 0, 0, 1, L"", 1},
 };
 
 static const struct notify_test_receive test_dont_convert_receive_data[] =
 {
-    {empty_w, sizeof(empty_w), ARRAY_SIZE(buffer), NULL, test_a, sizeof(test_a), -1, test_a, ARRAY_SIZE(buffer)},
-    {empty_w, sizeof(empty_w), ARRAY_SIZE(buffer), test_a, NULL, 0, -1, test_a, ARRAY_SIZE(buffer)},
+    {L"", sizeof(L""), ARRAY_SIZE(buffer), NULL, test_a, sizeof(test_a), -1, test_a, ARRAY_SIZE(buffer)},
+    {L"", sizeof(L""), ARRAY_SIZE(buffer), test_a, NULL, 0, -1, test_a, ARRAY_SIZE(buffer)},
 };
 
 static const struct notify_test_tooltip
 {
     /* Data for parent to write */
-    CHAR *write_sztext;
+    const CHAR *write_sztext;
     INT write_sztext_size;
-    CHAR *write_lpsztext;
+    const CHAR *write_lpsztext;
     HMODULE write_hinst;
     /* Data when message returned */
-    WCHAR *return_sztext;
+    const WCHAR *return_sztext;
     INT return_sztext_size;
-    WCHAR *return_lpsztext;
+    const WCHAR *return_lpsztext;
     HMODULE return_hinst;
     /* Data expected by parent */
-    CHAR *expect_sztext;
+    const CHAR *expect_sztext;
     /* Data send to parent */
-    WCHAR *send_sztext;
+    const WCHAR *send_sztext;
     INT send_sztext_size;
-    WCHAR *send_lpsztext;
+    const WCHAR *send_lpsztext;
 } test_tooltip_data[] =
 {
-    {NULL, 0, NULL, NULL, empty_w, -1, empty_w},
+    {NULL, 0, NULL, NULL, L"", -1, L""},
     {test_a, sizeof(test_a), NULL, NULL, test_w, -1, test_w},
     {test_a, sizeof(test_a), test_a, NULL, test_w, -1, test_w},
-    {test_a, sizeof(test_a), (CHAR *)1, (HMODULE)0xdeadbeef, empty_w, -1, (WCHAR *)1, (HMODULE)0xdeadbeef},
+    {test_a, sizeof(test_a), (CHAR *)1, (HMODULE)0xdeadbeef, L"", -1, (WCHAR *)1, (HMODULE)0xdeadbeef},
     {test_a, sizeof(test_a), test_a, (HMODULE)0xdeadbeef, test_w, -1, test_w, (HMODULE)0xdeadbeef},
     {NULL, 0, test_a, NULL, test_w, -1, test_w},
     {test_a, 2, test_a, NULL, test_w, -1, test_w},
     {NULL, 0, NULL, NULL, test_w, -1, test_w, NULL, test_a, test_w, sizeof(test_w)},
-    {NULL, 0, NULL, NULL, empty_w, -1, empty_w, NULL, empty_a, NULL, 0, test_w},
+    {NULL, 0, NULL, NULL, L"", -1, L"", NULL, "", NULL, 0, test_w},
     {NULL, 0, large_a, NULL, large_truncated_80_w, sizeof(large_truncated_80_w), large_w}
 };
 
 static const struct notify_test_datetime_format
 {
     /* Data send to parent */
-    WCHAR *send_pszformat;
+    const WCHAR *send_pszformat;
     /* Data expected by parent */
-    CHAR *expect_pszformat;
+    const CHAR *expect_pszformat;
     /* Data for parent to write */
-    CHAR *write_szdisplay;
+    const CHAR *write_szdisplay;
     INT write_szdisplay_size;
-    CHAR *write_pszdisplay;
+    const CHAR *write_pszdisplay;
     /* Data when message returned */
-    WCHAR *return_szdisplay;
+    const WCHAR *return_szdisplay;
     INT return_szdisplay_size;
-    WCHAR *return_pszdisplay;
+    const WCHAR *return_pszdisplay;
 } test_datetime_format_data[] =
 {
     {test_w, test_a},
-    {NULL, NULL, NULL, 0, test_a, empty_w, -1, test_w},
+    {NULL, NULL, NULL, 0, test_a, L"", -1, test_w},
     {NULL, NULL, test_a, sizeof(test_a), NULL, test_w, -1, test_w},
     {NULL, NULL, test_a, 2, test_a, (WCHAR *)te_a, -1, test_w},
     {NULL, NULL, NULL, 0, large_a, NULL, 0, large_w}
@@ -285,7 +270,7 @@ static const struct message set_pos_empty_seq[] = {
 static CHAR *heap_strdup(const CHAR *str)
 {
     int len = lstrlenA(str) + 1;
-    CHAR *ret = heap_alloc(len * sizeof(CHAR));
+    CHAR *ret = HeapAlloc(GetProcessHeap(), 0, len * sizeof(CHAR));
     lstrcpyA(ret, str);
     return ret;
 }
@@ -327,9 +312,9 @@ static LRESULT WINAPI parent_wnd_proc(HWND hwnd, UINT message, WPARAM wParam, LP
                 DWORD style = GetWindowLongA(nmpgcs->hdr.hwndFrom, GWL_STYLE);
 
                 if (style & PGS_HORZ)
-                    ok(nmpgcs->dwFlag == PGF_CALCWIDTH, "Unexpected flags %#x.\n", nmpgcs->dwFlag);
+                    ok(nmpgcs->dwFlag == PGF_CALCWIDTH, "Unexpected flags %#lx.\n", nmpgcs->dwFlag);
                 else
-                    ok(nmpgcs->dwFlag == PGF_CALCHEIGHT, "Unexpected flags %#x.\n", nmpgcs->dwFlag);
+                    ok(nmpgcs->dwFlag == PGF_CALCHEIGHT, "Unexpected flags %#lx.\n", nmpgcs->dwFlag);
                 break;
             }
             default:
@@ -464,14 +449,14 @@ static void test_pager(void)
     ok_sequence(sequences, PAGER_SEQ_INDEX, set_child_seq, "set child", FALSE);
     GetWindowRect( pager, &rect );
     ok( rect.right - rect.left == 100 && rect.bottom - rect.top == 100,
-        "pager resized %dx%d\n", rect.right - rect.left, rect.bottom - rect.top );
+        "pager resized %ldx%ld\n", rect.right - rect.left, rect.bottom - rect.top );
 
     flush_sequences(sequences, NUM_MSG_SEQUENCES);
     SendMessageA(pager, PGM_SETCHILD, 0, (LPARAM)child2_wnd);
     ok_sequence(sequences, PAGER_SEQ_INDEX, switch_child_seq, "switch to invisible child", FALSE);
     GetWindowRect(pager, &rect);
     ok(rect.right - rect.left == 100 && rect.bottom - rect.top == 100,
-        "pager resized %dx%d\n", rect.right - rect.left, rect.bottom - rect.top);
+        "pager resized %ldx%ld\n", rect.right - rect.left, rect.bottom - rect.top);
     ok(!IsWindowVisible(child2_wnd), "Child window 2 is visible\n");
 
     flush_sequences(sequences, NUM_MSG_SEQUENCES);
@@ -479,14 +464,14 @@ static void test_pager(void)
     ok_sequence(sequences, PAGER_SEQ_INDEX, set_child_seq, "switch to visible child", FALSE);
     GetWindowRect(pager, &rect);
     ok(rect.right - rect.left == 100 && rect.bottom - rect.top == 100,
-        "pager resized %dx%d\n", rect.right - rect.left, rect.bottom - rect.top);
+        "pager resized %ldx%ld\n", rect.right - rect.left, rect.bottom - rect.top);
 
     flush_sequences( sequences, NUM_MSG_SEQUENCES );
     SendMessageA( pager, PGM_SETPOS, 0, 10 );
     ok_sequence(sequences, PAGER_SEQ_INDEX, set_pos_seq, "set pos", TRUE);
     GetWindowRect( pager, &rect );
     ok( rect.right - rect.left == 100 && rect.bottom - rect.top == 100,
-        "pager resized %dx%d\n", rect.right - rect.left, rect.bottom - rect.top );
+        "pager resized %ldx%ld\n", rect.right - rect.left, rect.bottom - rect.top );
 
     flush_sequences( sequences, NUM_MSG_SEQUENCES );
     SendMessageA( pager, PGM_SETPOS, 0, 10 );
@@ -544,69 +529,64 @@ static LRESULT WINAPI test_notifyformat_proc(HWND hwnd, UINT message, WPARAM wPa
 
 static BOOL register_notifyformat_class(void)
 {
-    static const WCHAR class_w[] = {'P', 'a', 'g', 'e', 'r', ' ', 'n', 'o', 't', 'i', 'f', 'y', 'f',
-                                   'o', 'r', 'm', 'a', 't', ' ', 'c', 'l', 'a', 's', 's', 0};
     WNDCLASSW cls = {0};
 
     cls.lpfnWndProc = test_notifyformat_proc;
     cls.hInstance = GetModuleHandleW(NULL);
-    cls.lpszClassName = class_w;
+    cls.lpszClassName = L"Pager notifyformat class";
     return RegisterClassW(&cls);
 }
 
 static void test_wm_notifyformat(void)
 {
-    static const WCHAR class_w[] = {'P', 'a', 'g', 'e', 'r', ' ', 'n', 'o', 't', 'i', 'f', 'y', 'f',
-                                    'o', 'r', 'm', 'a', 't', ' ', 'c', 'l', 'a', 's', 's', 0};
-    static const WCHAR parent_w[] = {'p', 'a', 'r', 'e', 'n', 't', 0};
-    static const WCHAR pager_w[] = {'p', 'a', 'g', 'e', 'r', 0};
-    static const WCHAR child_w[] = {'c', 'h', 'i', 'l', 'd', 0};
     static const INT formats[] = {NFR_UNICODE, NFR_ANSI};
     HWND parent, pager, child;
     LRESULT ret;
+    BOOL bret;
     INT i;
 
-    ok(register_notifyformat_class(), "Register test class failed, error 0x%08x\n", GetLastError());
+    bret = register_notifyformat_class();
+    ok(bret, "Register test class failed, error 0x%08lx\n", GetLastError());
 
     for (i = 0; i < ARRAY_SIZE(formats); i++)
     {
         notify_format = formats[i];
-        parent = CreateWindowW(class_w, parent_w, WS_OVERLAPPED, 0, 0, 100, 100, 0, 0, GetModuleHandleW(0), 0);
+        parent = CreateWindowW(L"Pager notifyformat class", L"parent", WS_OVERLAPPED, 0, 0, 100, 100, 0, 0, GetModuleHandleW(0), 0);
         ok(parent != NULL, "CreateWindow failed\n");
-        pager = CreateWindowW(WC_PAGESCROLLERW, pager_w, WS_CHILD, 0, 0, 100, 100, parent, 0, GetModuleHandleW(0), 0);
+        pager = CreateWindowW(WC_PAGESCROLLERW, L"pager", WS_CHILD, 0, 0, 100, 100, parent, 0, GetModuleHandleW(0), 0);
         ok(pager != NULL, "CreateWindow failed\n");
-        child = CreateWindowW(class_w, child_w, WS_CHILD, 0, 0, 100, 100, pager, 0, GetModuleHandleW(0), 0);
+        child = CreateWindowW(L"Pager notifyformat class", L"child", WS_CHILD, 0, 0, 100, 100, pager, 0, GetModuleHandleW(0), 0);
         ok(child != NULL, "CreateWindow failed\n");
         SendMessageW(pager, PGM_SETCHILD, 0, (LPARAM)child);
 
         /* Test parent */
         notify_query_received = FALSE;
         ret = SendMessageW(pager, WM_NOTIFYFORMAT, (WPARAM)parent, NF_REQUERY);
-        ok(ret == notify_format, "Expect %d, got %ld\n", notify_format, ret);
+        ok(ret == notify_format, "Expect %d, got %Id\n", notify_format, ret);
         ok(notify_query_received, "Didn't receive notify\n");
 
         /* Send NF_QUERY directly to parent */
         notify_query_received = FALSE;
         ret = SendMessageW(parent, WM_NOTIFYFORMAT, (WPARAM)pager, NF_QUERY);
-        ok(ret == notify_format, "Expect %d, got %ld\n", notify_format, ret);
+        ok(ret == notify_format, "Expect %d, got %Id\n", notify_format, ret);
         ok(notify_query_received, "Didn't receive notify\n");
 
         /* Pager send notifications to its parent regardless of wParam */
         notify_query_received = FALSE;
         ret = SendMessageW(pager, WM_NOTIFYFORMAT, (WPARAM)parent_wnd, NF_REQUERY);
-        ok(ret == notify_format, "Expect %d, got %ld\n", notify_format, ret);
+        ok(ret == notify_format, "Expect %d, got %Id\n", notify_format, ret);
         ok(notify_query_received, "Didn't receive notify\n");
 
         /* Pager always wants Unicode notifications from children */
         ret = SendMessageW(child, WM_NOTIFYFORMAT, (WPARAM)pager, NF_REQUERY);
-        ok(ret == NFR_UNICODE, "Expect %d, got %ld\n", NFR_UNICODE, ret);
+        ok(ret == NFR_UNICODE, "Expect %d, got %Id\n", NFR_UNICODE, ret);
         ret = SendMessageW(pager, WM_NOTIFYFORMAT, (WPARAM)child, NF_QUERY);
-        ok(ret == NFR_UNICODE, "Expect %d, got %ld\n", NFR_UNICODE, ret);
+        ok(ret == NFR_UNICODE, "Expect %d, got %Id\n", NFR_UNICODE, ret);
 
         DestroyWindow(parent);
     }
 
-    UnregisterClassW(class_w, GetModuleHandleW(NULL));
+    UnregisterClassW(L"Pager notifyformat class", GetModuleHandleW(NULL));
 }
 
 static void notify_generic_text_handler(CHAR **text, INT *text_max)
@@ -622,7 +602,7 @@ static void notify_generic_text_handler(CHAR **text, INT *text_max)
         send_data = (notify_test_info.test_id == CONVERT_SEND ? test_convert_send_data : test_dont_convert_send_data)
                     + notify_test_info.sub_test_id;
         if (notify_test_info.flags & ZERO_SEND)
-            ok(!lstrcmpA(*text, empty_a), "Code 0x%08x test 0x%08x sub test %d expect empty text, got %s\n",
+            ok(!*text[0], "Code 0x%08x test 0x%08x sub test %d expect empty text, got %s\n",
                notify_test_info.unicode, notify_test_info.test_id, notify_test_info.sub_test_id, *text);
         else if (notify_test_info.flags & CONVERT_SEND)
             ok(!lstrcmpA(send_data->expect_text, *text), "Code 0x%08x test 0x%08x sub test %d expect %s, got %s\n",
@@ -656,12 +636,12 @@ static void notify_generic_text_handler(CHAR **text, INT *text_max)
         else if(notify_test_info.unicode == HDN_GETDISPINFOW)
             *text = heap_strdup(receive_data->write_pointer);
         else
-            *text = receive_data->write_pointer;
+            *text = (char *)receive_data->write_pointer;
         if (text_max && receive_data->write_text_max != -1) *text_max = receive_data->write_text_max;
         break;
     }
     case SEND_EMPTY_IF_NULL:
-        ok(!lstrcmpA(*text, empty_a), "Code 0x%08x test 0x%08x sub test %d expect empty text, got %s\n",
+        ok(!*text[0], "Code 0x%08x test 0x%08x sub test %d expect empty text, got %s\n",
            notify_test_info.unicode, notify_test_info.test_id, notify_test_info.sub_test_id, *text);
         break;
     case DONT_SEND_EMPTY_IF_NULL:
@@ -680,7 +660,7 @@ static void notify_tooltip_handler(NMTTDISPINFOA *nm)
         ok(!lstrcmpA(data->expect_sztext, nm->szText), "Sub test %d expect %s, got %s\n", notify_test_info.sub_test_id,
            data->expect_sztext, nm->szText);
     if (data->write_sztext) memcpy(nm->szText, data->write_sztext, data->write_sztext_size);
-    if (data->write_lpsztext) nm->lpszText = data->write_lpsztext;
+    if (data->write_lpsztext) nm->lpszText = (char *)data->write_lpsztext;
     if (data->write_hinst) nm->hinst = data->write_hinst;
 }
 
@@ -698,7 +678,6 @@ static void notify_datetime_handler(NMDATETIMEFORMATA *nm)
 
 static LRESULT WINAPI test_notify_proc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 {
-    static const WCHAR test[] = {'t', 'e', 's', 't', 0};
     switch (message)
     {
     case WM_NOTIFY:
@@ -709,9 +688,9 @@ static LRESULT WINAPI test_notify_proc(HWND hwnd, UINT message, WPARAM wParam, L
         if (!notify_test_info.unicode) break;
         ok(!notify_test_info.received, "Extra notification received\n");
 
-        ok(wParam == notify_test_info.id_from, "Expect %ld, got %ld\n", notify_test_info.id_from, wParam);
+        ok(wParam == notify_test_info.id_from, "Expect %Id, got %Id\n", notify_test_info.id_from, wParam);
         ok(hdr->code == notify_test_info.ansi, "Expect 0x%08x, got 0x%08x\n", notify_test_info.ansi, hdr->code);
-        ok(hdr->idFrom == notify_test_info.id_from, "Expect %ld, got %ld\n", notify_test_info.id_from, wParam);
+        ok(hdr->idFrom == notify_test_info.id_from, "Expect %Id, got %Id\n", notify_test_info.id_from, wParam);
         ok(hdr->hwndFrom == notify_test_info.hwnd_from, "Expect %p, got %p\n", notify_test_info.hwnd_from, hdr->hwndFrom);
 
         if (hdr->code != notify_test_info.ansi)
@@ -908,7 +887,7 @@ static LRESULT WINAPI test_notify_proc(HWND hwnd, UINT message, WPARAM wParam, L
         }
         notify_test_info.received = TRUE;
         ok(!lstrcmpA(test_a, "test"), "test_a got modified\n");
-        ok(!lstrcmpW(test_w, test), "test_w got modified\n");
+        ok(!lstrcmpW(test_w, L"test"), "test_w got modified\n");
         return 0;
     }
     case WM_NOTIFYFORMAT:
@@ -1023,8 +1002,9 @@ static void test_notify_generic_text_helper(HWND pager, const struct generic_tex
             if (data->return_text)
             {
                 if (para->flags & CONVERT_RECEIVE)
-                    ok(!lstrcmpW(data->return_text, *para->text), "Code 0x%08x sub test %d expect %s, got %s\n",
-                       para->code_unicode, i, wine_dbgstr_w((WCHAR *)data->return_text), wine_dbgstr_w(*para->text));
+                    ok(!wcsncmp(data->return_text, *para->text, *para->text_max),
+                       "Code 0x%08x sub test %d expect %s, got %s\n", para->code_unicode, i,
+                       wine_dbgstr_w((WCHAR *)data->return_text), wine_dbgstr_w(*para->text));
                 else
                     ok(!lstrcmpA(data->return_text, (CHAR *)*para->text), "Code 0x%08x sub test %d expect %s, got %s\n",
                        para->code_unicode, i, (CHAR *)data->return_text, (CHAR *)*para->text);
@@ -1104,8 +1084,8 @@ static void test_wm_notify_header(HWND pager)
     HD_TEXTFILTERW hdtf = {0};
 
     hdi.mask = HDI_TEXT | HDI_FILTER;
-    hdi.pszText = test_w;
-    hdtf.pszText = test_w;
+    hdi.pszText = (WCHAR *)test_w;
+    hdtf.pszText = (WCHAR *)test_w;
     nmh.pitem = &hdi;
     nmh.pitem->pvFilter = &hdtf;
     send_notify(pager, HDN_BEGINDRAG, HDN_BEGINDRAG, (LPARAM)&nmh, TRUE);
@@ -1140,7 +1120,7 @@ static void test_wm_notify_tooltip(HWND pager)
 
         memset(&nmttdi, 0, sizeof(nmttdi));
         if (data->send_sztext) memcpy(nmttdi.szText, data->send_sztext, data->send_sztext_size);
-        if (data->send_lpsztext) nmttdi.lpszText = data->send_lpsztext;
+        if (data->send_lpsztext) nmttdi.lpszText = (WCHAR *)data->send_lpsztext;
         send_notify(pager, TTN_GETDISPINFOW, TTN_GETDISPINFOA, (LPARAM)&nmttdi, FALSE);
         if (data->return_sztext)
         {
@@ -1230,7 +1210,7 @@ static void test_wm_notify(void)
          DONT_CONVERT_SEND | DONT_CONVERT_RECEIVE},
         {&nmtbr, sizeof(nmtbr), NULL, 0, (WCHAR **)&nmtbr.tbButton.iString, NULL, TBN_RESTORE, TBN_RESTORE,
          DONT_CONVERT_SEND | DONT_CONVERT_RECEIVE},
-        {&nmtbdi, sizeof(nmtbdi), &nmtbdi.dwMask, TBNF_TEXT, &nmtbdi.pszText, &nmtbdi.cchText, TBN_GETDISPINFOW,
+        {&nmtbdi, sizeof(nmtbdi), (UINT*)&nmtbdi.dwMask, TBNF_TEXT, &nmtbdi.pszText, &nmtbdi.cchText, TBN_GETDISPINFOW,
          TBN_GETDISPINFOW, DONT_CONVERT_SEND | DONT_CONVERT_RECEIVE},
         {&nmtb, sizeof(nmtb), NULL, 0, &nmtb.pszText, &nmtb.cchText, TBN_GETBUTTONINFOW, TBN_GETBUTTONINFOA,
          SEND_EMPTY_IF_NULL | CONVERT_SEND | CONVERT_RECEIVE},
@@ -1280,9 +1260,11 @@ static void test_wm_notify(void)
         {&nmtv, sizeof(nmtv), &nmtv.itemOld.mask, TVIF_TEXT, &nmtv.itemOld.pszText, &nmtv.itemOld.cchTextMax,
          TVN_SELCHANGEDW, TVN_SELCHANGEDA, CONVERT_SEND, TVITEM_OLD_HANDLER}
     };
+    BOOL bret;
     INT i;
 
-    ok(register_test_notify_class(), "Register test class failed, error 0x%08x\n", GetLastError());
+    bret = register_test_notify_class();
+    ok(bret, "Register test class failed, error 0x%08lx\n", GetLastError());
 
     parent = CreateWindowA(class, "parent", WS_OVERLAPPED, 0, 0, 100, 100, 0, 0, GetModuleHandleA(0), 0);
     ok(parent != NULL, "CreateWindow failed\n");
