@@ -33,7 +33,6 @@
 #include "mmsystem.h"
 #include "wingdi.h"
 #include "mmreg.h"
-#include "winternl.h"
 #include "wine/debug.h"
 #include "dsound.h"
 #include "ks.h"
@@ -48,7 +47,7 @@ void DSOUND_RecalcVolPan(PDSVOLUMEPAN volpan)
 	double temp;
 	TRACE("(%p)\n",volpan);
 
-	TRACE("Vol=%d Pan=%d\n", volpan->lVolume, volpan->lPan);
+	TRACE("Vol=%ld Pan=%ld\n", volpan->lVolume, volpan->lPan);
 	/* the AmpFactors are expressed in 16.16 fixed point */
 
 	/* FIXME: use calculated vol and pan ampfactors */
@@ -57,7 +56,7 @@ void DSOUND_RecalcVolPan(PDSVOLUMEPAN volpan)
 	temp = (double) (volpan->lVolume + (volpan->lPan < 0 ? volpan->lPan : 0));
 	volpan->dwTotalAmpFactor[1] = (ULONG) (pow(2.0, temp / 600.0) * 0xffff);
 
-	TRACE("left = %x, right = %x\n", volpan->dwTotalAmpFactor[0], volpan->dwTotalAmpFactor[1]);
+	TRACE("left = %lx, right = %lx\n", volpan->dwTotalAmpFactor[0], volpan->dwTotalAmpFactor[1]);
 }
 
 void DSOUND_AmpFactorToVolPan(PDSVOLUMEPAN volpan)
@@ -65,7 +64,7 @@ void DSOUND_AmpFactorToVolPan(PDSVOLUMEPAN volpan)
     double left,right;
     TRACE("(%p)\n",volpan);
 
-    TRACE("left=%x, right=%x\n",volpan->dwTotalAmpFactor[0],volpan->dwTotalAmpFactor[1]);
+    TRACE("left=%lx, right=%lx\n",volpan->dwTotalAmpFactor[0],volpan->dwTotalAmpFactor[1]);
     if (volpan->dwTotalAmpFactor[0]==0)
         left=-10000;
     else
@@ -84,7 +83,7 @@ void DSOUND_AmpFactorToVolPan(PDSVOLUMEPAN volpan)
     if (volpan->lPan < -10000)
         volpan->lPan=-10000;
 
-    TRACE("Vol=%d Pan=%d\n", volpan->lVolume, volpan->lPan);
+    TRACE("Vol=%ld Pan=%ld\n", volpan->lVolume, volpan->lPan);
 }
 
 /**
@@ -97,6 +96,7 @@ void DSOUND_RecalcFormat(IDirectSoundBufferImpl *dsb)
 {
 	DWORD ichannels = dsb->pwfx->nChannels;
 	DWORD ochannels = dsb->device->pwfx->nChannels;
+	LONG64 oldFreqAdjustDen = dsb->freqAdjustDen;
 	WAVEFORMATEXTENSIBLE *pwfxe;
 	BOOL ieee = FALSE;
 
@@ -130,8 +130,10 @@ void DSOUND_RecalcFormat(IDirectSoundBufferImpl *dsb)
 
 	/* calculate the 10ms write lead */
 	dsb->writelead = (dsb->freq / 100) * dsb->pwfx->nBlockAlign;
+	dsb->maxwritelead = (DSBFREQUENCY_MAX / 100) * dsb->pwfx->nBlockAlign;
 
-	dsb->freqAccNum = 0;
+	if (oldFreqAdjustDen)
+		dsb->freqAccNum = (dsb->freqAccNum * dsb->freqAdjustDen + oldFreqAdjustDen / 2) / oldFreqAdjustDen;
 
 	dsb->get_aux = ieee ? getbpp[4] : getbpp[dsb->pwfx->wBitsPerSample/8 - 1];
 	dsb->put_aux = putieee32;
@@ -143,7 +145,7 @@ void DSOUND_RecalcFormat(IDirectSoundBufferImpl *dsb)
 	{
 		dsb->mix_channels = ichannels;
 		if (ichannels > 32) {
-			FIXME("Copying %u channels is unsupported, limiting to first 32\n", ichannels);
+			FIXME("Copying %lu channels is unsupported, limiting to first 32\n", ichannels);
 			dsb->mix_channels = 32;
 		}
 	}
@@ -179,6 +181,12 @@ void DSOUND_RecalcFormat(IDirectSoundBufferImpl *dsb)
 		dsb->put = put_surround512stereo;
 		dsb->put_aux = putieee32_sum;
 	}
+	else if (ichannels == 8 && ochannels == 2)
+	{
+		dsb->mix_channels = 8;
+		dsb->put = put_surround712stereo;
+		dsb->put_aux = putieee32_sum;
+	}
 	else if (ichannels == 4 && ochannels == 2)
 	{
 		dsb->mix_channels = 4;
@@ -188,7 +196,7 @@ void DSOUND_RecalcFormat(IDirectSoundBufferImpl *dsb)
 	else
 	{
 		if (ichannels > 2)
-			FIXME("Conversion from %u to %u channels is not implemented, falling back to stereo\n", ichannels, ochannels);
+			FIXME("Conversion from %lu to %lu channels is not implemented, falling back to stereo\n", ichannels, ochannels);
 		dsb->mix_channels = 2;
 	}
 }
@@ -218,7 +226,6 @@ void DSOUND_CheckEvent(const IDirectSoundBufferImpl *dsb, DWORD playpos, int len
             TRACE("Signalling %p\n", dsb->notifies[left].hEventNotify);
             SetEvent(dsb->notifies[left].hEventNotify);
         }
-        return;
     }
 
     for(first = 0; first < dsb->nrofnotifies && dsb->notifies[first].dwOffset == DSBPN_OFFSETSTOP; ++first)
@@ -243,7 +250,7 @@ void DSOUND_CheckEvent(const IDirectSoundBufferImpl *dsb, DWORD playpos, int len
         }
     }
 
-    TRACE("Not stopped: first notify: %u (%u), left notify: %u (%u), range: [%u,%u)\n",
+    TRACE("Not stopped: first notify: %u (%lu), left notify: %u (%lu), range: [%lu,%lu)\n",
             first, dsb->notifies[first].dwOffset,
             left, dsb->notifies[left].dwOffset,
             playpos, (playpos + len) % dsb->buflen);
@@ -254,7 +261,7 @@ void DSOUND_CheckEvent(const IDirectSoundBufferImpl *dsb, DWORD playpos, int len
             if(dsb->notifies[check].dwOffset >= playpos + len)
                 break;
 
-            TRACE("Signalling %p (%u)\n", dsb->notifies[check].hEventNotify, dsb->notifies[check].dwOffset);
+            TRACE("Signalling %p (%lu)\n", dsb->notifies[check].hEventNotify, dsb->notifies[check].dwOffset);
             SetEvent(dsb->notifies[check].hEventNotify);
         }
     }
@@ -264,29 +271,45 @@ void DSOUND_CheckEvent(const IDirectSoundBufferImpl *dsb, DWORD playpos, int len
             if(dsb->notifies[check].dwOffset >= (playpos + len) % dsb->buflen)
                 break;
 
-            TRACE("Signalling %p (%u)\n", dsb->notifies[check].hEventNotify, dsb->notifies[check].dwOffset);
+            TRACE("Signalling %p (%lu)\n", dsb->notifies[check].hEventNotify, dsb->notifies[check].dwOffset);
             SetEvent(dsb->notifies[check].hEventNotify);
         }
     }
 }
 
 static inline float get_current_sample(const IDirectSoundBufferImpl *dsb,
-        DWORD mixpos, DWORD channel)
+        BYTE *buffer, DWORD buflen, DWORD mixpos, DWORD channel)
 {
-    if (mixpos >= dsb->buflen && !(dsb->playflags & DSBPLAY_LOOPING))
+    if (mixpos >= buflen && !(dsb->playflags & DSBPLAY_LOOPING))
         return 0.0f;
-    return dsb->get(dsb, mixpos % dsb->buflen, channel);
+    return dsb->get(dsb, buffer + (mixpos % buflen), channel);
 }
 
 static UINT cp_fields_noresample(IDirectSoundBufferImpl *dsb, UINT count)
 {
     UINT istride = dsb->pwfx->nBlockAlign;
     UINT ostride = dsb->device->pwfx->nChannels * sizeof(float);
+    UINT committed_samples = 0;
     DWORD channel, i;
-    for (i = 0; i < count; i++)
+
+    if (!secondarybuffer_is_audible(dsb))
+        return count;
+
+    if(dsb->use_committed) {
+        committed_samples = (dsb->writelead - dsb->committed_mixpos) / istride;
+        committed_samples = committed_samples <= count ? committed_samples : count;
+    }
+
+    for (i = 0; i < committed_samples; i++)
         for (channel = 0; channel < dsb->mix_channels; channel++)
-            dsb->put(dsb, i * ostride, channel, get_current_sample(dsb,
-                    dsb->sec_mixpos + i * istride, channel));
+            dsb->put(dsb, i * ostride, channel, get_current_sample(dsb, dsb->committedbuff,
+                dsb->writelead, dsb->committed_mixpos + i * istride, channel));
+
+    for (; i < count; i++)
+        for (channel = 0; channel < dsb->mix_channels; channel++)
+            dsb->put(dsb, i * ostride, channel, get_current_sample(dsb, dsb->buffer->memory,
+                dsb->buflen, dsb->sec_mixpos + i * istride, channel));
+
     return count;
 }
 
@@ -295,6 +318,7 @@ static UINT cp_fields_resample(IDirectSoundBufferImpl *dsb, UINT count, LONG64 *
     UINT i, channel;
     UINT istride = dsb->pwfx->nBlockAlign;
     UINT ostride = dsb->device->pwfx->nChannels * sizeof(float);
+    UINT committed_samples = 0;
 
     LONG64 freqAcc_start = *freqAccNum;
     LONG64 freqAcc_end = freqAcc_start + count * dsb->freqAdjustNum;
@@ -310,27 +334,40 @@ static UINT cp_fields_resample(IDirectSoundBufferImpl *dsb, UINT count, LONG64 *
     len += fir_cachesize;
     len *= sizeof(float);
 
+    *freqAccNum = freqAcc_end % dsb->freqAdjustDen;
+
+    if (!secondarybuffer_is_audible(dsb))
+        return max_ipos;
+
     if (!dsb->device->cp_buffer) {
-        dsb->device->cp_buffer = HeapAlloc(GetProcessHeap(), 0, len);
+        dsb->device->cp_buffer = malloc(len);
         dsb->device->cp_buffer_len = len;
     } else if (len > dsb->device->cp_buffer_len) {
-        dsb->device->cp_buffer = HeapReAlloc(GetProcessHeap(), 0, dsb->device->cp_buffer, len);
+        dsb->device->cp_buffer = realloc(dsb->device->cp_buffer, len);
         dsb->device->cp_buffer_len = len;
     }
 
     fir_copy = dsb->device->cp_buffer;
     intermediate = fir_copy + fir_cachesize;
 
+    if(dsb->use_committed) {
+        committed_samples = (dsb->writelead - dsb->committed_mixpos) / istride;
+        committed_samples = committed_samples <= required_input ? committed_samples : required_input;
+    }
 
     /* Important: this buffer MUST be non-interleaved
      * if you want -msse3 to have any effect.
      * This is good for CPU cache effects, too.
      */
     itmp = intermediate;
-    for (channel = 0; channel < channels; channel++)
-        for (i = 0; i < required_input; i++)
-            *(itmp++) = get_current_sample(dsb,
-                    dsb->sec_mixpos + i * istride, channel);
+    for (channel = 0; channel < channels; channel++) {
+        for (i = 0; i < committed_samples; i++)
+            *(itmp++) = get_current_sample(dsb, dsb->committedbuff,
+                dsb->writelead, dsb->committed_mixpos + i * istride, channel);
+        for (; i < required_input; i++)
+            *(itmp++) = get_current_sample(dsb, dsb->buffer->memory,
+                    dsb->buflen, dsb->sec_mixpos + i * istride, channel);
+    }
 
     for(i = 0; i < count; ++i) {
         UINT int_fir_steps = (freqAcc_start + i * dsb->freqAdjustNum) * dsbfirstep / dsb->freqAdjustDen;
@@ -359,8 +396,6 @@ static UINT cp_fields_resample(IDirectSoundBufferImpl *dsb, UINT count, LONG64 *
         }
     }
 
-    *freqAccNum = freqAcc_end % dsb->freqAdjustDen;
-
     return max_ipos;
 }
 
@@ -384,6 +419,12 @@ static void cp_fields(IDirectSoundBufferImpl *dsb, UINT count, LONG64 *freqAccNu
     }
 
     dsb->sec_mixpos = ipos;
+
+    if(dsb->use_committed) {
+        dsb->committed_mixpos += adv * dsb->pwfx->nBlockAlign;
+        if(dsb->committed_mixpos >= dsb->writelead)
+            dsb->use_committed = FALSE;
+    }
 }
 
 /**
@@ -423,10 +464,7 @@ static void DSOUND_MixToTemporary(IDirectSoundBufferImpl *dsb, DWORD frames)
 	if (dsb->device->tmp_buffer_len < size_bytes || !dsb->device->tmp_buffer)
 	{
 		dsb->device->tmp_buffer_len = size_bytes;
-		if (dsb->device->tmp_buffer)
-			dsb->device->tmp_buffer = HeapReAlloc(GetProcessHeap(), 0, dsb->device->tmp_buffer, size_bytes);
-		else
-			dsb->device->tmp_buffer = HeapAlloc(GetProcessHeap(), 0, size_bytes);
+		dsb->device->tmp_buffer = realloc(dsb->device->tmp_buffer, size_bytes);
 	}
 	if(dsb->put_aux == putieee32_sum)
 		memset(dsb->device->tmp_buffer, 0, dsb->device->tmp_buffer_len);
@@ -453,7 +491,7 @@ static void DSOUND_MixerVol(const IDirectSoundBufferImpl *dsb, INT frames)
 	UINT channels = dsb->device->pwfx->nChannels, chan;
 
 	TRACE("(%p,%d)\n",dsb,frames);
-	TRACE("left = %x, right = %x\n", dsb->volpan.dwTotalAmpFactor[0],
+	TRACE("left = %lx, right = %lx\n", dsb->volpan.dwTotalAmpFactor[0],
 		dsb->volpan.dwTotalAmpFactor[1]);
 
 	if ((!(dsb->dsbd.dwFlags & DSBCAPS_CTRLPAN) || (dsb->volpan.lPan == 0)) &&
@@ -494,22 +532,23 @@ static DWORD DSOUND_MixInBuffer(IDirectSoundBufferImpl *dsb, float *mix_buffer,
 	float *ibuf;
 	DWORD oldpos;
 
-	TRACE("sec_mixpos=%d/%d\n", dsb->sec_mixpos, dsb->buflen);
-	TRACE("(%p, frames=%d)\n",dsb,frames);
+	TRACE("sec_mixpos=%ld/%ld\n", dsb->sec_mixpos, dsb->buflen);
+	TRACE("(%p, frames=%ld)\n",dsb,frames);
 
 	/* Resample buffer to temporary buffer specifically allocated for this purpose, if needed */
 	oldpos = dsb->sec_mixpos;
 	DSOUND_MixToTemporary(dsb, frames);
 	ibuf = dsb->device->tmp_buffer;
 
-	/* Apply volume if needed */
-	DSOUND_MixerVol(dsb, frames);
+	if (secondarybuffer_is_audible(dsb)) {
+		/* Apply volume if needed */
+		DSOUND_MixerVol(dsb, frames);
 
-	mixieee32(ibuf, mix_buffer, frames * dsb->device->pwfx->nChannels);
+		mixieee32(ibuf, mix_buffer, frames * dsb->device->pwfx->nChannels);
+	}
 
 	/* check for notification positions */
-	if (dsb->dsbd.dwFlags & DSBCAPS_CTRLPOSITIONNOTIFY &&
-	    dsb->state != STATE_STARTING) {
+	if (dsb->dsbd.dwFlags & DSBCAPS_CTRLPOSITIONNOTIFY) {
 		INT ilen = DSOUND_BufPtrDiff(dsb->buflen, dsb->sec_mixpos, oldpos);
 		DSOUND_CheckEvent(dsb, oldpos, ilen);
 	}
@@ -532,30 +571,15 @@ static DWORD DSOUND_MixOne(IDirectSoundBufferImpl *dsb, float *mix_buffer, DWORD
 {
 	DWORD primary_done = 0;
 
-	TRACE("(%p, frames=%d)\n",dsb,frames);
-	TRACE("looping=%d, leadin=%d\n", dsb->playflags, dsb->leadin);
-
-	/* If leading in, only mix about 20 ms, and 'skip' mixing the rest, for more fluid pointer advancement */
-	/* FIXME: Is this needed? */
-	if (dsb->leadin && dsb->state == STATE_STARTING) {
-		if (frames > 2 * dsb->device->frag_frames) {
-			primary_done = frames - 2 * dsb->device->frag_frames;
-			frames = 2 * dsb->device->frag_frames;
-			dsb->sec_mixpos += primary_done *
-				dsb->pwfx->nBlockAlign * dsb->freqAdjustNum / dsb->freqAdjustDen;
-		}
-	}
-
-	dsb->leadin = FALSE;
-
-	TRACE("frames (primary) = %i\n", frames);
+	TRACE("(%p, frames=%ld)\n",dsb,frames);
+	TRACE("looping=%ld\n", dsb->playflags);
 
 	/* First try to mix to the end of the buffer if possible
 	 * Theoretically it would allow for better optimization
 	*/
 	primary_done += DSOUND_MixInBuffer(dsb, mix_buffer, frames);
 
-	TRACE("total mixed data=%d\n", primary_done);
+	TRACE("total mixed data=%ld\n", primary_done);
 
 	/* Report back the total prebuffered amount for this buffer */
 	return primary_done;
@@ -579,20 +603,16 @@ static void DSOUND_MixToPrimary(const DirectSoundDevice *device, float *mix_buff
 	/* unless we find a running buffer, all have stopped */
 	*all_stopped = TRUE;
 
-	TRACE("(frames %d)\n", frames);
+	TRACE("(frames %ld)\n", frames);
 	for (i = 0; i < device->nrofbuffers; i++) {
 		dsb = device->buffers[i];
 
-		TRACE("MixToPrimary for %p, state=%d\n", dsb, dsb->state);
+		TRACE("MixToPrimary for %p, state=%ld\n", dsb, dsb->state);
 
 		if (dsb->buflen && dsb->state) {
-			TRACE("Checking %p, frames=%d\n", dsb, frames);
-			RtlAcquireResourceShared(&dsb->lock, TRUE);
-			/* if buffer is stopping it is stopped now */
-			if (dsb->state == STATE_STOPPING) {
-				dsb->state = STATE_STOPPED;
-				DSOUND_CheckEvent(dsb, 0, 0);
-			} else if (dsb->state != STATE_STOPPED) {
+			TRACE("Checking %p, frames=%ld\n", dsb, frames);
+			AcquireSRWLockShared(&dsb->lock);
+			if (dsb->state != STATE_STOPPED) {
 
 				/* if the buffer was starting, it must be playing now */
 				if (dsb->state == STATE_STARTING)
@@ -603,7 +623,7 @@ static void DSOUND_MixToPrimary(const DirectSoundDevice *device, float *mix_buff
 
 				*all_stopped = FALSE;
 			}
-			RtlReleaseResource(&dsb->lock);
+			ReleaseSRWLockShared(&dsb->lock);
 		}
 	}
 }
@@ -627,7 +647,7 @@ static void DSOUND_WaveQueue(DirectSoundDevice *device, LPBYTE pos, DWORD bytes)
 
 	hr = IAudioRenderClient_GetBuffer(device->render, bytes / device->pwfx->nBlockAlign, &buffer);
 	if(FAILED(hr)){
-		WARN("GetBuffer failed: %08x\n", hr);
+		WARN("GetBuffer failed: %08lx\n", hr);
 		return;
 	}
 
@@ -635,7 +655,7 @@ static void DSOUND_WaveQueue(DirectSoundDevice *device, LPBYTE pos, DWORD bytes)
 
 	hr = IAudioRenderClient_ReleaseBuffer(device->render, bytes / device->pwfx->nBlockAlign, 0);
 	if(FAILED(hr)) {
-		ERR("ReleaseBuffer failed: %08x\n", hr);
+		ERR("ReleaseBuffer failed: %08lx\n", hr);
 		IAudioRenderClient_ReleaseBuffer(device->render, 0, 0);
 		return;
 	}
@@ -657,7 +677,8 @@ static void DSOUND_WaveQueue(DirectSoundDevice *device, LPBYTE pos, DWORD bytes)
  */
 static void DSOUND_PerformMix(DirectSoundDevice *device)
 {
-	DWORD block, pad_frames, pad_bytes, frames;
+	DWORD block, pad_bytes, frames;
+	UINT32 pad_frames;
 	HRESULT hr;
 
 	TRACE("(%p)\n", device);
@@ -667,7 +688,7 @@ static void DSOUND_PerformMix(DirectSoundDevice *device)
 
 	hr = IAudioClient_GetCurrentPadding(device->client, &pad_frames);
 	if(FAILED(hr)){
-		WARN("GetCurrentPadding failed: %08x\n", hr);
+		WARN("GetCurrentPadding failed: %08lx\n", hr);
 		LeaveCriticalSection(&device->mixlock);
 		return;
 	}
@@ -701,7 +722,7 @@ static void DSOUND_PerformMix(DirectSoundDevice *device)
 
 		hr = IAudioRenderClient_GetBuffer(device->render, frames, (BYTE **)&buffer);
 		if(FAILED(hr)){
-			WARN("GetBuffer failed: %08x\n", hr);
+			WARN("GetBuffer failed: %08lx\n", hr);
 			LeaveCriticalSection(&device->mixlock);
 			return;
 		}
@@ -721,7 +742,7 @@ static void DSOUND_PerformMix(DirectSoundDevice *device)
 
 		hr = IAudioRenderClient_ReleaseBuffer(device->render, frames, 0);
 		if(FAILED(hr))
-			ERR("ReleaseBuffer failed: %08x\n", hr);
+			ERR("ReleaseBuffer failed: %08lx\n", hr);
 
 		device->pad += frames * block;
 	} else if (!device->stopped) {
@@ -745,6 +766,7 @@ DWORD CALLBACK DSOUND_mixthread(void *p)
 {
 	DirectSoundDevice *dev = p;
 	TRACE("(%p)\n", dev);
+	SetThreadDescription(GetCurrentThread(), L"wine_dsound_mixer");
 
 	while (dev->ref) {
 		DWORD ret;
@@ -755,16 +777,15 @@ DWORD CALLBACK DSOUND_mixthread(void *p)
 		 */
 		ret = WaitForSingleObject(dev->sleepev, dev->sleeptime);
 		if (ret == WAIT_FAILED)
-			WARN("wait returned error %u %08x!\n", GetLastError(), GetLastError());
+			WARN("wait returned error %lu %08lx!\n", GetLastError(), GetLastError());
 		else if (ret != WAIT_OBJECT_0)
-			WARN("wait returned %08x!\n", ret);
+			WARN("wait returned %08lx!\n", ret);
 		if (!dev->ref)
 			break;
 
-		RtlAcquireResourceShared(&(dev->buffer_list_lock), TRUE);
+		AcquireSRWLockShared(&dev->buffer_list_lock);
 		DSOUND_PerformMix(dev);
-		RtlReleaseResource(&(dev->buffer_list_lock));
+		ReleaseSRWLockShared(&dev->buffer_list_lock);
 	}
-	SetEvent(dev->thread_finished);
 	return 0;
 }
