@@ -1142,7 +1142,8 @@ BOOL WINAPI SetBrushOrgEx( HDC hdc, INT x, INT y, POINT *oldorg )
  */
 BOOL WINAPI FixBrushOrgEx( HDC hdc, INT x, INT y, POINT *oldorg )
 {
-    return SetBrushOrgEx( hdc, x, y, oldorg );
+    /* From Windows 2000 this is a NOP returning FALSE */
+    return FALSE;
 }
 
 /***********************************************************************
@@ -1439,17 +1440,6 @@ INT WINAPI SetROP2( HDC hdc, INT mode )
     return ret;
 }
 
-/***********************************************************************
- *           GetMiterLimit  (GDI32.@)
- */
-BOOL WINAPI GetMiterLimit( HDC hdc, FLOAT *limit )
-{
-    DC_ATTR *dc_attr;
-    if (!(dc_attr = get_dc_attr( hdc ))) return FALSE;
-    if (limit) *limit = dc_attr->miter_limit;
-    return TRUE;
-}
-
 /*******************************************************************
  *           SetMiterLimit  (GDI32.@)
  */
@@ -1459,9 +1449,7 @@ BOOL WINAPI SetMiterLimit( HDC hdc, FLOAT limit, FLOAT *old_limit )
     if (!(dc_attr = get_dc_attr( hdc ))) return FALSE;
     if (dc_attr->emf && !EMFDC_SetMiterLimit( dc_attr, limit )) return 0;
     if (limit < 1.0f) return FALSE;
-    if (old_limit) *old_limit = dc_attr->miter_limit;
-    dc_attr->miter_limit = limit;
-    return TRUE;
+    return NtGdiSetMiterLimit( hdc, *(DWORD *)&limit, old_limit );
 }
 
 /***********************************************************************
@@ -2435,8 +2423,7 @@ BOOL WINAPI GdiSetPixelFormat( HDC hdc, INT format, const PIXELFORMATDESCRIPTOR
  */
 BOOL WINAPI CancelDC(HDC hdc)
 {
-    FIXME( "stub\n" );
-    return TRUE;
+    return NtGdiCancelDC( hdc );
 }
 
 /***********************************************************************
