@@ -18,55 +18,18 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-#include "wine/port.h"
-
 #include <stdarg.h>
 #include <stdio.h>
+#include <stdlib.h>
 
 #include "gphoto2_i.h"
 #include "wingdi.h"
 #include "winuser.h"
-#include "wine/library.h"
+#include "unixlib.h"
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(twain);
 
-#ifdef HAVE_GPHOTO2
-static void *libjpeg_handle;
-#define MAKE_FUNCPTR(f) static typeof(f) * p##f
-MAKE_FUNCPTR(jpeg_std_error);
-MAKE_FUNCPTR(jpeg_CreateDecompress);
-MAKE_FUNCPTR(jpeg_read_header);
-MAKE_FUNCPTR(jpeg_start_decompress);
-MAKE_FUNCPTR(jpeg_read_scanlines);
-MAKE_FUNCPTR(jpeg_finish_decompress);
-MAKE_FUNCPTR(jpeg_destroy_decompress);
-#undef MAKE_FUNCPTR
-
-static void *load_libjpeg(void)
-{
-    if((libjpeg_handle = wine_dlopen(SONAME_LIBJPEG, RTLD_NOW, NULL, 0)) != NULL) {
-
-#define LOAD_FUNCPTR(f) \
-    if((p##f = wine_dlsym(libjpeg_handle, #f, NULL, 0)) == NULL) { \
-        libjpeg_handle = NULL; \
-        return NULL; \
-    }
-
-        LOAD_FUNCPTR(jpeg_std_error);
-        LOAD_FUNCPTR(jpeg_CreateDecompress);
-        LOAD_FUNCPTR(jpeg_read_header);
-        LOAD_FUNCPTR(jpeg_start_decompress);
-        LOAD_FUNCPTR(jpeg_read_scanlines);
-        LOAD_FUNCPTR(jpeg_finish_decompress);
-        LOAD_FUNCPTR(jpeg_destroy_decompress);
-#undef LOAD_FUNCPTR
-    }
-    return libjpeg_handle;
-}
-
-
 /* for the jpeg decompressor source manager. */
 static void _jpeg_init_source(j_decompress_ptr cinfo) { }
 
@@ -86,7 +49,19 @@ static boolean _jpeg_resync_to_restart(j_decompress_ptr cinfo, int desired) {
     return FALSE;
 }
 static void _jpeg_term_source(j_decompress_ptr cinfo) { }
-#endif
+
+static void close_file( UINT64 handle )
+{
+    struct close_file_params params = { handle };
+    GPHOTO2_CALL( close_file, &params );
+}
+
+static void close_current_file(void)
+{
+    close_file( activeDS.file_handle );
+    activeDS.file_handle = 0;
+    free( activeDS.file_data );
+}
 
 /* DG_IMAGE/DAT_CIECOLOR/MSG_GET */
 TW_UINT16 GPHOTO2_CIEColorGet (pTW_IDENTITY pOrigin, 
@@ -133,47 +108,45 @@ TW_UINT16 GPHOTO2_ImageFileXferGet (pTW_IDENTITY pOrigin,
     return TWRC_FAILURE;
 }
 
-#ifdef HAVE_GPHOTO2
 static TW_UINT16 _get_image_and_startup_jpeg(void) {
-    const char *folder = NULL, *filename = NULL;
-    struct gphoto2_file *file;
-    const unsigned char *filedata;
-    unsigned long filesize;
+    unsigned int i;
     int ret;
+    struct open_file_params open_params;
+    struct get_file_data_params get_data_params;
 
-    if (activeDS.file) /* Already loaded. */
+    if (activeDS.file_handle) /* Already loaded. */
 	return TWRC_SUCCESS;
 
-    if(!libjpeg_handle) {
-	if(!load_libjpeg()) {
-	    FIXME("Failed reading JPEG because unable to find %s\n", SONAME_LIBJPEG);
-	    filedata = NULL;
-	    return TWRC_FAILURE;
+    for (i = 0; i < activeDS.file_count; i++)
+    {
+        if (activeDS.download_flags[i])
+        {
+            activeDS.download_flags[i] = FALSE; /* mark as done */
+            break;
 	}
     }
-
-    LIST_FOR_EACH_ENTRY( file, &activeDS.files, struct gphoto2_file, entry ) {
-	if (strstr(file->filename,".JPG") || strstr(file->filename,".jpg")) {
-	    filename = file->filename;
-	    folder = file->folder;
-	    TRACE("downloading %s/%s\n", folder, filename);
-	    if (file->download) {
-		file->download = FALSE; /* mark as done */
-		break;
-	    }
-	}
-    }
-    gp_file_new (&activeDS.file);
-    ret = gp_camera_file_get(activeDS.camera, folder, filename, GP_FILE_TYPE_NORMAL,
-			     activeDS.file, activeDS.context);
-    if (ret < GP_OK) {
-	FIXME("Failed to get file?\n");
+    if (i == activeDS.file_count)
+    {
 	activeDS.twCC = TWCC_SEQERROR;
 	return TWRC_FAILURE;
     }
-    ret = gp_file_get_data_and_size (activeDS.file, (const char**)&filedata, &filesize);
-    if (ret < GP_OK) {
-	FIXME("Failed to get file data?\n");
+
+    open_params.idx     = i;
+    open_params.preview = FALSE;
+    open_params.handle  = &activeDS.file_handle;
+    open_params.size    = &activeDS.file_size;
+    if (GPHOTO2_CALL( open_file, &open_params ))
+    {
+	activeDS.twCC = TWCC_SEQERROR;
+	return TWRC_FAILURE;
+    }
+
+    activeDS.file_data = malloc( activeDS.file_size );
+    get_data_params.handle = activeDS.file_handle;
+    get_data_params.data   = activeDS.file_data;
+    get_data_params.size   = activeDS.file_size;
+    if (GPHOTO2_CALL( get_file_data, &get_data_params ))
+    {
 	activeDS.twCC = TWCC_SEQERROR;
 	return TWRC_FAILURE;
     }
@@ -181,37 +154,34 @@ static TW_UINT16 _get_image_and_startup_jpeg(void) {
     /* This is basically so we can use in-memory data for jpeg decompression.
      * We need to have all the functions.
      */
-    activeDS.xjsm.next_input_byte	= filedata;
-    activeDS.xjsm.bytes_in_buffer	= filesize;
+    activeDS.xjsm.next_input_byte	= activeDS.file_data;
+    activeDS.xjsm.bytes_in_buffer	= activeDS.file_size;
     activeDS.xjsm.init_source	= _jpeg_init_source;
     activeDS.xjsm.fill_input_buffer	= _jpeg_fill_input_buffer;
     activeDS.xjsm.skip_input_data	= _jpeg_skip_input_data;
     activeDS.xjsm.resync_to_restart	= _jpeg_resync_to_restart;
     activeDS.xjsm.term_source	= _jpeg_term_source;
 
-    activeDS.jd.err = pjpeg_std_error(&activeDS.jerr);
+    activeDS.jd.err = jpeg_std_error(&activeDS.jerr);
     /* jpeg_create_decompress is a macro that expands to jpeg_CreateDecompress - see jpeglib.h
      * jpeg_create_decompress(&jd); */
-    pjpeg_CreateDecompress(&activeDS.jd, JPEG_LIB_VERSION, sizeof(struct jpeg_decompress_struct));
+    jpeg_CreateDecompress(&activeDS.jd, JPEG_LIB_VERSION, sizeof(struct jpeg_decompress_struct));
     activeDS.jd.src = &activeDS.xjsm;
-    ret=pjpeg_read_header(&activeDS.jd,TRUE);
+    ret=jpeg_read_header(&activeDS.jd,TRUE);
     activeDS.jd.out_color_space = JCS_RGB;
-    pjpeg_start_decompress(&activeDS.jd);
+    jpeg_start_decompress(&activeDS.jd);
     if (ret != JPEG_HEADER_OK) {
 	ERR("Jpeg image in stream has bad format, read header returned %d.\n",ret);
-	gp_file_unref (activeDS.file);
-	activeDS.file = NULL;
+        close_current_file();
 	return TWRC_FAILURE;
     }
     return TWRC_SUCCESS;
 }
-#endif
 
 /* DG_IMAGE/DAT_IMAGEINFO/MSG_GET */
 TW_UINT16 GPHOTO2_ImageInfoGet (pTW_IDENTITY pOrigin, 
                               TW_MEMREF pData)
 {
-#ifdef HAVE_GPHOTO2
     pTW_IMAGEINFO pImageInfo = (pTW_IMAGEINFO) pData;
 
     TRACE("DG_IMAGE/DAT_IMAGEINFO/MSG_GET\n");
@@ -247,9 +217,6 @@ TW_UINT16 GPHOTO2_ImageInfoGet (pTW_IDENTITY pOrigin,
     pImageInfo->ImageLength 	= activeDS.jd.output_height;
     pImageInfo->BitsPerPixel	= 24;
     return TWRC_SUCCESS;
-#else
-    return TWRC_FAILURE;
-#endif
 }
 
 /* DG_IMAGE/DAT_IMAGELAYOUT/MSG_GET */
@@ -292,7 +259,6 @@ TW_UINT16 GPHOTO2_ImageLayoutSet (pTW_IDENTITY pOrigin,
 TW_UINT16 GPHOTO2_ImageMemXferGet (pTW_IDENTITY pOrigin, 
                                  TW_MEMREF pData)
 {
-#ifdef HAVE_GPHOTO2
     TW_UINT16 twRC = TWRC_SUCCESS;
     pTW_IMAGEMEMXFER pImageMemXfer = (pTW_IMAGEMEMXFER) pData;
     LPBYTE buffer;
@@ -318,7 +284,7 @@ TW_UINT16 GPHOTO2_ImageMemXferGet (pTW_IDENTITY pOrigin,
 
         activeDS.currentState = 7;
     } else {
-	if (!activeDS.file) {
+	if (!activeDS.file_handle) {
     	    activeDS.twCC = TWRC_SUCCESS;
 	    return TWRC_XFERDONE;
 	}
@@ -338,7 +304,7 @@ TW_UINT16 GPHOTO2_ImageMemXferGet (pTW_IDENTITY pOrigin,
 	   ((pImageMemXfer->Memory.Length - curoff) > activeDS.jd.output_width*activeDS.jd.output_components)
     ) {
 	JSAMPROW row = buffer+curoff;
-	int x = pjpeg_read_scanlines(&activeDS.jd,&row,1);
+	int x = jpeg_read_scanlines(&activeDS.jd,&row,1);
 	if (x != 1) {
 		FIXME("failed to read current scanline?\n");
 		break;
@@ -355,10 +321,9 @@ TW_UINT16 GPHOTO2_ImageMemXferGet (pTW_IDENTITY pOrigin,
     TransferringDialogBox(activeDS.progressWnd,0);
 
     if (activeDS.jd.output_scanline == activeDS.jd.output_height) {
-        pjpeg_finish_decompress(&activeDS.jd);
-        pjpeg_destroy_decompress(&activeDS.jd);
-	gp_file_unref (activeDS.file);
-	activeDS.file = NULL;
+        jpeg_finish_decompress(&activeDS.jd);
+        jpeg_destroy_decompress(&activeDS.jd);
+        close_current_file();
 	TRACE("xfer is done!\n");
 
 	/*TransferringDialogBox(activeDS.progressWnd, -1);*/
@@ -368,38 +333,22 @@ TW_UINT16 GPHOTO2_ImageMemXferGet (pTW_IDENTITY pOrigin,
     if (pImageMemXfer->Memory.Flags & TWMF_HANDLE)
         LocalUnlock(pImageMemXfer->Memory.TheMem);
     return twRC;
-#else
-    return TWRC_FAILURE;
-#endif
 }
 
 /* DG_IMAGE/DAT_IMAGENATIVEXFER/MSG_GET */
 TW_UINT16 GPHOTO2_ImageNativeXferGet (pTW_IDENTITY pOrigin, 
                                     TW_MEMREF pData)
 {
-#ifdef HAVE_GPHOTO2
-    pTW_UINT32 pHandle = (pTW_UINT32) pData;
-    HBITMAP hDIB;
-    BITMAPINFO bmpInfo;
-    LPBYTE bits;
+    TW_HANDLE *pHandle = (TW_HANDLE *) pData;
+    HGLOBAL hDIB;
+    BITMAPINFO *bmpInfo;
     JSAMPROW samprow, oldsamprow;
+    UINT header_size; /* Number of bytes in the BITMAPINFOHEADER and the color palette (RGBQUAD) */
+    UINT bytes_per_line; /* Number of bytes per scan line including padding */
+    UINT image_size; /* Number of bytes in the image bits */
 
-    FIXME("DG_IMAGE/DAT_IMAGENATIVEXFER/MSG_GET: implemented, but expect program crash due to DIB.\n");
+    TRACE("DG_IMAGE/DAT_IMAGENATIVEXFER/MSG_GET\n");
 
-/*  NOTE NOTE NOTE NOTE NOTE NOTE NOTE
- *
- *  While this is a mandatory transfer mode and this function
- *  is correctly implemented and fully works, the calling program
- *  will likely crash after calling.
- *
- *  Reason is that there is a lot of example code that does:
- *  bmpinfo = GlobalLock(hBITMAP); ... pointer access to bmpinfo
- *
- *  Our current HBITMAP handles do not support getting GlobalLocked -> App Crash
- *
- *  This needs a GDI Handle rewrite, at least for DIB sections.
- *  - Marcus
- */
     if (activeDS.currentState != 6) {
         activeDS.twCC = TWCC_SEQERROR;
         return TWRC_FAILURE;
@@ -412,54 +361,90 @@ TW_UINT16 GPHOTO2_ImageNativeXferGet (pTW_IDENTITY pOrigin,
     TRACE("Acquiring image %dx%dx%d bits from gphoto.\n",
 	activeDS.jd.output_width, activeDS.jd.output_height,
 	activeDS.jd.output_components*8);
-    ZeroMemory (&bmpInfo, sizeof (BITMAPINFO));
-    bmpInfo.bmiHeader.biSize = sizeof (BITMAPINFOHEADER);
-    bmpInfo.bmiHeader.biWidth = activeDS.jd.output_width;
-    bmpInfo.bmiHeader.biHeight = -activeDS.jd.output_height;
-    bmpInfo.bmiHeader.biPlanes = 1;
-    bmpInfo.bmiHeader.biBitCount = activeDS.jd.output_components*8;
-    bmpInfo.bmiHeader.biCompression = BI_RGB;
-    bmpInfo.bmiHeader.biSizeImage = 0;
-    bmpInfo.bmiHeader.biXPelsPerMeter = 0;
-    bmpInfo.bmiHeader.biYPelsPerMeter = 0;
-    bmpInfo.bmiHeader.biClrUsed = 0;
-    bmpInfo.bmiHeader.biClrImportant = 0;
-    hDIB = CreateDIBSection (0, &bmpInfo, DIB_RGB_COLORS, (LPVOID)&bits, 0, 0);
+
+    header_size = sizeof(BITMAPINFOHEADER) + (activeDS.jd.output_components==1 ? 256*sizeof(RGBQUAD) : 0);
+    bytes_per_line = (activeDS.jd.output_width*activeDS.jd.output_components + 3) & ~3;
+    image_size = bytes_per_line * activeDS.jd.output_height;
+    hDIB = GlobalAlloc(GMEM_MOVEABLE, header_size + image_size);
     if (!hDIB) {
-	FIXME("Failed creating DIB.\n");
-	gp_file_unref (activeDS.file);
-	activeDS.file = NULL;
+	FIXME("Failed allocating memory for DIB.\n");
+        close_current_file();
 	activeDS.twCC = TWCC_LOWMEMORY;
 	return TWRC_FAILURE;
     }
+    bmpInfo = (BITMAPINFO *) GlobalLock(hDIB);
+    bmpInfo->bmiHeader.biSize = sizeof (BITMAPINFOHEADER);
+    bmpInfo->bmiHeader.biWidth = activeDS.jd.output_width;
+    bmpInfo->bmiHeader.biHeight = activeDS.jd.output_height;
+    bmpInfo->bmiHeader.biPlanes = 1;
+    bmpInfo->bmiHeader.biBitCount = activeDS.jd.output_components*8;
+    bmpInfo->bmiHeader.biCompression = BI_RGB;
+    bmpInfo->bmiHeader.biSizeImage = image_size;
+    bmpInfo->bmiHeader.biXPelsPerMeter = 0;
+    bmpInfo->bmiHeader.biYPelsPerMeter = 0;
+    bmpInfo->bmiHeader.biClrUsed = 0;
+    bmpInfo->bmiHeader.biClrImportant = 0;
+
+    if (bmpInfo->bmiHeader.biBitCount==8)
+      {
+	/* Create a grayscale color palette */
+	for (int i=0; i<255; i++)
+	  {
+	    bmpInfo->bmiColors[i].rgbBlue =
+	      bmpInfo->bmiColors[i].rgbGreen =
+	      bmpInfo->bmiColors[i].rgbRed = i;
+	    bmpInfo->bmiColors[i].rgbReserved = 0;
+	  }
+      }
+
     samprow = HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,activeDS.jd.output_width*activeDS.jd.output_components);
     oldsamprow = samprow;
     while ( activeDS.jd.output_scanline<activeDS.jd.output_height ) {
+	BYTE *bits;
+	BYTE *row;
         unsigned int i;
-        int x = pjpeg_read_scanlines(&activeDS.jd,&samprow,1);
-	if (x != 1) {
+
+	/* Calculate to the beginning of the current row before calling jpeg_read_scanlines */
+	row = ((BYTE *) bmpInfo)
+	  + header_size
+	  + (activeDS.jd.output_height - activeDS.jd.output_scanline - 1) * bytes_per_line;
+
+	/* Read next data row from the JPEG source */
+	if (jpeg_read_scanlines(&activeDS.jd, &samprow, 1) != 1) {
 		FIXME("failed to read current scanline?\n");
 		break;
 	}
-	/* We have to convert from RGB to BGR, see MSDN/ BITMAPINFOHEADER */
-	for(i=0;i<activeDS.jd.output_width;i++,samprow+=activeDS.jd.output_components) {
-	    *(bits++) = *(samprow+2);
-	    *(bits++) = *(samprow+1);
-	    *(bits++) = *(samprow);
-	}
-	bits = (LPBYTE)(((UINT_PTR)bits + 3) & ~3);
+
+	bits = row;
+	if (activeDS.jd.output_components==1)
+	  {
+	    /* Grayscale value, no RGB to BGR conversion */
+	    memcpy(row, samprow, activeDS.jd.output_width);
+	    bits += activeDS.jd.output_width;
+	  }
+	else if (activeDS.jd.output_components>=3)
+	  {
+	    /* RGB data, We have to convert from RGB to BGR, see MSDN/ BITMAPINFOHEADER */
+	    for(i=0;i<activeDS.jd.output_width;i++,samprow+=activeDS.jd.output_components) {
+	      *(bits++) = *(samprow+2);
+	      *(bits++) = *(samprow+1);
+	      *(bits++) = *(samprow);
+	    }
+	  }
+	/* set the padding-bytes at the end of the scan line to 0 */
+	while ((bits-row) & 3)
+	  {
+	    *bits++ = 0;
+	  }
 	samprow = oldsamprow;
     }
     HeapFree (GetProcessHeap(), 0, samprow);
-    gp_file_unref (activeDS.file);
-    activeDS.file = NULL;
-    *pHandle = (UINT_PTR)hDIB;
+    GlobalUnlock(hDIB);
+    close_current_file();
+    *pHandle = (TW_HANDLE *)hDIB;
     activeDS.twCC = TWCC_SUCCESS;
     activeDS.currentState = 7;
     return TWRC_XFERDONE;
-#else
-    return TWRC_FAILURE;
-#endif
 }
 
 /* DG_IMAGE/DAT_JPEGCOMPRESSION/MSG_GET */
@@ -553,47 +538,48 @@ TW_UINT16 GPHOTO2_RGBResponseSet (pTW_IDENTITY pOrigin,
     return TWRC_FAILURE;
 }
 
-#ifdef HAVE_GPHOTO2
+/* Called from ui.c in gphoto2.ds to display
+ * preview images of the photos on the camera
+ */
 TW_UINT16
-_get_gphoto2_file_as_DIB(
-    const char *folder, const char *filename, CameraFileType type,
-    HWND hwnd, HBITMAP *hDIB
-) {
-    const unsigned char *filedata;
-    unsigned long	filesize;
-    int			ret;
-    CameraFile		*file;
+_get_gphoto2_file_as_DIB( unsigned int idx, BOOL preview, HWND hwnd, HBITMAP *hDIB )
+{
+    unsigned char *filedata;
+    int ret;
     struct jpeg_source_mgr		xjsm;
     struct jpeg_decompress_struct	jd;
     struct jpeg_error_mgr		jerr;
     BITMAPINFO 		bmpInfo;
     LPBYTE		bits;
     JSAMPROW		samprow, oldsamprow;
+    struct open_file_params open_params;
+    struct get_file_data_params get_data_params;
+    UINT64 file_handle;
+    unsigned int filesize;
 
-    if(!libjpeg_handle) {
-	if(!load_libjpeg()) {
-	    FIXME("Failed reading JPEG because unable to find %s\n", SONAME_LIBJPEG);
-	    filedata = NULL;
-	    return TWRC_FAILURE;
-	}
-    }
-
-    gp_file_new (&file);
-    ret = gp_camera_file_get(activeDS.camera, folder, filename, type, file, activeDS.context);
-    if (ret < GP_OK) {
-	FIXME("Failed to get file?\n");
-	gp_file_unref (file);
+    open_params.idx     = idx;
+    open_params.preview = preview;
+    open_params.handle  = &file_handle;
+    open_params.size    = &filesize;
+    if (GPHOTO2_CALL( open_file, &open_params ))
+    {
+	FIXME( "Failed to get file %u\n", idx);
 	return TWRC_FAILURE;
     }
-    ret = gp_file_get_data_and_size (file, (const char**)&filedata, &filesize);
-    if (ret < GP_OK) {
-	FIXME("Failed to get file data?\n");
+    filedata = malloc( filesize );
+    get_data_params.handle = file_handle;
+    get_data_params.data   = filedata;
+    get_data_params.size   = filesize;
+    if (GPHOTO2_CALL( get_file_data, &get_data_params ))
+    {
+        close_file( file_handle );
+        free( filedata );
 	return TWRC_FAILURE;
     }
 
     /* FIXME: Actually we might get other types than JPEG ... But only handle JPEG for now */
     if (filedata[0] != 0xff) {
-	ERR("File %s/%s might not be JPEG, cannot decode!\n", folder, filename);
+	ERR("File %u might not be JPEG, cannot decode!\n", idx);
     }
 
     /* This is basically so we can use in-memory data for jpeg decompression.
@@ -607,17 +593,18 @@ _get_gphoto2_file_as_DIB(
     xjsm.resync_to_restart	= _jpeg_resync_to_restart;
     xjsm.term_source	= _jpeg_term_source;
 
-    jd.err = pjpeg_std_error(&jerr);
+    jd.err = jpeg_std_error(&jerr);
     /* jpeg_create_decompress is a macro that expands to jpeg_CreateDecompress - see jpeglib.h
      * jpeg_create_decompress(&jd); */
-    pjpeg_CreateDecompress(&jd, JPEG_LIB_VERSION, sizeof(struct jpeg_decompress_struct));
+    jpeg_CreateDecompress(&jd, JPEG_LIB_VERSION, sizeof(struct jpeg_decompress_struct));
     jd.src = &xjsm;
-    ret=pjpeg_read_header(&jd,TRUE);
+    ret=jpeg_read_header(&jd,TRUE);
     jd.out_color_space = JCS_RGB;
-    pjpeg_start_decompress(&jd);
+    jpeg_start_decompress(&jd);
     if (ret != JPEG_HEADER_OK) {
 	ERR("Jpeg image in stream has bad format, read header returned %d.\n",ret);
-	gp_file_unref (file);
+        close_file( file_handle );
+        free( filedata );
 	return TWRC_FAILURE;
     }
 
@@ -636,14 +623,15 @@ _get_gphoto2_file_as_DIB(
     *hDIB = CreateDIBSection(0, &bmpInfo, DIB_RGB_COLORS, (LPVOID)&bits, 0, 0);
     if (!*hDIB) {
 	FIXME("Failed creating DIB.\n");
-	gp_file_unref (file);
+        close_file( file_handle );
+        free( filedata );
 	return TWRC_FAILURE;
     }
     samprow = HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,jd.output_width*jd.output_components);
     oldsamprow = samprow;
     while ( jd.output_scanline<jd.output_height ) {
         unsigned int i;
-        int x = pjpeg_read_scanlines(&jd,&samprow,1);
+        int x = jpeg_read_scanlines(&jd,&samprow,1);
 	if (x != 1) {
 	    FIXME("failed to read current scanline?\n");
 	    break;
@@ -658,7 +646,7 @@ _get_gphoto2_file_as_DIB(
 	samprow = oldsamprow;
     }
     HeapFree (GetProcessHeap(), 0, samprow);
-    gp_file_unref (file);
+    close_file( file_handle );
+    free( filedata );
     return TWRC_SUCCESS;
 }
-#endif
