@@ -436,7 +436,7 @@ DWORD WINAPI DECLSPEC_HOTPATCH WaitForMultipleObjectsEx( DWORD count, const HAND
     }
     for (i = 0; i < count; i++) hloc[i] = normalize_std_handle( handles[i] );
 
-    status = NtWaitForMultipleObjects( count, hloc, !wait_all, alertable,
+    status = NtWaitForMultipleObjects( count, hloc, wait_all ? WaitAll : WaitAny, alertable,
                                        get_nt_timeout( &time, timeout ) );
     if (HIWORD(status))  /* is it an error code? */
     {
@@ -1824,3 +1824,50 @@ __ASM_STDCALL_FUNC(InterlockedDecrement, 4,
                   "ret $4")
 
 #endif  /* __i386__ */
+
+
+/***********************************************************************
+ * Synchronization barrier functions
+ ***********************************************************************/
+
+
+/***********************************************************************
+ *           InitializeSynchronizationBarrier  (kernelbase.@)
+ */
+BOOL WINAPI InitializeSynchronizationBarrier( SYNCHRONIZATION_BARRIER *barrier, LONG thread_count, LONG spin_count )
+{
+    if (thread_count <= 0 || spin_count < -1)
+    {
+        SetLastError( ERROR_INVALID_PARAMETER );
+        return FALSE;
+    }
+    return set_ntstatus( RtlInitBarrier( barrier, thread_count, spin_count ));
+}
+
+
+/***********************************************************************
+ *           DeleteSynchronizationBarrier  (kernelbase.@)
+ */
+BOOL WINAPI DeleteSynchronizationBarrier( SYNCHRONIZATION_BARRIER *barrier )
+{
+    RtlDeleteBarrier( barrier );
+    return TRUE;
+}
+
+
+/***********************************************************************
+ *           EnterSynchronizationBarrier  (kernelbase.@)
+ */
+BOOL WINAPI EnterSynchronizationBarrier( SYNCHRONIZATION_BARRIER *barrier, DWORD flags )
+{
+    static const DWORD valid_flags = SYNCHRONIZATION_BARRIER_FLAGS_SPIN_ONLY | SYNCHRONIZATION_BARRIER_FLAGS_BLOCK_ONLY
+                                     | SYNCHRONIZATION_BARRIER_FLAGS_NO_DELETE;
+    static unsigned int once;
+
+    if (flags & ~valid_flags) RtlRaiseStatus( STATUS_INVALID_PARAMETER );
+
+    if (flags & ~SYNCHRONIZATION_BARRIER_FLAGS_NO_DELETE && !once++)
+        FIXME( "flags %#lx are not implemented.\n", flags );
+
+    return RtlBarrier( barrier, flags & SYNCHRONIZATION_BARRIER_FLAGS_NO_DELETE ? 0 : 0x10000 );
+}
