@@ -34,7 +34,6 @@
 #include "localspl_private.h"
 
 #include "wine/debug.h"
-#include "wine/heap.h"
 #include "wine/list.h"
 
 
@@ -66,6 +65,9 @@ static CRITICAL_SECTION xcv_handles_cs = { &xcv_handles_cs_debug, -1, 0, 0, 0, 0
 typedef struct {
     struct list entry;
     DWORD   type;
+    HANDLE  hfile;
+    DWORD   thread_id;
+    INT64   doc_handle;
     WCHAR   nameW[1];
 } port_t;
 
@@ -95,6 +97,9 @@ BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
         case DLL_PROCESS_ATTACH:
             DisableThreadLibraryCalls( hinstDLL );
             localspl_instance = hinstDLL;
+            if (__wine_init_unix_call())
+                return FALSE;
+            UNIX_CALL(process_attach, NULL);
             break;
     }
     return TRUE;
@@ -117,7 +122,7 @@ static BOOL does_port_exist(LPCWSTR myname)
     TRACE("(%s)\n", debugstr_w(myname));
 
     id = EnumPortsW(NULL, 1, NULL, 0, &needed, &returned);
-    pi = heap_alloc(needed);
+    pi = malloc(needed);
     returned = 0;
     if (pi)
         id = EnumPortsW(NULL, 1, (LPBYTE) pi, needed, &needed, &returned);
@@ -128,13 +133,13 @@ static BOOL does_port_exist(LPCWSTR myname)
         {
             if (lstrcmpiW(myname, pi[id].pName) == 0) {
                 TRACE("(%lu) found %s\n", id, debugstr_w(pi[id].pName));
-                heap_free(pi);
+                free(pi);
                 return TRUE;
             }
         }
     }
 
-    heap_free(pi);
+    free(pi);
     return FALSE;
 }
 
@@ -249,7 +254,7 @@ getports_cleanup:
  * 
  */
 
-static DWORD get_type_from_name(LPCWSTR name)
+static DWORD get_type_from_name(LPCWSTR name, BOOL check_filename)
 {
     HANDLE  hfile;
 
@@ -274,6 +279,9 @@ static DWORD get_type_from_name(LPCWSTR name)
     if (!wcsncmp(name, L"LPR:", ARRAY_SIZE(L"LPR:") - 1))
         return PORT_IS_LPR;
 
+    if (!check_filename)
+        return PORT_IS_UNKNOWN;
+
     /* Must be a file or a directory. Does the file exist ? */
     hfile = CreateFileW(name, GENERIC_WRITE, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
     TRACE("%p for OPEN_EXISTING on %s\n", hfile, debugstr_w(name));
@@ -301,31 +309,34 @@ static DWORD get_type_from_local_name(LPCWSTR nameW)
     LPWSTR  myname = NULL;
     DWORD   needed = 0;
     DWORD   numentries = 0;
-    DWORD   id = 0;
+    DWORD   id;
 
-    TRACE("(%s)\n", debugstr_w(myname));
+    TRACE("(%s)\n", debugstr_w(nameW));
+
+    if ((id = get_type_from_name(nameW, FALSE)) >= PORT_IS_WINE)
+        return id;
 
     needed = get_ports_from_reg(1, NULL, 0, &numentries);
-    pi = heap_alloc(needed);
+    pi = malloc(needed);
     if (pi)
         needed = get_ports_from_reg(1, (LPBYTE) pi, needed, &numentries);
 
     if (pi && needed && numentries > 0) {
         /* we got a number of valid ports. */
 
-        while ((myname == NULL) && (id < numentries))
+        for (id = 0; id < numentries; id++)
         {
             if (lstrcmpiW(nameW, pi[id].pName) == 0) {
                 TRACE("(%lu) found %s\n", id, debugstr_w(pi[id].pName));
                 myname = pi[id].pName;
+                break;
             }
-            id++;
         }
     }
 
-    id = (myname) ? get_type_from_name(myname) : PORT_IS_UNKNOWN;
+    id = myname ? get_type_from_name(myname, TRUE) : PORT_IS_UNKNOWN;
 
-    heap_free(pi);
+    free(pi);
     return id;
 
 }
@@ -386,31 +397,6 @@ static BOOL WINAPI localmon_AddPortExW(LPWSTR pName, DWORD level, LPBYTE pBuffer
     return (res == ERROR_SUCCESS);
 }
 
-/*****************************************************
- * localmon_ClosePort [exported through MONITOREX]
- *
- * Close a
- *
- * PARAMS
- *  hPort  [i] The Handle to close
- *
- * RETURNS
- *  Success: TRUE
- *  Failure: FALSE
- *
- */
-static BOOL WINAPI localmon_ClosePort(HANDLE hPort)
-{
-    port_t * port = hPort;
-
-    TRACE("(%p)\n", port);
-    EnterCriticalSection(&port_handles_cs);
-    list_remove(&port->entry);
-    LeaveCriticalSection(&port_handles_cs);
-    heap_free(port);
-    return TRUE;
-}
-
 /*****************************************************
  *   localmon_EnumPortsW [exported through MONITOREX]
  *
@@ -498,10 +484,12 @@ static BOOL WINAPI localmon_OpenPortW(LPWSTR pName, PHANDLE phPort)
     type = get_type_from_local_name(pName);
     if (!type) return FALSE;
 
-    port = heap_alloc(FIELD_OFFSET(port_t, nameW[lstrlenW(pName) + 1]));
+    port = malloc(FIELD_OFFSET(port_t, nameW[wcslen(pName) + 1]));
     if (!port) return FALSE;
 
     port->type = type;
+    port->hfile = INVALID_HANDLE_VALUE;
+    port->doc_handle = 0;
     lstrcpyW(port->nameW, pName);
     *phPort = port;
 
@@ -513,6 +501,145 @@ static BOOL WINAPI localmon_OpenPortW(LPWSTR pName, PHANDLE phPort)
     return TRUE;
 }
 
+static BOOL WINAPI localmon_StartDocPort(HANDLE hport, WCHAR *printer_name,
+        DWORD job_id, DWORD level, BYTE *info)
+{
+    DOC_INFO_1W *doc_info = (DOC_INFO_1W *)info;
+    port_t *port = hport;
+
+    TRACE("(%p %s %ld %ld %p)\n", hport, debugstr_w(printer_name),
+            job_id, level, doc_info);
+
+    if (port->type >= PORT_IS_WINE)
+    {
+        struct start_doc_params params;
+
+        if (port->doc_handle)
+            return TRUE;
+
+        port->thread_id = GetCurrentThreadId();
+
+        params.type = port->type;
+        params.port = port->nameW;
+        params.document_title = doc_info ? doc_info->pDocName : NULL;
+        params.doc = &port->doc_handle;
+        return UNIX_CALL(start_doc, &params);
+    }
+
+    if (port->type != PORT_IS_FILE)
+    {
+        SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
+        return FALSE;
+    }
+
+    if (port->hfile != INVALID_HANDLE_VALUE)
+        return TRUE;
+
+    if (!doc_info || !doc_info->pOutputFile)
+    {
+        SetLastError(ERROR_INVALID_PARAMETER);
+        return FALSE;
+    }
+
+    port->hfile = CreateFileW(doc_info->pOutputFile, GENERIC_WRITE,
+            FILE_SHARE_READ, NULL, CREATE_ALWAYS, 0, NULL);
+    return port->hfile != INVALID_HANDLE_VALUE;
+}
+
+static BOOL WINAPI localmon_WritePort(HANDLE hport, BYTE *buf, DWORD size,
+        DWORD *written)
+{
+    port_t *port = hport;
+
+    TRACE("(%p %p %lu %p)\n", hport, buf, size, written);
+
+    if (port->type >= PORT_IS_WINE)
+    {
+        struct write_doc_params params;
+        BOOL ret;
+
+        if (!port->doc_handle)
+        {
+            SetLastError(ERROR_INVALID_HANDLE);
+            return FALSE;
+        }
+
+        if (port->type == PORT_IS_CUPS && port->thread_id != GetCurrentThreadId())
+        {
+            FIXME("used from other thread\n");
+            SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
+            return FALSE;
+        }
+
+        params.doc = port->doc_handle;
+        params.buf = buf;
+        params.size = size;
+        ret = UNIX_CALL(write_doc, &params);
+        *written = ret ? size : 0;
+        return ret;
+    }
+
+    return WriteFile(port->hfile, buf, size, written, NULL);
+}
+
+static BOOL WINAPI localmon_EndDocPort(HANDLE hport)
+{
+    port_t *port = hport;
+
+    TRACE("(%p)\n", hport);
+
+    if (port->type >= PORT_IS_WINE)
+    {
+        struct end_doc_params params;
+
+        if (!port->doc_handle)
+            return TRUE;
+
+        if (port->type == PORT_IS_CUPS && port->thread_id != GetCurrentThreadId())
+        {
+            FIXME("used from other thread\n");
+            SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
+            return FALSE;
+        }
+
+        params.doc = port->doc_handle;
+        port->doc_handle = 0;
+        return UNIX_CALL(end_doc, &params);
+    }
+
+    CloseHandle(port->hfile);
+    port->hfile = INVALID_HANDLE_VALUE;
+    return TRUE;
+}
+
+/*****************************************************
+ * localmon_ClosePort [exported through MONITOREX]
+ *
+ * Close a Port
+ *
+ * PARAMS
+ *  hport  [i] The Handle to close
+ *
+ * RETURNS
+ *  Success: TRUE
+ *  Failure: FALSE
+ *
+ */
+static BOOL WINAPI localmon_ClosePort(HANDLE hport)
+{
+    port_t *port = hport;
+
+    TRACE("(%p)\n", port);
+
+    localmon_EndDocPort(hport);
+
+    EnterCriticalSection(&port_handles_cs);
+    list_remove(&port->entry);
+    LeaveCriticalSection(&port_handles_cs);
+    free(port);
+    return TRUE;
+}
+
 /*****************************************************
  * localmon_XcvClosePort [exported through MONITOREX]
  *
@@ -535,7 +662,7 @@ static BOOL WINAPI localmon_XcvClosePort(HANDLE hXcv)
     EnterCriticalSection(&xcv_handles_cs);
     list_remove(&xcv->entry);
     LeaveCriticalSection(&xcv_handles_cs);
-    heap_free(xcv);
+    free(xcv);
     return TRUE;
 }
 
@@ -658,7 +785,7 @@ static DWORD WINAPI localmon_XcvDataPort(HANDLE hXcv, LPCWSTR pszDataName, PBYTE
 
     if (!lstrcmpW(pszDataName, L"PortIsValid")) {
         TRACE("InputData (%ld): %s\n", cbInputData, debugstr_w( (LPWSTR) pInputData));
-        res = get_type_from_name((LPCWSTR) pInputData);
+        res = get_type_from_name((LPCWSTR) pInputData, TRUE);
         TRACE("detected as %lu\n",  res);
         /* names, that we have recognized, are valid */
         if (res) return ERROR_SUCCESS;
@@ -679,7 +806,7 @@ static DWORD WINAPI localmon_XcvDataPort(HANDLE hXcv, LPCWSTR pszDataName, PBYTE
             ptr =  ((xcv_t *)hXcv)->nameW;
         }
         lstrcpynW(buffer, ptr, ARRAY_SIZE(buffer));
-        if (buffer[0]) buffer[lstrlenW(buffer)-1] = '\0';  /* remove the ':' */
+        if (buffer[0]) buffer[wcslen(buffer)-1] = '\0';  /* remove the ':' */
         res = SetDefaultCommConfigW(buffer, (LPCOMMCONFIG) pInputData, cbInputData);
         TRACE("got %lu with %lu\n", res, GetLastError() );
         return res ? ERROR_SUCCESS : GetLastError();
@@ -710,7 +837,7 @@ static BOOL WINAPI localmon_XcvOpenPort(LPCWSTR pName, ACCESS_MASK GrantedAccess
 
     TRACE("%s, 0x%lx, %p)\n", debugstr_w(pName), GrantedAccess, phXcv);
     /* No checks for any field is done in Windows */
-    xcv = heap_alloc(FIELD_OFFSET(xcv_t, nameW[lstrlenW(pName) + 1]));
+    xcv = malloc(FIELD_OFFSET(xcv_t, nameW[wcslen(pName) + 1]));
     if (xcv) {
         xcv->GrantedAccess = GrantedAccess;
         lstrcpyW(xcv->nameW, pName);
@@ -756,10 +883,10 @@ LPMONITOREX WINAPI InitializePrintMonitor(LPWSTR regroot)
             localmon_EnumPortsW,
             localmon_OpenPortW,
             NULL,       /* localmon_OpenPortExW */ 
-            NULL,       /* localmon_StartDocPortW */
-            NULL,       /* localmon_WritePortW */
+            localmon_StartDocPort,
+            localmon_WritePort,
             NULL,       /* localmon_ReadPortW */
-            NULL,       /* localmon_EndDocPortW */
+            localmon_EndDocPort,
             localmon_ClosePort,
             NULL,       /* Use AddPortUI in localui.dll */
             localmon_AddPortExW,
