@@ -582,9 +582,7 @@ static BOOL compile_cs(const WCHAR *source, const WCHAR *target, const WCHAR *ty
     ret = CreateProcessW(csc, cmdline, NULL, NULL, FALSE, DETACHED_PROCESS, NULL, NULL, &si, &pi);
     ok(ret, "Could not create process: %lu\n", GetLastError());
 
-    wait_child_process(pi.hProcess);
-    CloseHandle(pi.hThread);
-    CloseHandle(pi.hProcess);
+    wait_child_process(&pi);
 
     ret = PathFileExistsW(target);
     ok(ret, "Compilation failed\n");
