@@ -38,19 +38,11 @@ struct HTMLFormElement {
     HTMLElement element;
 
     IHTMLFormElement IHTMLFormElement_iface;
+    IWineHTMLFormPrivate IWineHTMLFormPrivate_iface;
 
     nsIDOMHTMLFormElement *nsform;
 };
 
-typedef struct {
-    IEnumVARIANT IEnumVARIANT_iface;
-
-    LONG ref;
-
-    ULONG iter;
-    HTMLFormElement *elem;
-} HTMLFormElementEnum;
-
 HRESULT return_nsform(nsresult nsres, nsIDOMHTMLFormElement *form, IHTMLFormElement **p)
 {
     nsIDOMNode *form_node;
@@ -118,135 +110,6 @@ static HRESULT htmlform_item(HTMLFormElement *This, int i, IDispatch **ret)
     return S_OK;
 }
 
-static inline HTMLFormElementEnum *impl_from_IEnumVARIANT(IEnumVARIANT *iface)
-{
-    return CONTAINING_RECORD(iface, HTMLFormElementEnum, IEnumVARIANT_iface);
-}
-
-static HRESULT WINAPI HTMLFormElementEnum_QueryInterface(IEnumVARIANT *iface, REFIID riid, void **ppv)
-{
-    HTMLFormElementEnum *This = impl_from_IEnumVARIANT(iface);
-
-    TRACE("(%p)->(%s %p)\n", This, debugstr_mshtml_guid(riid), ppv);
-
-    if(IsEqualGUID(riid, &IID_IUnknown)) {
-        *ppv = &This->IEnumVARIANT_iface;
-    }else if(IsEqualGUID(riid, &IID_IEnumVARIANT)) {
-        *ppv = &This->IEnumVARIANT_iface;
-    }else {
-        FIXME("(%p)->(%s %p)\n", This, debugstr_mshtml_guid(riid), ppv);
-        *ppv = NULL;
-        return E_NOINTERFACE;
-    }
-
-    IUnknown_AddRef((IUnknown*)*ppv);
-    return S_OK;
-}
-
-static ULONG WINAPI HTMLFormElementEnum_AddRef(IEnumVARIANT *iface)
-{
-    HTMLFormElementEnum *This = impl_from_IEnumVARIANT(iface);
-    LONG ref = InterlockedIncrement(&This->ref);
-
-    TRACE("(%p) ref=%ld\n", This, ref);
-
-    return ref;
-}
-
-static ULONG WINAPI HTMLFormElementEnum_Release(IEnumVARIANT *iface)
-{
-    HTMLFormElementEnum *This = impl_from_IEnumVARIANT(iface);
-    LONG ref = InterlockedDecrement(&This->ref);
-
-    TRACE("(%p) ref=%ld\n", This, ref);
-
-    if(!ref) {
-        IHTMLFormElement_Release(&This->elem->IHTMLFormElement_iface);
-        free(This);
-    }
-
-    return ref;
-}
-
-static HRESULT WINAPI HTMLFormElementEnum_Next(IEnumVARIANT *iface, ULONG celt, VARIANT *rgVar, ULONG *pCeltFetched)
-{
-    HTMLFormElementEnum *This = impl_from_IEnumVARIANT(iface);
-    nsresult nsres;
-    HRESULT hres;
-    ULONG num, i;
-    LONG len;
-
-    TRACE("(%p)->(%lu %p %p)\n", This, celt, rgVar, pCeltFetched);
-
-    nsres = nsIDOMHTMLFormElement_GetLength(This->elem->nsform, &len);
-    if(NS_FAILED(nsres))
-        return E_FAIL;
-    num = min(len - This->iter, celt);
-
-    for(i = 0; i < num; i++) {
-        hres = htmlform_item(This->elem, This->iter + i, &V_DISPATCH(&rgVar[i]));
-        if(FAILED(hres)) {
-            while(i--)
-                VariantClear(&rgVar[i]);
-            return hres;
-        }
-        V_VT(&rgVar[i]) = VT_DISPATCH;
-    }
-
-    This->iter += num;
-    if(pCeltFetched)
-        *pCeltFetched = num;
-    return num == celt ? S_OK : S_FALSE;
-}
-
-static HRESULT WINAPI HTMLFormElementEnum_Skip(IEnumVARIANT *iface, ULONG celt)
-{
-    HTMLFormElementEnum *This = impl_from_IEnumVARIANT(iface);
-    nsresult nsres;
-    LONG len;
-
-    TRACE("(%p)->(%lu)\n", This, celt);
-
-    nsres = nsIDOMHTMLFormElement_GetLength(This->elem->nsform, &len);
-    if(NS_FAILED(nsres))
-        return E_FAIL;
-
-    if(This->iter + celt > len) {
-        This->iter = len;
-        return S_FALSE;
-    }
-
-    This->iter += celt;
-    return S_OK;
-}
-
-static HRESULT WINAPI HTMLFormElementEnum_Reset(IEnumVARIANT *iface)
-{
-    HTMLFormElementEnum *This = impl_from_IEnumVARIANT(iface);
-
-    TRACE("(%p)->()\n", This);
-
-    This->iter = 0;
-    return S_OK;
-}
-
-static HRESULT WINAPI HTMLFormElementEnum_Clone(IEnumVARIANT *iface, IEnumVARIANT **ppEnum)
-{
-    HTMLFormElementEnum *This = impl_from_IEnumVARIANT(iface);
-    FIXME("(%p)->(%p)\n", This, ppEnum);
-    return E_NOTIMPL;
-}
-
-static const IEnumVARIANTVtbl HTMLFormElementEnumVtbl = {
-    HTMLFormElementEnum_QueryInterface,
-    HTMLFormElementEnum_AddRef,
-    HTMLFormElementEnum_Release,
-    HTMLFormElementEnum_Next,
-    HTMLFormElementEnum_Skip,
-    HTMLFormElementEnum_Reset,
-    HTMLFormElementEnum_Clone
-};
-
 static inline HTMLFormElement *impl_from_IHTMLFormElement(IHTMLFormElement *iface)
 {
     return CONTAINING_RECORD(iface, HTMLFormElement, IHTMLFormElement_iface);
@@ -633,23 +496,10 @@ static HRESULT WINAPI HTMLFormElement_get_length(IHTMLFormElement *iface, LONG *
 static HRESULT WINAPI HTMLFormElement__newEnum(IHTMLFormElement *iface, IUnknown **p)
 {
     HTMLFormElement *This = impl_from_IHTMLFormElement(iface);
-    HTMLFormElementEnum *ret;
 
     TRACE("(%p)->(%p)\n", This, p);
 
-    ret = malloc(sizeof(*ret));
-    if(!ret)
-        return E_OUTOFMEMORY;
-
-    ret->IEnumVARIANT_iface.lpVtbl = &HTMLFormElementEnumVtbl;
-    ret->ref = 1;
-    ret->iter = 0;
-
-    HTMLFormElement_AddRef(&This->IHTMLFormElement_iface);
-    ret->elem = This;
-
-    *p = (IUnknown*)&ret->IEnumVARIANT_iface;
-    return S_OK;
+    return create_enum_variant(&This->element.node.event_target.dispex, p);
 }
 
 static HRESULT WINAPI HTMLFormElement_item(IHTMLFormElement *iface, VARIANT name,
@@ -718,6 +568,68 @@ static const IHTMLFormElementVtbl HTMLFormElementVtbl = {
     HTMLFormElement_tags
 };
 
+static inline HTMLFormElement *impl_from_IWineHTMLFormPrivateVtbl(IWineHTMLFormPrivate *iface)
+{
+    return CONTAINING_RECORD(iface, HTMLFormElement, IWineHTMLFormPrivate_iface);
+}
+
+DISPEX_IDISPATCH_IMPL(HTMLFormElement_private, IWineHTMLFormPrivate,
+                      impl_from_IWineHTMLFormPrivateVtbl(iface)->element.node.event_target.dispex)
+
+static HRESULT WINAPI HTMLFormElement_private_put_enctype(IWineHTMLFormPrivate *iface, BSTR v)
+{
+    HTMLFormElement *This = impl_from_IWineHTMLFormPrivateVtbl(iface);
+
+    TRACE("(%p)->(%s)\n", This, debugstr_w(v));
+
+    return IHTMLFormElement_put_encoding(&This->IHTMLFormElement_iface, v);
+}
+
+static HRESULT WINAPI HTMLFormElement_private_get_enctype(IWineHTMLFormPrivate *iface, BSTR *ret)
+{
+    HTMLFormElement *This = impl_from_IWineHTMLFormPrivateVtbl(iface);
+
+    TRACE("(%p)->(%p)\n", This, ret);
+
+    return IHTMLFormElement_get_encoding(&This->IHTMLFormElement_iface, ret);
+}
+
+static HRESULT WINAPI HTMLFormElement_private_put_noValidate(IWineHTMLFormPrivate *iface, VARIANT_BOOL v)
+{
+    HTMLFormElement *This = impl_from_IWineHTMLFormPrivateVtbl(iface);
+    FIXME("(%p)->(%x)\n", This, v);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI HTMLFormElement_private_get_noValidate(IWineHTMLFormPrivate *iface, VARIANT_BOOL *ret)
+{
+    HTMLFormElement *This = impl_from_IWineHTMLFormPrivateVtbl(iface);
+    FIXME("(%p)->(%p)\n", This, ret);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI HTMLFormElement_private_checkValidity(IWineHTMLFormPrivate *iface, VARIANT_BOOL *ret)
+{
+    HTMLFormElement *This = impl_from_IWineHTMLFormPrivateVtbl(iface);
+    FIXME("(%p)->(%p)\n", This, ret);
+    return E_NOTIMPL;
+}
+
+static const IWineHTMLFormPrivateVtbl WineHTMLFormPrivateVtbl = {
+    HTMLFormElement_private_QueryInterface,
+    HTMLFormElement_private_AddRef,
+    HTMLFormElement_private_Release,
+    HTMLFormElement_private_GetTypeInfoCount,
+    HTMLFormElement_private_GetTypeInfo,
+    HTMLFormElement_private_GetIDsOfNames,
+    HTMLFormElement_private_Invoke,
+    HTMLFormElement_private_put_enctype,
+    HTMLFormElement_private_get_enctype,
+    HTMLFormElement_private_put_noValidate,
+    HTMLFormElement_private_get_noValidate,
+    HTMLFormElement_private_checkValidity,
+};
+
 static inline HTMLFormElement *impl_from_DispatchEx(DispatchEx *iface)
 {
     return CONTAINING_RECORD(iface, HTMLFormElement, element.node.event_target.dispex);
@@ -731,6 +643,8 @@ static void *HTMLFormElement_query_interface(DispatchEx *dispex, REFIID riid)
         return &This->IHTMLFormElement_iface;
     if(IsEqualGUID(&DIID_DispHTMLFormElement, riid))
         return &This->IHTMLFormElement_iface;
+    if(IsEqualGUID(&IID_IWineHTMLFormPrivate, riid))
+        return &This->IWineHTMLFormPrivate_iface;
 
     return HTMLElement_query_interface(&This->element.node.event_target.dispex, riid);
 }
@@ -869,6 +783,24 @@ static HRESULT HTMLFormElement_invoke(DispatchEx *dispex, DISPID id, LCID lcid,
     return S_OK;
 }
 
+static ULONG HTMLFormElement_collection_len(DispatchEx *dispex)
+{
+    HTMLFormElement *This = impl_from_DispatchEx(dispex);
+    nsresult nsres;
+    LONG len;
+
+    nsres = nsIDOMHTMLFormElement_GetLength(This->nsform, &len);
+    assert(nsres == NS_OK);
+    return len;
+}
+
+static HRESULT HTMLFormElement_collection_item(DispatchEx *dispex, ULONG index, IDispatch **p)
+{
+    HTMLFormElement *This = impl_from_DispatchEx(dispex);
+
+    return htmlform_item(This, index, p);
+}
+
 static HRESULT HTMLFormElement_handle_event(DispatchEx *dispex, DOMEvent *event, BOOL *prevent_default)
 {
     HTMLFormElement *This = impl_from_DispatchEx(dispex);
@@ -898,7 +830,15 @@ static void HTMLFormElement_init_dispex_info(dispex_data_t *info, compat_mode_t
         {DISPID_COLLECTION,                .noattr = TRUE},
         {DISPID_UNKNOWN}
     };
+    static const dispex_hook_t private_ie9_hooks[] = {
+        {DISPID_IWINEHTMLFORMPRIVATE_NOVALIDATE},
+        {DISPID_IWINEHTMLFORMPRIVATE_CHECKVALIDITY},
+        {DISPID_UNKNOWN}
+    };
+
     dispex_info_add_interface(info, IHTMLFormElement_tid, hooks);
+    if(mode >= COMPAT_MODE_IE9)
+        dispex_info_add_interface(info, IWineHTMLFormPrivate_tid, mode < COMPAT_MODE_IE10 ? private_ie9_hooks : NULL);
 
     HTMLElement_init_dispex_info(info, mode);
 }
@@ -906,13 +846,15 @@ static void HTMLFormElement_init_dispex_info(dispex_data_t *info, compat_mode_t
 static const event_target_vtbl_t HTMLFormElement_event_target_vtbl = {
     {
         HTMLELEMENT_DISPEX_VTBL_ENTRIES,
-        .query_interface= HTMLFormElement_query_interface,
-        .destructor     = HTMLElement_destructor,
-        .traverse       = HTMLFormElement_traverse,
-        .unlink         = HTMLFormElement_unlink,
-        .get_dispid     = HTMLFormElement_get_dispid,
-        .get_prop_desc  = dispex_index_prop_desc,
-        .invoke         = HTMLFormElement_invoke
+        .query_interface = HTMLFormElement_query_interface,
+        .destructor      = HTMLElement_destructor,
+        .traverse        = HTMLFormElement_traverse,
+        .unlink          = HTMLFormElement_unlink,
+        .get_dispid      = HTMLFormElement_get_dispid,
+        .get_prop_desc   = dispex_index_prop_desc,
+        .invoke          = HTMLFormElement_invoke,
+        .collection_len  = HTMLFormElement_collection_len,
+        .collection_item = HTMLFormElement_collection_item,
     },
     HTMLELEMENT_EVENT_TARGET_VTBL_ENTRIES,
     .handle_event       = HTMLFormElement_handle_event
@@ -936,6 +878,7 @@ HRESULT HTMLFormElement_Create(HTMLDocumentNode *doc, nsIDOMElement *nselem, HTM
         return E_OUTOFMEMORY;
 
     ret->IHTMLFormElement_iface.lpVtbl = &HTMLFormElementVtbl;
+    ret->IWineHTMLFormPrivate_iface.lpVtbl = &WineHTMLFormPrivateVtbl;
     ret->element.node.vtbl = &HTMLFormElementImplVtbl;
 
     HTMLElement_Init(&ret->element, doc, nselem, &HTMLFormElement_dispex);
