@@ -29,6 +29,7 @@
 #include "wine/debug.h"
 
 #include "mshtml_private.h"
+#include "htmlevent.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(mshtml);
 
@@ -43,57 +44,8 @@ static inline HTMLGenericElement *impl_from_IHTMLGenericElement(IHTMLGenericElem
     return CONTAINING_RECORD(iface, HTMLGenericElement, IHTMLGenericElement_iface);
 }
 
-static HRESULT WINAPI HTMLGenericElement_QueryInterface(IHTMLGenericElement *iface, REFIID riid, void **ppv)
-{
-    HTMLGenericElement *This = impl_from_IHTMLGenericElement(iface);
-
-    return IHTMLDOMNode_QueryInterface(&This->element.node.IHTMLDOMNode_iface, riid, ppv);
-}
-
-static ULONG WINAPI HTMLGenericElement_AddRef(IHTMLGenericElement *iface)
-{
-    HTMLGenericElement *This = impl_from_IHTMLGenericElement(iface);
-
-    return IHTMLDOMNode_AddRef(&This->element.node.IHTMLDOMNode_iface);
-}
-
-static ULONG WINAPI HTMLGenericElement_Release(IHTMLGenericElement *iface)
-{
-    HTMLGenericElement *This = impl_from_IHTMLGenericElement(iface);
-
-    return IHTMLDOMNode_Release(&This->element.node.IHTMLDOMNode_iface);
-}
-
-static HRESULT WINAPI HTMLGenericElement_GetTypeInfoCount(IHTMLGenericElement *iface, UINT *pctinfo)
-{
-    HTMLGenericElement *This = impl_from_IHTMLGenericElement(iface);
-    return IDispatchEx_GetTypeInfoCount(&This->element.node.event_target.dispex.IDispatchEx_iface, pctinfo);
-}
-
-static HRESULT WINAPI HTMLGenericElement_GetTypeInfo(IHTMLGenericElement *iface, UINT iTInfo,
-                                              LCID lcid, ITypeInfo **ppTInfo)
-{
-    HTMLGenericElement *This = impl_from_IHTMLGenericElement(iface);
-    return IDispatchEx_GetTypeInfo(&This->element.node.event_target.dispex.IDispatchEx_iface, iTInfo, lcid,
-            ppTInfo);
-}
-
-static HRESULT WINAPI HTMLGenericElement_GetIDsOfNames(IHTMLGenericElement *iface, REFIID riid,
-        LPOLESTR *rgszNames, UINT cNames, LCID lcid, DISPID *rgDispId)
-{
-    HTMLGenericElement *This = impl_from_IHTMLGenericElement(iface);
-    return IDispatchEx_GetIDsOfNames(&This->element.node.event_target.dispex.IDispatchEx_iface, riid, rgszNames,
-            cNames, lcid, rgDispId);
-}
-
-static HRESULT WINAPI HTMLGenericElement_Invoke(IHTMLGenericElement *iface, DISPID dispIdMember,
-        REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS *pDispParams,
-        VARIANT *pVarResult, EXCEPINFO *pExcepInfo, UINT *puArgErr)
-{
-    HTMLGenericElement *This = impl_from_IHTMLGenericElement(iface);
-    return IDispatchEx_Invoke(&This->element.node.event_target.dispex.IDispatchEx_iface, dispIdMember, riid,
-            lcid, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
-}
+DISPEX_IDISPATCH_IMPL(HTMLGenericElement, IHTMLGenericElement,
+                      impl_from_IHTMLGenericElement(iface)->element.node.event_target.dispex)
 
 static HRESULT WINAPI HTMLGenericElement_get_recordset(IHTMLGenericElement *iface, IDispatch **p)
 {
@@ -122,70 +74,66 @@ static const IHTMLGenericElementVtbl HTMLGenericElementVtbl = {
     HTMLGenericElement_namedRecordset
 };
 
-static inline HTMLGenericElement *impl_from_HTMLDOMNode(HTMLDOMNode *iface)
+static inline HTMLGenericElement *impl_from_DispatchEx(DispatchEx *iface)
 {
-    return CONTAINING_RECORD(iface, HTMLGenericElement, element.node);
+    return CONTAINING_RECORD(iface, HTMLGenericElement, element.node.event_target.dispex);
 }
 
-static HRESULT HTMLGenericElement_QI(HTMLDOMNode *iface, REFIID riid, void **ppv)
+static void *HTMLGenericElement_query_interface(DispatchEx *dispex, REFIID riid)
 {
-    HTMLGenericElement *This = impl_from_HTMLDOMNode(iface);
+    HTMLGenericElement *This = impl_from_DispatchEx(dispex);
 
-    *ppv = NULL;
+    if(IsEqualGUID(&IID_IHTMLGenericElement, riid))
+        return &This->IHTMLGenericElement_iface;
 
-    if(IsEqualGUID(&IID_IHTMLGenericElement, riid)) {
-        TRACE("(%p)->(IID_IHTMLGenericElement %p)\n", This, ppv);
-        *ppv = &This->IHTMLGenericElement_iface;
-    }else {
-        return HTMLElement_QI(&This->element.node, riid, ppv);
-    }
-
-    IUnknown_AddRef((IUnknown*)*ppv);
-    return S_OK;
-}
-
-static void HTMLGenericElement_destructor(HTMLDOMNode *iface)
-{
-    HTMLGenericElement *This = impl_from_HTMLDOMNode(iface);
-
-    HTMLElement_destructor(&This->element.node);
+    return HTMLElement_query_interface(&This->element.node.event_target.dispex, riid);
 }
 
 static const NodeImplVtbl HTMLGenericElementImplVtbl = {
-    &CLSID_HTMLGenericElement,
-    HTMLGenericElement_QI,
-    HTMLGenericElement_destructor,
-    HTMLElement_cpc,
-    HTMLElement_clone,
-    HTMLElement_handle_event,
-    HTMLElement_get_attr_col
+    .clsid                 = &CLSID_HTMLGenericElement,
+    .cpc_entries           = HTMLElement_cpc,
+    .clone                 = HTMLElement_clone,
+    .get_attr_col          = HTMLElement_get_attr_col
 };
 
-static const tid_t HTMLGenericElement_iface_tids[] = {
-    HTMLELEMENT_TIDS,
+static const event_target_vtbl_t HTMLGenericElement_event_target_vtbl = {
+    {
+        HTMLELEMENT_DISPEX_VTBL_ENTRIES,
+        .query_interface= HTMLGenericElement_query_interface,
+        .destructor     = HTMLElement_destructor,
+        .traverse       = HTMLElement_traverse,
+        .unlink         = HTMLElement_unlink
+    },
+    HTMLELEMENT_EVENT_TARGET_VTBL_ENTRIES,
+    .handle_event       = HTMLElement_handle_event
+};
+
+static const tid_t HTMLUnknownElement_iface_tids[] = {
     IHTMLGenericElement_tid,
     0
 };
 
-static dispex_static_data_t HTMLGenericElement_dispex = {
-    NULL,
-    DispHTMLGenericElement_tid,
-    HTMLGenericElement_iface_tids,
-    HTMLElement_init_dispex_info
+dispex_static_data_t HTMLUnknownElement_dispex = {
+    .id           = OBJID_HTMLUnknownElement,
+    .prototype_id = OBJID_HTMLElement,
+    .vtbl         = &HTMLGenericElement_event_target_vtbl.dispex_vtbl,
+    .disp_tid     = DispHTMLGenericElement_tid,
+    .iface_tids   = HTMLUnknownElement_iface_tids,
+    .init_info    = HTMLElement_init_dispex_info,
 };
 
 HRESULT HTMLGenericElement_Create(HTMLDocumentNode *doc, nsIDOMElement *nselem, HTMLElement **elem)
 {
     HTMLGenericElement *ret;
 
-    ret = heap_alloc_zero(sizeof(HTMLGenericElement));
+    ret = calloc(1, sizeof(HTMLGenericElement));
     if(!ret)
         return E_OUTOFMEMORY;
 
     ret->IHTMLGenericElement_iface.lpVtbl = &HTMLGenericElementVtbl;
     ret->element.node.vtbl = &HTMLGenericElementImplVtbl;
 
-    HTMLElement_Init(&ret->element, doc, nselem, &HTMLGenericElement_dispex);
+    HTMLElement_Init(&ret->element, doc, nselem, &HTMLUnknownElement_dispex);
 
     *elem = &ret->element;
     return S_OK;
