@@ -19,6 +19,7 @@
 #include <stdio.h>
 #include <process.h>
 
+#include "stdint.h"
 #include "windef.h"
 #include "winbase.h"
 #include "wine/test.h"
@@ -28,6 +29,38 @@ typedef struct
     SRWLOCK srwlock;
 } shared_mutex;
 
+enum tzdb_error
+{
+    TZDB_ERROR_SUCCESS,
+    TZDB_ERROR_WIN,
+    TZDB_ERROR_ICU,
+};
+
+struct tzdb_time_zones
+{
+    enum tzdb_error error;
+    char *ver;
+    unsigned int count;
+    char **names;
+    char **links;
+};
+
+struct tzdb_current_zone
+{
+    enum tzdb_error error;
+    char *name;
+};
+
+struct tzdb_leap_second
+{
+    uint16_t year;
+    uint16_t month;
+    uint16_t day;
+    uint16_t hour;
+    uint16_t negative;
+    uint16_t reserved;
+};
+
 static unsigned int (__stdcall *p___std_parallel_algorithms_hw_threads)(void);
 
 static void (__stdcall *p___std_bulk_submit_threadpool_work)(PTP_WORK, size_t);
@@ -41,6 +74,12 @@ static void (__stdcall *p___std_execution_wait_on_uchar)(void *, unsigned char);
 static void (__stdcall *p___std_execution_wake_by_address_all)(void *);
 static shared_mutex* (__stdcall *p___std_acquire_shared_mutex_for_instance)(void*);
 static void (__stdcall *p___std_release_shared_mutex_for_instance)(void*);
+static struct tzdb_time_zones * (__stdcall *p___std_tzdb_get_time_zones)(void);
+static void (__stdcall *p___std_tzdb_delete_time_zones)(struct tzdb_time_zones *);
+static struct tzdb_current_zone * (__stdcall *p___std_tzdb_get_current_zone)(void);
+static void (__stdcall *p___std_tzdb_delete_current_zone)(struct tzdb_current_zone *);
+static struct tzdb_leap_second * (__stdcall *p___std_tzdb_get_leap_seconds)(size_t, size_t *);
+static void (__stdcall *p___std_tzdb_delete_leap_seconds)(struct tzdb_leap_second *);
 
 #define SETNOFAIL(x,y) x = (void*)GetProcAddress(msvcp,y)
 #define SET(x,y) do { SETNOFAIL(x,y); ok(x != NULL, "Export '%s' not found\n", y); } while(0)
@@ -64,6 +103,12 @@ static HMODULE init(void)
     SET(p___std_execution_wake_by_address_all, "__std_execution_wake_by_address_all");
     SET(p___std_acquire_shared_mutex_for_instance, "__std_acquire_shared_mutex_for_instance");
     SET(p___std_release_shared_mutex_for_instance, "__std_release_shared_mutex_for_instance");
+    SET(p___std_tzdb_get_time_zones, "__std_tzdb_get_time_zones");
+    SET(p___std_tzdb_delete_time_zones, "__std_tzdb_delete_time_zones");
+    SET(p___std_tzdb_get_current_zone, "__std_tzdb_get_current_zone");
+    SET(p___std_tzdb_delete_current_zone, "__std_tzdb_delete_current_zone");
+    SET(p___std_tzdb_get_leap_seconds, "__std_tzdb_get_leap_seconds");
+    SET(p___std_tzdb_delete_leap_seconds, "__std_tzdb_delete_leap_seconds");
     return msvcp;
 }
 
@@ -325,6 +370,54 @@ static void test___std_acquire_shared_mutex_for_instance(void)
     p___std_release_shared_mutex_for_instance(NULL);
 }
 
+static void test___std_tzdb(void)
+{
+    struct tzdb_current_zone *c;
+    struct tzdb_time_zones *z;
+    void *leap_seconds;
+    unsigned int i;
+    size_t size;
+
+    if (!p___std_tzdb_get_time_zones)
+    {
+        win_skip("__std_tzdb_get_time_zones is not available, skipping tests.\n");
+        return;
+    }
+
+    z = p___std_tzdb_get_time_zones();
+    ok(!!z, "got NULL.\n");
+    ok(!z->error || broken(z->error == TZDB_ERROR_WIN && !z->ver && !z->count), "got %u.\n", z->error);
+    if (z->error)
+    {
+        win_skip("__std_tzdb_get_time_zones empty result, skipping remaining tests.\n");
+        p___std_tzdb_delete_time_zones(z);
+        return;
+    }
+    ok(!!z->ver, "got NULL.\n");
+    ok(z->count, "got 0.\n");
+
+    trace("ver %s.\n", debugstr_a(z->ver));
+    c = p___std_tzdb_get_current_zone();
+    ok(!!c, "got NULL.\n");
+    ok(!!c->name, "got NULL.\n");
+    ok(!c->error, "got %u.\n", c->error);
+    for (i = 0; i < z->count; ++i)
+    {
+        if (!strcmp(c->name, z->names[i]))
+            break;
+    }
+    ok(i < z->count, "current zone %s not found.\n", c->name);
+
+    p___std_tzdb_delete_current_zone(c);
+    p___std_tzdb_delete_time_zones(z);
+
+    size = 0xdeadbeef;
+    leap_seconds = p___std_tzdb_get_leap_seconds(100, &size);
+    ok(!leap_seconds, "got %p.\n", leap_seconds);
+    ok(!size, "got %#Ix.\n", size);
+    p___std_tzdb_delete_leap_seconds(leap_seconds);
+}
+
 START_TEST(msvcp140_atomic_wait)
 {
     HMODULE msvcp;
@@ -338,5 +431,6 @@ START_TEST(msvcp140_atomic_wait)
     test___std_atomic_wait_direct();
     test___std_execution();
     test___std_acquire_shared_mutex_for_instance();
+    test___std_tzdb();
     FreeLibrary(msvcp);
 }
