@@ -189,7 +189,7 @@ LRESULT VFWAPI ICGetInfo16(HIC16 hic, ICINFO16 * picinfo, DWORD cb)
 
     TRACE("(0x%08lx,%p,%ld)\n", (DWORD) hic, picinfo, cb);
     ret = ICSendMessage16(hic, ICM_GETINFO, (DWORD) picinfo, cb);
-    TRACE("	-> 0x%08lx\n", ret);
+    TRACE("	-> 0x%08Ix\n", ret);
     return ret;
 }
 
@@ -714,7 +714,7 @@ static  LRESULT CALLBACK  IC_Callback3216(DWORD pfn16, HIC hic, HDRVR hdrv, UINT
 
 #define MAX_THUNKS      32
 
-#include "pshpack1.h"
+#pragma pack(push,1)
 static struct msvideo_thunk
 {
     BYTE        popl_eax;        /* popl  %eax (return address) */
@@ -725,7 +725,7 @@ static struct msvideo_thunk
     DWORD       callback;
     HIC16       hIC16;           /* driver's handle */
 } *MSVIDEO_Thunks;
-#include "poppack.h"
+#pragma pack(pop)
 
 static CRITICAL_SECTION msvideo_cs;
 static CRITICAL_SECTION_DEBUG critsect_debug =
