@@ -20,15 +20,11 @@
  */
 #define COBJMACROS
 
-#include "config.h"
-
 #include <stdarg.h>
-#ifdef HAVE_LIBXML2
-# include <libxml/parser.h>
-# include <libxml/xmlerror.h>
-# include <libxml/SAX2.h>
-# include <libxml/parserInternals.h>
-#endif
+#include <libxml/parser.h>
+#include <libxml/xmlerror.h>
+#include <libxml/SAX2.h>
+#include <libxml/parserInternals.h>
 
 #include "windef.h"
 #include "winbase.h"
@@ -45,8 +41,6 @@
 
 #include "msxml_private.h"
 
-#ifdef HAVE_LIBXML2
-
 WINE_DEFAULT_DEBUG_CHANNEL(msxml);
 
 typedef enum
@@ -69,37 +63,6 @@ typedef enum
     LexicalHandlerParEntities    = 1 << 15
 } saxreader_feature;
 
-/* feature names */
-static const WCHAR FeatureExternalGeneralEntitiesW[] = {
-    'h','t','t','p',':','/','/','x','m','l','.','o','r','g','/','s','a','x','/',
-    'f','e','a','t','u','r','e','s','/','e','x','t','e','r','n','a','l','-','g','e','n','e','r','a','l',
-    '-','e','n','t','i','t','i','e','s',0
-};
-
-static const WCHAR FeatureExternalParameterEntitiesW[] = {
-    'h','t','t','p',':','/','/','x','m','l','.','o','r','g','/','s','a','x','/','f','e','a','t','u','r','e','s',
-    '/','e','x','t','e','r','n','a','l','-','p','a','r','a','m','e','t','e','r','-','e','n','t','i','t','i','e','s',0
-};
-
-static const WCHAR FeatureLexicalHandlerParEntitiesW[] = {
-    'h','t','t','p',':','/','/','x','m','l','.','o','r','g','/','s','a','x','/','f','e','a','t','u','r','e','s',
-    '/','l','e','x','i','c','a','l','-','h','a','n','d','l','e','r','/','p','a','r','a','m','e','t','e','r','-','e','n','t','i','t','i','e','s',0
-};
-
-static const WCHAR FeatureProhibitDTDW[] = {
-    'p','r','o','h','i','b','i','t','-','d','t','d',0
-};
-
-static const WCHAR FeatureNamespacesW[] = {
-    'h','t','t','p',':','/','/','x','m','l','.','o','r','g','/','s','a','x','/','f','e','a','t','u','r','e','s',
-    '/','n','a','m','e','s','p','a','c','e','s',0
-};
-
-static const WCHAR FeatureNamespacePrefixesW[] = {
-    'h','t','t','p',':','/','/','x','m','l','.','o','r','g','/','s','a','x','/','f','e','a','t','u','r','e','s',
-    '/','n','a','m','e','s','p','a','c','e','-','p','r','e','f','i','x','e','s',0
-};
-
 struct saxreader_feature_pair
 {
     saxreader_feature feature;
@@ -107,12 +70,14 @@ struct saxreader_feature_pair
 };
 
 static const struct saxreader_feature_pair saxreader_feature_map[] = {
-    { ExternalGeneralEntities, FeatureExternalGeneralEntitiesW },
-    { ExternalParameterEntities, FeatureExternalParameterEntitiesW },
-    { LexicalHandlerParEntities, FeatureLexicalHandlerParEntitiesW },
-    { NamespacePrefixes, FeatureNamespacePrefixesW },
-    { Namespaces, FeatureNamespacesW },
-    { ProhibitDTD, FeatureProhibitDTDW }
+    { ExhaustiveErrors, L"exhaustive-errors" },
+    { ExternalGeneralEntities, L"http://xml.org/sax/features/external-general-entities" },
+    { ExternalParameterEntities, L"http://xml.org/sax/features/external-parameter-entities" },
+    { LexicalHandlerParEntities, L"http://xml.org/sax/features/lexical-handler/parameter-entities" },
+    { NamespacePrefixes, L"http://xml.org/sax/features/namespace-prefixes" },
+    { Namespaces, L"http://xml.org/sax/features/namespaces" },
+    { ProhibitDTD, L"prohibit-dtd" },
+    { SchemaValidation, L"schema-validation" },
 };
 
 static saxreader_feature get_saxreader_feature(const WCHAR *name)
@@ -120,13 +85,13 @@ static saxreader_feature get_saxreader_feature(const WCHAR *name)
     int min, max, n, c;
 
     min = 0;
-    max = sizeof(saxreader_feature_map)/sizeof(struct saxreader_feature_pair) - 1;
+    max = ARRAY_SIZE(saxreader_feature_map) - 1;
 
     while (min <= max)
     {
         n = (min+max)/2;
 
-        c = strcmpW(saxreader_feature_map[n].name, name);
+        c = wcscmp(saxreader_feature_map[n].name, name);
         if (!c)
             return saxreader_feature_map[n].feature;
 
@@ -139,6 +104,8 @@ static saxreader_feature get_saxreader_feature(const WCHAR *name)
     return FeatureUnknown;
 }
 
+static const WCHAR empty_str;
+
 struct bstrpool
 {
     BSTR *pool;
@@ -364,50 +331,9 @@ static HRESULT saxreader_saxcharacters(saxlocator *locator, BSTR chars)
 }
 
 /* property names */
-static const WCHAR PropertyCharsetW[] = {
-    'c','h','a','r','s','e','t',0
-};
-static const WCHAR PropertyXmlDeclVersionW[] = {
-    'x','m','l','d','e','c','l','-','v','e','r','s','i','o','n',0
-};
-static const WCHAR PropertyDeclHandlerW[] = {
-    'h','t','t','p',':','/','/','x','m','l','.','o','r','g','/',
-    's','a','x','/','p','r','o','p','e','r','t','i','e','s','/',
-    'd','e','c','l','a','r','a','t','i','o','n',
-    '-','h','a','n','d','l','e','r',0
-};
-static const WCHAR PropertyDomNodeW[] = {
-    'h','t','t','p',':','/','/','x','m','l','.','o','r','g','/',
-    's','a','x','/','p','r','o','p','e','r','t','i','e','s','/',
-    'd','o','m','-','n','o','d','e',0
-};
-static const WCHAR PropertyInputSourceW[] = {
-    'i','n','p','u','t','-','s','o','u','r','c','e',0
-};
-static const WCHAR PropertyLexicalHandlerW[] = {
-    'h','t','t','p',':','/','/','x','m','l','.','o','r','g','/',
-    's','a','x','/','p','r','o','p','e','r','t','i','e','s','/',
-    'l','e','x','i','c','a','l','-','h','a','n','d','l','e','r',0
-};
-static const WCHAR PropertyMaxElementDepthW[] = {
-    'm','a','x','-','e','l','e','m','e','n','t','-','d','e','p','t','h',0
-};
-static const WCHAR PropertyMaxXMLSizeW[] = {
-    'm','a','x','-','x','m','l','-','s','i','z','e',0
-};
-static const WCHAR PropertySchemaDeclHandlerW[] = {
-    's','c','h','e','m','a','-','d','e','c','l','a','r','a','t','i','o','n','-',
-    'h','a','n','d','l','e','r',0
-};
-static const WCHAR PropertyXMLDeclEncodingW[] = {
-    'x','m','l','d','e','c','l','-','e','n','c','o','d','i','n','g',0
-};
-static const WCHAR PropertyXMLDeclStandaloneW[] = {
-    'x','m','l','d','e','c','l','-','s','t','a','n','d','a','l','o','n','e',0
-};
-static const WCHAR PropertyXMLDeclVersionW[] = {
-    'x','m','l','d','e','c','l','-','v','e','r','s','i','o','n',0
-};
+static const WCHAR PropertyDeclHandlerW[] = L"http://xml.org/sax/properties/declaration-handler";
+static const WCHAR PropertyDomNodeW[] = L"http://xml.org/sax/properties/dom-node";
+static const WCHAR PropertyLexicalHandlerW[] = L"http://xml.org/sax/properties/lexical-handler";
 
 static inline HRESULT set_feature_value(saxreader *reader, saxreader_feature feature, VARIANT_BOOL value)
 {
@@ -444,10 +370,10 @@ static BSTR build_qname(BSTR prefix, BSTR local)
         WCHAR *ptr;
 
         ptr = qname;
-        strcpyW(ptr, prefix);
+        lstrcpyW(ptr, prefix);
         ptr += SysStringLen(prefix);
         *ptr++ = ':';
-        strcpyW(ptr, local);
+        lstrcpyW(ptr, local);
         return qname;
     }
     else
@@ -460,13 +386,13 @@ static element_entry* alloc_element_entry(const xmlChar *local, const xmlChar *p
     element_entry *ret;
     int i;
 
-    ret = heap_alloc(sizeof(*ret));
+    ret = malloc(sizeof(*ret));
     if (!ret) return ret;
 
     ret->local  = bstr_from_xmlChar(local);
     ret->prefix = bstr_from_xmlChar(prefix);
     ret->qname  = build_qname(ret->prefix, ret->local);
-    ret->ns = nb_ns ? heap_alloc(nb_ns*sizeof(ns)) : NULL;
+    ret->ns = nb_ns ? malloc(nb_ns * sizeof(ns)) : NULL;
     ret->ns_count = nb_ns;
 
     for (i=0; i < nb_ns; i++)
@@ -492,8 +418,8 @@ static void free_element_entry(element_entry *element)
     SysFreeString(element->local);
     SysFreeString(element->qname);
 
-    heap_free(element->ns);
-    heap_free(element);
+    free(element->ns);
+    free(element);
 }
 
 static void push_element_ns(saxlocator *locator, element_entry *element)
@@ -524,7 +450,7 @@ static BSTR find_element_uri(saxlocator *locator, const xmlChar *uri)
     LIST_FOR_EACH_ENTRY(element, &locator->elements, element_entry, entry)
     {
         for (i=0; i < element->ns_count; i++)
-            if (!strcmpW(uriW, element->ns[i].uri))
+            if (!wcscmp(uriW, element->ns[i].uri))
             {
                 SysFreeString(uriW);
                 return element->ns[i].uri;
@@ -561,7 +487,7 @@ static BOOL bstr_pool_insert(struct bstrpool *pool, BSTR pool_entry)
 {
     if (!pool->pool)
     {
-        pool->pool = HeapAlloc(GetProcessHeap(), 0, 16 * sizeof(*pool->pool));
+        pool->pool = malloc(16 * sizeof(*pool->pool));
         if (!pool->pool)
             return FALSE;
 
@@ -570,12 +496,12 @@ static BOOL bstr_pool_insert(struct bstrpool *pool, BSTR pool_entry)
     }
     else if (pool->index == pool->len)
     {
-        BSTR *realloc = HeapReAlloc(GetProcessHeap(), 0, pool->pool, pool->len * 2 * sizeof(*realloc));
+        BSTR *new_pool = realloc(pool->pool, pool->len * 2 * sizeof(*new_pool));
 
-        if (!realloc)
+        if (!new_pool)
             return FALSE;
 
-        pool->pool = realloc;
+        pool->pool = new_pool;
         pool->len *= 2;
     }
 
@@ -590,7 +516,7 @@ static void free_bstr_pool(struct bstrpool *pool)
     for (i = 0; i < pool->index; i++)
         SysFreeString(pool->pool[i]);
 
-    HeapFree(GetProcessHeap(), 0, pool->pool);
+    free(pool->pool);
 
     pool->pool = NULL;
     pool->index = pool->len = 0;
@@ -668,7 +594,7 @@ static void format_error_message_from_id(saxlocator *This, HRESULT hr)
     {
         WCHAR msg[1024];
         if(!FormatMessageW(FORMAT_MESSAGE_FROM_SYSTEM,
-                    NULL, hr, 0, msg, sizeof(msg)/sizeof(msg[0]), NULL))
+                    NULL, hr, 0, msg, ARRAY_SIZE(msg), NULL))
         {
             FIXME("MSXML errors not yet supported.\n");
             msg[0] = '\0';
@@ -743,9 +669,7 @@ static HRESULT WINAPI ivbsaxattributes_GetTypeInfo(
     IVBSAXAttributes *iface,
     UINT iTInfo, LCID lcid, ITypeInfo** ppTInfo )
 {
-    saxlocator *This = impl_from_IVBSAXAttributes( iface );
-
-    TRACE("(%p)->(%u %u %p)\n", This, iTInfo, lcid, ppTInfo);
+    TRACE("%p, %u, %lx, %p.\n", iface, iTInfo, lcid, ppTInfo);
 
     return get_typeinfo(IVBSAXAttributes_tid, ppTInfo);
 }
@@ -758,11 +682,10 @@ static HRESULT WINAPI ivbsaxattributes_GetIDsOfNames(
     LCID lcid,
     DISPID* rgDispId)
 {
-    saxlocator *This = impl_from_IVBSAXAttributes( iface );
     ITypeInfo *typeinfo;
     HRESULT hr;
 
-    TRACE("(%p)->(%s %p %u %u %p)\n", This, debugstr_guid(riid), rgszNames, cNames,
+    TRACE("%p, %s, %p, %u, %lx %p.\n", iface, debugstr_guid(riid), rgszNames, cNames,
           lcid, rgDispId);
 
     if(!rgszNames || cNames == 0 || !rgDispId)
@@ -789,18 +712,16 @@ static HRESULT WINAPI ivbsaxattributes_Invoke(
     EXCEPINFO* pExcepInfo,
     UINT* puArgErr)
 {
-    saxlocator *This = impl_from_IVBSAXAttributes( iface );
     ITypeInfo *typeinfo;
     HRESULT hr;
 
-    TRACE("(%p)->(%d %s %d %d %p %p %p %p)\n", This, dispIdMember, debugstr_guid(riid),
+    TRACE("%p, %ld, %s, %lx, %d, %p, %p, %p, %p.\n", iface, dispIdMember, debugstr_guid(riid),
           lcid, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
 
     hr = get_typeinfo(IVBSAXAttributes_tid, &typeinfo);
     if(SUCCEEDED(hr))
     {
-        hr = ITypeInfo_Invoke(typeinfo, &This->IVBSAXAttributes_iface, dispIdMember, wFlags,
-                pDispParams, pVarResult, pExcepInfo, puArgErr);
+        hr = ITypeInfo_Invoke(typeinfo, iface, dispIdMember, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
         ITypeInfo_Release(typeinfo);
     }
 
@@ -1387,7 +1308,7 @@ static const struct ISAXAttributesVtbl isaxattributes_vtbl =
    escaped to '&#38;'. This function takes care of ampersands only. */
 static BSTR saxreader_get_unescaped_value(const xmlChar *buf, int len)
 {
-    static const WCHAR ampescW[] = {'&','#','3','8',';',0};
+    static const WCHAR ampescW[] = L"&#38;";
     WCHAR *dest, *ptrW, *str;
     DWORD str_len;
     BSTR bstr;
@@ -1398,29 +1319,29 @@ static BSTR saxreader_get_unescaped_value(const xmlChar *buf, int len)
     str_len = MultiByteToWideChar(CP_UTF8, 0, (LPCSTR)buf, len, NULL, 0);
     if (len != -1) str_len++;
 
-    str = heap_alloc(str_len*sizeof(WCHAR));
+    str = malloc(str_len * sizeof(WCHAR));
     if (!str) return NULL;
 
     MultiByteToWideChar(CP_UTF8, 0, (LPCSTR)buf, len, str, str_len);
     if (len != -1) str[str_len-1] = 0;
 
     ptrW = str;
-    while ((dest = strstrW(ptrW, ampescW)))
+    while ((dest = wcsstr(ptrW, ampescW)))
     {
         WCHAR *src;
 
         /* leave first '&' from a reference as a value */
-        src = dest + (sizeof(ampescW)/sizeof(WCHAR) - 1);
+        src = dest + ARRAY_SIZE(ampescW) - 1;
         dest++;
 
         /* move together with null terminator */
-        memmove(dest, src, (strlenW(src) + 1)*sizeof(WCHAR));
+        memmove(dest, src, (lstrlenW(src) + 1)*sizeof(WCHAR));
 
         ptrW++;
     }
 
     bstr = SysAllocString(str);
-    heap_free(str);
+    free(str);
 
     return bstr;
 }
@@ -1447,7 +1368,6 @@ static HRESULT SAXAttributes_populate(saxlocator *locator,
         int nb_attributes, const xmlChar **xmlAttributes)
 {
     static const xmlChar xmlns[] = "xmlns";
-    static const WCHAR xmlnsW[] = { 'x','m','l','n','s',0 };
 
     struct _attributes *attrs;
     int i;
@@ -1460,13 +1380,15 @@ static HRESULT SAXAttributes_populate(saxlocator *locator,
     if(locator->attr_count > locator->attr_alloc_count)
     {
         int new_size = locator->attr_count * 2;
-        attrs = heap_realloc_zero(locator->attributes, new_size * sizeof(struct _attributes));
+        attrs = realloc(locator->attributes, new_size * sizeof(*locator->attributes));
         if(!attrs)
         {
             free_attribute_values(locator);
             locator->attr_count = 0;
             return E_OUTOFMEMORY;
         }
+        memset(attrs + locator->attr_alloc_count, 0,
+                (new_size - locator->attr_alloc_count) * sizeof(*locator->attributes));
         locator->attributes = attrs;
         locator->attr_alloc_count = new_size;
     }
@@ -1487,7 +1409,7 @@ static HRESULT SAXAttributes_populate(saxlocator *locator,
 
         SysFreeString(attrs[nb_attributes+i].szQName);
         if(!xmlNamespaces[2*i])
-            attrs[nb_attributes+i].szQName = SysAllocString(xmlnsW);
+            attrs[nb_attributes+i].szQName = SysAllocString(L"xmlns");
         else
             attrs[nb_attributes+i].szQName = QName_from_xmlChar(xmlns, xmlNamespaces[2*i]);
     }
@@ -1655,8 +1577,8 @@ static void libxmlStartElementNS(
                     &uri, &local, &element->qname, &This->IVBSAXAttributes_iface);
         else
             hr = ISAXContentHandler_startElement(handler->handler,
-                    uri, SysStringLen(uri),
-                    local, SysStringLen(local),
+                    uri ? uri : &empty_str, SysStringLen(uri),
+                    local ? local : &empty_str, SysStringLen(local),
                     element->qname, SysStringLen(element->qname),
                     &This->ISAXAttributes_iface);
 
@@ -1729,8 +1651,8 @@ static void libxmlEndElementNS(
     else
         hr = ISAXContentHandler_endElement(
                 handler->handler,
-                uri, SysStringLen(uri),
-                local, SysStringLen(local),
+                uri ? uri : &empty_str, SysStringLen(uri),
+                local ? local : &empty_str, SysStringLen(local),
                 element->qname, SysStringLen(element->qname));
 
     free_attribute_values(This);
@@ -1910,7 +1832,7 @@ static void libxmlComment(void *ctx, const xmlChar *value)
         format_error_message_from_id(This, hr);
 }
 
-static void libxmlFatalError(void *ctx, const char *msg, ...)
+static void WINAPIV libxmlFatalError(void *ctx, const char *msg, ...)
 {
     saxlocator *This = ctx;
     struct saxerrorhandler_iface *handler = saxreader_get_errorhandler(This->saxreader);
@@ -1929,7 +1851,7 @@ static void libxmlFatalError(void *ctx, const char *msg, ...)
     va_end(args);
 
     len = MultiByteToWideChar(CP_UNIXCP, 0, message, -1, NULL, 0);
-    error = heap_alloc(sizeof(WCHAR)*len);
+    error = malloc(sizeof(WCHAR) * len);
     if(error)
     {
         MultiByteToWideChar(CP_UNIXCP, 0, message, -1, error, len);
@@ -1940,7 +1862,7 @@ static void libxmlFatalError(void *ctx, const char *msg, ...)
     {
         xmlStopParser(This->pParserCtxt);
         This->ret = E_FAIL;
-        heap_free(error);
+        free(error);
         return;
     }
 
@@ -1956,7 +1878,7 @@ static void libxmlFatalError(void *ctx, const char *msg, ...)
     else
         ISAXErrorHandler_fatalError(handler->handler, &This->ISAXLocator_iface, error, E_FAIL);
 
-    heap_free(error);
+    free(error);
 
     xmlStopParser(This->pParserCtxt);
     This->ret = E_FAIL;
@@ -2078,6 +2000,71 @@ static void libxml_cdatablock(void *ctx, const xmlChar *value, int len)
         format_error_message_from_id(locator, hr);
 }
 
+static void libxml_pi(void *ctx, const xmlChar *_target, const xmlChar *_data)
+{
+    saxlocator *locator = ctx;
+    struct saxcontenthandler_iface *handler = saxreader_get_contenthandler(locator->saxreader);
+    BSTR target, data;
+    HRESULT hr = S_OK;
+
+    update_position(locator, FALSE);
+    if (!saxreader_has_handler(locator, SAXContentHandler)) return;
+
+    target = pooled_bstr_from_xmlChar(&locator->saxreader->pool, _target);
+    data = pooled_bstr_from_xmlChar(&locator->saxreader->pool, _data);
+
+    if (locator->vbInterface)
+        hr = IVBSAXContentHandler_processingInstruction(handler->vbhandler, &target, &data);
+    else
+        hr = ISAXContentHandler_processingInstruction(handler->handler, target, SysStringLen(target),
+                data, SysStringLen(data));
+
+    if (FAILED(hr))
+        format_error_message_from_id(locator, hr);
+}
+
+static void libxml_internalsubset(void *ctx, const xmlChar *_name, const xmlChar *external_id, const xmlChar *system_id)
+{
+    saxlocator *locator = ctx;
+    struct saxlexicalhandler_iface *lexical = saxreader_get_lexicalhandler(locator->saxreader);
+    BSTR name, pubid, sysid;
+    HRESULT hr;
+
+    update_position(locator, FALSE);
+    if (!saxreader_has_handler(locator, SAXLexicalHandler)) return;
+
+    name = pooled_bstr_from_xmlChar(&locator->saxreader->pool, _name);
+    pubid = pooled_bstr_from_xmlChar(&locator->saxreader->pool, external_id);
+    sysid = pooled_bstr_from_xmlChar(&locator->saxreader->pool, system_id);
+
+    if (locator->vbInterface)
+        hr = IVBSAXLexicalHandler_startDTD(lexical->vbhandler, &name, &pubid, &sysid);
+    else
+        hr = ISAXLexicalHandler_startDTD(lexical->handler, name, SysStringLen(name),
+                pubid, SysStringLen(pubid), sysid, SysStringLen(sysid));
+
+    if (FAILED(hr))
+        format_error_message_from_id(locator, hr);
+}
+
+static void libxml_externalsubset(void *ctx, const xmlChar *name, const xmlChar *external_id, const xmlChar *system_id)
+{
+    saxlocator *locator = ctx;
+    struct saxlexicalhandler_iface *lexical = saxreader_get_lexicalhandler(locator->saxreader);
+    HRESULT hr = S_OK;
+
+    update_position(locator, FALSE);
+    if (!saxreader_has_handler(locator, SAXLexicalHandler)) return;
+
+    if (locator->vbInterface)
+        hr = IVBSAXLexicalHandler_endDTD(lexical->vbhandler);
+    else
+        hr = ISAXLexicalHandler_endDTD(lexical->handler);
+
+    if (FAILED(hr))
+        format_error_message_from_id(locator, hr);
+}
+
 static xmlParserInputPtr libxmlresolveentity(void *ctx, const xmlChar *publicid, const xmlChar *systemid)
 {
     FIXME("entity resolving not implemented, %s, %s\n", publicid, systemid);
@@ -2144,9 +2131,7 @@ static HRESULT WINAPI ivbsaxlocator_GetTypeInfo(
     IVBSAXLocator *iface,
     UINT iTInfo, LCID lcid, ITypeInfo** ppTInfo )
 {
-    saxlocator *This = impl_from_IVBSAXLocator( iface );
-
-    TRACE("(%p)->(%u %u %p)\n", This, iTInfo, lcid, ppTInfo);
+    TRACE("%p, %u, %lx, %p.\n", iface, iTInfo, lcid, ppTInfo);
 
     return get_typeinfo(IVBSAXLocator_tid, ppTInfo);
 }
@@ -2159,11 +2144,10 @@ static HRESULT WINAPI ivbsaxlocator_GetIDsOfNames(
     LCID lcid,
     DISPID* rgDispId)
 {
-    saxlocator *This = impl_from_IVBSAXLocator( iface );
     ITypeInfo *typeinfo;
     HRESULT hr;
 
-    TRACE("(%p)->(%s %p %u %u %p)\n", This, debugstr_guid(riid), rgszNames, cNames,
+    TRACE("%p, %s, %p, %u, %lx, %p.\n", iface, debugstr_guid(riid), rgszNames, cNames,
           lcid, rgDispId);
 
     if(!rgszNames || cNames == 0 || !rgDispId)
@@ -2190,18 +2174,16 @@ static HRESULT WINAPI ivbsaxlocator_Invoke(
     EXCEPINFO* pExcepInfo,
     UINT* puArgErr)
 {
-    saxlocator *This = impl_from_IVBSAXLocator( iface );
     ITypeInfo *typeinfo;
     HRESULT hr;
 
-    TRACE("(%p)->(%d %s %d %d %p %p %p %p)\n", This, dispIdMember, debugstr_guid(riid),
+    TRACE("%p, %ld, %s, %lx, %d, %p, %p, %p, %p.\n", iface, dispIdMember, debugstr_guid(riid),
           lcid, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
 
     hr = get_typeinfo(IVBSAXLocator_tid, &typeinfo);
     if(SUCCEEDED(hr))
     {
-        hr = ITypeInfo_Invoke(typeinfo, &This->IVBSAXLocator_iface, dispIdMember, wFlags,
-                pDispParams, pVarResult, pExcepInfo, puArgErr);
+        hr = ITypeInfo_Invoke(typeinfo, iface, dispIdMember, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
         ITypeInfo_Release(typeinfo);
     }
 
@@ -2312,7 +2294,7 @@ static ULONG WINAPI isaxlocator_AddRef(ISAXLocator* iface)
 {
     saxlocator *This = impl_from_ISAXLocator( iface );
     ULONG ref = InterlockedIncrement( &This->ref );
-    TRACE("(%p)->(%d)\n", This, ref);
+    TRACE("%p, refcount %lu.\n", iface, ref);
     return ref;
 }
 
@@ -2320,11 +2302,11 @@ static ULONG WINAPI isaxlocator_Release(
         ISAXLocator* iface)
 {
     saxlocator *This = impl_from_ISAXLocator( iface );
-    LONG ref = InterlockedDecrement( &This->ref );
+    ULONG ref = InterlockedDecrement( &This->ref );
 
-    TRACE("(%p)->(%d)\n", This, ref );
+    TRACE("%p, refcount %ld.\n", iface, ref );
 
-    if (ref == 0)
+    if (!ref)
     {
         element_entry *element, *element2;
         int index;
@@ -2339,7 +2321,7 @@ static ULONG WINAPI isaxlocator_Release(
             SysFreeString(This->attributes[index].szValue);
             SysFreeString(This->attributes[index].szQName);
         }
-        heap_free(This->attributes);
+        free(This->attributes);
 
         /* element stack */
         LIST_FOR_EACH_ENTRY_SAFE(element, element2, &This->elements, element_entry, entry)
@@ -2349,7 +2331,7 @@ static ULONG WINAPI isaxlocator_Release(
         }
 
         ISAXXMLReader_Release(&This->saxreader->ISAXXMLReader_iface);
-        heap_free( This );
+        free(This);
     }
 
     return ref;
@@ -2433,12 +2415,9 @@ static const struct ISAXLocatorVtbl SAXLocatorVtbl =
 
 static HRESULT SAXLocator_create(saxreader *reader, saxlocator **ppsaxlocator, BOOL vbInterface)
 {
-    static const WCHAR w3xmlns[] = { 'h','t','t','p',':','/','/', 'w','w','w','.','w','3','.',
-        'o','r','g','/','2','0','0','0','/','x','m','l','n','s','/',0 };
-
     saxlocator *locator;
 
-    locator = heap_alloc( sizeof (*locator) );
+    locator = malloc(sizeof(*locator));
     if( !locator )
         return E_OUTOFMEMORY;
 
@@ -2459,24 +2438,24 @@ static HRESULT SAXLocator_create(saxreader *reader, saxlocator **ppsaxlocator, B
     locator->column = 0;
     locator->ret = S_OK;
     if (locator->saxreader->version >= MSXML6)
-        locator->namespaceUri = SysAllocString(w3xmlns);
+        locator->namespaceUri = SysAllocString(L"http://www.w3.org/2000/xmlns/");
     else
         locator->namespaceUri = SysAllocStringLen(NULL, 0);
     if(!locator->namespaceUri)
     {
         ISAXXMLReader_Release(&reader->ISAXXMLReader_iface);
-        heap_free(locator);
+        free(locator);
         return E_OUTOFMEMORY;
     }
 
     locator->attr_alloc_count = 8;
     locator->attr_count = 0;
-    locator->attributes = heap_alloc_zero(sizeof(struct _attributes)*locator->attr_alloc_count);
+    locator->attributes = calloc(locator->attr_alloc_count, sizeof(*locator->attributes));
     if(!locator->attributes)
     {
         ISAXXMLReader_Release(&reader->ISAXXMLReader_iface);
         SysFreeString(locator->namespaceUri);
-        heap_free(locator);
+        free(locator);
         return E_OUTOFMEMORY;
     }
 
@@ -2505,18 +2484,9 @@ static HRESULT internal_parseBuffer(saxreader *This, const char *buffer, int siz
 
     if (size >= 4)
     {
-        const unsigned char *buff = (unsigned char*)buffer;
-
         encoding = xmlDetectCharEncoding((xmlChar*)buffer, 4);
         enc_name = (xmlChar*)xmlGetCharEncodingName(encoding);
         TRACE("detected encoding: %s\n", enc_name);
-        /* skip BOM, parser won't switch encodings and so won't skip it on its own */
-        if ((encoding == XML_CHAR_ENCODING_UTF8) &&
-            buff[0] == 0xEF && buff[1] == 0xBB && buff[2] == 0xBF)
-        {
-            buffer += 3;
-            size -= 3;
-        }
     }
 
     /* if libxml2 detection failed try to guess */
@@ -2641,7 +2611,7 @@ static HRESULT internal_parse(
         case VT_BSTR|VT_BYREF:
         {
             BSTR str = V_ISBYREF(&varInput) ? *V_BSTRREF(&varInput) : V_BSTR(&varInput);
-            hr = internal_parseBuffer(This, (const char*)str, strlenW(str)*sizeof(WCHAR), vbInterface);
+            hr = internal_parseBuffer(This, (const char*)str, lstrlenW(str)*sizeof(WCHAR), vbInterface);
             break;
         }
         case VT_ARRAY|VT_UI1: {
@@ -2722,23 +2692,23 @@ static HRESULT internal_onDataAvailable(void *obj, char *ptr, DWORD len)
     return internal_parseBuffer(This, ptr, len, FALSE);
 }
 
-static HRESULT internal_parseURL(
-        saxreader* This,
-        const WCHAR *url,
-        BOOL vbInterface)
+static HRESULT internal_parseURL(saxreader *reader, const WCHAR *url, BOOL vbInterface)
 {
     IMoniker *mon;
     bsc_t *bsc;
     HRESULT hr;
 
-    TRACE("(%p)->(%s)\n", This, debugstr_w(url));
+    TRACE("%p, %s.\n", reader, debugstr_w(url));
+
+    if (!url && reader->version < MSXML4)
+        return E_INVALIDARG;
 
     hr = create_moniker_from_url(url, &mon);
     if(FAILED(hr))
         return hr;
 
-    if(vbInterface) hr = bind_url(mon, internal_vbonDataAvailable, This, &bsc);
-    else hr = bind_url(mon, internal_onDataAvailable, This, &bsc);
+    if(vbInterface) hr = bind_url(mon, internal_vbonDataAvailable, reader, &bsc);
+    else hr = bind_url(mon, internal_onDataAvailable, reader, &bsc);
     IMoniker_Release(mon);
 
     if(FAILED(hr))
@@ -2811,14 +2781,14 @@ static HRESULT internal_putProperty(
     if(!memcmp(prop, PropertyLexicalHandlerW, sizeof(PropertyLexicalHandlerW)))
         return saxreader_put_handler_from_variant(This, SAXLexicalHandler, v, vbInterface);
 
-    if(!memcmp(prop, PropertyMaxXMLSizeW, sizeof(PropertyMaxXMLSizeW)))
+    if(!memcmp(prop, L"max-xml-size", sizeof(L"max-xml-size")))
     {
         if (V_VT(v) == VT_I4 && V_I4(v) == 0) return S_OK;
         FIXME("(%p)->(%s): max-xml-size unsupported\n", This, debugstr_variant(v));
         return E_NOTIMPL;
     }
 
-    if(!memcmp(prop, PropertyMaxElementDepthW, sizeof(PropertyMaxElementDepthW)))
+    if(!memcmp(prop, L"max-element-depth", sizeof(L"max-element-depth")))
     {
         if (V_VT(v) == VT_I4 && V_I4(v) == 0) return S_OK;
         FIXME("(%p)->(%s): max-element-depth unsupported\n", This, debugstr_variant(v));
@@ -2827,25 +2797,25 @@ static HRESULT internal_putProperty(
 
     FIXME("(%p)->(%s:%s): unsupported property\n", This, debugstr_w(prop), debugstr_variant(v));
 
-    if(!memcmp(prop, PropertyCharsetW, sizeof(PropertyCharsetW)))
+    if(!memcmp(prop, L"charset", sizeof(L"charset")))
         return E_NOTIMPL;
 
     if(!memcmp(prop, PropertyDomNodeW, sizeof(PropertyDomNodeW)))
         return E_FAIL;
 
-    if(!memcmp(prop, PropertyInputSourceW, sizeof(PropertyInputSourceW)))
+    if(!memcmp(prop, L"input-source", sizeof(L"input-source")))
         return E_NOTIMPL;
 
-    if(!memcmp(prop, PropertySchemaDeclHandlerW, sizeof(PropertySchemaDeclHandlerW)))
+    if(!memcmp(prop, L"schema-declaration-handler", sizeof(L"schema-declaration-handler")))
         return E_NOTIMPL;
 
-    if(!memcmp(prop, PropertyXMLDeclEncodingW, sizeof(PropertyXMLDeclEncodingW)))
+    if(!memcmp(prop, L"xmldecl-encoding", sizeof(L"xmldecl-encoding")))
         return E_FAIL;
 
-    if(!memcmp(prop, PropertyXMLDeclStandaloneW, sizeof(PropertyXMLDeclStandaloneW)))
+    if(!memcmp(prop, L"xmldecl-standalone", sizeof(L"xmldecl-standalone")))
         return E_FAIL;
 
-    if(!memcmp(prop, PropertyXMLDeclVersionW, sizeof(PropertyXMLDeclVersionW)))
+    if(!memcmp(prop, L"xmldecl-version", sizeof(L"xmldecl-version")))
         return E_FAIL;
 
     return E_INVALIDARG;
@@ -2871,7 +2841,7 @@ static HRESULT internal_getProperty(const saxreader* This, const WCHAR *prop, VA
         return S_OK;
     }
 
-    if (!memcmp(PropertyXmlDeclVersionW, prop, sizeof(PropertyXmlDeclVersionW)))
+    if (!memcmp(L"xmldecl-version", prop, sizeof(L"xmldecl-version")))
     {
         V_VT(value) = VT_BSTR;
         V_BSTR(value) = SysAllocString(This->xmldecl_version);
@@ -2952,7 +2922,7 @@ static ULONG WINAPI saxxmlreader_Release(
         SysFreeString(This->xmldecl_version);
         free_bstr_pool(&This->pool);
 
-        heap_free( This );
+        free(This);
     }
 
     return ref;
@@ -3216,7 +3186,14 @@ static HRESULT WINAPI isaxxmlreader_getFeature(
     TRACE("(%p)->(%s %p)\n", This, debugstr_w(feature_name), value);
 
     feature = get_saxreader_feature(feature_name);
-    if (feature == Namespaces || feature == NamespacePrefixes)
+
+    if (This->version < MSXML4 && (feature == ExhaustiveErrors || feature == SchemaValidation))
+        return E_INVALIDARG;
+
+    if (feature == Namespaces ||
+            feature == NamespacePrefixes ||
+            feature == ExhaustiveErrors ||
+            feature == SchemaValidation)
         return get_feature_value(This, feature, value);
 
     FIXME("(%p)->(%s %p) stub\n", This, debugstr_w(feature_name), value);
@@ -3236,15 +3213,18 @@ static HRESULT WINAPI isaxxmlreader_putFeature(
     feature = get_saxreader_feature(feature_name);
 
     /* accepted cases */
-    if ((feature == ExternalGeneralEntities   && value == VARIANT_FALSE) ||
-        (feature == ExternalParameterEntities && value == VARIANT_FALSE) ||
+    if ((feature == ExhaustiveErrors && value == VARIANT_FALSE) ||
+        (feature == SchemaValidation && value == VARIANT_FALSE) ||
          feature == Namespaces ||
          feature == NamespacePrefixes)
     {
         return set_feature_value(This, feature, value);
     }
 
-    if (feature == LexicalHandlerParEntities || feature == ProhibitDTD)
+    if (feature == LexicalHandlerParEntities ||
+            feature == ProhibitDTD ||
+            feature == ExternalGeneralEntities ||
+            feature == ExternalParameterEntities)
     {
         FIXME("(%p)->(%s %x) stub\n", This, debugstr_w(feature_name), value);
         return set_feature_value(This, feature, value);
@@ -3431,7 +3411,7 @@ HRESULT SAXXMLReader_create(MSXML_VERSION version, LPVOID *ppObj)
 
     TRACE("(%p)\n", ppObj);
 
-    reader = heap_alloc( sizeof (*reader) );
+    reader = malloc(sizeof(*reader));
     if( !reader )
         return E_OUTOFMEMORY;
 
@@ -3462,6 +3442,9 @@ HRESULT SAXXMLReader_create(MSXML_VERSION version, LPVOID *ppObj)
     reader->sax.fatalError = libxmlFatalError;
     reader->sax.cdataBlock = libxml_cdatablock;
     reader->sax.resolveEntity = libxmlresolveentity;
+    reader->sax.processingInstruction = libxml_pi;
+    reader->sax.internalSubset = libxml_internalsubset;
+    reader->sax.externalSubset = libxml_externalsubset;
 
     *ppObj = &reader->IVBSAXXMLReader_iface;
 
@@ -3469,14 +3452,3 @@ HRESULT SAXXMLReader_create(MSXML_VERSION version, LPVOID *ppObj)
 
     return S_OK;
 }
-
-#else
-
-HRESULT SAXXMLReader_create(MSXML_VERSION version, LPVOID *ppObj)
-{
-    MESSAGE("This program tried to use a SAX XML Reader object, but\n"
-            "libxml2 support was not present at compile time.\n");
-    return E_NOTIMPL;
-}
-
-#endif
