@@ -18,11 +18,16 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#define NONAMELESSUNION
-#define NONAMELESSSTRUCT
-#include "ntdll_test.h"
+#include <stdarg.h>
+
+#include "ntstatus.h"
+#define WIN32_NO_STATUS
+#include "windef.h"
+#include "winbase.h"
+#include "winternl.h"
 #include "ddk/wdm.h"
 #include "intrin.h"
+#include "wine/test.h"
 
 #define TICKSPERSEC        10000000
 #define TICKSPERMSEC       10000
@@ -40,6 +45,10 @@ static BOOL     (WINAPI *pRtlQueryUnbiasedInterruptTime)( ULONGLONG *time );
 static BOOL     (WINAPI *pRtlQueryPerformanceCounter)(LARGE_INTEGER*);
 static BOOL     (WINAPI *pRtlQueryPerformanceFrequency)(LARGE_INTEGER*);
 
+static NTSTATUS (WINAPI *pNtConvertBetweenAuxiliaryCounterAndPerformanceCounter)(ULONG, ULONGLONG *, ULONGLONG *, ULONGLONG *);
+static HRESULT (WINAPI *pConvertAuxiliaryCounterToPerformanceCounter)(ULONGLONG, ULONGLONG *, ULONGLONG *);
+static HRESULT (WINAPI *pConvertPerformanceCounterToAuxiliaryCounter)(ULONGLONG, ULONGLONG *, ULONGLONG *);
+
 static const int MonthLengths[2][12] =
 {
 	{ 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 },
@@ -127,7 +136,7 @@ static void test_NtQueryPerformanceCounter(void)
     ok(status == STATUS_SUCCESS, "expected STATUS_SUCCESS, got %08lx\n", status);
 }
 
-#if defined(__i386__) || defined(__x86_64__)
+#if (defined(__i386__) || defined(__x86_64__)) && !defined(__arm64ec__)
 
 struct hypervisor_shared_data
 {
@@ -147,7 +156,7 @@ static UINT64 multiply_tsc(UINT64 a, UINT64 b)
 static void test_RtlQueryPerformanceCounter(void)
 {
     struct hypervisor_shared_data *hsd;
-    KSHARED_USER_DATA *usd = (void *)0x7ffe0000;
+    KUSER_SHARED_DATA *usd = (void *)0x7ffe0000;
     LARGE_INTEGER frequency, counter;
     NTSTATUS status;
     UINT64 tsc0, tsc1;
@@ -160,18 +169,18 @@ static void test_RtlQueryPerformanceCounter(void)
         return;
     }
 
-    if (!(usd->u3.s.QpcBypassEnabled & SHARED_GLOBAL_FLAGS_QPC_BYPASS_ENABLED))
+    if (!(usd->QpcBypassEnabled & SHARED_GLOBAL_FLAGS_QPC_BYPASS_ENABLED))
     {
         todo_wine win_skip("QpcBypassEnabled is not set, skipping tests\n");
         return;
     }
 
-    if ((usd->u3.s.QpcBypassEnabled & SHARED_GLOBAL_FLAGS_QPC_BYPASS_USE_HV_PAGE))
+    if ((usd->QpcBypassEnabled & SHARED_GLOBAL_FLAGS_QPC_BYPASS_USE_HV_PAGE))
     {
-        ok( usd->u3.s.QpcBypassEnabled == (SHARED_GLOBAL_FLAGS_QPC_BYPASS_ENABLED|SHARED_GLOBAL_FLAGS_QPC_BYPASS_USE_HV_PAGE|SHARED_GLOBAL_FLAGS_QPC_BYPASS_USE_RDTSCP),
-            "unexpected QpcBypassEnabled %x, expected 0x83\n", usd->u3.s.QpcBypassEnabled );
+        ok( usd->QpcBypassEnabled == (SHARED_GLOBAL_FLAGS_QPC_BYPASS_ENABLED|SHARED_GLOBAL_FLAGS_QPC_BYPASS_USE_HV_PAGE|SHARED_GLOBAL_FLAGS_QPC_BYPASS_USE_RDTSCP),
+            "unexpected QpcBypassEnabled %x, expected 0x83\n", usd->QpcBypassEnabled );
         ok( usd->QpcFrequency == 10000000, "unexpected QpcFrequency %I64d, expected 10000000\n", usd->QpcFrequency );
-        ok( !usd->u3.s.QpcShift, "unexpected QpcShift %d, expected 0\n", usd->u3.s.QpcShift );
+        ok( !usd->QpcShift, "unexpected QpcShift %d, expected 0\n", usd->QpcShift );
 
         hsd = NULL;
         status = pNtQuerySystemInformation( SystemHypervisorSharedPageInformation, &hsd, sizeof(void *), &len );
@@ -192,7 +201,7 @@ static void test_RtlQueryPerformanceCounter(void)
     }
     else
     {
-        ok( usd->u3.s.QpcShift == 10, "unexpected QpcShift %d, expected 10\n", usd->u3.s.QpcShift );
+        ok( usd->QpcShift == 10, "unexpected QpcShift %d, expected 10\n", usd->QpcShift );
 
         tsc0 = __rdtsc();
         ret = pRtlQueryPerformanceCounter( &counter );
@@ -200,9 +209,9 @@ static void test_RtlQueryPerformanceCounter(void)
         ok( ret, "RtlQueryPerformanceCounter failed\n" );
 
         tsc0 += usd->QpcBias;
-        tsc0 >>= usd->u3.s.QpcShift;
+        tsc0 >>= usd->QpcShift;
         tsc1 += usd->QpcBias;
-        tsc1 >>= usd->u3.s.QpcShift;
+        tsc1 >>= usd->QpcShift;
 
         ok( tsc0 <= counter.QuadPart, "rdtscp %I64d and RtlQueryPerformanceCounter %I64d are out of order\n", tsc0, counter.QuadPart );
         ok( counter.QuadPart <= tsc1, "RtlQueryPerformanceCounter %I64d and rdtscp %I64d are out of order\n", counter.QuadPart, tsc1 );
@@ -409,7 +418,7 @@ static ULONGLONG read_ksystem_time(volatile KSYSTEM_TIME *time)
 
 static void test_user_shared_data_time(void)
 {
-    KSHARED_USER_DATA *user_shared_data = (void *)0x7ffe0000;
+    KUSER_SHARED_DATA *user_shared_data = (void *)0x7ffe0000;
     SYSTEM_TIMEOFDAY_INFORMATION timeofday;
     ULONGLONG t1, t2, t3;
     NTSTATUS status;
@@ -422,7 +431,7 @@ static void test_user_shared_data_time(void)
         if (user_shared_data->NtMajorVersion <= 5 && user_shared_data->NtMinorVersion <= 1)
             t2 = (DWORD)((*(volatile ULONG*)&user_shared_data->TickCountLowDeprecated * (ULONG64)user_shared_data->TickCountMultiplier) >> 24);
         else
-            t2 = (DWORD)((read_ksystem_time(&user_shared_data->u.TickCount) * user_shared_data->TickCountMultiplier) >> 24);
+            t2 = (DWORD)((read_ksystem_time(&user_shared_data->TickCount) * user_shared_data->TickCountMultiplier) >> 24);
         t3 = GetTickCount();
     } while(t3 < t1 && i++ < 1); /* allow for wrap, but only once */
 
@@ -474,9 +483,53 @@ static void test_user_shared_data_time(void)
             t1, timeofday.TimeZoneBias.QuadPart);
 }
 
+static void test_NtConvertBetweenAuxiliaryCounterAndPerformanceCounter(void)
+{
+    ULONGLONG qpc, error, value;
+    NTSTATUS status;
+    HRESULT hr;
+
+    if (!pNtConvertBetweenAuxiliaryCounterAndPerformanceCounter)
+    {
+        win_skip("NtConvertBetweenAuxiliaryCounterAndPerformanceCounter not found.\n");
+        return;
+    }
+
+    status = pNtConvertBetweenAuxiliaryCounterAndPerformanceCounter(0, NULL, NULL, NULL);
+    ok(status == STATUS_ACCESS_VIOLATION, "got %#lx.\n", status);
+    qpc = error = value = 0xdeadbeef;
+    status = pNtConvertBetweenAuxiliaryCounterAndPerformanceCounter(0, &value, &qpc, NULL);
+    ok(status == STATUS_NOT_SUPPORTED, "got %#lx.\n", status);
+    ok(value == 0xdeadbeef, "got %#I64x.\n", value);
+    ok(qpc == 0xdeadbeef, "got %#I64x.\n", qpc);
+    ok(error == 0xdeadbeef, "got %#I64x.\n", error);
+    status = pNtConvertBetweenAuxiliaryCounterAndPerformanceCounter(1, &value, &qpc, &error);
+    ok(status == STATUS_NOT_SUPPORTED, "got %#lx.\n", status);
+    ok(value == 0xdeadbeef, "got %#I64x.\n", value);
+    ok(qpc == 0xdeadbeef, "got %#I64x.\n", qpc);
+    ok(error == 0xdeadbeef, "got %#I64x.\n", error);
+    status = pNtConvertBetweenAuxiliaryCounterAndPerformanceCounter(2, &value, &qpc, &error);
+    ok(status == STATUS_NOT_SUPPORTED, "got %#lx.\n", status);
+    ok(value == 0xdeadbeef, "got %#I64x.\n", value);
+    ok(qpc == 0xdeadbeef, "got %#I64x.\n", qpc);
+    ok(error == 0xdeadbeef, "got %#I64x.\n", error);
+
+    hr = pConvertAuxiliaryCounterToPerformanceCounter(1, &qpc, &error);
+    ok(hr == E_NOTIMPL, "got %#lx.\n", hr);
+    ok(value == 0xdeadbeef, "got %#I64x.\n", value);
+    ok(qpc == 0xdeadbeef, "got %#I64x.\n", qpc);
+    ok(error == 0xdeadbeef, "got %#I64x.\n", error);
+    QueryPerformanceCounter((LARGE_INTEGER *)&qpc);
+    hr = pConvertPerformanceCounterToAuxiliaryCounter(qpc, &value, &error);
+    ok(hr == E_NOTIMPL, "got %#lx.\n", hr);
+    ok(value == 0xdeadbeef, "got %#I64x.\n", value);
+    ok(error == 0xdeadbeef, "got %#I64x.\n", error);
+}
+
 START_TEST(time)
 {
     HMODULE mod = GetModuleHandleA("ntdll.dll");
+    HMODULE hkernelbase = GetModuleHandleA("kernelbase.dll");
     pRtlTimeToTimeFields = (void *)GetProcAddress(mod,"RtlTimeToTimeFields");
     pRtlTimeFieldsToTime = (void *)GetProcAddress(mod,"RtlTimeFieldsToTime");
     pNtQueryPerformanceCounter = (void *)GetProcAddress(mod, "NtQueryPerformanceCounter");
@@ -488,6 +541,11 @@ START_TEST(time)
     pRtlQueryUnbiasedInterruptTime = (void *)GetProcAddress(mod, "RtlQueryUnbiasedInterruptTime");
     pRtlQueryPerformanceCounter = (void *)GetProcAddress(mod, "RtlQueryPerformanceCounter");
     pRtlQueryPerformanceFrequency = (void *)GetProcAddress(mod, "RtlQueryPerformanceFrequency");
+    pNtConvertBetweenAuxiliaryCounterAndPerformanceCounter =
+        (void *)GetProcAddress(mod, "NtConvertBetweenAuxiliaryCounterAndPerformanceCounter");
+
+    pConvertAuxiliaryCounterToPerformanceCounter = (void *)GetProcAddress(hkernelbase, "ConvertAuxiliaryCounterToPerformanceCounter");
+    pConvertPerformanceCounterToAuxiliaryCounter = (void *)GetProcAddress(hkernelbase, "ConvertPerformanceCounterToAuxiliaryCounter");
 
     if (pRtlTimeToTimeFields && pRtlTimeFieldsToTime)
         test_pRtlTimeToTimeFields();
@@ -496,8 +554,9 @@ START_TEST(time)
     test_NtQueryPerformanceCounter();
     test_RtlQueryTimeZoneInformation();
     test_user_shared_data_time();
-#if defined(__i386__) || defined(__x86_64__)
+#if (defined(__i386__) || defined(__x86_64__)) && !defined(__arm64ec__)
     test_RtlQueryPerformanceCounter();
 #endif
     test_TimerResolution();
+    test_NtConvertBetweenAuxiliaryCounterAndPerformanceCounter();
 }
