@@ -18,6 +18,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
+#define COBJMACROS
+
 #include "windows.h"
 #include "wtypes.h"
 #include "ddeml.h"
@@ -44,28 +46,28 @@ static const struct valid_mapping
 {
     { PROP_V0 , PROP_INV, PROP_INV, PROP_INV }, /* VT_EMPTY */
     { PROP_V0 , PROP_INV, PROP_INV, PROP_INV }, /* VT_NULL */
-    { PROP_V0 , PROP_V1 | PROP_TODO , PROP_V0 , PROP_V1 | PROP_TODO  }, /* VT_I2 */
-    { PROP_V0 , PROP_V1 | PROP_TODO , PROP_V0 , PROP_V1 | PROP_TODO  }, /* VT_I4 */
-    { PROP_V0 , PROP_V1 | PROP_TODO , PROP_V0 , PROP_V1 | PROP_TODO  }, /* VT_R4 */
-    { PROP_V0 , PROP_V1 | PROP_TODO , PROP_V0 , PROP_V1 | PROP_TODO  }, /* VT_R8 */
-    { PROP_V0 , PROP_V1 | PROP_TODO , PROP_V0 , PROP_V1 | PROP_TODO  }, /* VT_CY */
-    { PROP_V0 , PROP_V1 | PROP_TODO , PROP_V0 , PROP_V1 | PROP_TODO  }, /* VT_DATE */
-    { PROP_V0 , PROP_V1 | PROP_TODO , PROP_V0 , PROP_V1 | PROP_TODO  }, /* VT_BSTR */
-    { PROP_V1 , PROP_V1 | PROP_TODO , PROP_INV, PROP_V1 | PROP_TODO  }, /* VT_DISPATCH */
-    { PROP_V0 , PROP_V1 | PROP_TODO , PROP_V0 , PROP_V1 | PROP_TODO  }, /* VT_ERROR */
-    { PROP_V0 , PROP_V1 | PROP_TODO , PROP_V0 , PROP_V1 | PROP_TODO  }, /* VT_BOOL */
-    { PROP_V1 | PROP_TODO , PROP_V1 | PROP_TODO , PROP_V0 , PROP_V1 | PROP_TODO  }, /* VT_VARIANT */
-    { PROP_V1 , PROP_V1 | PROP_TODO , PROP_INV, PROP_V1 | PROP_TODO  }, /* VT_UNKNOWN */
-    { PROP_V1 , PROP_V1 | PROP_TODO , PROP_INV, PROP_V1 | PROP_TODO  }, /* VT_DECIMAL */
+    { PROP_V0 , PROP_V1 , PROP_V0 , PROP_V1 | PROP_TODO  }, /* VT_I2 */
+    { PROP_V0 , PROP_V1 , PROP_V0 , PROP_V1 | PROP_TODO  }, /* VT_I4 */
+    { PROP_V0 , PROP_V1 , PROP_V0 , PROP_V1 | PROP_TODO  }, /* VT_R4 */
+    { PROP_V0 , PROP_V1 , PROP_V0 , PROP_V1 | PROP_TODO  }, /* VT_R8 */
+    { PROP_V0 , PROP_V1 , PROP_V0 , PROP_V1 | PROP_TODO  }, /* VT_CY */
+    { PROP_V0 , PROP_V1 , PROP_V0 , PROP_V1 | PROP_TODO  }, /* VT_DATE */
+    { PROP_V0 , PROP_V1 , PROP_V0 , PROP_V1 | PROP_TODO  }, /* VT_BSTR */
+    { PROP_V1 , PROP_V1 , PROP_INV, PROP_V1 | PROP_TODO  }, /* VT_DISPATCH */
+    { PROP_V0 , PROP_V1 , PROP_V0 , PROP_V1 | PROP_TODO  }, /* VT_ERROR */
+    { PROP_V0 , PROP_V1 , PROP_V0 , PROP_V1 | PROP_TODO  }, /* VT_BOOL */
+    { PROP_V1 | PROP_TODO , PROP_V1 , PROP_V0 , PROP_V1 | PROP_TODO  }, /* VT_VARIANT */
+    { PROP_V1 , PROP_V1, PROP_INV, PROP_V1 | PROP_TODO  }, /* VT_UNKNOWN */
+    { PROP_V1 , PROP_V1 , PROP_INV, PROP_V1 | PROP_TODO  }, /* VT_DECIMAL */
     { PROP_INV, PROP_INV, PROP_INV, PROP_INV }, /* 15 */
-    { PROP_V1 , PROP_V1 | PROP_TODO , PROP_V1 , PROP_V1 | PROP_TODO  }, /* VT_I1 */
-    { PROP_V0 , PROP_V1 | PROP_TODO , PROP_V0 , PROP_V1 | PROP_TODO  }, /* VT_UI1 */
-    { PROP_V0 , PROP_V1 | PROP_TODO , PROP_V0 , PROP_V1 | PROP_TODO  }, /* VT_UI2 */
-    { PROP_V0 , PROP_V1 | PROP_TODO , PROP_V0 , PROP_V1 | PROP_TODO  }, /* VT_UI4 */
+    { PROP_V1 , PROP_V1 , PROP_V1 , PROP_V1 | PROP_TODO  }, /* VT_I1 */
+    { PROP_V0 , PROP_V1 , PROP_V0 , PROP_V1 | PROP_TODO  }, /* VT_UI1 */
+    { PROP_V0 , PROP_V1 , PROP_V0 , PROP_V1 | PROP_TODO  }, /* VT_UI2 */
+    { PROP_V0 , PROP_V1 , PROP_V0 , PROP_V1 | PROP_TODO  }, /* VT_UI4 */
     { PROP_V0 , PROP_V1A | PROP_TODO, PROP_V0 , PROP_V1A | PROP_TODO }, /* VT_I8 */
     { PROP_V0 , PROP_V1A | PROP_TODO, PROP_V0 , PROP_V1A | PROP_TODO }, /* VT_UI8 */
-    { PROP_V1 , PROP_V1 | PROP_TODO , PROP_INV, PROP_V1 | PROP_TODO  }, /* VT_INT */
-    { PROP_V1 , PROP_V1 | PROP_TODO , PROP_INV, PROP_V1 | PROP_TODO  }, /* VT_UINT */
+    { PROP_V1 , PROP_V1 , PROP_INV, PROP_V1 | PROP_TODO  }, /* VT_INT */
+    { PROP_V1 , PROP_V1 , PROP_INV, PROP_V1 | PROP_TODO  }, /* VT_UINT */
     { PROP_INV, PROP_INV, PROP_INV, PROP_INV }, /* VT_VOID */
     { PROP_INV, PROP_INV, PROP_INV, PROP_INV }, /* VT_HRESULT */
     { PROP_INV, PROP_INV, PROP_INV, PROP_INV }, /* VT_PTR */
@@ -160,17 +162,17 @@ static void expect(HRESULT hr, VARTYPE vt, BOOL copy, int line)
     if(flags == PROP_INV)
     {
         if (copy && (vt & VT_VECTOR))
-            ok(hr == DISP_E_BADVARTYPE || hr == STG_E_INVALIDPARAMETER, "%s (%s): got %08x (line %d)\n", wine_vtypes[idx], modifier, hr, line);
+            ok(hr == DISP_E_BADVARTYPE || hr == STG_E_INVALIDPARAMETER, "%s (%s): got %08lx (line %d)\n", wine_vtypes[idx], modifier, hr, line);
         else
-            ok(hr == (copy ? DISP_E_BADVARTYPE : STG_E_INVALIDPARAMETER), "%s (%s): got %08x (line %d)\n", wine_vtypes[idx], modifier, hr, line);
+            ok(hr == (copy ? DISP_E_BADVARTYPE : STG_E_INVALIDPARAMETER), "%s (%s): got %08lx (line %d)\n", wine_vtypes[idx], modifier, hr, line);
     }
     else if(flags == PROP_V0)
-        ok(hr == S_OK, "%s (%s): got %08x\n", wine_vtypes[idx], modifier, hr);
+        ok(hr == S_OK, "%s (%s): got %08lx\n", wine_vtypes[idx], modifier, hr);
     else todo_wine_if(flags & PROP_TODO)
     {
         if(hr != S_OK)
             win_skip("%s (%s): unsupported\n", wine_vtypes[idx], modifier);
-        else ok(hr == S_OK, "%s (%s): got %08x\n", wine_vtypes[idx], modifier, hr);
+        else ok(hr == S_OK, "%s (%s): got %08lx\n", wine_vtypes[idx], modifier, hr);
     }
 }
 
@@ -184,10 +186,10 @@ static void test_validtypes(void)
 
     memset(&propvar, 0x55, sizeof(propvar));
     hr = PropVariantClear(&propvar);
-    ok(hr == STG_E_INVALIDPARAMETER, "expected STG_E_INVALIDPARAMETER, got %08x\n", hr);
+    ok(hr == STG_E_INVALIDPARAMETER, "expected STG_E_INVALIDPARAMETER, got %08lx\n", hr);
     ok(propvar.vt == 0, "expected 0, got %d\n", propvar.vt);
-    ok(U(propvar).uhVal.QuadPart == 0, "expected 0, got %#x/%#x\n",
-       U(propvar).uhVal.u.LowPart, U(propvar).uhVal.u.HighPart);
+    ok(propvar.uhVal.QuadPart == 0, "expected 0, got %#lx/%#lx\n",
+       propvar.uhVal.u.LowPart, propvar.uhVal.u.HighPart);
 
     for (i = 0; i < ARRAY_SIZE(valid_types); i++)
     {
@@ -198,11 +200,11 @@ static void test_validtypes(void)
             memset(&propvar, 0, sizeof(propvar));
         else if (i == VT_BLOB || i == VT_BLOB_OBJECT)
         {
-            U(propvar).blob.cbSize = 0;
-            U(propvar).blob.pBlobData = NULL;
+            propvar.blob.cbSize = 0;
+            propvar.blob.pBlobData = NULL;
         }
         else
-            U(propvar).pszVal = NULL;
+            propvar.pszVal = NULL;
         vt = propvar.vt = i;
         memset(&copy, 0x77, sizeof(copy));
         hr = PropVariantCopy(&copy, &propvar);
@@ -210,9 +212,9 @@ static void test_validtypes(void)
         if (hr == S_OK)
         {
             ok(copy.vt == propvar.vt, "expected %d, got %d\n", propvar.vt, copy.vt);
-            ok(U(copy).uhVal.QuadPart == U(propvar).uhVal.QuadPart, "%u: expected %#x/%#x, got %#x/%#x\n",
-               i, U(propvar).uhVal.u.LowPart, U(propvar).uhVal.u.HighPart,
-               U(copy).uhVal.u.LowPart, U(copy).uhVal.u.HighPart);
+            ok(copy.uhVal.QuadPart == propvar.uhVal.QuadPart, "%u: expected %#lx/%#lx, got %#lx/%#lx\n",
+               i, propvar.uhVal.u.LowPart, propvar.uhVal.u.HighPart,
+               copy.uhVal.u.LowPart, copy.uhVal.u.HighPart);
         }
         else
         {
@@ -222,11 +224,11 @@ static void test_validtypes(void)
         hr = PropVariantClear(&propvar);
         expect(hr, vt, FALSE, __LINE__);
         ok(propvar.vt == 0, "expected 0, got %d\n", propvar.vt);
-        ok(U(propvar).uhVal.QuadPart == 0, "%u: expected 0, got %#x/%#x\n",
-           i, U(propvar).uhVal.u.LowPart, U(propvar).uhVal.u.HighPart);
+        ok(propvar.uhVal.QuadPart == 0, "%u: expected 0, got %#lx/%#lx\n",
+           i, propvar.uhVal.u.LowPart, propvar.uhVal.u.HighPart);
 
         memset(&propvar, 0x55, sizeof(propvar));
-        U(propvar).pszVal = NULL;
+        propvar.pszVal = NULL;
         vt = propvar.vt = i | VT_ARRAY;
         memset(&copy, 0x77, sizeof(copy));
         hr = PropVariantCopy(&copy, &propvar);
@@ -234,8 +236,8 @@ static void test_validtypes(void)
         if (hr == S_OK)
         {
             ok(copy.vt == propvar.vt, "expected %d, got %d\n", propvar.vt, copy.vt);
-            ok(U(copy).uhVal.QuadPart == 0, "%u: expected 0, got %#x/%#x\n",
-               i, U(copy).uhVal.u.LowPart, U(copy).uhVal.u.HighPart);
+            ok(copy.uhVal.QuadPart == 0, "%u: expected 0, got %#lx/%#lx\n",
+               i, copy.uhVal.u.LowPart, copy.uhVal.u.HighPart);
         }
         else
         {
@@ -245,12 +247,12 @@ static void test_validtypes(void)
         hr = PropVariantClear(&propvar);
         expect(hr, vt, FALSE, __LINE__);
         ok(propvar.vt == 0, "expected 0, got %d\n", propvar.vt);
-        ok(U(propvar).uhVal.QuadPart == 0, "%u: expected 0, got %#x/%#x\n",
-           i, U(propvar).uhVal.u.LowPart, U(propvar).uhVal.u.HighPart);
+        ok(propvar.uhVal.QuadPart == 0, "%u: expected 0, got %#lx/%#lx\n",
+           i, propvar.uhVal.u.LowPart, propvar.uhVal.u.HighPart);
 
         memset(&propvar, 0x55, sizeof(propvar));
-        U(propvar).caub.cElems = 0;
-        U(propvar).caub.pElems = NULL;
+        propvar.caub.cElems = 0;
+        propvar.caub.pElems = NULL;
         vt = propvar.vt = i | VT_VECTOR;
         memset(&copy, 0x77, sizeof(copy));
         hr = PropVariantCopy(&copy, &propvar);
@@ -258,8 +260,8 @@ static void test_validtypes(void)
         if (hr == S_OK)
         {
             ok(copy.vt == propvar.vt, "expected %d, got %d\n", propvar.vt, copy.vt);
-            ok(!U(copy).caub.cElems, "%u: expected 0, got %d\n", i, U(copy).caub.cElems);
-            ok(!U(copy).caub.pElems, "%u: expected NULL, got %p\n", i, U(copy).caub.pElems);
+            ok(!copy.caub.cElems, "%u: expected 0, got %ld\n", i, copy.caub.cElems);
+            ok(!copy.caub.pElems, "%u: expected NULL, got %p\n", i, copy.caub.pElems);
         }
         else
         {
@@ -269,11 +271,11 @@ static void test_validtypes(void)
         hr = PropVariantClear(&propvar);
         expect(hr, vt, FALSE, __LINE__);
         ok(propvar.vt == 0, "expected 0, got %d\n", propvar.vt);
-        ok(U(propvar).uhVal.QuadPart == 0, "%u: expected 0, got %#x/%#x\n",
-           i, U(propvar).uhVal.u.LowPart, U(propvar).uhVal.u.HighPart);
+        ok(propvar.uhVal.QuadPart == 0, "%u: expected 0, got %#lx/%#lx\n",
+           i, propvar.uhVal.u.LowPart, propvar.uhVal.u.HighPart);
 
         memset(&propvar, 0x55, sizeof(propvar));
-        U(propvar).pszVal = NULL;
+        propvar.pszVal = NULL;
         vt = propvar.vt = i | VT_BYREF;
         memset(&copy, 0x77, sizeof(copy));
         hr = PropVariantCopy(&copy, &propvar);
@@ -281,9 +283,9 @@ static void test_validtypes(void)
         if (hr == S_OK)
         {
             ok(copy.vt == propvar.vt, "expected %d, got %d\n", propvar.vt, copy.vt);
-            ok(U(copy).uhVal.QuadPart == U(propvar).uhVal.QuadPart, "%u: expected %#x/%#x, got %#x/%#x\n",
-               i, U(propvar).uhVal.u.LowPart, U(propvar).uhVal.u.HighPart,
-               U(copy).uhVal.u.LowPart, U(copy).uhVal.u.HighPart);
+            ok(copy.uhVal.QuadPart == propvar.uhVal.QuadPart, "%u: expected %#lx/%#lx, got %#lx/%#lx\n",
+               i, propvar.uhVal.u.LowPart, propvar.uhVal.u.HighPart,
+               copy.uhVal.u.LowPart, copy.uhVal.u.HighPart);
         }
         else
         {
@@ -293,47 +295,141 @@ static void test_validtypes(void)
         hr = PropVariantClear(&propvar);
         expect(hr, vt, FALSE, __LINE__);
         ok(propvar.vt == 0, "expected 0, got %d\n", propvar.vt);
-        ok(U(propvar).uhVal.QuadPart == 0, "%u: expected 0, got %#x/%#x\n",
-           i, U(propvar).uhVal.u.LowPart, U(propvar).uhVal.u.HighPart);
+        ok(propvar.uhVal.QuadPart == 0, "%u: expected 0, got %#lx/%#lx\n",
+           i, propvar.uhVal.u.LowPart, propvar.uhVal.u.HighPart);
     }
 }
 
+struct unk_impl
+{
+    IUnknown IUnknown_iface;
+    LONG ref;
+};
+
+static inline struct unk_impl *impl_from_IUnknown(IUnknown *iface)
+{
+    return CONTAINING_RECORD(iface, struct unk_impl, IUnknown_iface);
+}
+
+static HRESULT WINAPI unk_QueryInterface(IUnknown *iface, REFIID riid, void **ppv)
+{
+    struct unk_impl *This = impl_from_IUnknown(iface);
+    if(winetest_debug > 1)
+        trace("Call to unk_QueryInterface()\n");
+    *ppv = &This->IUnknown_iface;
+    IUnknown_AddRef(iface);
+    return S_OK;
+}
+
+static ULONG WINAPI unk_AddRef(IUnknown *iface)
+{
+    struct unk_impl *This = impl_from_IUnknown(iface);
+    if(winetest_debug > 1)
+        trace("Call to unk_AddRef()\n");
+    return InterlockedIncrement(&This->ref);
+}
+
+static ULONG WINAPI unk_Release(IUnknown *iface)
+{
+    struct unk_impl *This = impl_from_IUnknown(iface);
+    if(winetest_debug > 1)
+        trace("Call to unk_Release()\n");
+    return InterlockedDecrement(&This->ref);
+}
+
+static const IUnknownVtbl unk_vtbl =
+{
+    unk_QueryInterface,
+    unk_AddRef,
+    unk_Release
+};
+
 static void test_copy(void)
 {
     static char szTestString[] = "Test String";
     static WCHAR wszTestString[] = {'T','e','s','t',' ','S','t','r','i','n','g',0};
+    struct unk_impl unk_obj = {{&unk_vtbl}, 1};
     PROPVARIANT propvarSrc;
     PROPVARIANT propvarDst;
+    SAFEARRAY *sa;
+    SAFEARRAYBOUND sabound;
+    LONG saindex;
     HRESULT hr;
 
     propvarSrc.vt = VT_BSTR;
-    U(propvarSrc).bstrVal = SysAllocString(wszTestString);
+    propvarSrc.bstrVal = SysAllocString(wszTestString);
 
     hr = PropVariantCopy(&propvarDst, &propvarSrc);
     ok(hr == S_OK, "PropVariantCopy(...VT_BSTR...) failed\n");
-    ok(!lstrcmpW(U(propvarSrc).bstrVal, U(propvarDst).bstrVal), "BSTR not copied properly\n");
+    ok(!lstrcmpW(propvarSrc.bstrVal, propvarDst.bstrVal), "BSTR not copied properly\n");
     hr = PropVariantClear(&propvarSrc);
     ok(hr == S_OK, "PropVariantClear(...VT_BSTR...) failed\n");
     hr = PropVariantClear(&propvarDst);
     ok(hr == S_OK, "PropVariantClear(...VT_BSTR...) failed\n");
 
+    /* BSTR with embedded null */
+    propvarSrc.vt = VT_BSTR;
+    propvarSrc.bstrVal = SysAllocStringLen(L"Test Str\0ing", 12);
+    hr = PropVariantCopy(&propvarDst, &propvarSrc);
+    ok(hr == S_OK, "Failed to copy propvar, hr %#lx.\n", hr);
+    ok(SysStringLen(propvarDst.bstrVal) == 8, "Unexpected copy length.\n");
+    ok(SysStringLen(propvarSrc.bstrVal) == 12, "Unexpected source length.\n");
+    ok(!lstrcmpW(propvarSrc.bstrVal, propvarDst.bstrVal), "BSTR not copied properly\n");
+    hr = PropVariantClear(&propvarSrc);
+    ok(hr == S_OK, "Failed to clear propvar, hr %#lx.\n", hr);
+    hr = PropVariantClear(&propvarDst);
+    ok(hr == S_OK, "Failed to clear propvar, hr %#lx.\n", hr);
+
     propvarSrc.vt = VT_LPWSTR;
-    U(propvarSrc).pwszVal = wszTestString;
+    propvarSrc.pwszVal = wszTestString;
     hr = PropVariantCopy(&propvarDst, &propvarSrc);
     ok(hr == S_OK, "PropVariantCopy(...VT_LPWSTR...) failed\n");
-    ok(!lstrcmpW(U(propvarSrc).pwszVal, U(propvarDst).pwszVal), "Wide string not copied properly\n");
+    ok(!lstrcmpW(propvarSrc.pwszVal, propvarDst.pwszVal), "Wide string not copied properly\n");
     hr = PropVariantClear(&propvarDst);
     ok(hr == S_OK, "PropVariantClear(...VT_LPWSTR...) failed\n");
     memset(&propvarSrc, 0, sizeof(propvarSrc));
 
     propvarSrc.vt = VT_LPSTR;
-    U(propvarSrc).pszVal = szTestString;
+    propvarSrc.pszVal = szTestString;
     hr = PropVariantCopy(&propvarDst, &propvarSrc);
     ok(hr == S_OK, "PropVariantCopy(...VT_LPSTR...) failed\n");
-    ok(!strcmp(U(propvarSrc).pszVal, U(propvarDst).pszVal), "String not copied properly\n");
+    ok(!strcmp(propvarSrc.pszVal, propvarDst.pszVal), "String not copied properly\n");
     hr = PropVariantClear(&propvarDst);
     ok(hr == S_OK, "PropVariantClear(...VT_LPSTR...) failed\n");
     memset(&propvarSrc, 0, sizeof(propvarSrc));
+
+    propvarSrc.vt = VT_UNKNOWN;
+    propvarSrc.punkVal = &unk_obj.IUnknown_iface;
+    hr = PropVariantCopy(&propvarDst, &propvarSrc);
+    ok(hr == S_OK, "PropVariantCopy(...VT_UNKNOWN...) failed: 0x%08lx.\n", hr);
+    ok(propvarDst.punkVal == &unk_obj.IUnknown_iface, "Got wrong IUnknown pointer\n");
+    ok(unk_obj.ref == 2, "got wrong refcount: %ld.\n", unk_obj.ref);
+    hr = PropVariantClear(&propvarDst);
+    ok(hr == S_OK, "PropVariantClear(...VT_UNKNOWN...) failed: 0x%08lx.\n", hr);
+    ok(unk_obj.ref == 1, "got wrong refcount: %ld.\n", unk_obj.ref);
+    memset(&propvarSrc, 0, sizeof(propvarSrc));
+
+    sabound.lLbound = 0;
+    sabound.cElements = 2;
+    sa = SafeArrayCreate(VT_UNKNOWN, 1, &sabound);
+    saindex = 0;
+    SafeArrayPutElement(sa, &saindex, &unk_obj.IUnknown_iface);
+    saindex = 1;
+    SafeArrayPutElement(sa, &saindex, &unk_obj.IUnknown_iface);
+    ok(unk_obj.ref == 3, "got wrong refcount: %ld.\n", unk_obj.ref);
+
+    propvarSrc.vt = VT_ARRAY | VT_UNKNOWN;
+    propvarSrc.parray = sa;
+    hr = PropVariantCopy(&propvarDst, &propvarSrc);
+    ok(hr == S_OK, "PropVariantCopy(...VT_ARRAY|VT_UNKNOWN...) failed: 0x%08lx.\n", hr);
+    ok(unk_obj.ref == 5, "got wrong refcount: %ld.\n", unk_obj.ref);
+    hr = PropVariantClear(&propvarDst);
+    ok(hr == S_OK, "PropVariantClear(...VT_ARRAY|VT_UNKNOWN...) failed: 0x%08lx.\n", hr);
+    ok(unk_obj.ref == 3, "got wrong refcount: %ld.\n", unk_obj.ref);
+    hr = PropVariantClear(&propvarSrc);
+    ok(hr == S_OK, "PropVariantClear(...VT_ARRAY|VT_UNKNOWN...) failed: 0x%08lx.\n", hr);
+    ok(unk_obj.ref == 1, "got wrong refcount: %ld.\n", unk_obj.ref);
+    memset(&propvarSrc, 0, sizeof(propvarSrc));
 }
 
 struct _PMemoryAllocator_vtable {
@@ -357,7 +453,7 @@ static void WINAPI PMemoryAllocator_Free(PMemoryAllocator *_this, void *pv)
 
 #ifdef __i386__
 
-#include "pshpack1.h"
+#pragma pack(push,1)
 typedef struct
 {
     BYTE pop_eax;  /* popl  %eax  */
@@ -366,7 +462,7 @@ typedef struct
     BYTE jmp_func; /* jmp   $func */
     DWORD func;
 } THISCALL_TO_STDCALL_THUNK;
-#include "poppack.h"
+#pragma pack(pop)
 
 static THISCALL_TO_STDCALL_THUNK *wrapperCodeMem = NULL;
 
@@ -434,6 +530,29 @@ static const char serialized_bstr_mb[] = {
     0,0,0,0
 };
 
+static const char serialized_bstr_vec[] = {
+    VT_LPWSTR, VT_VECTOR >> 8, 0, 0,
+    3, 0, 0, 0,
+    2, 0, 0, 0, 'a', 0, 0, 0,
+    3, 0, 0, 0, 'b', 0, 'c', 0, 0, 0, 0, 0,
+    4, 0, 0, 0, 'd', 0, 'e', 0, 'f', 0, 0, 0
+};
+
+static const char serialized_i1_vec[] = {
+    VT_I1, VT_VECTOR >> 8, 0, 0,
+    3, 0, 0, 0,
+    1, 2, 3, 0
+};
+
+static const char serialized_variant_vec[] = {
+    VT_VARIANT, VT_VECTOR >> 8, 0, 0,
+    2, 0, 0, 0,
+    VT_I1, 0, 0, 0,
+    1, 0, 0, 0,
+    VT_I4, 0, 0, 0,
+    2, 0, 0, 0
+};
+
 static void test_propertytovariant(void)
 {
     HANDLE hole32;
@@ -474,14 +593,14 @@ static void test_propertytovariant(void)
 
     ok(ret == 0, "StgConvertPropertyToVariant returned %i\n", ret);
     ok(propvar.vt == VT_I4, "unexpected vt %x\n", propvar.vt);
-    ok(U(propvar).lVal == 0xfeabcdef, "unexpected lVal %x\n", U(propvar).lVal);
+    ok(propvar.lVal == 0xfeabcdef, "unexpected lVal %lx\n", propvar.lVal);
 
     ret = pStgConvertPropertyToVariant((SERIALIZEDPROPERTYVALUE*)serialized_bstr_wc,
         CP_WINUNICODE, &propvar, &allocator);
 
     ok(ret == 0, "StgConvertPropertyToVariant returned %i\n", ret);
     ok(propvar.vt == VT_BSTR, "unexpected vt %x\n", propvar.vt);
-    ok(!lstrcmpW(U(propvar).bstrVal, test_string), "unexpected string value\n");
+    ok(!lstrcmpW(propvar.bstrVal, test_string), "unexpected string value\n");
     PropVariantClear(&propvar);
 
     ret = pStgConvertPropertyToVariant((SERIALIZEDPROPERTYVALUE*)serialized_bstr_mb,
@@ -489,7 +608,44 @@ static void test_propertytovariant(void)
 
     ok(ret == 0, "StgConvertPropertyToVariant returned %i\n", ret);
     ok(propvar.vt == VT_BSTR, "unexpected vt %x\n", propvar.vt);
-    ok(!lstrcmpW(U(propvar).bstrVal, test_string), "unexpected string value\n");
+    ok(!lstrcmpW(propvar.bstrVal, test_string), "unexpected string value\n");
+    PropVariantClear(&propvar);
+
+    ret = pStgConvertPropertyToVariant((SERIALIZEDPROPERTYVALUE*)serialized_i1_vec,
+        CP_WINUNICODE, &propvar, &allocator);
+
+    ok(!ret, "StgConvertPropertyToVariant returned %i\n", ret);
+    ok(propvar.vt == (VT_VECTOR | VT_I1), "unexpected vt %x\n", propvar.vt);
+    ok(propvar.cac.cElems == 3, "unexpected cElems %lu\n", propvar.cac.cElems);
+    ok(propvar.cac.pElems[0] == 1, "pElems[0] = %d\n", propvar.cac.pElems[0]);
+    ok(propvar.cac.pElems[1] == 2, "pElems[0] = %d\n", propvar.cac.pElems[1]);
+    ok(propvar.cac.pElems[2] == 3, "pElems[0] = %d\n", propvar.cac.pElems[2]);
+    PropVariantClear(&propvar);
+
+    ret = pStgConvertPropertyToVariant((SERIALIZEDPROPERTYVALUE*)serialized_bstr_vec,
+        CP_WINUNICODE, &propvar, &allocator);
+
+    ok(!ret, "StgConvertPropertyToVariant returned %i\n", ret);
+    ok(propvar.vt == (VT_VECTOR | VT_LPWSTR), "unexpected vt %x\n", propvar.vt);
+    ok(propvar.calpwstr.cElems == 3, "unexpected cElems %lu\n", propvar.calpwstr.cElems);
+    ok(!wcscmp(propvar.calpwstr.pElems[0], L"a"), "pElems[0] = %s\n",
+            debugstr_w(propvar.calpwstr.pElems[0]));
+    ok(!wcscmp(propvar.calpwstr.pElems[1], L"bc"), "pElems[1] = %s\n",
+            debugstr_w(propvar.calpwstr.pElems[1]));
+    ok(!wcscmp(propvar.calpwstr.pElems[2], L"def"), "pElems[2] = %s\n",
+            debugstr_w(propvar.calpwstr.pElems[2]));
+    PropVariantClear(&propvar);
+
+    ret = pStgConvertPropertyToVariant((SERIALIZEDPROPERTYVALUE*)serialized_variant_vec,
+        CP_WINUNICODE, &propvar, &allocator);
+
+    ok(!ret, "StgConvertPropertyToVariant returned %i\n", ret);
+    ok(propvar.vt == (VT_VECTOR | VT_VARIANT), "unexpected vt %x\n", propvar.vt);
+    ok(propvar.capropvar.cElems == 2, "unexpected cElems %lu\n", propvar.calpwstr.cElems);
+    ok(propvar.capropvar.pElems[0].vt == VT_I1, "pElems[0].vt = %d\n", propvar.capropvar.pElems[0].vt);
+    ok(propvar.capropvar.pElems[0].cVal == 1, "pElems[0].cVal = %d\n", propvar.capropvar.pElems[0].cVal);
+    ok(propvar.capropvar.pElems[1].vt == VT_I4, "pElems[1].vt = %d\n", propvar.capropvar.pElems[1].vt);
+    ok(propvar.capropvar.pElems[1].lVal == 2, "pElems[1].lVal = %ld\n", propvar.capropvar.pElems[1].lVal);
     PropVariantClear(&propvar);
 }
 
@@ -514,23 +670,23 @@ static void test_varianttoproperty(void)
         return;
     }
 
-    own_propvalue = HeapAlloc(GetProcessHeap(), 0, sizeof(SERIALIZEDPROPERTYVALUE) + 20);
+    own_propvalue = malloc(sizeof(SERIALIZEDPROPERTYVALUE) + 20);
 
     PropVariantInit(&propvar);
 
     propvar.vt = VT_I4;
-    U(propvar).lVal = 0xfeabcdef;
+    propvar.lVal = 0xfeabcdef;
 
     len = 0xdeadbeef;
     propvalue = pStgConvertVariantToProperty(&propvar, CP_WINUNICODE, NULL, &len,
         0, FALSE, 0);
 
     ok(propvalue == NULL, "got nonnull propvalue\n");
-    todo_wine ok(len == 8, "unexpected length %d\n", len);
+    todo_wine ok(len == 8, "unexpected length %ld\n", len);
 
     if (len == 0xdeadbeef)
     {
-        HeapFree(GetProcessHeap(), 0, own_propvalue);
+        free(own_propvalue);
         return;
     }
 
@@ -539,7 +695,7 @@ static void test_varianttoproperty(void)
         0, FALSE, 0);
 
     ok(propvalue == own_propvalue, "unexpected propvalue %p\n", propvalue);
-    ok(len == 8, "unexpected length %d\n", len);
+    ok(len == 8, "unexpected length %ld\n", len);
     ok(!memcmp(propvalue, serialized_i4, 8), "got wrong data\n");
 
     propvar.vt = VT_EMPTY;
@@ -549,9 +705,9 @@ static void test_varianttoproperty(void)
         0, FALSE, 0);
 
     ok(propvalue == own_propvalue, "unexpected propvalue %p\n", propvalue);
-    ok(len == 4 || broken(len == 0) /* before Vista */, "unexpected length %d\n", len);
+    ok(len == 4 || broken(len == 0) /* before Vista */, "unexpected length %ld\n", len);
     if (len) ok(!memcmp(propvalue, serialized_empty, 4), "got wrong data\n");
-    else ok(propvalue->dwType == 0xdeadbeef, "unexpected type %d\n", propvalue->dwType);
+    else ok(propvalue->dwType == 0xdeadbeef, "unexpected type %ld\n", propvalue->dwType);
 
     propvar.vt = VT_NULL;
     len = 20;
@@ -559,19 +715,19 @@ static void test_varianttoproperty(void)
         0, FALSE, 0);
 
     ok(propvalue == own_propvalue, "unexpected propvalue %p\n", propvalue);
-    ok(len == 4, "unexpected length %d\n", len);
+    ok(len == 4, "unexpected length %ld\n", len);
     ok(!memcmp(propvalue, serialized_null, 4), "got wrong data\n");
 
     test_string_bstr = SysAllocString(test_string);
 
     propvar.vt = VT_BSTR;
-    U(propvar).bstrVal = test_string_bstr;
+    propvar.bstrVal = test_string_bstr;
     len = 20;
     propvalue = pStgConvertVariantToProperty(&propvar, CP_WINUNICODE, own_propvalue, &len,
         0, FALSE, 0);
 
     ok(propvalue == own_propvalue, "unexpected propvalue %p\n", propvalue);
-    ok(len == 20, "unexpected length %d\n", len);
+    ok(len == 20, "unexpected length %ld\n", len);
     ok(!memcmp(propvalue, serialized_bstr_wc, 20), "got wrong data\n");
 
     len = 20;
@@ -579,12 +735,12 @@ static void test_varianttoproperty(void)
         0, FALSE, 0);
 
     ok(propvalue == own_propvalue, "unexpected propvalue %p\n", propvalue);
-    ok(len == 16, "unexpected length %d\n", len);
+    ok(len == 16, "unexpected length %ld\n", len);
     ok(!memcmp(propvalue, serialized_bstr_mb, 16), "got wrong data\n");
 
     SysFreeString(test_string_bstr);
 
-    HeapFree(GetProcessHeap(), 0, own_propvalue);
+    free(own_propvalue);
 }
 
 START_TEST(propvariant)
