@@ -300,7 +300,7 @@ typedef struct {
  *
  */
 
-#include "pshpack1.h"
+#pragma pack(push,1)
 
 typedef struct {
 /*00*/	DWORD SLTG_magic;	/* 0x47544c53  == "SLTG" */
@@ -309,10 +309,7 @@ typedef struct {
 /*08*/  WORD res08;             /* some kind of len/offset ?? */
 /*0a*/	WORD first_blk;		/* 1 based index into blk entries that
 				   corresponds to first block in file */
-/*0c*/	DWORD res0c;		/* always 0x000204ff */
-/*10*/  DWORD res10;		/* always 0x00000000 */
-/*14*/	DWORD res14;		/* always 0x000000c0 */
-/*18*/	DWORD res18;		/* always 0x46000000 */
+/*0c*/	GUID guid;		/* always 000204ff-0000-0000-c000-000000000046 */
 /*1c*/	DWORD res1c;		/* always 0x00000044 */
 /*20*/	DWORD res20;		/* always 0xffff0000 */
 } SLTG_Header;
@@ -329,12 +326,11 @@ typedef struct {
 
 /* These then get followed by this magic */
 typedef struct {
-/*00*/ BYTE res00;		/* always 0x01 */
-/*01*/ CHAR CompObj_magic[8];	/* always "CompObj" */
+/*00*/ CHAR CompObj_magic[9];	/* always "\1CompObj" */
 /*09*/ CHAR dir_magic[4];	/* always "dir" */
 } SLTG_Magic;
 
-#define SLTG_COMPOBJ_MAGIC "CompObj"
+#define SLTG_COMPOBJ_MAGIC "\1CompObj"
 #define SLTG_DIR_MAGIC "dir"
 
 /* Next we have SLTG_Header.nrOfFileBlks - 2 of Index strings.  These
@@ -386,18 +382,18 @@ typedef struct {
 /* we then get 0x40 bytes worth of 0xffff or small numbers followed by
    nrOfFileBlks - 2 of these */
 typedef struct {
-	WORD small_no;
 	SLTG_Name index_name; /* This refers to a name in the directory */
 	SLTG_Name other_name; /* Another one of these weird names */
 	WORD res1a;	      /* 0xffff */
 	WORD name_offs;	      /* offset to name in name table */
-	WORD more_bytes;      /* if this is non-zero we get this many
+	WORD hlpstr_len;      /* if this is non-zero we get this many
 				 bytes before the next element, which seem
 				 to reference the docstring of the type ? */
 	WORD res20;	      /* 0xffff */
 	DWORD helpcontext;
 	WORD res26;	      /* 0xffff */
         GUID uuid;
+        WORD typekind;
 } SLTG_OtherTypeInfo;
 
 /* Next we get WORD 0x0003 followed by a DWORD which if we add to
@@ -593,7 +589,7 @@ WORD offset from start of block to SAFEARRAY
 WORD typeofarray
 */
 
-#include "poppack.h"
+#pragma pack(pop)
 
 /* The OLE Automation ProxyStub Interface Class (aka Typelib Marshaler) */
 DEFINE_OLEGUID( CLSID_PSDispatch,    0x00020420, 0x0000, 0x0000 );
