@@ -170,8 +170,8 @@ static void opc_content_release(struct opc_content *content)
 
     if (!refcount)
     {
-        heap_free(content->data);
-        heap_free(content);
+        free(content->data);
+        free(content);
     }
 }
 
@@ -199,7 +199,7 @@ static ULONG WINAPI opc_part_enum_AddRef(IOpcPartEnumerator *iface)
     struct opc_part_enum *part_enum = impl_from_IOpcPartEnumerator(iface);
     ULONG refcount = InterlockedIncrement(&part_enum->refcount);
 
-    TRACE("%p increasing refcount to %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     return refcount;
 }
@@ -209,12 +209,12 @@ static ULONG WINAPI opc_part_enum_Release(IOpcPartEnumerator *iface)
     struct opc_part_enum *part_enum = impl_from_IOpcPartEnumerator(iface);
     ULONG refcount = InterlockedDecrement(&part_enum->refcount);
 
-    TRACE("%p decreasing refcount to %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     if (!refcount)
     {
         IOpcPartSet_Release(&part_enum->part_set->IOpcPartSet_iface);
-        heap_free(part_enum);
+        free(part_enum);
     }
 
     return refcount;
@@ -237,7 +237,10 @@ static HRESULT WINAPI opc_part_enum_MoveNext(IOpcPartEnumerator *iface, BOOL *ha
     if (has_part_collection_changed(part_enum))
         return OPC_E_ENUM_COLLECTION_CHANGED;
 
-    if (part_enum->part_set->count && (part_enum->pos == ~(size_t)0 || part_enum->pos < part_enum->part_set->count))
+    if (part_enum->pos == part_enum->part_set->count)
+        return OPC_E_ENUM_CANNOT_MOVE_NEXT;
+
+    if (part_enum->pos == ~(size_t)0 || part_enum->pos < part_enum->part_set->count)
         part_enum->pos++;
 
     *has_next = part_enum->pos < part_enum->part_set->count;
@@ -257,10 +260,10 @@ static HRESULT WINAPI opc_part_enum_MovePrevious(IOpcPartEnumerator *iface, BOOL
     if (has_part_collection_changed(part_enum))
         return OPC_E_ENUM_COLLECTION_CHANGED;
 
-    if (part_enum->pos != ~(size_t)0)
-        part_enum->pos--;
+    if (part_enum->pos == ~(size_t)0)
+        return OPC_E_ENUM_CANNOT_MOVE_PREVIOUS;
 
-    *has_previous = part_enum->pos != ~(size_t)0;
+    *has_previous = --part_enum->pos != ~(size_t)0;
 
     return S_OK;
 }
@@ -321,7 +324,7 @@ static HRESULT opc_part_enum_create(struct opc_part_set *part_set, IOpcPartEnume
 {
     struct opc_part_enum *part_enum;
 
-    if (!(part_enum = heap_alloc_zero(sizeof(*part_enum))))
+    if (!(part_enum = calloc(1, sizeof(*part_enum))))
         return E_OUTOFMEMORY;
 
     part_enum->IOpcPartEnumerator_iface.lpVtbl = &opc_part_enum_vtbl;
@@ -360,7 +363,7 @@ static ULONG WINAPI opc_rel_enum_AddRef(IOpcRelationshipEnumerator *iface)
     struct opc_rel_enum *rel_enum = impl_from_IOpcRelationshipEnumerator(iface);
     ULONG refcount = InterlockedIncrement(&rel_enum->refcount);
 
-    TRACE("%p increasing refcount to %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     return refcount;
 }
@@ -370,12 +373,12 @@ static ULONG WINAPI opc_rel_enum_Release(IOpcRelationshipEnumerator *iface)
     struct opc_rel_enum *rel_enum = impl_from_IOpcRelationshipEnumerator(iface);
     ULONG refcount = InterlockedDecrement(&rel_enum->refcount);
 
-    TRACE("%p decreasing refcount to %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     if (!refcount)
     {
         IOpcRelationshipSet_Release(&rel_enum->rel_set->IOpcRelationshipSet_iface);
-        heap_free(rel_enum);
+        free(rel_enum);
     }
 
     return refcount;
@@ -482,7 +485,7 @@ static HRESULT opc_rel_enum_create(struct opc_relationship_set *rel_set, IOpcRel
 {
     struct opc_rel_enum *rel_enum;
 
-    if (!(rel_enum = heap_alloc_zero(sizeof(*rel_enum))))
+    if (!(rel_enum = calloc(1, sizeof(*rel_enum))))
         return E_OUTOFMEMORY;
 
     rel_enum->IOpcRelationshipEnumerator_iface.lpVtbl = &opc_rel_enum_vtbl;
@@ -520,7 +523,7 @@ static ULONG WINAPI opc_content_stream_AddRef(IStream *iface)
     struct opc_content_stream *stream = impl_from_IStream(iface);
     ULONG refcount = InterlockedIncrement(&stream->refcount);
 
-    TRACE("%p increasing refcount to %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     return refcount;
 }
@@ -530,12 +533,12 @@ static ULONG WINAPI opc_content_stream_Release(IStream *iface)
     struct opc_content_stream *stream = impl_from_IStream(iface);
     ULONG refcount = InterlockedDecrement(&stream->refcount);
 
-    TRACE("%p decreasing refcount to %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     if (!refcount)
     {
         opc_content_release(stream->content);
-        heap_free(stream);
+        free(stream);
     }
 
     return refcount;
@@ -546,7 +549,7 @@ static HRESULT WINAPI opc_content_stream_Read(IStream *iface, void *buff, ULONG
     struct opc_content_stream *stream = impl_from_IStream(iface);
     DWORD read = 0;
 
-    TRACE("iface %p, buff %p, size %u, num_read %p.\n", iface, buff, size, num_read);
+    TRACE("%p, %p, %lu, %p.\n", iface, buff, size, num_read);
 
     if (!num_read)
         num_read = &read;
@@ -569,7 +572,7 @@ static HRESULT WINAPI opc_content_stream_Write(IStream *iface, const void *data,
     struct opc_content_stream *stream = impl_from_IStream(iface);
     DWORD written = 0;
 
-    TRACE("iface %p, data %p, size %u, num_written %p.\n", iface, data, size, num_written);
+    TRACE("%p, %p, %lu, %p.\n", iface, data, size, num_written);
 
     if (!num_written)
         num_written = &written;
@@ -578,7 +581,7 @@ static HRESULT WINAPI opc_content_stream_Write(IStream *iface, const void *data,
 
     if (size > stream->content->size.QuadPart - stream->pos.QuadPart)
     {
-        void *ptr = heap_realloc(stream->content->data, stream->pos.QuadPart + size);
+        void *ptr = realloc(stream->content->data, stream->pos.QuadPart + size);
         if (!ptr)
             return E_OUTOFMEMORY;
         stream->content->data = ptr;
@@ -597,7 +600,7 @@ static HRESULT WINAPI opc_content_stream_Seek(IStream *iface, LARGE_INTEGER move
     struct opc_content_stream *stream = impl_from_IStream(iface);
     ULARGE_INTEGER pos;
 
-    TRACE("iface %p, move %s, origin %d, newpos %p.\n", iface, wine_dbgstr_longlong(move.QuadPart), origin, newpos);
+    TRACE("%p, %s, %ld, %p.\n", iface, wine_dbgstr_longlong(move.QuadPart), origin, newpos);
 
     switch (origin)
     {
@@ -611,7 +614,7 @@ static HRESULT WINAPI opc_content_stream_Seek(IStream *iface, LARGE_INTEGER move
         pos.QuadPart = stream->content->size.QuadPart + move.QuadPart;
         break;
     default:
-        WARN("Unknown origin mode %d.\n", origin);
+        WARN("Unknown origin mode %ld.\n", origin);
         return E_INVALIDARG;
     }
 
@@ -641,7 +644,7 @@ static HRESULT WINAPI opc_content_stream_CopyTo(IStream *iface, IStream *dest, U
 
 static HRESULT WINAPI opc_content_stream_Commit(IStream *iface, DWORD flags)
 {
-    FIXME("iface %p, flags %#x stub!\n", iface, flags);
+    FIXME("iface %p, flags %#lx stub!\n", iface, flags);
 
     return E_NOTIMPL;
 }
@@ -656,7 +659,7 @@ static HRESULT WINAPI opc_content_stream_Revert(IStream *iface)
 static HRESULT WINAPI opc_content_stream_LockRegion(IStream *iface, ULARGE_INTEGER offset,
         ULARGE_INTEGER size, DWORD lock_type)
 {
-    FIXME("iface %p, offset %s, size %s, lock_type %d stub!\n", iface, wine_dbgstr_longlong(offset.QuadPart),
+    FIXME("iface %p, offset %s, size %s, lock_type %ld stub!\n", iface, wine_dbgstr_longlong(offset.QuadPart),
             wine_dbgstr_longlong(size.QuadPart), lock_type);
 
     return E_NOTIMPL;
@@ -665,7 +668,7 @@ static HRESULT WINAPI opc_content_stream_LockRegion(IStream *iface, ULARGE_INTEG
 static HRESULT WINAPI opc_content_stream_UnlockRegion(IStream *iface, ULARGE_INTEGER offset, ULARGE_INTEGER size,
         DWORD lock_type)
 {
-    FIXME("iface %p, offset %s, size %s, lock_type %d stub!\n", iface, wine_dbgstr_longlong(offset.QuadPart),
+    FIXME("iface %p, offset %s, size %s, lock_type %ld stub!\n", iface, wine_dbgstr_longlong(offset.QuadPart),
             wine_dbgstr_longlong(size.QuadPart), lock_type);
 
     return E_NOTIMPL;
@@ -673,7 +676,7 @@ static HRESULT WINAPI opc_content_stream_UnlockRegion(IStream *iface, ULARGE_INT
 
 static HRESULT WINAPI opc_content_stream_Stat(IStream *iface, STATSTG *statstg, DWORD flag)
 {
-    FIXME("iface %p, statstg %p, flag %d stub!\n", iface, statstg, flag);
+    FIXME("iface %p, statstg %p, flag %ld stub!\n", iface, statstg, flag);
 
     return E_NOTIMPL;
 }
@@ -707,7 +710,7 @@ static HRESULT opc_content_stream_create(struct opc_content *content, IStream **
 {
     struct opc_content_stream *stream;
 
-    if (!(stream = heap_alloc_zero(sizeof(*stream))))
+    if (!(stream = calloc(1, sizeof(*stream))))
         return E_OUTOFMEMORY;
 
     stream->IStream_iface.lpVtbl = &opc_content_stream_vtbl;
@@ -761,7 +764,7 @@ static ULONG WINAPI opc_part_AddRef(IOpcPart *iface)
     struct opc_part *part = impl_from_IOpcPart(iface);
     ULONG refcount = InterlockedIncrement(&part->refcount);
 
-    TRACE("%p increasing refcount to %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     return refcount;
 }
@@ -771,7 +774,7 @@ static ULONG WINAPI opc_part_Release(IOpcPart *iface)
     struct opc_part *part = impl_from_IOpcPart(iface);
     ULONG refcount = InterlockedDecrement(&part->refcount);
 
-    TRACE("%p decreasing refcount to %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     if (!refcount)
     {
@@ -780,7 +783,7 @@ static ULONG WINAPI opc_part_Release(IOpcPart *iface)
         IOpcPartUri_Release(part->name);
         CoTaskMemFree(part->content_type);
         opc_content_release(part->content);
-        heap_free(part);
+        free(part);
     }
 
     return refcount;
@@ -866,7 +869,7 @@ static HRESULT opc_part_create(struct opc_part_set *set, IOpcPartUri *name, cons
     if (!opc_array_reserve((void **)&set->parts, &set->size, set->count + 1, sizeof(*set->parts)))
         return E_OUTOFMEMORY;
 
-    if (!(part = heap_alloc_zero(sizeof(*part))))
+    if (!(part = calloc(1, sizeof(*part))))
         return E_OUTOFMEMORY;
 
     part->IOpcPart_iface.lpVtbl = &opc_part_vtbl;
@@ -880,7 +883,7 @@ static HRESULT opc_part_create(struct opc_part_set *set, IOpcPartUri *name, cons
         return E_OUTOFMEMORY;
     }
 
-    part->content = heap_alloc_zero(sizeof(*part->content));
+    part->content = calloc(1, sizeof(*part->content));
     if (!part->content)
     {
         IOpcPart_Release(&part->IOpcPart_iface);
@@ -933,7 +936,7 @@ static ULONG WINAPI opc_part_set_AddRef(IOpcPartSet *iface)
     struct opc_part_set *part_set = impl_from_IOpcPartSet(iface);
     ULONG refcount = InterlockedIncrement(&part_set->refcount);
 
-    TRACE("%p increasing refcount to %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     return refcount;
 }
@@ -943,7 +946,7 @@ static ULONG WINAPI opc_part_set_Release(IOpcPartSet *iface)
     struct opc_part_set *part_set = impl_from_IOpcPartSet(iface);
     ULONG refcount = InterlockedDecrement(&part_set->refcount);
 
-    TRACE("%p decreasing refcount to %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     if (!refcount)
     {
@@ -951,8 +954,8 @@ static ULONG WINAPI opc_part_set_Release(IOpcPartSet *iface)
 
         for (i = 0; i < part_set->count; ++i)
             IOpcPart_Release(&part_set->parts[i]->IOpcPart_iface);
-        heap_free(part_set->parts);
-        heap_free(part_set);
+        free(part_set->parts);
+        free(part_set);
     }
 
     return refcount;
@@ -1070,7 +1073,7 @@ static ULONG WINAPI opc_relationship_AddRef(IOpcRelationship *iface)
     struct opc_relationship *relationship = impl_from_IOpcRelationship(iface);
     ULONG refcount = InterlockedIncrement(&relationship->refcount);
 
-    TRACE("%p increasing refcount to %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     return refcount;
 }
@@ -1080,7 +1083,7 @@ static ULONG WINAPI opc_relationship_Release(IOpcRelationship *iface)
     struct opc_relationship *relationship = impl_from_IOpcRelationship(iface);
     ULONG refcount = InterlockedDecrement(&relationship->refcount);
 
-    TRACE("%p decreasing refcount to %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     if (!refcount)
     {
@@ -1088,7 +1091,7 @@ static ULONG WINAPI opc_relationship_Release(IOpcRelationship *iface)
         CoTaskMemFree(relationship->type);
         IOpcUri_Release(relationship->source_uri);
         IUri_Release(relationship->target);
-        heap_free(relationship);
+        free(relationship);
     }
 
     return refcount;
@@ -1183,7 +1186,7 @@ static HRESULT opc_relationship_create(struct opc_relationship_set *set, const W
     if (!opc_array_reserve((void **)&set->relationships, &set->size, set->count + 1, sizeof(*set->relationships)))
         return E_OUTOFMEMORY;
 
-    if (!(relationship = heap_alloc_zero(sizeof(*relationship))))
+    if (!(relationship = calloc(1, sizeof(*relationship))))
         return E_OUTOFMEMORY;
 
     relationship->IOpcRelationship_iface.lpVtbl = &opc_relationship_vtbl;
@@ -1201,12 +1204,11 @@ static HRESULT opc_relationship_create(struct opc_relationship_set *set, const W
         relationship->id = CoTaskMemAlloc(10 * sizeof(WCHAR));
         if (relationship->id)
         {
-            static const WCHAR fmtW[] = {'R','%','0','8','X',0};
             DWORD generated;
 
             /* FIXME: test that generated id is unique */
             RtlGenRandom(&generated, sizeof(generated));
-            swprintf(relationship->id, 10, fmtW, generated);
+            swprintf(relationship->id, 10, L"R%08X", generated);
 
             if (opc_relationshipset_get_rel(set, relationship->id))
             {
@@ -1254,7 +1256,7 @@ static ULONG WINAPI opc_relationship_set_AddRef(IOpcRelationshipSet *iface)
     struct opc_relationship_set *relationship_set = impl_from_IOpcRelationshipSet(iface);
     ULONG refcount = InterlockedIncrement(&relationship_set->refcount);
 
-    TRACE("%p increasing refcount to %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     return refcount;
 }
@@ -1264,7 +1266,7 @@ static ULONG WINAPI opc_relationship_set_Release(IOpcRelationshipSet *iface)
     struct opc_relationship_set *relationship_set = impl_from_IOpcRelationshipSet(iface);
     ULONG refcount = InterlockedDecrement(&relationship_set->refcount);
 
-    TRACE("%p decreasing refcount to %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     if (!refcount)
     {
@@ -1273,8 +1275,8 @@ static ULONG WINAPI opc_relationship_set_Release(IOpcRelationshipSet *iface)
         for (i = 0; i < relationship_set->count; ++i)
             IOpcRelationship_Release(&relationship_set->relationships[i]->IOpcRelationship_iface);
         IOpcUri_Release(relationship_set->source_uri);
-        heap_free(relationship_set->relationships);
-        heap_free(relationship_set);
+        free(relationship_set->relationships);
+        free(relationship_set);
     }
 
     return refcount;
@@ -1399,7 +1401,7 @@ static HRESULT opc_relationship_set_create(IOpcUri *source_uri, IOpcRelationship
 {
     struct opc_relationship_set *relationship_set;
 
-    if (!(relationship_set = heap_alloc_zero(sizeof(*relationship_set))))
+    if (!(relationship_set = calloc(1, sizeof(*relationship_set))))
         return E_OUTOFMEMORY;
 
     relationship_set->IOpcRelationshipSet_iface.lpVtbl = &opc_relationship_set_vtbl;
@@ -1433,7 +1435,7 @@ static ULONG WINAPI opc_package_AddRef(IOpcPackage *iface)
     struct opc_package *package = impl_from_IOpcPackage(iface);
     ULONG refcount = InterlockedIncrement(&package->refcount);
 
-    TRACE("%p increasing refcount to %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     return refcount;
 }
@@ -1443,7 +1445,7 @@ static ULONG WINAPI opc_package_Release(IOpcPackage *iface)
     struct opc_package *package = impl_from_IOpcPackage(iface);
     ULONG refcount = InterlockedDecrement(&package->refcount);
 
-    TRACE("%p decreasing refcount to %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     if (!refcount)
     {
@@ -1453,7 +1455,7 @@ static ULONG WINAPI opc_package_Release(IOpcPackage *iface)
             IOpcRelationshipSet_Release(package->relationship_set);
         if (package->source_uri)
             IOpcUri_Release(package->source_uri);
-        heap_free(package);
+        free(package);
     }
 
     return refcount;
@@ -1467,7 +1469,7 @@ static HRESULT WINAPI opc_package_GetPartSet(IOpcPackage *iface, IOpcPartSet **p
 
     if (!package->part_set)
     {
-        struct opc_part_set *part_set = heap_alloc_zero(sizeof(*part_set));
+        struct opc_part_set *part_set = calloc(1, sizeof(*part_set));
         if (!part_set)
             return E_OUTOFMEMORY;
 
@@ -1516,7 +1518,7 @@ HRESULT opc_package_create(IOpcFactory *factory, IOpcPackage **out)
     struct opc_package *package;
     HRESULT hr;
 
-    if (!(package = heap_alloc_zero(sizeof(*package))))
+    if (!(package = calloc(1, sizeof(*package))))
         return E_OUTOFMEMORY;
 
     package->IOpcPackage_iface.lpVtbl = &opc_package_vtbl;
@@ -1524,7 +1526,7 @@ HRESULT opc_package_create(IOpcFactory *factory, IOpcPackage **out)
 
     if (FAILED(hr = IOpcFactory_CreatePackageRootUri(factory, &package->source_uri)))
     {
-        heap_free(package);
+        free(package);
         return hr;
     }
 
@@ -1567,7 +1569,7 @@ static HRESULT opc_package_add_override_content_type(struct content_types *types
 {
     struct content_type *type;
 
-    if (!(type = heap_alloc(sizeof(*type))))
+    if (!(type = malloc(sizeof(*type))))
         return E_OUTOFMEMORY;
 
     type->element = CONTENT_TYPE_OVERRIDE;
@@ -1584,7 +1586,7 @@ static HRESULT opc_package_add_default_content_type(struct content_types *types,
 {
     struct content_type *type;
 
-    if (!(type = heap_alloc(sizeof(*type))))
+    if (!(type = malloc(sizeof(*type))))
         return E_OUTOFMEMORY;
 
     type->element = CONTENT_TYPE_DEFAULT;
@@ -1594,7 +1596,7 @@ static HRESULT opc_package_add_default_content_type(struct content_types *types,
     {
         CoTaskMemFree(type->u.def.ext);
         CoTaskMemFree(type->u.def.type);
-        heap_free(type);
+        free(type);
         return E_OUTOFMEMORY;
     }
 
@@ -1606,7 +1608,8 @@ static HRESULT opc_package_add_default_content_type(struct content_types *types,
 static HRESULT opc_package_add_content_type(struct content_types *types, IOpcPart *part)
 {
     struct content_type *cur;
-    BSTR ext, content_type;
+    BSTR ext;
+    WCHAR *content_type;
     BOOL added = FALSE;
     IOpcPartUri *name;
     HRESULT hr;
@@ -1650,7 +1653,7 @@ static HRESULT opc_package_add_content_type(struct content_types *types, IOpcPar
         hr = opc_package_add_default_content_type(types, ext, content_type);
 
     SysFreeString(ext);
-    SysFreeString(content_type);
+    CoTaskMemFree(content_type);
 
     return hr;
 }
@@ -1733,31 +1736,18 @@ static HRESULT opc_package_collect_content_types(IOpcPackage *package, struct co
 
 static HRESULT opc_package_write_default_type(const WCHAR *ext, const WCHAR *type, IXmlWriter *writer)
 {
-    static const WCHAR contenttypeW[] = {'C','o','n','t','e','n','t','T','y','p','e',0};
-    static const WCHAR extensionW[] = {'E','x','t','e','n','s','i','o','n',0};
-    static const WCHAR defaultW[] = {'D','e','f','a','u','l','t',0};
     HRESULT hr;
 
-    hr = IXmlWriter_WriteStartElement(writer, NULL, defaultW, NULL);
+    hr = IXmlWriter_WriteStartElement(writer, NULL, L"Default", NULL);
     if (SUCCEEDED(hr))
-        hr = IXmlWriter_WriteAttributeString(writer, NULL, extensionW, NULL, ext);
+        hr = IXmlWriter_WriteAttributeString(writer, NULL, L"Extension", NULL, ext);
     if (SUCCEEDED(hr))
-        hr = IXmlWriter_WriteAttributeString(writer, NULL, contenttypeW, NULL, type);
+        hr = IXmlWriter_WriteAttributeString(writer, NULL, L"ContentType", NULL, type);
     return hr;
 }
 
 static HRESULT opc_package_write_contenttypes(IOpcPackage *package, struct zip_archive *archive, IXmlWriter *writer)
 {
-    static const WCHAR uriW[] = {'h','t','t','p',':','/','/','s','c','h','e','m','a','s','.','o','p','e','n','x','m','l','f','o','r','m','a','t','s','.','o','r','g','/',
-            'p','a','c','k','a','g','e','/','2','0','0','6','/','c','o','n','t','e','n','t','-','t','y','p','e','s',0};
-    static const WCHAR relstypeW[] = {'a','p','p','l','i','c','a','t','i','o','n','/','v','n','d','.','o','p','e','n','x','m','l','f','o','r','m','a','t','s','-',
-            'p','a','c','k','a','g','e','.','r','e','l','a','t','i','o','n','s','h','i','p','s','+','x','m','l',0};
-    static const WCHAR contenttypesW[] = {'[','C','o','n','t','e','n','t','_','T','y','p','e','s',']','.','x','m','l',0};
-    static const WCHAR contenttypeW[] = {'C','o','n','t','e','n','t','T','y','p','e',0};
-    static const WCHAR overrideW[] = {'O','v','e','r','r','i','d','e',0};
-    static const WCHAR partnameW[] = {'P','a','r','t','N','a','m','e',0};
-    static const WCHAR typesW[] = {'T','y','p','e','s',0};
-    static const WCHAR relsW[] = {'r','e','l','s',0};
     struct content_type *content_type, *content_type2;
     struct content_types types;
     IStream *content = NULL;
@@ -1774,11 +1764,13 @@ static HRESULT opc_package_write_contenttypes(IOpcPackage *package, struct zip_a
     if (SUCCEEDED(hr))
         hr = IXmlWriter_WriteStartDocument(writer, XmlStandalone_Omit);
     if (SUCCEEDED(hr))
-        hr = IXmlWriter_WriteStartElement(writer, NULL, typesW, uriW);
+        hr = IXmlWriter_WriteStartElement(writer, NULL, L"Types",
+                L"http://schemas.openxmlformats.org/package/2006/content-types");
 
     if (SUCCEEDED(hr) && types.has_rels_part)
     {
-        hr = opc_package_write_default_type(relsW, relstypeW, writer);
+        hr = opc_package_write_default_type(L"rels",
+                L"application/vnd.openxmlformats-package.relationships+xml", writer);
         if (SUCCEEDED(hr))
             hr = IXmlWriter_WriteEndElement(writer);
     }
@@ -1799,17 +1791,17 @@ static HRESULT opc_package_write_contenttypes(IOpcPackage *package, struct zip_a
             BSTR name = NULL;
 
             if (SUCCEEDED(hr))
-                hr = IXmlWriter_WriteStartElement(writer, NULL, overrideW, NULL);
+                hr = IXmlWriter_WriteStartElement(writer, NULL, L"Override", NULL);
             if (SUCCEEDED(hr))
                 hr = IOpcPart_GetName(content_type->u.override.part, &uri);
             if (SUCCEEDED(hr))
                 hr = IOpcPartUri_GetRawUri(uri, &name);
             if (SUCCEEDED(hr))
-                hr = IXmlWriter_WriteAttributeString(writer, NULL, partnameW, NULL, name);
+                hr = IXmlWriter_WriteAttributeString(writer, NULL, L"PartName", NULL, name);
             if (SUCCEEDED(hr))
                 hr = IOpcPart_GetContentType(content_type->u.override.part, &type);
             if (SUCCEEDED(hr))
-                hr = IXmlWriter_WriteAttributeString(writer, NULL, contenttypeW, NULL, type);
+                hr = IXmlWriter_WriteAttributeString(writer, NULL, L"ContentType", NULL, type);
 
             if (uri)
                 IOpcPartUri_Release(uri);
@@ -1822,7 +1814,7 @@ static HRESULT opc_package_write_contenttypes(IOpcPackage *package, struct zip_a
             hr = IXmlWriter_WriteEndElement(writer);
 
         list_remove(&content_type->entry);
-        heap_free(content_type);
+        free(content_type);
     }
 
     if (SUCCEEDED(hr))
@@ -1831,7 +1823,7 @@ static HRESULT opc_package_write_contenttypes(IOpcPackage *package, struct zip_a
         hr = IXmlWriter_Flush(writer);
 
     if (SUCCEEDED(hr))
-        hr = compress_add_file(archive, contenttypesW, content, OPC_COMPRESSION_NORMAL);
+        hr = compress_add_file(archive, L"[Content_Types].xml", content, OPC_COMPRESSION_NORMAL);
 
     if (content)
         IStream_Release(content);
@@ -1841,16 +1833,12 @@ static HRESULT opc_package_write_contenttypes(IOpcPackage *package, struct zip_a
 
 static HRESULT opc_package_write_rel(IOpcRelationship *rel, IXmlWriter *writer)
 {
-    static const WCHAR relationshipW[] = {'R','e','l','a','t','i','o','n','s','h','i','p',0};
-    static const WCHAR targetW[] = {'T','a','r','g','e','t',0};
-    static const WCHAR typeW[] = {'T','y','p','e',0};
-    static const WCHAR idW[] = {'I','d',0};
     BSTR target_uri;
     HRESULT hr;
     WCHAR *str;
     IUri *uri;
 
-    if (FAILED(hr = IXmlWriter_WriteStartElement(writer, NULL, relationshipW, NULL)))
+    if (FAILED(hr = IXmlWriter_WriteStartElement(writer, NULL, L"Relationship", NULL)))
         return hr;
 
     if (FAILED(hr = IOpcRelationship_GetTargetUri(rel, &uri)))
@@ -1859,7 +1847,7 @@ static HRESULT opc_package_write_rel(IOpcRelationship *rel, IXmlWriter *writer)
     IUri_GetRawUri(uri, &target_uri);
     IUri_Release(uri);
 
-    hr = IXmlWriter_WriteAttributeString(writer, NULL, targetW, NULL, target_uri);
+    hr = IXmlWriter_WriteAttributeString(writer, NULL, L"Target", NULL, target_uri);
     SysFreeString(target_uri);
     if (FAILED(hr))
         return hr;
@@ -1867,7 +1855,7 @@ static HRESULT opc_package_write_rel(IOpcRelationship *rel, IXmlWriter *writer)
     if (FAILED(hr = IOpcRelationship_GetId(rel, &str)))
         return hr;
 
-    hr = IXmlWriter_WriteAttributeString(writer, NULL, idW, NULL, str);
+    hr = IXmlWriter_WriteAttributeString(writer, NULL, L"Id", NULL, str);
     CoTaskMemFree(str);
     if (FAILED(hr))
         return hr;
@@ -1875,7 +1863,7 @@ static HRESULT opc_package_write_rel(IOpcRelationship *rel, IXmlWriter *writer)
     if (FAILED(hr = IOpcRelationship_GetRelationshipType(rel, &str)))
         return hr;
 
-    hr = IXmlWriter_WriteAttributeString(writer, NULL, typeW, NULL, str);
+    hr = IXmlWriter_WriteAttributeString(writer, NULL, L"Type", NULL, str);
     CoTaskMemFree(str);
     if (FAILED(hr))
         return hr;
@@ -1886,9 +1874,6 @@ static HRESULT opc_package_write_rel(IOpcRelationship *rel, IXmlWriter *writer)
 static HRESULT opc_package_write_rels(struct zip_archive *archive, IOpcRelationshipSet *rels,
         IOpcUri *uri, IXmlWriter *writer)
 {
-    static const WCHAR uriW[] = {'h','t','t','p',':','/','/','s','c','h','e','m','a','s','.','o','p','e','n','x','m','l','f','o','r','m','a','t','s','.','o','r','g','/',
-            'p','a','c','k','a','g','e','/','2','0','0','6','/','r','e','l','a','t','i','o','n','s','h','i','p','s',0};
-    static const WCHAR relationshipsW[] = {'R','e','l','a','t','i','o','n','s','h','i','p','s',0};
     IOpcRelationshipEnumerator *enumerator;
     BSTR rels_part_uri = NULL;
     IOpcPartUri *rels_uri;
@@ -1916,7 +1901,8 @@ static HRESULT opc_package_write_rels(struct zip_archive *archive, IOpcRelations
     if (SUCCEEDED(hr))
         hr = IXmlWriter_WriteStartDocument(writer, XmlStandalone_Yes);
     if (SUCCEEDED(hr))
-        hr = IXmlWriter_WriteStartElement(writer, NULL, relationshipsW, uriW);
+        hr = IXmlWriter_WriteStartElement(writer, NULL, L"Relationships",
+                L"http://schemas.openxmlformats.org/package/2006/relationships");
 
     while (has_next)
     {
@@ -2034,13 +2020,10 @@ HRESULT opc_package_write(IOpcPackage *package, OPC_WRITE_FLAGS flags, IStream *
     IXmlWriter *writer;
     HRESULT hr;
 
-    if (flags != OPC_WRITE_FORCE_ZIP32)
-        FIXME("Unsupported write flags %#x.\n", flags);
-
     if (FAILED(hr = CreateXmlWriter(&IID_IXmlWriter, (void **)&writer, NULL)))
         return hr;
 
-    if (FAILED(hr = compress_create_archive(stream, &archive)))
+    if (FAILED(hr = compress_create_archive(stream, flags != OPC_WRITE_FORCE_ZIP32, &archive)))
     {
         IXmlWriter_Release(writer);
         return hr;
@@ -2058,13 +2041,15 @@ HRESULT opc_package_write(IOpcPackage *package, OPC_WRITE_FLAGS flags, IStream *
     /* Parts. */
     if (SUCCEEDED(hr))
         hr = opc_package_write_parts(archive, package, writer);
+    if (SUCCEEDED(hr))
+        hr = compress_finalize_archive(archive);
 
     if (rels)
         IOpcRelationshipSet_Release(rels);
     if (uri)
         IOpcUri_Release(uri);
 
-    compress_finalize_archive(archive);
+    compress_release_archive(archive);
     IXmlWriter_Release(writer);
 
     return hr;
