@@ -18,120 +18,73 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-#include "wine/port.h"
-
 #include <stdarg.h>
 #include <stdio.h>
 
 #include "sane_i.h"
 #include "wine/debug.h"
-#include "wine/library.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(twain);
 
 struct tagActiveDS activeDS;
 
 DSMENTRYPROC SANE_dsmentry;
-
-#ifdef SONAME_LIBSANE
-#define MAKE_FUNCPTR(f) typeof(f) * p##f;
-MAKE_FUNCPTR(sane_init)
-MAKE_FUNCPTR(sane_exit)
-MAKE_FUNCPTR(sane_get_devices)
-MAKE_FUNCPTR(sane_open)
-MAKE_FUNCPTR(sane_close)
-MAKE_FUNCPTR(sane_get_option_descriptor)
-MAKE_FUNCPTR(sane_control_option)
-MAKE_FUNCPTR(sane_get_parameters)
-MAKE_FUNCPTR(sane_start)
-MAKE_FUNCPTR(sane_read)
-MAKE_FUNCPTR(sane_cancel)
-MAKE_FUNCPTR(sane_set_io_mode)
-MAKE_FUNCPTR(sane_get_select_fd)
-MAKE_FUNCPTR(sane_strstatus)
-#undef MAKE_FUNCPTR
-
 HINSTANCE SANE_instance;
 
-static void *libsane_handle;
-
-static void close_libsane(void *h)
-{
-    if (h)
-        wine_dlclose(h, NULL, 0);
-}
-
-static void *open_libsane(void)
-{
-    void *h;
-
-    h = wine_dlopen(SONAME_LIBSANE, RTLD_GLOBAL | RTLD_NOW, NULL, 0);
-    if (!h)
-    {
-        WARN("dlopen(%s) failed\n", SONAME_LIBSANE);
-        return NULL;
-    }
-
-#define LOAD_FUNCPTR(f) \
-    if((p##f = wine_dlsym(h, #f, NULL, 0)) == NULL) { \
-        close_libsane(h); \
-        ERR("Could not dlsym %s\n", #f); \
-        return NULL; \
-    }
-
-    LOAD_FUNCPTR(sane_init)
-    LOAD_FUNCPTR(sane_exit)
-    LOAD_FUNCPTR(sane_get_devices)
-    LOAD_FUNCPTR(sane_open)
-    LOAD_FUNCPTR(sane_close)
-    LOAD_FUNCPTR(sane_get_option_descriptor)
-    LOAD_FUNCPTR(sane_control_option)
-    LOAD_FUNCPTR(sane_get_parameters)
-    LOAD_FUNCPTR(sane_start)
-    LOAD_FUNCPTR(sane_read)
-    LOAD_FUNCPTR(sane_cancel)
-    LOAD_FUNCPTR(sane_set_io_mode)
-    LOAD_FUNCPTR(sane_get_select_fd)
-    LOAD_FUNCPTR(sane_strstatus)
-#undef LOAD_FUNCPTR
-
-    return h;
-}
-
 BOOL WINAPI DllMain (HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
 {
-    TRACE("%p,%x,%p\n", hinstDLL, fdwReason, lpvReserved);
+    TRACE("%p,%lx,%p\n", hinstDLL, fdwReason, lpvReserved);
 
     switch (fdwReason)
     {
         case DLL_PROCESS_ATTACH: {
-	    SANE_Int version_code;
-
-            libsane_handle = open_libsane();
-            if (! libsane_handle)
-                return FALSE;
-
-	    psane_init (&version_code, NULL);
 	    SANE_instance = hinstDLL;
             DisableThreadLibraryCalls(hinstDLL);
+            if (__wine_init_unix_call()) return FALSE;
+            SANE_CALL( process_attach, NULL );
             break;
 	}
         case DLL_PROCESS_DETACH:
             if (lpvReserved) break;
-            TRACE("calling sane_exit()\n");
-	    psane_exit ();
-            close_libsane(libsane_handle);
+            SANE_CALL( process_detach, NULL );
             break;
     }
 
     return TRUE;
 }
 
-static TW_UINT16 SANE_GetIdentity( pTW_IDENTITY, pTW_IDENTITY);
-static TW_UINT16 SANE_OpenDS( pTW_IDENTITY, pTW_IDENTITY);
+static TW_UINT16 SANE_OpenDS( pTW_IDENTITY pOrigin, pTW_IDENTITY self)
+{
+    if (SANE_dsmentry == NULL)
+    {
+        HMODULE moddsm = GetModuleHandleW(L"twain_32");
 
-#endif /* SONAME_LIBSANE */
+        if (moddsm)
+            SANE_dsmentry = (void*)GetProcAddress(moddsm, "DSM_Entry");
+
+        if (!SANE_dsmentry)
+        {
+            ERR("can't find DSM entry point\n");
+            return TWRC_FAILURE;
+        }
+    }
+
+    if (SANE_CALL( open_ds, self )) return TWRC_FAILURE;
+
+    activeDS.twCC = SANE_SaneSetDefaults();
+    if (activeDS.twCC == TWCC_SUCCESS)
+    {
+        strcpy(activeDS.identity.Manufacturer, self->Manufacturer);
+        strcpy(activeDS.identity.ProductFamily, self->ProductFamily);
+        strcpy(activeDS.identity.ProductName, self->ProductName);
+        activeDS.currentState = 4;
+        activeDS.identity.Id = self->Id;
+        activeDS.appIdentity = *pOrigin;
+        return TWRC_SUCCESS;
+    }
+    SANE_CALL( close_ds, NULL );
+    return TWRC_FAILURE;
+}
 
 static TW_UINT16 SANE_SetEntryPoint (pTW_IDENTITY pOrigin, TW_MEMREF pData);
 
@@ -149,29 +102,20 @@ static TW_UINT16 SANE_SourceControlHandler (
 	    switch (MSG)
 	    {
 		case MSG_CLOSEDS:
-#ifdef SONAME_LIBSANE
-		     psane_close (activeDS.deviceHandle);
-#else
-		     twRC = TWRC_FAILURE;
-                     activeDS.twCC = TWCC_CAPUNSUPPORTED;
-#endif
-		     break;
+                    SANE_CALL( close_ds, NULL );
+                    break;
 		case MSG_OPENDS:
-#ifdef SONAME_LIBSANE
 		     twRC = SANE_OpenDS( pOrigin, (pTW_IDENTITY)pData);
-#else
-		     twRC = TWRC_FAILURE;
-                     activeDS.twCC = TWCC_CAPUNSUPPORTED;
-#endif
 		     break;
 		case MSG_GET:
-#ifdef SONAME_LIBSANE
-		     twRC = SANE_GetIdentity( pOrigin, (pTW_IDENTITY)pData);
-#else
-		     twRC = TWRC_FAILURE;
-                     activeDS.twCC = TWCC_CAPUNSUPPORTED;
-#endif
-		     break;
+                {
+                    if (SANE_CALL( get_identity, pData ))
+                    {
+                        activeDS.twCC = TWCC_CAPUNSUPPORTED;
+                        twRC = TWRC_FAILURE;
+                    }
+                    break;
+                }
 	    }
 	    break;
         case DAT_CAPABILITY:
@@ -390,7 +334,7 @@ DS_Entry ( pTW_IDENTITY pOrigin,
 {
     TW_UINT16 twRC = TWRC_SUCCESS;  /* Return Code */
 
-    TRACE("(DG=%d DAT=%d MSG=%d)\n", DG, DAT, MSG);
+    TRACE("(DG=%ld DAT=%d MSG=%d)\n", DG, DAT, MSG);
 
     switch (DG)
     {
@@ -427,132 +371,3 @@ TW_UINT16 SANE_SetEntryPoint (pTW_IDENTITY pOrigin, TW_MEMREF pData)
 
     return TWRC_SUCCESS;
 }
-
-#ifdef SONAME_LIBSANE
-/* Sane returns device names that are longer than the 32 bytes allowed
-   by TWAIN.  However, it colon separates them, and the last bit is
-   the most interesting.  So we use the last bit, and add a signature
-   to ensure uniqueness */
-static void copy_sane_short_name(const char *in, char *out, size_t outsize)
-{
-    const char *p;
-    int  signature = 0;
-
-    if (strlen(in) <= outsize - 1)
-    {
-        strcpy(out, in);
-        return;
-    }
-
-    for (p = in; *p; p++)
-        signature += *p;
-
-    p = strrchr(in, ':');
-    if (!p)
-        p = in;
-    else
-        p++;
-
-    if (strlen(p) > outsize - 7 - 1)
-        p += strlen(p) - (outsize - 7 - 1);
-
-    strcpy(out, p);
-    sprintf(out + strlen(out), "(%04X)", signature % 0x10000);
-
-}
-
-static const SANE_Device **sane_devlist;
-
-static void
-detect_sane_devices(void) {
-    if (sane_devlist && sane_devlist[0]) return;
-    TRACE("detecting sane...\n");
-    if (psane_get_devices (&sane_devlist, SANE_FALSE) != SANE_STATUS_GOOD)
-	return;
-}
-
-static TW_UINT16
-SANE_GetIdentity( pTW_IDENTITY pOrigin, pTW_IDENTITY self) {
-    static int cursanedev = 0;
-
-    detect_sane_devices();
-    if (!sane_devlist[cursanedev])
-	return TWRC_FAILURE;
-    self->ProtocolMajor = TWON_PROTOCOLMAJOR;
-    self->ProtocolMinor = TWON_PROTOCOLMINOR;
-    self->SupportedGroups = DG_CONTROL | DG_IMAGE | DF_DS2;
-    copy_sane_short_name(sane_devlist[cursanedev]->name, self->ProductName, sizeof(self->ProductName) - 1);
-    lstrcpynA (self->Manufacturer, sane_devlist[cursanedev]->vendor, sizeof(self->Manufacturer) - 1);
-    lstrcpynA (self->ProductFamily, sane_devlist[cursanedev]->model, sizeof(self->ProductFamily) - 1);
-    cursanedev++;
-
-    if (!sane_devlist[cursanedev] 		||
-	!sane_devlist[cursanedev]->model	||
-	!sane_devlist[cursanedev]->vendor	||
-	!sane_devlist[cursanedev]->name
-    )
-	cursanedev = 0; /* wrap to begin */
-    return TWRC_SUCCESS;
-}
-
-static TW_UINT16 SANE_OpenDS( pTW_IDENTITY pOrigin, pTW_IDENTITY self) {
-    SANE_Status status;
-    int i;
-
-    if (SANE_dsmentry == NULL)
-    {
-        static const WCHAR twain32W[] = {'t','w','a','i','n','_','3','2',0};
-        HMODULE moddsm = GetModuleHandleW(twain32W);
-
-        if (moddsm)
-            SANE_dsmentry = (void*)GetProcAddress(moddsm, "DSM_Entry");
-
-        if (!SANE_dsmentry)
-        {
-            ERR("can't find DSM entry point\n");
-            return TWRC_FAILURE;
-        }
-    }
-
-    detect_sane_devices();
-    if (!sane_devlist[0]) {
-	ERR("No scanners? We should not get to OpenDS?\n");
-	return TWRC_FAILURE;
-    }
-
-    for (i=0; sane_devlist[i] && sane_devlist[i]->model; i++) {
-	TW_STR32 name;
-
-	/* To make string as short as above */
-	lstrcpynA(name, sane_devlist[i]->vendor, sizeof(name)-1);
-	if (*self->Manufacturer && strcmp(name, self->Manufacturer))
-	    continue;
-	lstrcpynA(name, sane_devlist[i]->model, sizeof(name)-1);
-	if (*self->ProductFamily && strcmp(name, self->ProductFamily))
-	    continue;
-        copy_sane_short_name(sane_devlist[i]->name, name, sizeof(name) - 1);
-	if (*self->ProductName && strcmp(name, self->ProductName))
-	    continue;
-	break;
-    }
-    if (!sane_devlist[i]) {
-	WARN("Scanner not found.\n");
-	return TWRC_FAILURE;
-    }
-    status = psane_open(sane_devlist[i]->name,&activeDS.deviceHandle);
-    if (status == SANE_STATUS_GOOD) {
-        activeDS.twCC = SANE_SaneSetDefaults();
-        if (activeDS.twCC == TWCC_SUCCESS) {
-	    activeDS.currentState = 4;
-            activeDS.identity.Id = self->Id;
-            activeDS.appIdentity = *pOrigin;
-	    return TWRC_SUCCESS;
-        }
-        else
-            psane_close(activeDS.deviceHandle);
-    }
-    else
-        ERR("sane_open(%s): %s\n", sane_devlist[i]->name, psane_strstatus (status));
-    return TWRC_FAILURE;
-}
-#endif
