@@ -23,6 +23,7 @@
 #include <stdio.h>
 
 #include "sane_i.h"
+#include "winreg.h"
 #include "winuser.h"
 #include "winnls.h"
 #include "wingdi.h"
@@ -36,6 +37,23 @@ WINE_DEFAULT_DEBUG_CHANNEL(twain);
 #define ID_EDIT_BASE 0x1000
 #define ID_STATIC_BASE 0x2000
 
+#define OPTION_VALUE_MAX 255
+#define OPTION_NAME_MAX 64
+
+typedef struct
+{
+    INT   opt_type;
+    DWORD reg_type;
+    CHAR  name[OPTION_NAME_MAX];
+    INT   optno;
+    BYTE  value[OPTION_VALUE_MAX];
+    DWORD size;
+    BOOL  is_enabled;
+} ScannerOption;
+
+static char reg_path[MAX_PATH];
+static int gOptCount;
+
 static INT_PTR CALLBACK DialogProc (HWND , UINT , WPARAM , LPARAM );
 static INT CALLBACK PropSheetProc(HWND, UINT,LPARAM);
 
@@ -485,6 +503,8 @@ BOOL DoScannerUI(void)
 
     hdc = CreateCompatibleDC(0);
 
+    gOptCount = optcount;
+
     while (index < optcount)
     {
         struct option_descriptor opt;
@@ -508,13 +528,19 @@ BOOL DoScannerUI(void)
             psp[page_count].lParam = (LPARAM)&activeDS;
             page_count ++;
         }
-       
+
         index ++;
     }
- 
+
     len = lstrlenA(activeDS.identity.Manufacturer)
          + lstrlenA(activeDS.identity.ProductName) + 2;
     szCaption = malloc(len *sizeof(WCHAR));
+
+    snprintf(reg_path, MAX_PATH, "Software\\ScannersSettings\\%s\\%s_%s",
+             activeDS.identity.Manufacturer,
+             activeDS.identity.ProductFamily,
+             activeDS.identity.ProductName);
+
     MultiByteToWideChar(CP_ACP,0,activeDS.identity.Manufacturer,-1,
             szCaption,len);
     szCaption[lstrlenA(activeDS.identity.Manufacturer)] = ' ';
@@ -548,6 +574,142 @@ BOOL DoScannerUI(void)
         return FALSE;
 }
 
+static BOOL save_to_reg( DWORD reg_type, CHAR* name, const BYTE* value, DWORD size )
+{
+    HKEY h_key;
+    LSTATUS res;
+
+    if (RegCreateKeyExA( HKEY_CURRENT_USER, reg_path, 0, NULL, REG_OPTION_NON_VOLATILE,
+                         KEY_ALL_ACCESS, NULL, &h_key, NULL ))
+        return FALSE;
+
+    res = RegSetValueExA( h_key, name, 0, reg_type, value, size );
+    RegCloseKey( h_key );
+    return res == ERROR_SUCCESS;
+}
+
+static BOOL load_from_reg( int opt_type, CHAR* name, void* value )
+{
+    HKEY h_key;
+    DWORD flag, size;
+    LSTATUS res;
+
+    if (RegOpenKeyExA( HKEY_CURRENT_USER, reg_path, 0, KEY_ALL_ACCESS, &h_key )) return FALSE;
+
+    switch( opt_type )
+    {
+        case TYPE_INT:
+        case TYPE_FIXED:
+        case TYPE_BOOL:
+            flag = RRF_RT_REG_DWORD;
+            size = sizeof(DWORD);
+            break;
+        case TYPE_STRING:
+            flag = RRF_RT_REG_SZ;
+            size = OPTION_VALUE_MAX;
+            break;
+        default:
+            RegCloseKey( h_key );
+            ERR( "Unknown type: %d\n", opt_type );
+            return FALSE;
+    }
+
+    res = RegGetValueA( h_key, NULL, name, flag, NULL, value, &size );
+    RegCloseKey( h_key );
+    return res == ERROR_SUCCESS;
+}
+
+static BOOL get_option(struct option_descriptor* opt, ScannerOption* option)
+{
+    lstrcpynA(option->name, opt->name, OPTION_NAME_MAX);
+    option->is_enabled = opt->is_active;
+    option->optno = opt->optno;
+
+    if (opt->type ==TYPE_STRING && opt->constraint_type != CONSTRAINT_NONE)
+    {
+        CHAR buffer[255];
+        option->reg_type = REG_SZ;
+        option->opt_type = opt->type;
+        sane_option_get_value(opt->optno, buffer);
+        lstrcpynA((CHAR*)option->value, buffer, OPTION_VALUE_MAX);
+        option->size = (DWORD)(strlen(buffer) + 1);
+    }
+    else if (opt->type == TYPE_BOOL)
+    {
+        BOOL b;
+        option->opt_type = opt->type;
+        option->reg_type = REG_DWORD;
+        sane_option_get_value(opt->optno, &b);
+        memcpy(option->value, &b, sizeof(BOOL));
+        option->size = sizeof(b);
+    }
+    else if (opt->type == TYPE_INT && opt->constraint_type == CONSTRAINT_WORD_LIST)
+    {
+        int val;
+        option->opt_type = opt->type;
+        option->reg_type = REG_DWORD;
+        sane_option_get_value(opt->optno, &val);
+        memcpy(option->value, &val, sizeof(INT));
+        option->size = sizeof(val);
+    }
+    else if (opt->constraint_type == CONSTRAINT_RANGE)
+    {
+        if (opt->type == TYPE_INT)
+        {
+            int si;
+            option->opt_type = opt->type;
+            option->reg_type = REG_DWORD;
+            sane_option_get_value(opt->optno, &si);
+            if (opt->constraint.range.quant)
+            {
+                si = si / opt->constraint.range.quant;
+            }
+            memcpy(option->value, &si, sizeof(INT));
+            option->size = sizeof(si);
+        }
+        else if (opt->type == TYPE_FIXED)
+        {
+            int pos, *sf;
+            option->opt_type = opt->type;
+            option->reg_type = REG_DWORD;
+            sf = calloc( opt->size, sizeof(int) );
+            sane_option_get_value(opt->optno, sf );
+            if (opt->constraint.range.quant)
+                pos = *sf / opt->constraint.range.quant;
+            else
+                pos = MulDiv( *sf, 100, 65536 );
+            memcpy(option->value, &pos, sizeof(INT));
+            option->size = sizeof(pos);
+            free(sf);
+        }
+        else
+        {
+            FIXME("Unhandled option type %d with constraint\n", opt->type);
+            return FALSE;
+        }
+    }
+    else
+    {
+        FIXME("Unhandled option type %d\n", opt->type);
+        return FALSE;
+    }
+
+    return TRUE;
+}
+
+static BOOL save_option(int optno)
+{
+    ScannerOption option;
+    struct option_descriptor opt;
+
+    opt.optno = optno;
+    SANE_CALL(option_get_descriptor, &opt);
+
+    if (!get_option(&opt, &option)) return FALSE;
+
+    return save_to_reg(option.reg_type, option.name, option.value, option.size);
+}
+
 static void UpdateRelevantEdit(HWND hwnd, const struct option_descriptor *opt, int position)
 {
     WCHAR buffer[244];
@@ -607,6 +769,7 @@ static BOOL UpdateSaneScrollOption(const struct option_descriptor *opt, DWORD po
             si = position;
 
         sane_option_set_value( opt->optno, &si, &result );
+        save_option(opt->optno);
         break;
     }
     case TYPE_FIXED:
@@ -616,6 +779,7 @@ static BOOL UpdateSaneScrollOption(const struct option_descriptor *opt, DWORD po
             si = MulDiv( position, 65536, 100 );
 
         sane_option_set_value( opt->optno, &si, &result );
+        save_option(opt->optno);
         break;
     default:
         break;
@@ -635,8 +799,10 @@ static INT_PTR InitializeDialog(HWND hwnd)
     if (rc != TWCC_SUCCESS)
     {
         ERR("Unable to read number of options\n");
-        return FALSE;
+        optcount = gOptCount;
     }
+    else
+        gOptCount = optcount;
 
     for ( i = 1; i < optcount; i++)
     {
@@ -662,16 +828,32 @@ static INT_PTR InitializeDialog(HWND hwnd)
 
             for (p = opt.constraint.strings; *p; p += lstrlenW(p) + 1)
                 SendMessageW( control,CB_ADDSTRING,0, (LPARAM)p );
+
+            if (load_from_reg(opt.type, opt.name, buffer))
+            {
+                for (p = opt.constraint.strings; *p; p += lstrlenW(p) + 1)
+                {
+                    CHAR param[256];
+                    WideCharToMultiByte(CP_UTF8, 0, p, -1, param, sizeof(param), NULL, NULL);
+                    if (!strcmp(param, buffer))
+                    {
+                        sane_option_set_value(opt.optno, buffer, NULL);
+                        break;
+                    }
+                }
+                if (!*p) ERR("%s=%s is incorrect. The default value is set!", opt.name, buffer);
+            }
+
             sane_option_get_value( i, buffer );
             SendMessageA(control,CB_SELECTSTRING,0,(LPARAM)buffer);
         }
         else if (opt.type == TYPE_BOOL)
         {
             BOOL b;
-            sane_option_get_value( i, &b );
-            if (b)
-                SendMessageA(control,BM_SETCHECK,BST_CHECKED,0);
 
+            if (load_from_reg(opt.type, opt.name, &b)) sane_option_set_value( i, &b, NULL );
+            sane_option_get_value( i, &b );
+            SendMessageA(control,BM_SETCHECK, b ? BST_CHECKED : BST_UNCHECKED,0);
         }
         else if (opt.type == TYPE_INT && opt.constraint_type == CONSTRAINT_WORD_LIST)
         {
@@ -683,6 +865,20 @@ static INT_PTR InitializeDialog(HWND hwnd)
                 sprintf(buffer, "%d", opt.constraint.word_list[j]);
                 SendMessageA(control, CB_ADDSTRING, 0, (LPARAM)buffer);
             }
+
+            if (load_from_reg(opt.type, opt.name, &val))
+            {
+                for (j=1; j<=count; j++)
+                {
+                    if (opt.constraint.word_list[j] == val)
+                    {
+                        sane_option_set_value( i, &val, NULL );
+                        break;
+                    }
+                }
+                if (j > count) ERR("%s=%d is incorrect. The default value is set!\n", opt.name, val);
+            }
+
             sane_option_get_value( i, &val );
             sprintf(buffer, "%d", val);
             SendMessageA(control,CB_SELECTSTRING,0,(LPARAM)buffer);
@@ -702,6 +898,12 @@ static INT_PTR InitializeDialog(HWND hwnd)
 
                 SendMessageA(control,SBM_SETRANGE,min,max);
 
+                if (load_from_reg(opt.type, opt.name, &si))
+                {
+                    if (si >= min && si <= max) sane_option_set_value( i, &si, NULL);
+                    else ERR("%s=%d is out of range [%d..%d]. The default value is used!\n", opt.name, si, min, max);
+                }
+
                 sane_option_get_value( i, &si );
                 if (opt.constraint.range.quant)
                     si = si / opt.constraint.range.quant;
@@ -711,7 +913,7 @@ static INT_PTR InitializeDialog(HWND hwnd)
             }
             else if (opt.type == TYPE_FIXED)
             {
-                int pos, min, max, *sf;
+                int pos, min, max, *sf, val;
 
                 if (opt.constraint.range.quant)
                 {
@@ -726,6 +928,19 @@ static INT_PTR InitializeDialog(HWND hwnd)
 
                 SendMessageA(control,SBM_SETRANGE,min,max);
 
+                if (load_from_reg(opt.type, opt.name, &val))
+                {
+                    if (val >= min && val <= max)
+                    {
+                        int valSet;
+                        if (opt.constraint.range.quant)
+                            valSet = val * opt.constraint.range.quant;
+                        else
+                            valSet = MulDiv(val, 65536, 100);
+                        sane_option_set_value(i, &valSet, NULL);
+                    }
+                    else ERR("%s = %d is out of range [%d..%d]. The default value is used!\n", opt.name, val, min, max);
+                }
 
                 sf = calloc( opt.size, sizeof(int) );
                 sane_option_get_value( i, sf );
@@ -817,7 +1032,12 @@ static void ButtonClicked(HWND hwnd, INT id, HWND control)
     {
         BOOL r = SendMessageW(control,BM_GETCHECK,0,0)==BST_CHECKED;
         sane_option_set_value( opt.optno, &r, &changed );
-        if (changed) InitializeDialog(hwnd);
+
+        if (changed)
+        {
+            save_option(opt.optno);
+            InitializeDialog(hwnd);
+        }
     }
 }
 
@@ -851,11 +1071,15 @@ static void ComboChanged(HWND hwnd, INT id, HWND control)
         int val = atoi( value );
         sane_option_set_value( opt.optno, &val, &changed );
     }
-    if (changed) InitializeDialog(hwnd);
+
+    if (changed)
+    {
+        save_option(opt.optno);
+        InitializeDialog(hwnd);
+    }
     free( value );
 }
 
-
 static INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam)
 {
     switch (msg)
