@@ -24,9 +24,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-#include "wine/port.h"
-
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
@@ -35,7 +32,6 @@
 #include "windef.h"
 #include "winbase.h"
 #include "wingdi.h"
-#include "wine/unicode.h"
 #include "winnls.h"
 #include "controls.h"
 #include "usp10.h"
@@ -95,7 +91,7 @@ WINE_DEFAULT_DEBUG_CHANNEL(text);
 #define FORWARD_SLASH '/'
 #define BACK_SLASH '\\'
 
-static const WCHAR ELLIPSISW[] = {'.','.','.', 0};
+static const WCHAR ELLIPSISW[] = L"...";
 
 typedef struct tag_ellipsis_data
 {
@@ -142,7 +138,7 @@ static void TEXT_Ellipsify (HDC hdc, WCHAR *str, unsigned int max_len,
     unsigned int len_ellipsis;
     unsigned int lo, mid, hi;
 
-    len_ellipsis = strlenW (ELLIPSISW);
+    len_ellipsis = lstrlenW (ELLIPSISW);
     if (len_ellipsis > max_len) len_ellipsis = max_len;
     if (*len_str > max_len - len_ellipsis)
         *len_str = max_len - len_ellipsis;
@@ -239,7 +235,7 @@ static void TEXT_PathEllipsify (HDC hdc, WCHAR *str, unsigned int max_len,
     int len_under;
     WCHAR *lastBkSlash, *lastFwdSlash, *lastSlash;
 
-    len_ellipsis = strlenW (ELLIPSISW);
+    len_ellipsis = lstrlenW (ELLIPSISW);
     if (!max_len) return;
     if (len_ellipsis >= max_len) len_ellipsis = max_len - 1;
     if (*len_str + len_ellipsis >= max_len)
@@ -249,8 +245,8 @@ static void TEXT_PathEllipsify (HDC hdc, WCHAR *str, unsigned int max_len,
          */
     str[*len_str] = '\0'; /* to simplify things */
 
-    lastBkSlash  = strrchrW (str, BACK_SLASH);
-    lastFwdSlash = strrchrW (str, FORWARD_SLASH);
+    lastBkSlash  = wcsrchr (str, BACK_SLASH);
+    lastFwdSlash = wcsrchr (str, FORWARD_SLASH);
     lastSlash = lastBkSlash > lastFwdSlash ? lastBkSlash : lastFwdSlash;
     if (!lastSlash) lastSlash = str;
     len_trailing = *len_str - (lastSlash - str);
@@ -388,9 +384,9 @@ static void TEXT_WordBreak (HDC hdc, WCHAR *str, unsigned int max_str,
     }
     else
     {
-        while (i > 0 && !sla[(--i)+1].fSoftBreak) p--;
+        while (i > 0 && !sla[(--i)+1].fWhiteSpace) p--;
         p--;
-        word_fits = (i != 0 || sla[i+1].fSoftBreak );
+        word_fits = (i != 0 || sla[i+1].fWhiteSpace );
     }
 
     /* If there was one. */
@@ -677,7 +673,7 @@ static const WCHAR *TEXT_NextLineW( HDC hdc, const WCHAR *str, int *count,
             {
                 /* Throw away katakana access keys */
                 (*count)--, i++; /* skip the prefix */
-                (*count)--, i++; /* skip the letter */
+                (*count)--; i++; /* skip the letter */
             }
             else
             {
@@ -875,11 +871,10 @@ INT WINAPI DrawTextExW( HDC hdc, LPWSTR str, INT i_count,
     WCHAR *retstr;
     size_t size_retstr;
     WCHAR line[MAX_BUFFER];
-    int len, lh, count=i_count;
+    int len, lh, old_count, count = i_count;
     TEXTMETRICW tm;
     int lmargin = 0, rmargin = 0;
-    int x = rect->left, y = rect->top;
-    int width = rect->right - rect->left;
+    int x, y, width;
     int max_width = 0;
     int last_line;
     int tabwidth /* to keep gcc happy */ = 0;
@@ -901,7 +896,13 @@ INT WINAPI DrawTextExW( HDC hdc, LPWSTR str, INT i_count,
     if (flags & DT_SINGLELINE)
         flags &= ~DT_WORDBREAK;
 
-    GetTextMetricsW(hdc, &tm);
+    if (!GetTextMetricsW(hdc, &tm))
+        return 0;
+
+    x = rect->left;
+    y = rect->top;
+    width = rect->right - rect->left;
+
     if (flags & DT_EXTERNALLEADING)
 	lh = tm.tmHeight + tm.tmExternalLeading;
     else
@@ -915,7 +916,7 @@ INT WINAPI DrawTextExW( HDC hdc, LPWSTR str, INT i_count,
 
     if (count == -1)
     {
-        count = strlenW(str);
+        count = lstrlenW(str);
         if (count == 0)
         {
             if( flags & DT_CALCRECT)
@@ -977,7 +978,11 @@ INT WINAPI DrawTextExW( HDC hdc, LPWSTR str, INT i_count,
             last_line = !(flags & DT_NOCLIP) && y - ((flags & DT_EDITCONTROL) ? 2*lh-1 : lh) < rect->bottom;
 	else
             last_line = !(flags & DT_NOCLIP) && y + ((flags & DT_EDITCONTROL) ? 2*lh-1 : lh) > rect->bottom;
+
+        old_count = count;
 	strPtr = TEXT_NextLineW(hdc, strPtr, &count, line, &len, width, flags, &size, last_line, retstr, tabwidth, &prefix_offset, &ellip);
+        if (dtp)
+            dtp->uiLengthDrawn += old_count - count;
 
         if (flags & DT_CENTER)
             x = (rect->left + lmargin + rect->right - rmargin - size.cx) / 2;
@@ -1050,8 +1055,6 @@ INT WINAPI DrawTextExW( HDC hdc, LPWSTR str, INT i_count,
 	    y -= lh;
         else
 	    y += lh;
-        if (dtp)
-            dtp->uiLengthDrawn += len;
     }
     while (strPtr && !last_line);
 
@@ -1090,18 +1093,24 @@ INT WINAPI DrawTextExA( HDC hdc, LPSTR str, INT count,
    DWORD wmax;
    DWORD amax;
    UINT cp;
+   TEXTMETRICA tm;
+
+   if (!GetTextMetricsA(hdc, &tm))
+   {
+       SetLastError(ERROR_INVALID_HANDLE);
+       return 0;
+   }
 
    if (!count) return 0;
    if (!str && count > 0) return 0;
    if( !str || ((count == -1) && !(count = strlen(str))))
    {
         int lh;
-        TEXTMETRICA tm;
 
         if (dtp && dtp->cbSize != sizeof(DRAWTEXTPARAMS))
             return 0;
 
-        GetTextMetricsA(hdc, &tm);
+
         if (flags & DT_EXTERNALLEADING)
             lh = tm.tmHeight + tm.tmExternalLeading;
         else
@@ -1189,7 +1198,7 @@ INT WINAPI DrawTextA( HDC hdc, LPCSTR str, INT count, LPRECT rect, UINT flags )
  *           GrayString functions
  */
 
-/* callback for ASCII gray string proc */
+/* callback for ANSI gray string proc */
 static BOOL CALLBACK gray_string_callbackA( HDC hdc, LPARAM param, INT len )
 {
     return TextOutA( hdc, 0, 0, (LPCSTR)param, len );
@@ -1286,7 +1295,7 @@ BOOL WINAPI GrayStringW( HDC hdc, HBRUSH hbr, GRAYSTRINGPROC gsprc,
                          LPARAM lParam, INT cch, INT x, INT y,
                          INT cx, INT cy )
 {
-    if (!cch) cch = strlenW( (LPCWSTR)lParam );
+    if (!cch) cch = lstrlenW( (LPCWSTR)lParam );
     if ((cx == 0 || cy == 0) && cch != -1)
     {
         SIZE s;
@@ -1385,7 +1394,9 @@ static LONG TEXT_TabbedTextOut( HDC hdc, INT x, INT y, LPCWSTR lpstr,
             }
         } else
             x += extent.cx;
-        
+
+        if (!extent.cy) extent.cy = tm.tmHeight;
+
         if (fDisplayText)
         {
             r.top    = y;
@@ -1398,9 +1409,6 @@ static LONG TEXT_TabbedTextOut( HDC hdc, INT x, INT y, LPCWSTR lpstr,
         lpstr += j;
     }
 
-    if(!extent.cy)
-        extent.cy = tm.tmHeight;
-
     return MAKELONG(x - start, extent.cy);
 }
 
