@@ -16,10 +16,6 @@
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
- *
- * NOTE:
- * This code is duplicated in shlwapi. If you change something here make sure
- * to change it in shlwapi too.
  */
 
 #include <stdarg.h>
@@ -30,6 +26,7 @@
 #include "winbase.h"
 #include "wingdi.h"
 #include "winuser.h"
+#include "winnls.h"
 
 #include "wine/debug.h"
 
@@ -74,9 +71,6 @@ typedef union {
     LONGLONG int_view;
 } WPRINTF_DATA;
 
-static const CHAR null_stringA[] = "(null)";
-static const WCHAR null_stringW[] = { '(', 'n', 'u', 'l', 'l', ')', 0 };
-
 /***********************************************************************
  *           WPRINTF_ParseFormatA
  *
@@ -245,7 +239,7 @@ static INT WPRINTF_ParseFormatW( LPCWSTR format, WPRINTF_FORMAT *res )
  *           WPRINTF_GetLen
  */
 static UINT WPRINTF_GetLen( WPRINTF_FORMAT *format, WPRINTF_DATA *arg,
-                              LPSTR number, UINT maxlen )
+                            LPSTR number, UINT maxlen, BOOL dst_is_wide )
 {
     UINT len;
 
@@ -254,18 +248,45 @@ static UINT WPRINTF_GetLen( WPRINTF_FORMAT *format, WPRINTF_DATA *arg,
     switch(format->type)
     {
     case WPR_CHAR:
-    case WPR_WCHAR:
         return (format->precision = 1);
+    case WPR_WCHAR:
+        if (dst_is_wide) len = 1;
+        else len = WideCharToMultiByte( CP_ACP, 0, &arg->wchar_view, 1, NULL, 0, NULL, NULL );
+        return (format->precision = len);
     case WPR_STRING:
-        if (!arg->lpcstr_view) arg->lpcstr_view = null_stringA;
-        for (len = 0; !format->precision || (len < format->precision); len++)
-            if (!*(arg->lpcstr_view + len)) break;
+        if (!arg->lpcstr_view) arg->lpcstr_view = "(null)";
+        if (dst_is_wide)
+        {
+            LPCSTR p = arg->lpcstr_view;
+            for (len = 0; (!format->precision || len < format->precision) && *p; p++)
+            {
+                /* This isn't applicable for UTF-8 and UTF-7 */
+                if (IsDBCSLeadByte( *p )) p++;
+                len++;
+                if (!*p) break;
+            }
+        }
+        else
+        {
+            for (len = 0; !format->precision || (len < format->precision); len++)
+                if (!*(arg->lpcstr_view + len)) break;
+        }
         if (len > maxlen) len = maxlen;
         return (format->precision = len);
     case WPR_WSTRING:
-        if (!arg->lpcwstr_view) arg->lpcwstr_view = null_stringW;
-        for (len = 0; !format->precision || (len < format->precision); len++)
-            if (!*(arg->lpcwstr_view + len)) break;
+        if (!arg->lpcwstr_view) arg->lpcwstr_view = L"(null)";
+        if (dst_is_wide)
+        {
+            for (len = 0; !format->precision || (len < format->precision); len++)
+                if (!*(arg->lpcwstr_view + len)) break;
+        }
+        else
+        {
+            LPCWSTR p = arg->lpcwstr_view;
+            for (len = 0; (!format->precision || len < format->precision) && *p; p++)
+                len += WideCharToMultiByte( CP_ACP, 0, p, 1, NULL, 0, NULL, NULL );
+            if (format->precision && len > format->precision) len = format->precision;
+        }
         if (len > maxlen) len = maxlen;
         return (format->precision = len);
     case WPR_SIGNED:
@@ -311,7 +332,7 @@ static UINT WPRINTF_GetLen( WPRINTF_FORMAT *format, WPRINTF_DATA *arg,
 /***********************************************************************
  *           wvsnprintfA   (internal)
  */
-static INT wvsnprintfA( LPSTR buffer, UINT maxlen, LPCSTR spec, __ms_va_list args )
+static INT wvsnprintfA( LPSTR buffer, UINT maxlen, LPCSTR spec, va_list args )
 {
     WPRINTF_FORMAT format;
     LPSTR p = buffer;
@@ -354,7 +375,7 @@ static INT wvsnprintfA( LPSTR buffer, UINT maxlen, LPCSTR spec, __ms_va_list arg
             break;
         }
 
-        len = WPRINTF_GetLen( &format, &argData, number, maxlen - 1 );
+        len = WPRINTF_GetLen( &format, &argData, number, maxlen - 1, FALSE );
         sign = 0;
         if (!(format.flags & WPRINTF_LEFTALIGN))
             for (i = format.precision; i < format.width; i++, maxlen--)
@@ -362,7 +383,14 @@ static INT wvsnprintfA( LPSTR buffer, UINT maxlen, LPCSTR spec, __ms_va_list arg
         switch(format.type)
         {
         case WPR_WCHAR:
-            *p++ = argData.wchar_view;
+            {
+                CHAR mb[5];
+                if (WideCharToMultiByte( CP_ACP, 0, &argData.wchar_view, 1, mb, sizeof(mb), NULL, NULL ))
+                {
+                    memcpy( p, mb, len );
+                    p += len;
+                }
+            }
             break;
         case WPR_CHAR:
             *p++ = argData.char_view;
@@ -374,7 +402,15 @@ static INT wvsnprintfA( LPSTR buffer, UINT maxlen, LPCSTR spec, __ms_va_list arg
         case WPR_WSTRING:
             {
                 LPCWSTR ptr = argData.lpcwstr_view;
-                for (i = 0; i < len; i++) *p++ = (CHAR)*ptr++;
+                for (i = 0; i < len; ptr++)
+                {
+                    CHAR mb[5]; /* 5 is MB_LEN_MAX */
+                    int ret = WideCharToMultiByte( CP_ACP, 0, ptr, 1, mb, sizeof(mb), NULL, NULL );
+                    if (ret > len - i) ret = len - i;
+                    i += ret;
+                    memcpy( p, mb, ret );
+                    p += ret;
+                }
             }
             break;
         case WPR_HEXA:
@@ -416,7 +452,7 @@ static INT wvsnprintfA( LPSTR buffer, UINT maxlen, LPCSTR spec, __ms_va_list arg
 /***********************************************************************
  *           wvsnprintfW   (internal)
  */
-static INT wvsnprintfW( LPWSTR buffer, UINT maxlen, LPCWSTR spec, __ms_va_list args )
+static INT wvsnprintfW( LPWSTR buffer, UINT maxlen, LPCWSTR spec, va_list args )
 {
     WPRINTF_FORMAT format;
     LPWSTR p = buffer;
@@ -459,7 +495,7 @@ static INT wvsnprintfW( LPWSTR buffer, UINT maxlen, LPCWSTR spec, __ms_va_list a
             break;
         }
 
-        len = WPRINTF_GetLen( &format, &argData, number, maxlen - 1 );
+        len = WPRINTF_GetLen( &format, &argData, number, maxlen - 1, TRUE );
         sign = 0;
         if (!(format.flags & WPRINTF_LEFTALIGN))
             for (i = format.precision; i < format.width; i++, maxlen--)
@@ -470,12 +506,26 @@ static INT wvsnprintfW( LPWSTR buffer, UINT maxlen, LPCWSTR spec, __ms_va_list a
             *p++ = argData.wchar_view;
             break;
         case WPR_CHAR:
-            *p++ = argData.char_view;
-            break;
+            {
+                WCHAR wc;
+                if (!IsDBCSLeadByte( (BYTE)argData.char_view )
+                    && MultiByteToWideChar( CP_ACP, 0, &argData.char_view, 1, &wc, 1 ) > 0)
+                    *p++ = wc;
+                else
+                    *p++ = 0;
+                break;
+            }
         case WPR_STRING:
             {
                 LPCSTR ptr = argData.lpcstr_view;
-                for (i = 0; i < len; i++) *p++ = (BYTE)*ptr++;
+                for (i = 0; i < len; i++)
+                {
+                    WCHAR buf[2]; /* for LeadByte + NUL case, we need 2 WCHARs. */
+                    int ret, mb_len = IsDBCSLeadByte( *ptr ) ? 2 : 1;
+                    ret = MultiByteToWideChar( CP_ACP, 0, ptr, mb_len, buf, ARRAY_SIZE( buf ));
+                    *p++ = buf[ret - 1];
+                    ptr += mb_len;
+                }
             }
             break;
         case WPR_WSTRING:
@@ -520,7 +570,7 @@ static INT wvsnprintfW( LPWSTR buffer, UINT maxlen, LPCWSTR spec, __ms_va_list a
 /***********************************************************************
  *           wvsprintfA   (USER32.@)
  */
-INT WINAPI wvsprintfA( LPSTR buffer, LPCSTR spec, __ms_va_list args )
+INT WINAPI wvsprintfA( LPSTR buffer, LPCSTR spec, va_list args )
 {
     INT res = wvsnprintfA( buffer, 1024, spec, args );
     return ( res == -1 ) ? 1024 : res;
@@ -530,7 +580,7 @@ INT WINAPI wvsprintfA( LPSTR buffer, LPCSTR spec, __ms_va_list args )
 /***********************************************************************
  *           wvsprintfW   (USER32.@)
  */
-INT WINAPI wvsprintfW( LPWSTR buffer, LPCWSTR spec, __ms_va_list args )
+INT WINAPI wvsprintfW( LPWSTR buffer, LPCWSTR spec, va_list args )
 {
     INT res = wvsnprintfW( buffer, 1024, spec, args );
     return ( res == -1 ) ? 1024 : res;
@@ -542,12 +592,12 @@ INT WINAPI wvsprintfW( LPWSTR buffer, LPCWSTR spec, __ms_va_list args )
  */
 INT WINAPIV wsprintfA( LPSTR buffer, LPCSTR spec, ... )
 {
-    __ms_va_list valist;
+    va_list valist;
     INT res;
 
-    __ms_va_start( valist, spec );
+    va_start( valist, spec );
     res = wvsnprintfA( buffer, 1024, spec, valist );
-    __ms_va_end( valist );
+    va_end( valist );
     return ( res == -1 ) ? 1024 : res;
 }
 
@@ -557,11 +607,11 @@ INT WINAPIV wsprintfA( LPSTR buffer, LPCSTR spec, ... )
  */
 INT WINAPIV wsprintfW( LPWSTR buffer, LPCWSTR spec, ... )
 {
-    __ms_va_list valist;
+    va_list valist;
     INT res;
 
-    __ms_va_start( valist, spec );
+    va_start( valist, spec );
     res = wvsnprintfW( buffer, 1024, spec, valist );
-    __ms_va_end( valist );
+    va_end( valist );
     return ( res == -1 ) ? 1024 : res;
 }
