@@ -17,6 +17,7 @@
  */
 
 #include <stdarg.h>
+#include <stdlib.h>
 
 #include "windef.h"
 #include "winbase.h"
@@ -25,7 +26,6 @@
 #include "webservices.h"
 
 #include "wine/debug.h"
-#include "wine/heap.h"
 #include "wine/list.h"
 #include "webservices_private.h"
 
@@ -42,6 +42,10 @@ static const struct prop_desc msg_props[] =
     { sizeof(WS_XML_READER *), TRUE },          /* WS_MESSAGE_PROPERTY_BODY_READER */
     { sizeof(WS_XML_WRITER *), TRUE },          /* WS_MESSAGE_PROPERTY_BODY_WRITER */
     { sizeof(BOOL), TRUE },                     /* WS_MESSAGE_PROPERTY_IS_ADDRESSED */
+    { sizeof(WS_HEAP_PROPERTIES), TRUE },       /* WS_MESSAGE_PROPERTY_HEAP_PROPERTIES */
+    { sizeof(WS_XML_READER_PROPERTIES), TRUE }, /* WS_MESSAGE_PROPERTY_XML_READER_PROPERTIES */
+    { sizeof(WS_XML_WRITER_PROPERTIES), TRUE }, /* WS_MESSAGE_PROPERTY_XML_WRITER_PROPERTIES */
+    { sizeof(BOOL), FALSE },                    /* WS_MESSAGE_PROPERTY_IS_FAULT */
 };
 
 struct header
@@ -94,17 +98,17 @@ static struct msg *alloc_msg(void)
     struct msg *ret;
     ULONG size = sizeof(*ret) + prop_size( msg_props, count );
 
-    if (!(ret = heap_alloc_zero( size ))) return NULL;
-    if (!(ret->header = heap_alloc( HEADER_ARRAY_SIZE * sizeof(struct header *) )))
+    if (!(ret = calloc( 1, size ))) return NULL;
+    if (!(ret->header = malloc( HEADER_ARRAY_SIZE * sizeof(struct header *) )))
     {
-        heap_free( ret );
+        free( ret );
         return NULL;
     }
     ret->magic       = MSG_MAGIC;
     ret->state       = WS_MESSAGE_STATE_EMPTY;
     ret->header_size = HEADER_ARRAY_SIZE;
 
-    InitializeCriticalSection( &ret->cs );
+    InitializeCriticalSectionEx( &ret->cs, 0, RTL_CRITICAL_SECTION_FLAG_FORCE_DEBUG_INFO );
     ret->cs.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": msg.cs");
 
     prop_init( msg_props, count, ret->prop, &ret[1] );
@@ -114,14 +118,15 @@ static struct msg *alloc_msg(void)
 
 static void free_header( struct header *header )
 {
-    heap_free( header->name.bytes );
-    heap_free( header->ns.bytes );
+    free( header->name.bytes );
+    free( header->ns.bytes );
     if (header->mapped) free_xml_string( header->u.text );
-    heap_free( header );
+    free( header );
 }
 
 static void reset_msg( struct msg *msg )
 {
+    BOOL isfault = FALSE;
     ULONG i;
 
     msg->state         = WS_MESSAGE_STATE_EMPTY;
@@ -129,7 +134,7 @@ static void reset_msg( struct msg *msg )
     UuidCreate( &msg->id );
     memset( &msg->id_req, 0, sizeof(msg->id_req) );
     msg->is_addressed  = FALSE;
-    heap_free( msg->addr.chars );
+    free( msg->addr.chars );
     msg->addr.chars    = NULL;
     msg->addr.length   = 0;
 
@@ -150,6 +155,8 @@ static void reset_msg( struct msg *msg )
 
     memset( &msg->ctx_send, 0, sizeof(msg->ctx_send) );
     memset( &msg->ctx_receive, 0, sizeof(msg->ctx_receive) );
+
+    prop_set( msg->prop, msg->prop_count, WS_MESSAGE_PROPERTY_IS_FAULT, &isfault, sizeof(isfault) );
 }
 
 static void free_msg( struct msg *msg )
@@ -159,11 +166,11 @@ static void free_msg( struct msg *msg )
     WsFreeWriter( msg->writer );
     WsFreeReader( msg->reader );
     WsFreeHeap( msg->heap );
-    heap_free( msg->header );
+    free( msg->header );
 
     msg->cs.DebugInfo->Spare[0] = 0;
     DeleteCriticalSection( &msg->cs );
-    heap_free( msg );
+    free( msg );
 }
 
 #define HEAP_MAX_SIZE (1 << 16)
@@ -216,7 +223,7 @@ HRESULT WINAPI WsCreateMessage( WS_ENVELOPE_VERSION env_version, WS_ADDRESSING_V
 {
     HRESULT hr;
 
-    TRACE( "%u %u %p %u %p %p\n", env_version, addr_version, properties, count, handle, error );
+    TRACE( "%u %u %p %lu %p %p\n", env_version, addr_version, properties, count, handle, error );
     if (error) FIXME( "ignoring error parameter\n" );
 
     if (!handle || !env_version || !addr_version ||
@@ -240,7 +247,7 @@ HRESULT WINAPI WsCreateMessageForChannel( WS_CHANNEL *channel_handle, const WS_M
     WS_ADDRESSING_VERSION version_addr;
     HRESULT hr;
 
-    TRACE( "%p %p %u %p %p\n", channel_handle, properties, count, handle, error );
+    TRACE( "%p %p %lu %p %p\n", channel_handle, properties, count, handle, error );
     if (error) FIXME( "ignoring error parameter\n" );
 
     if (!channel_handle || !handle) return E_INVALIDARG;
@@ -307,7 +314,7 @@ HRESULT WINAPI WsResetMessage( WS_MESSAGE *handle, WS_ERROR *error )
     reset_msg( msg );
 
     LeaveCriticalSection( &msg->cs );
-    TRACE( "returning %08x\n", hr );
+    TRACE( "returning %#lx\n", hr );
     return hr;
 }
 
@@ -320,7 +327,7 @@ HRESULT WINAPI WsGetMessageProperty( WS_MESSAGE *handle, WS_MESSAGE_PROPERTY_ID
     struct msg *msg = (struct msg *)handle;
     HRESULT hr = S_OK;
 
-    TRACE( "%p %u %p %u %p\n", handle, id, buf, size, error );
+    TRACE( "%p %u %p %lu %p\n", handle, id, buf, size, error );
     if (error) FIXME( "ignoring error parameter\n" );
 
     if (!msg) return E_INVALIDARG;
@@ -375,12 +382,19 @@ HRESULT WINAPI WsGetMessageProperty( WS_MESSAGE *handle, WS_MESSAGE_PROPERTY_ID
         else *(BOOL *)buf = msg->is_addressed;
         break;
 
+    case WS_MESSAGE_PROPERTY_HEAP_PROPERTIES:
+    case WS_MESSAGE_PROPERTY_XML_READER_PROPERTIES:
+    case WS_MESSAGE_PROPERTY_XML_WRITER_PROPERTIES:
+        FIXME( "property %u not supported\n", id );
+        hr = E_NOTIMPL;
+        break;
+
     default:
         hr = prop_get( msg->prop, msg->prop_count, id, buf, size );
     }
 
     LeaveCriticalSection( &msg->cs );
-    TRACE( "returning %08x\n", hr );
+    TRACE( "returning %#lx\n", hr );
     return hr;
 }
 
@@ -393,7 +407,7 @@ HRESULT WINAPI WsSetMessageProperty( WS_MESSAGE *handle, WS_MESSAGE_PROPERTY_ID
     struct msg *msg = (struct msg *)handle;
     HRESULT hr;
 
-    TRACE( "%p %u %p %u\n", handle, id, value, size );
+    TRACE( "%p %u %p %lu %p\n", handle, id, value, size, error );
     if (error) FIXME( "ignoring error parameter\n" );
 
     if (!msg) return E_INVALIDARG;
@@ -421,7 +435,7 @@ HRESULT WINAPI WsSetMessageProperty( WS_MESSAGE *handle, WS_MESSAGE_PROPERTY_ID
     }
 
     LeaveCriticalSection( &msg->cs );
-    TRACE( "returning %08x\n", hr );
+    TRACE( "returning %#lx\n", hr );
     return hr;
 }
 
@@ -454,7 +468,7 @@ HRESULT WINAPI WsAddressMessage( WS_MESSAGE *handle, const WS_ENDPOINT_ADDRESS *
     if (msg->state < WS_MESSAGE_STATE_INITIALIZED || msg->is_addressed) hr = WS_E_INVALID_OPERATION;
     else if (addr && addr->url.length)
     {
-        if (!(msg->addr.chars = heap_alloc( addr->url.length * sizeof(WCHAR) ))) hr = E_OUTOFMEMORY;
+        if (!(msg->addr.chars = malloc( addr->url.length * sizeof(WCHAR) ))) hr = E_OUTOFMEMORY;
         else
         {
             memcpy( msg->addr.chars, addr->url.chars, addr->url.length * sizeof(WCHAR) );
@@ -464,7 +478,7 @@ HRESULT WINAPI WsAddressMessage( WS_MESSAGE *handle, const WS_ENDPOINT_ADDRESS *
     if (hr == S_OK) msg->is_addressed = TRUE;
 
     LeaveCriticalSection( &msg->cs );
-    TRACE( "returning %08x\n", hr );
+    TRACE( "returning %#lx\n", hr );
     return hr;
 }
 
@@ -611,9 +625,9 @@ static HRESULT write_relatesto_header( WS_XML_WRITER *writer, const WS_XML_STRIN
     return WsWriteEndElement( writer, NULL ); /* </a:RelatesTo> */
 }
 
-static HRESULT write_headers( struct msg *msg, WS_XML_WRITER *writer, const WS_XML_STRING *prefix_env,
-                              const WS_XML_STRING *ns_env, const WS_XML_STRING *prefix_addr,
-                              const WS_XML_STRING *ns_addr )
+static HRESULT write_headers( struct msg *msg, WS_MESSAGE_INITIALIZATION init, WS_XML_WRITER *writer,
+                              const WS_XML_STRING *prefix_env, const WS_XML_STRING *ns_env,
+                              const WS_XML_STRING *prefix_addr, const WS_XML_STRING *ns_addr )
 {
     static const WS_XML_STRING header = {6, (BYTE *)"Header"};
     HRESULT hr;
@@ -625,22 +639,25 @@ static HRESULT write_headers( struct msg *msg, WS_XML_WRITER *writer, const WS_X
     if ((hr = write_action_header( writer, prefix_env, ns_env, prefix_addr, ns_addr, msg->action )) != S_OK)
         return hr;
 
-    if (msg->init == WS_REPLY_MESSAGE)
+    if (init == WS_REPLY_MESSAGE || init == WS_FAULT_MESSAGE)
     {
         if ((hr = write_relatesto_header( writer, prefix_env, ns_env, prefix_addr, ns_addr, &msg->id_req )) != S_OK)
             return hr;
     }
-    else if (msg->addr.length)
-    {
-        if ((hr = write_to_header( writer, prefix_env, ns_env, prefix_addr, ns_addr, &msg->addr )) != S_OK)
-            return hr;
-    }
     else
     {
-        if ((hr = write_msgid_header( writer, prefix_env, ns_env, prefix_addr, ns_addr, &msg->id )) != S_OK)
+        if (init == WS_REQUEST_MESSAGE)
+        {
+            if ((hr = write_msgid_header(writer, prefix_env, ns_env, prefix_addr, ns_addr, &msg->id)) != S_OK)
+                return hr;
+            if (msg->version_addr == WS_ADDRESSING_VERSION_0_9 &&
+                (hr = write_replyto_header(writer, prefix_env, ns_env, prefix_addr, ns_addr)) != S_OK)
+                return hr;
+        }
+
+        if (msg->addr.length &&
+            (hr = write_to_header(writer, prefix_env, ns_env, prefix_addr, ns_addr, &msg->addr)) != S_OK)
             return hr;
-        if (msg->version_addr == WS_ADDRESSING_VERSION_0_9 &&
-            (hr = write_replyto_header( writer, prefix_env, ns_env, prefix_addr, ns_addr )) != S_OK) return hr;
     }
 
     for (i = 0; i < msg->header_count; i++)
@@ -685,7 +702,7 @@ static HRESULT write_headers_transport( struct msg *msg, WS_XML_WRITER *writer,
     return hr;
 }
 
-static HRESULT write_envelope_start( struct msg *msg, WS_XML_WRITER *writer )
+static HRESULT write_envelope_start( struct msg *msg, WS_MESSAGE_INITIALIZATION init, WS_XML_WRITER *writer )
 {
     static const WS_XML_STRING envelope = {8, (BYTE *)"Envelope"}, body = {4, (BYTE *)"Body"};
     static const WS_XML_STRING prefix_s = {1, (BYTE *)"s"}, prefix_a = {1, (BYTE *)"a"};
@@ -700,7 +717,7 @@ static HRESULT write_envelope_start( struct msg *msg, WS_XML_WRITER *writer )
     if (msg->version_addr == WS_ADDRESSING_VERSION_TRANSPORT)
         hr = write_headers_transport( msg, writer, prefix_env, ns_env );
     else
-        hr = write_headers( msg, writer, prefix_env, ns_env, prefix_addr, ns_addr );
+        hr = write_headers( msg, init, writer, prefix_env, ns_env, prefix_addr, ns_addr );
     if (hr != S_OK) return hr;
 
     return WsWriteStartElement( writer, prefix_env, &body, ns_env, NULL ); /* <s:Body> */
@@ -713,13 +730,13 @@ static HRESULT write_envelope_end( WS_XML_WRITER *writer )
     return WsWriteEndElement( writer, NULL ); /* </s:Envelope> */
 }
 
-static HRESULT write_envelope( struct msg *msg )
+static HRESULT write_envelope( struct msg *msg, WS_MESSAGE_INITIALIZATION init )
 {
     HRESULT hr;
     if (!msg->writer && (hr = WsCreateWriter( NULL, 0, &msg->writer, NULL )) != S_OK) return hr;
     if (!msg->buf && (hr = WsCreateXmlBuffer( msg->heap, NULL, 0, &msg->buf, NULL )) != S_OK) return hr;
     if ((hr = WsSetOutputToBuffer( msg->writer, msg->buf, NULL, 0, NULL )) != S_OK) return hr;
-    if ((hr = write_envelope_start( msg, msg->writer )) != S_OK) return hr;
+    if ((hr = write_envelope_start( msg, init, msg->writer )) != S_OK) return hr;
     return write_envelope_end( msg->writer );
 }
 
@@ -751,15 +768,15 @@ HRESULT WINAPI WsWriteEnvelopeStart( WS_MESSAGE *handle, WS_XML_WRITER *writer,
     }
 
     if (msg->state != WS_MESSAGE_STATE_INITIALIZED) hr = WS_E_INVALID_OPERATION;
-    else if ((hr = write_envelope( msg )) == S_OK &&
-             (hr = write_envelope_start( msg, writer )) == S_OK)
+    else if ((hr = write_envelope( msg, msg->init )) == S_OK &&
+             (hr = write_envelope_start( msg, msg->init, writer )) == S_OK)
     {
         msg->writer_body = writer;
         msg->state       = WS_MESSAGE_STATE_WRITING;
     }
 
     LeaveCriticalSection( &msg->cs );
-    TRACE( "returning %08x\n", hr );
+    TRACE( "returning %#lx\n", hr );
     return hr;
 }
 
@@ -789,7 +806,7 @@ HRESULT WINAPI WsWriteEnvelopeEnd( WS_MESSAGE *handle, WS_ERROR *error )
         msg->state = WS_MESSAGE_STATE_DONE;
 
     LeaveCriticalSection( &msg->cs );
-    TRACE( "returning %08x\n", hr );
+    TRACE( "returning %#lx\n", hr );
     return hr;
 }
 
@@ -802,7 +819,7 @@ HRESULT WINAPI WsWriteBody( WS_MESSAGE *handle, const WS_ELEMENT_DESCRIPTION *de
     struct msg *msg = (struct msg *)handle;
     HRESULT hr;
 
-    TRACE( "%p %p %08x %p %u %p\n", handle, desc, option, value, size, error );
+    TRACE( "%p %p %u %p %lu %p\n", handle, desc, option, value, size, error );
     if (error) FIXME( "ignoring error parameter\n" );
 
     if (!msg || !desc) return E_INVALIDARG;
@@ -832,7 +849,7 @@ HRESULT WINAPI WsWriteBody( WS_MESSAGE *handle, const WS_ELEMENT_DESCRIPTION *de
 
 done:
     LeaveCriticalSection( &msg->cs );
-    TRACE( "returning %08x\n", hr );
+    TRACE( "returning %#lx\n", hr );
     return hr;
 }
 
@@ -844,7 +861,7 @@ HRESULT WINAPI WsFlushBody( WS_MESSAGE *handle, ULONG size, const WS_ASYNC_CONTE
     struct msg *msg = (struct msg *)handle;
     HRESULT hr;
 
-    TRACE( "%p %u %p %p\n", handle, size, ctx, error );
+    TRACE( "%p %lu %p %p\n", handle, size, ctx, error );
 
     if (!msg) return E_INVALIDARG;
 
@@ -859,7 +876,7 @@ HRESULT WINAPI WsFlushBody( WS_MESSAGE *handle, ULONG size, const WS_ASYNC_CONTE
     hr = WsFlushWriter( msg->writer_body, size, ctx, error );
 
     LeaveCriticalSection( &msg->cs );
-    TRACE( "returning %08x\n", hr );
+    TRACE( "returning %#lx\n", hr );
     return hr;
 }
 
@@ -874,6 +891,18 @@ static BOOL match_current_element( WS_XML_READER *reader, const WS_XML_STRING *l
     return WsXmlStringEquals( elem->localName, localname, NULL ) == S_OK;
 }
 
+static BOOL match_current_element_with_ns( WS_XML_READER *reader, const WS_XML_STRING *localname, const WS_XML_STRING *ns )
+{
+    const WS_XML_NODE *node;
+    const WS_XML_ELEMENT_NODE *elem;
+
+    if (WsGetReaderNode( reader, &node, NULL ) != S_OK) return FALSE;
+    if (node->nodeType != WS_XML_NODE_TYPE_ELEMENT) return FALSE;
+    elem = (const WS_XML_ELEMENT_NODE *)node;
+    return WsXmlStringEquals( elem->localName, localname, NULL ) == S_OK &&
+           WsXmlStringEquals( elem->ns, ns, NULL ) == S_OK;
+}
+
 static HRESULT read_message_id( WS_XML_READER *reader, GUID *ret )
 {
     const WS_XML_NODE *node;
@@ -911,6 +940,9 @@ static HRESULT read_envelope_start( struct msg *msg, WS_XML_READER *reader )
 {
     static const WS_XML_STRING envelope = {8, (BYTE *)"Envelope"}, body = {4, (BYTE *)"Body"};
     static const WS_XML_STRING header = {6, (BYTE *)"Header"}, msgid = {9, (BYTE *)"MessageID"};
+    static const WS_XML_STRING fault = {5, (BYTE *)"Fault"};
+    const WS_XML_STRING *ns_env = get_env_namespace( msg->version_env );
+    BOOL isfault;
     HRESULT hr;
 
     if ((hr = WsReadNode( reader, NULL )) != S_OK) return hr;
@@ -927,7 +959,10 @@ static HRESULT read_envelope_start( struct msg *msg, WS_XML_READER *reader )
         }
     }
     if (!match_current_element( reader, &body )) return WS_E_INVALID_FORMAT;
-    return WsReadNode( reader, NULL );
+    if ((hr = WsReadNode( reader, NULL )) != S_OK) return hr;
+
+    isfault = match_current_element_with_ns( reader, &fault, ns_env );
+    return prop_set( msg->prop, msg->prop_count, WS_MESSAGE_PROPERTY_IS_FAULT, &isfault, sizeof(isfault) );
 }
 
 /**************************************************************************
@@ -966,7 +1001,7 @@ HRESULT WINAPI WsReadEnvelopeStart( WS_MESSAGE *handle, WS_XML_READER *reader, W
     }
 
     LeaveCriticalSection( &msg->cs );
-    TRACE( "returning %08x\n", hr );
+    TRACE( "returning %#lx\n", hr );
     return hr;
 }
 
@@ -996,7 +1031,7 @@ HRESULT WINAPI WsReadEnvelopeEnd( WS_MESSAGE *handle, WS_ERROR *error )
         msg->state = WS_MESSAGE_STATE_DONE;
 
     LeaveCriticalSection( &msg->cs );
-    TRACE( "returning %08x\n", hr );
+    TRACE( "returning %#lx\n", hr );
     return hr;
 }
 
@@ -1007,9 +1042,11 @@ HRESULT WINAPI WsReadBody( WS_MESSAGE *handle, const WS_ELEMENT_DESCRIPTION *des
                            WS_HEAP *heap, void *value, ULONG size, WS_ERROR *error )
 {
     struct msg *msg = (struct msg *)handle;
+    WS_ELEMENT_DESCRIPTION tmp;
+    WS_FAULT_DESCRIPTION fault_desc;
     HRESULT hr;
 
-    TRACE( "%p %p %08x %p %p %u %p\n", handle, desc, option, heap, value, size, error );
+    TRACE( "%p %p %u %p %p %lu %p\n", handle, desc, option, heap, value, size, error );
     if (error) FIXME( "ignoring error parameter\n" );
 
     if (!msg || !desc) return E_INVALIDARG;
@@ -1023,10 +1060,26 @@ HRESULT WINAPI WsReadBody( WS_MESSAGE *handle, const WS_ELEMENT_DESCRIPTION *des
     }
 
     if (msg->state != WS_MESSAGE_STATE_READING) hr = WS_E_INVALID_OPERATION;
-    else hr = WsReadElement( msg->reader_body, desc, option, heap, value, size, NULL );
+    else
+    {
+        if (!desc->typeDescription)
+        {
+            if (desc->type == WS_FAULT_TYPE)
+            {
+                memcpy( &tmp, desc, sizeof(*desc) );
+                fault_desc.envelopeVersion = msg->version_env;
+                tmp.typeDescription = &fault_desc;
+                desc = &tmp;
+            }
+            else if (desc->type == WS_ENDPOINT_ADDRESS_TYPE)
+                FIXME( "!desc->typeDescription with WS_ENDPOINT_ADDRESS_TYPE\n" );
+        }
+
+        hr = WsReadElement( msg->reader_body, desc, option, heap, value, size, NULL );
+    }
 
     LeaveCriticalSection( &msg->cs );
-    TRACE( "returning %08x\n", hr );
+    TRACE( "returning %#lx\n", hr );
     return hr;
 }
 
@@ -1038,7 +1091,7 @@ HRESULT WINAPI WsFillBody( WS_MESSAGE *handle, ULONG size, const WS_ASYNC_CONTEX
     struct msg *msg = (struct msg *)handle;
     HRESULT hr;
 
-    TRACE( "%p %u %p %p\n", handle, size, ctx, error );
+    TRACE( "%p %lu %p %p\n", handle, size, ctx, error );
 
     if (!msg) return E_INVALIDARG;
 
@@ -1053,7 +1106,7 @@ HRESULT WINAPI WsFillBody( WS_MESSAGE *handle, ULONG size, const WS_ASYNC_CONTEX
     hr = WsFillReader( msg->reader_body, size, ctx, error );
 
     LeaveCriticalSection( &msg->cs );
-    TRACE( "returning %08x\n", hr );
+    TRACE( "returning %#lx\n", hr );
     return hr;
 }
 
@@ -1074,7 +1127,7 @@ HRESULT WINAPI WsInitializeMessage( WS_MESSAGE *handle, WS_MESSAGE_INITIALIZATIO
         return E_NOTIMPL;
     }
 
-    if (!msg || init > WS_FAULT_MESSAGE) return E_INVALIDARG;
+    if (!msg || (UINT)init > WS_FAULT_MESSAGE) return E_INVALIDARG;
 
     EnterCriticalSection( &msg->cs );
 
@@ -1085,14 +1138,14 @@ HRESULT WINAPI WsInitializeMessage( WS_MESSAGE *handle, WS_MESSAGE_INITIALIZATIO
     }
 
     if (msg->state >= WS_MESSAGE_STATE_INITIALIZED) hr = WS_E_INVALID_OPERATION;
-    else if ((hr = write_envelope( msg )) == S_OK)
+    else if ((hr = write_envelope( msg, init )) == S_OK)
     {
         msg->init  = init;
         msg->state = WS_MESSAGE_STATE_INITIALIZED;
     }
 
     LeaveCriticalSection( &msg->cs );
-    TRACE( "returning %08x\n", hr );
+    TRACE( "returning %#lx\n", hr );
     return hr;
 }
 
@@ -1100,8 +1153,7 @@ static HRESULT grow_header_array( struct msg *msg, ULONG size )
 {
     struct header **tmp;
     if (size <= msg->header_size) return S_OK;
-    if (!(tmp = heap_realloc( msg->header, 2 * msg->header_size * sizeof(struct header *) )))
-        return E_OUTOFMEMORY;
+    if (!(tmp = realloc( msg->header, 2 * msg->header_size * sizeof(struct header *) ))) return E_OUTOFMEMORY;
     msg->header = tmp;
     msg->header_size *= 2;
     return S_OK;
@@ -1111,10 +1163,10 @@ static struct header *alloc_header( WS_HEADER_TYPE type, BOOL mapped, const WS_X
                                     const WS_XML_STRING *ns )
 {
     struct header *ret;
-    if (!(ret = heap_alloc_zero( sizeof(*ret) ))) return NULL;
+    if (!(ret = calloc( 1, sizeof(*ret) ))) return NULL;
     if (name && name->length)
     {
-        if (!(ret->name.bytes = heap_alloc( name->length )))
+        if (!(ret->name.bytes = malloc( name->length )))
         {
             free_header( ret );
             return NULL;
@@ -1124,7 +1176,7 @@ static struct header *alloc_header( WS_HEADER_TYPE type, BOOL mapped, const WS_X
     }
     if (ns && ns->length)
     {
-        if (!(ret->ns.bytes = heap_alloc( ns->length )))
+        if (!(ret->ns.bytes = malloc( ns->length )))
         {
             free_header( ret );
             return NULL;
@@ -1194,7 +1246,7 @@ HRESULT WINAPI WsSetHeader( WS_MESSAGE *handle, WS_HEADER_TYPE type, WS_TYPE val
     HRESULT hr;
     ULONG i;
 
-    TRACE( "%p %u %u %08x %p %u %p\n", handle, type, value_type, option, value, size, error );
+    TRACE( "%p %u %u %u %p %lu %p\n", handle, type, value_type, option, value, size, error );
     if (error) FIXME( "ignoring error parameter\n" );
 
     if (!msg || type < WS_ACTION_HEADER || type > WS_FAULT_TO_HEADER) return E_INVALIDARG;
@@ -1235,11 +1287,11 @@ HRESULT WINAPI WsSetHeader( WS_MESSAGE *handle, WS_HEADER_TYPE type, WS_TYPE val
     else free_header( msg->header[i] );
 
     msg->header[i] = header;
-    hr = write_envelope( msg );
+    hr = write_envelope( msg, msg->init );
 
 done:
     LeaveCriticalSection( &msg->cs );
-    TRACE( "returning %08x\n", hr );
+    TRACE( "returning %#lx\n", hr );
     return hr;
 }
 
@@ -1288,7 +1340,7 @@ HRESULT WINAPI WsGetHeader( WS_MESSAGE *handle, WS_HEADER_TYPE type, WS_TYPE val
     struct msg *msg = (struct msg *)handle;
     HRESULT hr;
 
-    TRACE( "%p %u %u %08x %p %p %u %p\n", handle, type, value_type, option, heap, value, size, error );
+    TRACE( "%p %u %u %u %p %p %lu %p\n", handle, type, value_type, option, heap, value, size, error );
     if (error) FIXME( "ignoring error parameter\n" );
 
     if (!msg || type < WS_ACTION_HEADER || type > WS_FAULT_TO_HEADER || option < WS_READ_REQUIRED_VALUE ||
@@ -1306,7 +1358,7 @@ HRESULT WINAPI WsGetHeader( WS_MESSAGE *handle, WS_HEADER_TYPE type, WS_TYPE val
     else hr = get_standard_header( msg, type, value_type, option, heap, value, size );
 
     LeaveCriticalSection( &msg->cs );
-    TRACE( "returning %08x\n", hr );
+    TRACE( "returning %#lx\n", hr );
     return hr;
 }
 
@@ -1353,11 +1405,11 @@ HRESULT WINAPI WsRemoveHeader( WS_MESSAGE *handle, WS_HEADER_TYPE type, WS_ERROR
                 break;
             }
         }
-        if (removed) hr = write_envelope( msg );
+        if (removed) hr = write_envelope( msg, msg->init );
     }
 
     LeaveCriticalSection( &msg->cs );
-    TRACE( "returning %08x\n", hr );
+    TRACE( "returning %#lx\n", hr );
     return hr;
 }
 
@@ -1490,7 +1542,7 @@ HRESULT WINAPI WsAddMappedHeader( WS_MESSAGE *handle, const WS_XML_STRING *name,
     struct msg *msg = (struct msg *)handle;
     HRESULT hr;
 
-    TRACE( "%p %s %u %08x %p %u %p\n", handle, debugstr_xmlstr(name), type, option, value, size, error );
+    TRACE( "%p %s %u %u %p %lu %p\n", handle, debugstr_xmlstr(name), type, option, value, size, error );
     if (error) FIXME( "ignoring error parameter\n" );
 
     if (!msg || !name) return E_INVALIDARG;
@@ -1507,7 +1559,7 @@ HRESULT WINAPI WsAddMappedHeader( WS_MESSAGE *handle, const WS_XML_STRING *name,
     else hr = add_mapped_header( msg, name, type, option, value, size );
 
     LeaveCriticalSection( &msg->cs );
-    TRACE( "returning %08x\n", hr );
+    TRACE( "returning %#lx\n", hr );
     return hr;
 }
 
@@ -1548,7 +1600,7 @@ HRESULT WINAPI WsRemoveMappedHeader( WS_MESSAGE *handle, const WS_XML_STRING *na
     }
 
     LeaveCriticalSection( &msg->cs );
-    TRACE( "returning %08x\n", hr );
+    TRACE( "returning %#lx\n", hr );
     return hr;
 }
 
@@ -1636,7 +1688,7 @@ HRESULT WINAPI WsGetMappedHeader( WS_MESSAGE *handle, const WS_XML_STRING *name,
     struct msg *msg = (struct msg *)handle;
     HRESULT hr;
 
-    TRACE( "%p %s %u %u %u %u %p %p %u %p\n", handle, debugstr_xmlstr(name), option, index, type, read_option,
+    TRACE( "%p %s %u %lu %u %u %p %p %lu %p\n", handle, debugstr_xmlstr(name), option, index, type, read_option,
            heap, value, size, error );
     if (error) FIXME( "ignoring error parameter\n" );
     if (option != WS_SINGLETON_HEADER)
@@ -1659,7 +1711,7 @@ HRESULT WINAPI WsGetMappedHeader( WS_MESSAGE *handle, const WS_XML_STRING *name,
     else hr = get_mapped_header( msg, name, type, read_option, heap, value, size );
 
     LeaveCriticalSection( &msg->cs );
-    TRACE( "returning %08x\n", hr );
+    TRACE( "returning %#lx\n", hr );
     return hr;
 }
 
@@ -1707,7 +1759,7 @@ HRESULT WINAPI WsAddCustomHeader( WS_MESSAGE *handle, const WS_ELEMENT_DESCRIPTI
     struct header *header;
     HRESULT hr;
 
-    TRACE( "%p %p %08x %p %u %08x %p\n", handle, desc, option, value, size, attrs, error );
+    TRACE( "%p %p %u %p %lu %#lx %p\n", handle, desc, option, value, size, attrs, error );
     if (error) FIXME( "ignoring error parameter\n" );
 
     if (!msg || !desc) return E_INVALIDARG;
@@ -1730,11 +1782,11 @@ HRESULT WINAPI WsAddCustomHeader( WS_MESSAGE *handle, const WS_ELEMENT_DESCRIPTI
     if ((hr = build_custom_header( msg, desc->elementLocalName, desc->elementNs, desc->type,
                                    desc->typeDescription, option, value, size, &header )) != S_OK) goto done;
     msg->header[msg->header_count++] = header;
-    hr = write_envelope( msg );
+    hr = write_envelope( msg, msg->init );
 
 done:
     LeaveCriticalSection( &msg->cs );
-    TRACE( "returning %08x\n", hr );
+    TRACE( "returning %#lx\n", hr );
     return hr;
 }
 
@@ -1761,7 +1813,7 @@ HRESULT WINAPI WsGetCustomHeader( WS_MESSAGE *handle, const WS_ELEMENT_DESCRIPTI
     struct msg *msg = (struct msg *)handle;
     HRESULT hr;
 
-    TRACE( "%p %p %08x %u %08x %p %p %u %p %p\n", handle, desc, repeat_option, index, option, heap, value,
+    TRACE( "%p %p %u %lu %u %p %p %lu %p %p\n", handle, desc, repeat_option, index, option, heap, value,
            size, attrs, error );
     if (error) FIXME( "ignoring error parameter\n" );
 
@@ -1791,7 +1843,7 @@ HRESULT WINAPI WsGetCustomHeader( WS_MESSAGE *handle, const WS_ELEMENT_DESCRIPTI
     else hr = get_custom_header( msg, desc, option, heap, value, size );
 
     LeaveCriticalSection( &msg->cs );
-    TRACE( "returning %08x\n", hr );
+    TRACE( "returning %#lx\n", hr );
     return hr;
 }
 
@@ -1833,18 +1885,18 @@ HRESULT WINAPI WsRemoveCustomHeader( WS_MESSAGE *handle, const WS_XML_STRING *na
                 i--;
             }
         }
-        if (removed) hr = write_envelope( msg );
+        if (removed) hr = write_envelope( msg, msg->init );
     }
 
     LeaveCriticalSection( &msg->cs );
-    TRACE( "returning %08x\n", hr );
+    TRACE( "returning %#lx\n", hr );
     return hr;
 }
 
 static WCHAR *build_http_header( const WCHAR *name, const WCHAR *value, ULONG *ret_len )
 {
     int len_name = lstrlenW( name ), len_value = lstrlenW( value );
-    WCHAR *ret = heap_alloc( (len_name + len_value + 2) * sizeof(WCHAR) );
+    WCHAR *ret = malloc( (len_name + len_value + 2) * sizeof(WCHAR) );
 
     if (!ret) return NULL;
     memcpy( ret, name, len_name * sizeof(WCHAR) );
@@ -1865,7 +1917,7 @@ static WCHAR *from_xml_string( const WS_XML_STRING *str )
 {
     WCHAR *ret;
     int len = MultiByteToWideChar( CP_UTF8, 0, (char *)str->bytes, str->length, NULL, 0 );
-    if (!(ret = heap_alloc( (len + 1) * sizeof(*ret) ))) return NULL;
+    if (!(ret = malloc( (len + 1) * sizeof(*ret) ))) return NULL;
     MultiByteToWideChar( CP_UTF8, 0, (char *)str->bytes, str->length, ret, len );
     ret[len] = 0;
     return ret;
@@ -1884,16 +1936,16 @@ static HRESULT insert_mapped_headers( struct msg *msg, HINTERNET req )
         if (!(name = from_xml_string( &msg->header[i]->name ))) return E_OUTOFMEMORY;
         if (!(value = from_xml_string( msg->header[i]->u.text )))
         {
-            heap_free( name );
+            free( name );
             return E_OUTOFMEMORY;
         }
         header = build_http_header( name, value, &len );
-        heap_free( name );
-        heap_free( value );
+        free( name );
+        free( value );
         if (!header) return E_OUTOFMEMORY;
 
         hr = insert_http_header( req, header, len, WINHTTP_ADDREQ_FLAG_ADD|WINHTTP_ADDREQ_FLAG_REPLACE );
-        heap_free( header );
+        free( header );
         if (hr != S_OK) return hr;
     }
     return S_OK;
@@ -1901,14 +1953,6 @@ static HRESULT insert_mapped_headers( struct msg *msg, HINTERNET req )
 
 HRESULT message_insert_http_headers( WS_MESSAGE *handle, HINTERNET req )
 {
-    static const WCHAR contenttypeW[] =
-        {'C','o','n','t','e','n','t','-','T','y','p','e',0};
-    static const WCHAR soapxmlW[] =
-        {'a','p','p','l','i','c','a','t','i','o','n','/','s','o','a','p','+','x','m','l',0};
-    static const WCHAR textxmlW[] =
-        {'t','e','x','t','/','x','m','l',0};
-    static const WCHAR charsetW[] =
-        {'c','h','a','r','s','e','t','=','u','t','f','-','8',0};
     struct msg *msg = (struct msg *)handle;
     HRESULT hr = E_OUTOFMEMORY;
     WCHAR *header = NULL, *buf;
@@ -1925,11 +1969,11 @@ HRESULT message_insert_http_headers( WS_MESSAGE *handle, HINTERNET req )
     switch (msg->version_env)
     {
     case WS_ENVELOPE_VERSION_SOAP_1_1:
-        header = build_http_header( contenttypeW, textxmlW, &len );
+        header = build_http_header( L"Content-Type", L"text/xml", &len );
         break;
 
     case WS_ENVELOPE_VERSION_SOAP_1_2:
-        header = build_http_header( contenttypeW, soapxmlW, &len );
+        header = build_http_header( L"Content-Type", L"application/soap+xml", &len );
         break;
 
     default:
@@ -1939,33 +1983,31 @@ HRESULT message_insert_http_headers( WS_MESSAGE *handle, HINTERNET req )
     if (!header) goto done;
 
     if ((hr = insert_http_header( req, header, len, WINHTTP_ADDREQ_FLAG_ADD )) != S_OK) goto done;
-    heap_free( header );
+    free( header );
 
     hr = E_OUTOFMEMORY;
-    if (!(header = build_http_header( contenttypeW, charsetW, &len ))) goto done;
+    if (!(header = build_http_header( L"Content-Type", L"charset=utf-8", &len ))) goto done;
     if ((hr = insert_http_header( req, header, len, WINHTTP_ADDREQ_FLAG_COALESCE_WITH_SEMICOLON )) != S_OK)
         goto done;
-    heap_free( header );
+    free( header );
     header = NULL;
 
     switch (msg->version_env)
     {
     case WS_ENVELOPE_VERSION_SOAP_1_1:
     {
-        static const WCHAR soapactionW[] = {'S','O','A','P','A','c','t','i','o','n',0};
-
         if (!(len = MultiByteToWideChar( CP_UTF8, 0, (char *)msg->action->bytes, msg->action->length, NULL, 0 )))
             break;
 
         hr = E_OUTOFMEMORY;
-        if (!(buf = heap_alloc( (len + 3) * sizeof(WCHAR) ))) goto done;
+        if (!(buf = malloc( (len + 3) * sizeof(WCHAR) ))) goto done;
         buf[0] = '"';
         MultiByteToWideChar( CP_UTF8, 0, (char *)msg->action->bytes, msg->action->length, buf + 1, len );
         buf[len + 1] = '"';
         buf[len + 2] = 0;
 
-        header = build_http_header( soapactionW, buf, &len );
-        heap_free( buf );
+        header = build_http_header( L"SOAPAction", buf, &len );
+        free( buf );
         if (!header) goto done;
 
         hr = insert_http_header( req, header, len, WINHTTP_ADDREQ_FLAG_ADD );
@@ -1973,22 +2015,21 @@ HRESULT message_insert_http_headers( WS_MESSAGE *handle, HINTERNET req )
     }
     case WS_ENVELOPE_VERSION_SOAP_1_2:
     {
-        static const WCHAR actionW[] = {'a','c','t','i','o','n','=','"'};
-        ULONG len_action = ARRAY_SIZE( actionW );
+        ULONG len_action = ARRAY_SIZE( L"action=\"" ) - 1;
 
         if (!(len = MultiByteToWideChar( CP_UTF8, 0, (char *)msg->action->bytes, msg->action->length, NULL, 0 )))
             break;
 
         hr = E_OUTOFMEMORY;
-        if (!(buf = heap_alloc( (len + len_action + 2) * sizeof(WCHAR) ))) goto done;
-        memcpy( buf, actionW, len_action * sizeof(WCHAR) );
+        if (!(buf = malloc( (len + len_action + 2) * sizeof(WCHAR) ))) goto done;
+        memcpy( buf, L"action=\"", len_action * sizeof(WCHAR) );
         MultiByteToWideChar( CP_UTF8, 0, (char *)msg->action->bytes, msg->action->length, buf + len_action, len );
         len += len_action;
         buf[len++] = '"';
         buf[len] = 0;
 
-        header = build_http_header( contenttypeW, buf, &len );
-        heap_free( buf );
+        header = build_http_header( L"Content-Type", buf, &len );
+        free( buf );
         if (!header) goto done;
 
         hr = insert_http_header( req, header, len, WINHTTP_ADDREQ_FLAG_COALESCE_WITH_SEMICOLON );
@@ -2002,9 +2043,9 @@ HRESULT message_insert_http_headers( WS_MESSAGE *handle, HINTERNET req )
     if (hr == S_OK) hr = insert_mapped_headers( msg, req );
 
 done:
-    heap_free( header );
+    free( header );
     LeaveCriticalSection( &msg->cs );
-    TRACE( "returning %08x\n", hr );
+    TRACE( "returning %#lx\n", hr );
     return hr;
 }
 
@@ -2022,26 +2063,26 @@ static HRESULT map_http_response_headers( struct msg *msg, HINTERNET req, const
             GetLastError() == ERROR_INSUFFICIENT_BUFFER)
         {
             HRESULT hr;
-            if (!(value = heap_alloc( size )))
+            if (!(value = malloc( size )))
             {
-                heap_free( name );
+                free( name );
                 return E_OUTOFMEMORY;
             }
             if (!WinHttpQueryHeaders( req, WINHTTP_QUERY_CUSTOM, name, value, &size, NULL ))
             {
-                heap_free( name );
+                free( name );
                 return HRESULT_FROM_WIN32( GetLastError() );
             }
             hr = add_mapped_header( msg, &mapping->responseHeaderMappings[i]->headerName, WS_WSZ_TYPE,
                                     WS_WRITE_REQUIRED_POINTER, &value, sizeof(value) );
-            heap_free( value );
+            free( value );
             if (hr != S_OK)
             {
-                heap_free( name );
+                free( name );
                 return hr;
             }
         }
-        heap_free( name );
+        free( name );
     }
     return S_OK;
 }
@@ -2062,7 +2103,7 @@ HRESULT message_map_http_response_headers( WS_MESSAGE *handle, HINTERNET req, co
     hr = map_http_response_headers( msg, req, mapping );
 
     LeaveCriticalSection( &msg->cs );
-    TRACE( "returning %08x\n", hr );
+    TRACE( "returning %#lx\n", hr );
     return hr;
 }
 
@@ -2119,7 +2160,7 @@ void message_do_send_callback( WS_MESSAGE *handle )
         HRESULT hr;
         TRACE( "executing callback %p\n", msg->ctx_send.callback );
         hr = msg->ctx_send.callback( handle, msg->heap, msg->ctx_send.state, NULL );
-        TRACE( "callback %p returned %08x\n", msg->ctx_send.callback, hr );
+        TRACE( "callback %p returned %#lx\n", msg->ctx_send.callback, hr );
     }
 
     LeaveCriticalSection( &msg->cs );
@@ -2142,7 +2183,7 @@ void message_do_receive_callback( WS_MESSAGE *handle )
         HRESULT hr;
         TRACE( "executing callback %p\n", msg->ctx_receive.callback );
         hr = msg->ctx_receive.callback( handle, msg->heap, msg->ctx_receive.state, NULL );
-        TRACE( "callback %p returned %08x\n", msg->ctx_receive.callback, hr );
+        TRACE( "callback %p returned %#lx\n", msg->ctx_receive.callback, hr );
     }
 
     LeaveCriticalSection( &msg->cs );
@@ -2218,3 +2259,44 @@ HRESULT message_set_request_id( WS_MESSAGE *handle, const GUID *id )
     LeaveCriticalSection( &msg->cs );
     return hr;
 }
+
+/* Attempt to read a fault message. If the message is a fault, WS_E_ENDPOINT_FAULT_RECEIVED will be returned. */
+HRESULT message_read_fault( WS_MESSAGE *handle, WS_HEAP *heap, WS_ERROR *error )
+{
+    static const WS_ELEMENT_DESCRIPTION desc = { NULL, NULL, WS_FAULT_TYPE, NULL };
+    BOOL isfault;
+    WS_FAULT fault = {0};
+    WS_XML_STRING action;
+    HRESULT hr;
+
+    if ((hr = WsGetMessageProperty( handle, WS_MESSAGE_PROPERTY_IS_FAULT, &isfault, sizeof(isfault), NULL )) != S_OK)
+        return hr;
+    if (!isfault)
+        return S_OK;
+
+    if ((hr = WsReadBody( handle, &desc, WS_READ_REQUIRED_VALUE, heap, &fault, sizeof(fault), NULL )) != S_OK ||
+        (hr = WsReadEnvelopeEnd( handle, NULL )) != S_OK)
+        goto done;
+
+    if (!error) goto done;
+
+    if ((hr = WsSetFaultErrorProperty( error, WS_FAULT_ERROR_PROPERTY_FAULT, &fault, sizeof(fault) )) != S_OK)
+        goto done;
+
+    if ((hr = WsGetHeader( handle, WS_ACTION_HEADER, WS_XML_STRING_TYPE, WS_READ_REQUIRED_VALUE,
+                           heap, &action, sizeof(action), NULL )) != S_OK)
+    {
+        if (hr == WS_E_INVALID_FORMAT)
+        {
+            memset( &action, 0, sizeof(action) );
+            hr = S_OK;
+        }
+        else
+            goto done;
+    }
+    hr = WsSetFaultErrorProperty( error, WS_FAULT_ERROR_PROPERTY_ACTION, &action, sizeof(action) );
+
+done:
+    free_fault_fields( heap, &fault );
+    return hr != S_OK ? hr : WS_E_ENDPOINT_FAULT_RECEIVED;
+}
