@@ -211,6 +211,16 @@ HBRUSH SYSCALL_API NtGdiCreateSolidBrush( COLORREF color, HBRUSH brush )
     SYSCALL_FUNC( NtGdiCreateSolidBrush );
 }
 
+NTSTATUS SYSCALL_API NtGdiDdDDIAcquireKeyedMutex( D3DKMT_ACQUIREKEYEDMUTEX *params )
+{
+    SYSCALL_FUNC( NtGdiDdDDIAcquireKeyedMutex );
+}
+
+NTSTATUS SYSCALL_API NtGdiDdDDIAcquireKeyedMutex2( D3DKMT_ACQUIREKEYEDMUTEX2 *params )
+{
+    SYSCALL_FUNC( NtGdiDdDDIAcquireKeyedMutex2 );
+}
+
 NTSTATUS SYSCALL_API NtGdiDdDDICheckOcclusion( const D3DKMT_CHECKOCCLUSION *desc )
 {
     SYSCALL_FUNC( NtGdiDdDDICheckOcclusion );
@@ -406,6 +416,16 @@ NTSTATUS SYSCALL_API NtGdiDdDDIQueryVideoMemoryInfo( D3DKMT_QUERYVIDEOMEMORYINFO
     SYSCALL_FUNC( NtGdiDdDDIQueryVideoMemoryInfo );
 }
 
+NTSTATUS SYSCALL_API NtGdiDdDDIReleaseKeyedMutex( D3DKMT_RELEASEKEYEDMUTEX *params )
+{
+    SYSCALL_FUNC( NtGdiDdDDIReleaseKeyedMutex );
+}
+
+NTSTATUS SYSCALL_API NtGdiDdDDIReleaseKeyedMutex2( D3DKMT_RELEASEKEYEDMUTEX2 *params )
+{
+    SYSCALL_FUNC( NtGdiDdDDIReleaseKeyedMutex2 );
+}
+
 NTSTATUS SYSCALL_API NtGdiDdDDISetQueuedLimit( D3DKMT_SETQUEUEDLIMIT *desc )
 {
     SYSCALL_FUNC( NtGdiDdDDISetQueuedLimit );
@@ -1824,6 +1844,11 @@ HDC SYSCALL_API NtUserGetWindowDC( HWND hwnd )
     SYSCALL_FUNC( NtUserGetWindowDC );
 }
 
+BOOL SYSCALL_API NtUserGetWindowDisplayAffinity( HWND hwnd, DWORD *affinity )
+{
+    SYSCALL_FUNC( NtUserGetWindowDisplayAffinity );
+}
+
 BOOL SYSCALL_API NtUserGetWindowPlacement( HWND hwnd, WINDOWPLACEMENT *placement )
 {
     SYSCALL_FUNC( NtUserGetWindowPlacement );
