@@ -26,6 +26,8 @@
 #endif
 #include "macdrv_cocoa.h"
 
+#pragma GCC diagnostic ignored "-Wdeclaration-after-statement"
+
 static uint64_t dedicated_gpu_id;
 static uint64_t integrated_gpu_id;
 
@@ -58,9 +60,8 @@ static inline void convert_display_rect(CGRect* out_rect, NSRect in_rect,
  */
 int macdrv_get_displays(struct macdrv_display** displays, int* count)
 {
-    int ret = -1;
-    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
-
+@autoreleasepool
+{
     NSArray* screens = [NSScreen screens];
     if (screens)
     {
@@ -74,14 +75,14 @@ int macdrv_get_displays(struct macdrv_display** displays, int* count)
             NSUInteger i;
             for (i = 0; i < num_screens; i++)
             {
-                NSScreen* screen = [screens objectAtIndex:i];
+                NSScreen* screen = screens[i];
                 NSRect frame = [screen frame];
                 NSRect visible_frame = [screen visibleFrame];
 
                 if (i == 0)
                     primary_frame = frame;
 
-                disps[i].displayID = [[[screen deviceDescription] objectForKey:@"NSScreenNumber"] unsignedIntValue];
+                disps[i].displayID = [[screen deviceDescription][@"NSScreenNumber"] unsignedIntValue];
                 convert_display_rect(&disps[i].frame, frame, primary_frame);
                 convert_display_rect(&disps[i].work_frame, visible_frame,
                                      primary_frame);
@@ -91,12 +92,12 @@ int macdrv_get_displays(struct macdrv_display** displays, int* count)
 
             *displays = disps;
             *count = num_screens;
-            ret = 0;
+            return 0;
         }
     }
 
-    [pool release];
-    return ret;
+    return -1;
+}
 }
 
 
@@ -190,12 +191,13 @@ done:
  * Returns non-zero value on failure.
  */
 static int macdrv_get_gpu_info_from_entry(struct macdrv_gpu* gpu, io_registry_entry_t entry)
+{
+@autoreleasepool
 {
     io_registry_entry_t parent_entry;
     io_registry_entry_t gpu_entry;
     kern_return_t result;
     int ret = -1;
-    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
 
     gpu_entry = entry;
     while (![@"IOPCIDevice" isEqualToString:[(NSString*)IOObjectCopyClass(gpu_entry) autorelease]])
@@ -211,7 +213,6 @@ static int macdrv_get_gpu_info_from_entry(struct macdrv_gpu* gpu, io_registry_en
         }
         else if (result != kIOReturnSuccess)
         {
-            [pool release];
             return ret;
         }
 
@@ -232,9 +233,9 @@ static int macdrv_get_gpu_info_from_entry(struct macdrv_gpu* gpu, io_registry_en
 done:
     if (gpu_entry != entry)
         IOObjectRelease(gpu_entry);
-    [pool release];
     return ret;
 }
+}
 
 #ifdef HAVE_MTLDEVICE_REGISTRYID
 
@@ -250,12 +251,45 @@ static int macdrv_get_gpu_info_from_registry_id(struct macdrv_gpu* gpu, uint64_t
     int ret;
     io_registry_entry_t entry;
 
-    entry = IOServiceGetMatchingService(kIOMasterPortDefault, IORegistryEntryIDMatching(registry_id));
+    entry = IOServiceGetMatchingService(0, IORegistryEntryIDMatching(registry_id));
     ret = macdrv_get_gpu_info_from_entry(gpu, entry);
     IOObjectRelease(entry);
     return ret;
 }
 
+/***********************************************************************
+ *              macdrv_get_gpu_info_from_mtldevice
+ *
+ * Get GPU information from a Metal device that responds to the registryID selector.
+ *
+ * Returns non-zero value on failure.
+ */
+static int macdrv_get_gpu_info_from_mtldevice(struct macdrv_gpu* gpu, id<MTLDevice> device)
+{
+    int ret;
+    if ((ret = macdrv_get_gpu_info_from_registry_id(gpu, [device registryID])))
+        return ret;
+#if defined(MAC_OS_X_VERSION_10_15) && MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_15
+    /* Apple GPUs aren't PCI devices and therefore have no device ID
+     * Use the Metal GPUFamily as the device ID */
+    if (!gpu->device_id && [device respondsToSelector:@selector(supportsFamily:)] && [device supportsFamily:MTLGPUFamilyApple1])
+    {
+        MTLGPUFamily highest = MTLGPUFamilyApple1;
+        while (1)
+        {
+            /* Apple2, etc are all in order */
+            MTLGPUFamily next = highest + 1;
+            if ([device supportsFamily:next])
+                highest = next;
+            else
+                break;
+        }
+        gpu->device_id = highest;
+    }
+#endif
+    return 0;
+}
+
 /***********************************************************************
  *              macdrv_get_gpus_from_metal
  *
@@ -264,6 +298,8 @@ static int macdrv_get_gpu_info_from_registry_id(struct macdrv_gpu* gpu, uint64_t
  * Returns non-zero value on failure with parameters unchanged and zero on success.
  */
 static int macdrv_get_gpus_from_metal(struct macdrv_gpu** new_gpus, int* count)
+{
+@autoreleasepool
 {
     struct macdrv_gpu* gpus = NULL;
     struct macdrv_gpu primary_gpu;
@@ -271,26 +307,24 @@ static int macdrv_get_gpus_from_metal(struct macdrv_gpu** new_gpus, int* count)
     BOOL hide_integrated = FALSE;
     int primary_index = 0, i;
     int gpu_count = 0;
-    int ret = -1;
-    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
 
     /* Test if Metal is available */
     if (&MTLCopyAllDevices == NULL)
-        goto done;
+        return -1;
     NSArray<id<MTLDevice>>* devices = [MTLCopyAllDevices() autorelease];
     if (!devices.count || ![devices[0] respondsToSelector:@selector(registryID)])
-        goto done;
+        return -1;
 
     gpus = calloc(devices.count, sizeof(*gpus));
     if (!gpus)
-        goto done;
+        return -1;
 
     /* Use MTLCreateSystemDefaultDevice instead of CGDirectDisplayCopyCurrentMetalDevice(CGMainDisplayID()) to get
      * the primary GPU because we need to hide the integrated GPU for an automatic graphic switching pair to avoid apps
      * using the integrated GPU. This is the behavior of Windows on a Mac. */
     primary_device = [MTLCreateSystemDefaultDevice() autorelease];
-    if (macdrv_get_gpu_info_from_registry_id(&primary_gpu, primary_device.registryID))
-        goto done;
+    if (macdrv_get_gpu_info_from_mtldevice(&primary_gpu, primary_device))
+        goto fail;
 
     /* Hide the integrated GPU if the system default device is a dedicated GPU */
     if (!primary_device.isLowPower)
@@ -301,8 +335,8 @@ static int macdrv_get_gpus_from_metal(struct macdrv_gpu** new_gpus, int* count)
 
     for (i = 0; i < devices.count; i++)
     {
-        if (macdrv_get_gpu_info_from_registry_id(&gpus[gpu_count], devices[i].registryID))
-            goto done;
+        if (macdrv_get_gpu_info_from_mtldevice(&gpus[gpu_count], devices[i]))
+            goto fail;
 
         if (hide_integrated && devices[i].isLowPower)
         {
@@ -327,12 +361,11 @@ static int macdrv_get_gpus_from_metal(struct macdrv_gpu** new_gpus, int* count)
 
     *new_gpus = gpus;
     *count = gpu_count;
-    ret = 0;
-done:
-    if (ret)
-        macdrv_free_gpus(gpus);
-    [pool release];
-    return ret;
+    return 0;
+fail:
+    macdrv_free_gpus(gpus);
+    return -1;
+}
 }
 
 /***********************************************************************
@@ -343,22 +376,21 @@ done:
  * Returns non-zero value on failure.
  */
 static int macdrv_get_gpu_info_from_display_id_using_metal(struct macdrv_gpu* gpu, CGDirectDisplayID display_id)
+{
+@autoreleasepool
 {
     id<MTLDevice> device;
-    int ret = -1;
-    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
 
     /* Test if Metal is available */
     if (&CGDirectDisplayCopyCurrentMetalDevice == NULL)
-        goto done;
+        return -1;
 
     device = [CGDirectDisplayCopyCurrentMetalDevice(display_id) autorelease];
     if (device && [device respondsToSelector:@selector(registryID)])
-        ret = macdrv_get_gpu_info_from_registry_id(gpu, device.registryID);
-
-done:
-    [pool release];
-    return ret;
+        return macdrv_get_gpu_info_from_registry_id(gpu, device.registryID);
+    else
+        return -1;
+}
 }
 
 #else
@@ -422,7 +454,7 @@ static int macdrv_get_gpus_from_iokit(struct macdrv_gpu** new_gpus, int* count)
     if (!gpus)
         goto done;
 
-    if (IOServiceGetMatchingServices(kIOMasterPortDefault, IOServiceMatching("IOPCIDevice"), &iterator)
+    if (IOServiceGetMatchingServices(0, IOServiceMatching("IOPCIDevice"), &iterator)
         != kIOReturnSuccess)
         goto done;
 
@@ -575,6 +607,7 @@ int macdrv_get_adapters(uint64_t gpu_id, struct macdrv_adapter** new_adapters, i
         if (gpu.id == gpu_id || (gpu_id == dedicated_gpu_id && gpu.id == integrated_gpu_id))
         {
             adapters[adapter_count].id = display_ids[i];
+            adapters[adapter_count].state_flags = DISPLAY_DEVICE_ATTACHED_TO_DESKTOP;
 
             if (CGDisplayIsMain(display_ids[i]))
             {
@@ -582,9 +615,6 @@ int macdrv_get_adapters(uint64_t gpu_id, struct macdrv_adapter** new_adapters, i
                 primary_index = adapter_count;
             }
 
-            if (CGDisplayIsActive(display_ids[i]))
-                adapters[adapter_count].state_flags |= DISPLAY_DEVICE_ATTACHED_TO_DESKTOP;
-
             adapter_count++;
         }
     }
@@ -646,65 +676,52 @@ int macdrv_get_monitors(uint32_t adapter_id, struct macdrv_monitor** new_monitor
     if (!monitors)
         return -1;
 
-    /* Report an inactive monitor */
-    if (!CGDisplayIsActive(adapter_id) && !CGDisplayIsInMirrorSet(adapter_id))
-    {
-        strcpy(monitors[monitor_count].name, "Generic Non-PnP Monitor");
-        monitors[monitor_count].state_flags = DISPLAY_DEVICE_ATTACHED;
-        monitor_count++;
-    }
-    /* Report active and mirrored monitors in the same mirroring set */
-    else
-    {
-        if (CGGetOnlineDisplayList(sizeof(display_ids) / sizeof(display_ids[0]), display_ids, &display_id_count)
-            != kCGErrorSuccess)
-            goto done;
+    if (CGGetOnlineDisplayList(sizeof(display_ids) / sizeof(display_ids[0]), display_ids, &display_id_count)
+        != kCGErrorSuccess)
+        goto done;
 
-        if (macdrv_get_displays(&displays, &display_count))
-            goto done;
+    if (macdrv_get_displays(&displays, &display_count))
+        goto done;
 
-        for (i = 0; i < display_id_count; i++)
+    for (i = 0; i < display_id_count; i++)
+    {
+        if (display_ids[i] != adapter_id && CGDisplayMirrorsDisplay(display_ids[i]) != adapter_id)
+            continue;
+
+        /* Find and fill in monitor info */
+        for (j = 0; j < display_count; j++)
         {
-            if (display_ids[i] != adapter_id && CGDisplayMirrorsDisplay(display_ids[i]) != adapter_id)
-                continue;
-
-            /* Find and fill in monitor info */
-            for (j = 0; j < display_count; j++)
+            if (displays[j].displayID == display_ids[i]
+                || CGDisplayMirrorsDisplay(display_ids[i]) == displays[j].displayID)
             {
-                if (displays[j].displayID == display_ids[i]
-                    || CGDisplayMirrorsDisplay(display_ids[i]) == displays[j].displayID)
+                /* Allocate more space if needed */
+                if (monitor_count >= capacity)
                 {
-                    /* Allocate more space if needed */
-                    if (monitor_count >= capacity)
-                    {
-                        capacity *= 2;
-                        realloc_monitors = realloc(monitors, sizeof(*monitors) * capacity);
-                        if (!realloc_monitors)
-                            goto done;
-                        monitors = realloc_monitors;
-                    }
-
-                    if (j == 0)
-                        primary_index = monitor_count;
-
-                    strcpy(monitors[monitor_count].name, "Generic Non-PnP Monitor");
-                    monitors[monitor_count].state_flags = DISPLAY_DEVICE_ATTACHED | DISPLAY_DEVICE_ACTIVE;
-                    monitors[monitor_count].rc_monitor = displays[j].frame;
-                    monitors[monitor_count].rc_work = displays[j].work_frame;
-                    monitor_count++;
-                    break;
+                    capacity *= 2;
+                    realloc_monitors = realloc(monitors, sizeof(*monitors) * capacity);
+                    if (!realloc_monitors)
+                        goto done;
+                    monitors = realloc_monitors;
                 }
+
+                if (j == 0)
+                    primary_index = monitor_count;
+
+                monitors[monitor_count].rc_monitor = displays[j].frame;
+                monitors[monitor_count].rc_work = displays[j].work_frame;
+                monitor_count++;
+                break;
             }
         }
+    }
 
-        /* Make sure the first monitor on primary adapter is primary */
-        if (primary_index)
-        {
-            struct macdrv_monitor tmp;
-            tmp = monitors[0];
-            monitors[0] = monitors[primary_index];
-            monitors[primary_index] = tmp;
-        }
+    /* Make sure the first monitor on primary adapter is primary */
+    if (primary_index)
+    {
+        struct macdrv_monitor tmp;
+        tmp = monitors[0];
+        monitors[0] = monitors[primary_index];
+        monitors[primary_index] = tmp;
     }
 
     *new_monitors = monitors;
