@@ -275,8 +275,6 @@ static const struct user_driver_funcs macdrv_funcs =
     .pDesktopWindowProc = macdrv_DesktopWindowProc,
     .pDestroyCursorIcon = macdrv_DestroyCursorIcon,
     .pDestroyWindow = macdrv_DestroyWindow,
-    .pGetCurrentDisplaySettings = macdrv_GetCurrentDisplaySettings,
-    .pGetDisplayDepth = macdrv_GetDisplayDepth,
     .pUpdateDisplayDevices = macdrv_UpdateDisplayDevices,
     .pGetCursorPos = macdrv_GetCursorPos,
     .pGetKeyboardLayoutList = macdrv_GetKeyboardLayoutList,
@@ -288,7 +286,7 @@ static const struct user_driver_funcs macdrv_funcs =
     .pSetCursor = macdrv_SetCursor,
     .pSetCursorPos = macdrv_SetCursorPos,
     .pSetDesktopWindow = macdrv_SetDesktopWindow,
-    .pSetFocus = macdrv_SetFocus,
+    .pActivateWindow = macdrv_ActivateWindow,
     .pSetLayeredWindowAttributes = macdrv_SetLayeredWindowAttributes,
     .pSetParent = macdrv_SetParent,
     .pSetWindowRgn = macdrv_SetWindowRgn,
@@ -305,11 +303,14 @@ static const struct user_driver_funcs macdrv_funcs =
     .pVkKeyScanEx = macdrv_VkKeyScanEx,
     .pImeProcessKey = macdrv_ImeProcessKey,
     .pNotifyIMEStatus = macdrv_NotifyIMEStatus,
+    .pSetIMECompositionRect = macdrv_SetIMECompositionRect,
     .pWindowMessage = macdrv_WindowMessage,
     .pWindowPosChanged = macdrv_WindowPosChanged,
     .pWindowPosChanging = macdrv_WindowPosChanging,
-    .pwine_get_vulkan_driver = macdrv_wine_get_vulkan_driver,
-    .pwine_get_wgl_driver = macdrv_wine_get_wgl_driver,
+    .pGetWindowStyleMasks = macdrv_GetWindowStyleMasks,
+    .pCreateWindowSurface = macdrv_CreateWindowSurface,
+    .pVulkanInit = macdrv_VulkanInit,
+    .pOpenGLInit = macdrv_OpenGLInit,
 };
 
 
