@@ -319,6 +319,32 @@ static BOOL ReadString(FILE *file, CHAR buffer[], INT bufsize, LPCSTR key,
     return TRUE;
 }
 
+static BOOL ReadStringW(FILE *file, CHAR *buffer, int size, const char *key, WCHAR **out)
+{
+    char *outA;
+    int len;
+
+    if (!ReadString(file, buffer, size, key, &outA))
+        return FALSE;
+    if (!outA)
+    {
+        *out = NULL;
+        return TRUE;
+    }
+
+    len = MultiByteToWideChar(CP_ACP, 0, outA, -1, NULL, 0);
+    if (len)
+        *out = HeapAlloc(PSDRV_Heap, 0, len * sizeof(WCHAR));
+    if (!len || !*out)
+    {
+        HeapFree(PSDRV_Heap, 0, outA);
+        return FALSE;
+    }
+    MultiByteToWideChar(CP_ACP, 0, outA, -1, *out, len);
+    HeapFree(PSDRV_Heap, 0, outA);
+    return TRUE;
+}
+
 /*******************************************************************************
  *  ReadBBox
  *
@@ -457,7 +483,7 @@ static BOOL ReadFixedPitch(FILE *file, CHAR buffer[], INT bufsize, AFM *afm,
 	return TRUE;
     }
 
-    if (_strnicmp(sz, "false", -1) == 0)
+    if (stricmp(sz, "false") == 0)
     {
     	afm->IsFixedPitch = FALSE;
 	*p_found = TRUE;
@@ -465,7 +491,7 @@ static BOOL ReadFixedPitch(FILE *file, CHAR buffer[], INT bufsize, AFM *afm,
 	return TRUE;
     }
 
-    if (_strnicmp(sz, "true", -1) == 0)
+    if (stricmp(sz, "true") == 0)
     {
     	afm->IsFixedPitch = TRUE;
 	*p_found = TRUE;
@@ -514,41 +540,7 @@ static BOOL ReadFontMetrics(FILE *file, CHAR buffer[], INT bufsize, AFM **p_afm)
     if (retval == FALSE || found == FALSE)
     	goto cleanup_afm;
 
-    retval = ReadFloat(file, buffer, bufsize, "UnderlinePosition",
-    	    &(afm->UnderlinePosition), &found);
-    if (retval == FALSE || found == FALSE)
-    	goto cleanup_afm;
-
-    retval = ReadFloat(file, buffer, bufsize, "UnderlineThickness",
-    	    &(afm->UnderlineThickness), &found);
-    if (retval == FALSE || found == FALSE)
-    	goto cleanup_afm;
-
-    retval = ReadFloat(file, buffer, bufsize, "Ascender",    	/* optional */
-    	    &(afm->Ascender), &found);
-    if (retval == FALSE)
-    	goto cleanup_afm;
-
-    retval = ReadFloat(file, buffer, bufsize, "Descender",   	/* optional */
-    	    &(afm->Descender), &found);
-    if (retval == FALSE)
-    	goto cleanup_afm;
-
     afm->WinMetrics.usUnitsPerEm = 1000;
-    afm->WinMetrics.sTypoAscender = (SHORT)Round(afm->Ascender);
-    afm->WinMetrics.sTypoDescender = (SHORT)Round(afm->Descender);
-
-    if (afm->WinMetrics.sTypoAscender == 0)
-    	afm->WinMetrics.sTypoAscender = (SHORT)Round(afm->FontBBox.ury);
-
-    if (afm->WinMetrics.sTypoDescender == 0)
-    	afm->WinMetrics.sTypoDescender = (SHORT)Round(afm->FontBBox.lly);
-
-    afm->WinMetrics.sTypoLineGap = 1200 -
-    	    (afm->WinMetrics.sTypoAscender - afm->WinMetrics.sTypoDescender);
-    if (afm->WinMetrics.sTypoLineGap < 0)
-    	afm->WinMetrics.sTypoLineGap = 0;
-
     return TRUE;
 
     cleanup_afm:    	    	    	/* handle fatal or non-fatal errors */
@@ -557,40 +549,6 @@ static BOOL ReadFontMetrics(FILE *file, CHAR buffer[], INT bufsize, AFM **p_afm)
 	return retval;
 }
 
-/*******************************************************************************
- *  ParseC
- *
- *  Fatal error:    	return FALSE (none defined)
- *
- *  Non-fatal error:	leave metrics->C set to INT_MAX
- *
- */
-static BOOL ParseC(LPSTR sz, OLD_AFMMETRICS *metrics)
-{
-    int     base = 10;
-    long    l;
-    CHAR    *cp, *end_ptr;
-
-    cp = sz + 1;
-
-    if (*cp == 'H')
-    {
-    	base = 16;
-	++cp;
-    }
-
-    errno = 0;
-    l = strtol(cp, &end_ptr, base);
-    if (end_ptr == cp || errno != 0 || l > INT_MAX || l < INT_MIN)
-    {
-    	WARN("Error parsing character code '%s'\n", sz);
-	return TRUE;
-    }
-
-    metrics->C = (INT)l;
-    return TRUE;
-}
-
 /*******************************************************************************
  *  ParseW
  *
@@ -699,9 +657,16 @@ static BOOL ParseB(LPSTR sz, OLD_AFMMETRICS *metrics)
  *  Non-fatal error:	leave metrics-> set to NULL
  *
  */
+static int __cdecl ug_name_cmp(const void *a, const void *b)
+{
+    return strcmp(((const UNICODEGLYPH *)a)->name,
+            ((const UNICODEGLYPH *)b)->name);
+}
+
 static BOOL ParseN(LPSTR sz, OLD_AFMMETRICS *metrics)
 {
     CHAR    save, *cp, *end_ptr;
+    UNICODEGLYPH ug, *pug;
 
     cp = sz + 1;
 
@@ -722,9 +687,18 @@ static BOOL ParseN(LPSTR sz, OLD_AFMMETRICS *metrics)
     save = *end_ptr;
     *end_ptr = '\0';
 
-    metrics->N = PSDRV_GlyphName(cp);
-    if (metrics->N == NULL)
-    	return FALSE;
+    ug.name = cp;
+    pug = bsearch(&ug, PSDRV_AGLbyName, PSDRV_AGLbyNameSize,
+            sizeof(ug), ug_name_cmp);
+    if (!pug)
+    {
+        FIXME("unsupported glyph name: %s\n", cp);
+        metrics->UV = -1;
+    }
+    else
+    {
+        metrics->UV = pug->UV;
+    }
 
     *end_ptr = save;
     return TRUE;
@@ -739,12 +713,9 @@ static BOOL ParseN(LPSTR sz, OLD_AFMMETRICS *metrics)
  */
 static const OLD_AFMMETRICS badmetrics =
 {
-    INT_MAX,	    	    	    	    	    /* C */
-    INT_MAX,	    	    	    	    	    /* UV */
+    -1,		    	    	    	    	    /* UV */
     FLT_MAX,	    	    	    	    	    /* WX */
-    NULL,   	    	    	    	    	    /* N */
     { FLT_MAX, FLT_MAX, FLT_MAX, FLT_MAX }, 	    /* B */
-    NULL    	    	    	    	    	    /* L */
 };
 
 static BOOL ParseCharMetrics(LPSTR buffer, INT len, OLD_AFMMETRICS *metrics)
@@ -760,10 +731,6 @@ static BOOL ParseCharMetrics(LPSTR buffer, INT len, OLD_AFMMETRICS *metrics)
 
 	switch(*cp)
 	{
-	    case 'C':	if (ParseC(cp, metrics) == FALSE)
-	    	    	    return FALSE;
-	    	    	break;
-
 	    case 'W':	if (ParseW(cp, metrics) == FALSE)
 	    	    	    return FALSE;
 	    	    	break;
@@ -787,8 +754,7 @@ static BOOL ParseCharMetrics(LPSTR buffer, INT len, OLD_AFMMETRICS *metrics)
 	++cp;
     }
 
-    if (metrics->C == INT_MAX || metrics->WX == FLT_MAX || metrics->N == NULL ||
-    	    metrics->B.ury == FLT_MAX)
+    if (metrics->WX == FLT_MAX || metrics->UV == -1 || metrics->B.ury == FLT_MAX)
     {
     	*metrics = badmetrics;
 	return TRUE;
@@ -841,68 +807,32 @@ static inline BOOL IsWinANSI(LONG uv)
  *  Also does some font metric calculations that require UVs to be known.
  *
  */
-static int __cdecl UnicodeGlyphByNameIndex(const void *a, const void *b)
-{
-    return ((const UNICODEGLYPH *)a)->name->index -
-    	    ((const UNICODEGLYPH *)b)->name->index;
-}
-
 static VOID Unicodify(AFM *afm, OLD_AFMMETRICS *metrics)
 {
     INT     i;
 
-    if (strcmp(afm->EncodingScheme, "FontSpecific") == 0)
+    if (wcscmp(afm->EncodingScheme, L"FontSpecific") == 0)
     {
-    	for (i = 0; i < afm->NumofMetrics; ++i)
-	{
-	    if (metrics[i].C >= 0x20 && metrics[i].C <= 0xff)
-	    {
-		metrics[i].UV = metrics[i].C | 0xf000L;
-	    }
-	    else
-	    {
-	    	TRACE("Unencoded glyph '%s'\n", metrics[i].N->sz);
-		metrics[i].UV = -1L;
-	    }
-	}
-
 	afm->WinMetrics.sAscender = (SHORT)Round(afm->FontBBox.ury);
 	afm->WinMetrics.sDescender = (SHORT)Round(afm->FontBBox.lly);
     }
     else    	    	    	    	    	/* non-FontSpecific encoding */
     {
-    	UNICODEGLYPH	ug, *p_ug;
-
-	PSDRV_IndexGlyphList();     	/* for fast searching of glyph names */
-
 	afm->WinMetrics.sAscender = afm->WinMetrics.sDescender = 0;
 
 	for (i = 0; i < afm->NumofMetrics; ++i)
 	{
-	    ug.name = metrics[i].N;
-	    p_ug = bsearch(&ug, PSDRV_AGLbyName, PSDRV_AGLbyNameSize,
-	    	    sizeof(ug), UnicodeGlyphByNameIndex);
-	    if (p_ug == NULL)
-	    {
-	    	TRACE("Glyph '%s' not in Adobe Glyph List\n", ug.name->sz);
-		metrics[i].UV = -1L;
-	    }
-	    else
-	    {
-	    	metrics[i].UV = p_ug->UV;
+            if (IsWinANSI(metrics[i].UV))
+            {
+                SHORT   ury = (SHORT)Round(metrics[i].B.ury);
+                SHORT   lly = (SHORT)Round(metrics[i].B.lly);
 
-		if (IsWinANSI(p_ug->UV))
-		{
-		    SHORT   ury = (SHORT)Round(metrics[i].B.ury);
-		    SHORT   lly = (SHORT)Round(metrics[i].B.lly);
-
-		    if (ury > afm->WinMetrics.sAscender)
-		    	afm->WinMetrics.sAscender = ury;
-		    if (lly < afm->WinMetrics.sDescender)
-		    	afm->WinMetrics.sDescender = lly;
-		}
-	    }
-	}
+                if (ury > afm->WinMetrics.sAscender)
+                    afm->WinMetrics.sAscender = ury;
+                if (lly < afm->WinMetrics.sDescender)
+                    afm->WinMetrics.sDescender = lly;
+            }
+        }
 
 	if (afm->WinMetrics.sAscender == 0)
 	    afm->WinMetrics.sAscender = (SHORT)Round(afm->FontBBox.ury);
@@ -962,7 +892,7 @@ static BOOL ReadCharMetrics(FILE *file, CHAR buffer[], INT bufsize, AFM *afm,
 	if(len > 0)
 	{
 	    retval = ParseCharMetrics(buffer, len, old_metrics + i);
-	    if (retval == FALSE || old_metrics[i].C == INT_MAX)
+	    if (retval == FALSE || old_metrics[i].UV == -1)
 	    	goto cleanup_old_metrics;
 
 	    continue;
@@ -998,10 +928,8 @@ static BOOL ReadCharMetrics(FILE *file, CHAR buffer[], INT bufsize, AFM *afm,
 
     for (i = 0; i < afm->NumofMetrics; ++i, ++metrics, ++encoded_metrics)
     {
-    	metrics->C = encoded_metrics->C;
 	metrics->UV = encoded_metrics->UV;
 	metrics->WX = encoded_metrics->WX;
-	metrics->N = encoded_metrics->N;
     }
 
     HeapFree(PSDRV_Heap, 0, old_metrics);
@@ -1028,8 +956,9 @@ static BOOL BuildAFM(FILE *file)
     CHAR    	buffer[258];    	/* allow for <cr>, <lf>, and <nul> */
     AFM     	*afm;
     AFMMETRICS	*metrics;
-    LPSTR   	font_name, full_name, family_name, encoding_scheme;
-    BOOL    	retval, added;
+    WCHAR *family_name, *encoding_scheme;
+    char *font_name;
+    BOOL retval, added;
 
     retval = ReadFontMetrics(file, buffer, sizeof(buffer), &afm);
     if (retval == FALSE || afm == NULL)
@@ -1039,22 +968,17 @@ static BOOL BuildAFM(FILE *file)
     if (retval == FALSE || font_name == NULL)
     	goto cleanup_afm;
 
-    retval = ReadString(file, buffer, sizeof(buffer), "FullName", &full_name);
-    if (retval == FALSE || full_name == NULL)
-    	goto cleanup_font_name;
-
-    retval = ReadString(file, buffer, sizeof(buffer), "FamilyName",
+    retval = ReadStringW(file, buffer, sizeof(buffer), "FamilyName",
     	    &family_name);
     if (retval == FALSE || family_name == NULL)
-    	goto cleanup_full_name;
+    	goto cleanup_font_name;
 
-    retval = ReadString(file, buffer, sizeof(buffer), "EncodingScheme",
+    retval = ReadStringW(file, buffer, sizeof(buffer), "EncodingScheme",
     	    &encoding_scheme);
     if (retval == FALSE || encoding_scheme == NULL)
     	goto cleanup_family_name;
 
     afm->FontName = font_name;
-    afm->FullName = full_name;
     afm->FamilyName = family_name;
     afm->EncodingScheme = encoding_scheme;
 
@@ -1074,8 +998,6 @@ static BOOL BuildAFM(FILE *file)
     	HeapFree(PSDRV_Heap, 0, encoding_scheme);
     cleanup_family_name:
     	HeapFree(PSDRV_Heap, 0, family_name);
-    cleanup_full_name:
-    	HeapFree(PSDRV_Heap, 0, full_name);
     cleanup_font_name:
     	HeapFree(PSDRV_Heap, 0, font_name);
     cleanup_afm:
@@ -1093,13 +1015,12 @@ static BOOL BuildAFM(FILE *file)
  */
 static BOOL ReadAFMFile(LPCWSTR filename)
 {
-    static const WCHAR rW[] = {'r',0};
     FILE    *f;
     BOOL    retval;
 
     TRACE("%s\n", debugstr_w(filename));
 
-    f = _wfopen(filename, rW);
+    f = _wfopen(filename, L"r");
     if (f == NULL)
     {
     	WARN("%s: %s\n", debugstr_w(filename), strerror(errno));
@@ -1118,27 +1039,17 @@ static BOOL ReadAFMFile(LPCWSTR filename)
  *  Reads all Type 1 AFM files in a directory.
  *
  */
-static BOOL ReadAFMDir(LPCSTR dirname)
+static BOOL ReadAFMDir(const WCHAR *dirname)
 {
-    static const WCHAR starW[] = {'*',0};
-    static const WCHAR afmW[] = {'.','a','f','m',0};
-    WCHAR *path = wine_get_dos_file_name( dirname );
     struct _wfinddata_t data;
     intptr_t handle;
     WCHAR *p, *filename;
     BOOL ret = TRUE;
+    DWORD size = 8 + wcslen(dirname) + 2 + 1;
 
-    if (!path) return TRUE;
-    if (!(filename = HeapAlloc( GetProcessHeap(), 0, lstrlenW(path) + 256 )))
-    {
-        HeapFree( GetProcessHeap(), 0, path );
-        return FALSE;
-    }
-    lstrcpyW( filename, path );
-    HeapFree( GetProcessHeap(), 0, path );
-    p = filename + lstrlenW(filename);
-    *p++ = '\\';
-    lstrcpyW( p, starW );
+    if (!(filename = malloc( size * sizeof(WCHAR) ))) return FALSE;
+    swprintf( filename, size, L"\\\\?\\unix%s\\*", dirname );
+    p = filename + wcslen(filename) - 1;
 
     handle = _wfindfirst( filename, &data );
     if (handle != -1)
@@ -1146,14 +1057,13 @@ static BOOL ReadAFMDir(LPCSTR dirname)
         do
         {
             WCHAR *ext = wcschr( data.name, '.' );
-            if (!ext || wcsicmp(ext, afmW)) continue;
+            if (!ext || wcsicmp(ext, L".afm")) continue;
             lstrcpyW( p, data.name );
             if (!(ret = ReadAFMFile( filename ))) break;
         } while (!_wfindnext( handle, &data ));
     }
     _findclose( handle );
-
-    HeapFree( GetProcessHeap(), 0, filename );
+    free( filename );
     return ret;
 }
 
@@ -1170,41 +1080,32 @@ static BOOL ReadAFMDir(LPCSTR dirname)
  */
 BOOL PSDRV_GetType1Metrics(void)
 {
-    static const WCHAR pathW[] = {'A','F','M','P','a','t','h',0};
     HKEY hkey;
     DWORD len;
-    LPWSTR valueW;
-    LPSTR valueA, ptr;
+    WCHAR *value, *ptr, *next;
 
     /* @@ Wine registry key: HKCU\Software\Wine\Fonts */
     if (RegOpenKeyA(HKEY_CURRENT_USER, "Software\\Wine\\Fonts", &hkey) != ERROR_SUCCESS)
         return TRUE;
 
-    if (RegQueryValueExW( hkey, pathW, NULL, NULL, NULL, &len ) == ERROR_SUCCESS)
+    if (RegQueryValueExW( hkey, L"AFMPath", NULL, NULL, NULL, &len ) == ERROR_SUCCESS)
     {
         len += sizeof(WCHAR);
-        valueW = HeapAlloc( PSDRV_Heap, 0, len );
-        if (RegQueryValueExW( hkey, pathW, NULL, NULL, (LPBYTE)valueW, &len ) == ERROR_SUCCESS)
+        value = HeapAlloc( PSDRV_Heap, 0, len );
+        if (RegQueryValueExW( hkey, L"AFMPath", NULL, NULL, (BYTE *)value, &len ) == ERROR_SUCCESS)
         {
-            len = WideCharToMultiByte( CP_UNIXCP, 0, valueW, -1, NULL, 0, NULL, NULL );
-            valueA = HeapAlloc( PSDRV_Heap, 0, len );
-            WideCharToMultiByte( CP_UNIXCP, 0, valueW, -1, valueA, len, NULL, NULL );
-            TRACE( "got AFM font path %s\n", debugstr_a(valueA) );
-            ptr = valueA;
-            while (ptr)
+            for (ptr = value; ptr; ptr = next)
             {
-                LPSTR next = strchr( ptr, ':' );
+                next = wcschr( ptr, ':' );
                 if (next) *next++ = 0;
                 if (!ReadAFMDir( ptr ))
                 {
                     RegCloseKey(hkey);
                     return FALSE;
                 }
-                ptr = next;
             }
-            HeapFree( PSDRV_Heap, 0, valueA );
         }
-        HeapFree( PSDRV_Heap, 0, valueW );
+        HeapFree( PSDRV_Heap, 0, value );
     }
 
     RegCloseKey(hkey);
