@@ -399,6 +399,56 @@ NTSTATUS WINAPI wow64_NtGdiCreateSolidBrush( UINT *args )
     return HandleToUlong( NtGdiCreateSolidBrush( color, brush ));
 }
 
+NTSTATUS WINAPI wow64_NtGdiDdDDIAcquireKeyedMutex( UINT *args )
+{
+    struct
+    {
+        D3DKMT_HANDLE hKeyedMutex;
+        UINT64 Key;
+        ULONG pTimeout;
+        UINT64 FenceValue;
+    } *desc32 = get_ptr( &args );
+    D3DKMT_ACQUIREKEYEDMUTEX desc;
+    NTSTATUS status;
+
+    if (!desc32) return STATUS_INVALID_PARAMETER;
+    desc.hKeyedMutex = desc32->hKeyedMutex;
+    desc.Key = desc32->Key;
+    desc.pTimeout = UlongToHandle( desc32->pTimeout );
+    desc.FenceValue = desc32->FenceValue;
+    status = NtGdiDdDDIAcquireKeyedMutex( &desc );
+    desc32->FenceValue = desc.FenceValue;
+
+    return status;
+}
+
+NTSTATUS WINAPI wow64_NtGdiDdDDIAcquireKeyedMutex2( UINT *args )
+{
+    struct
+    {
+        D3DKMT_HANDLE hKeyedMutex;
+        UINT64 Key;
+        ULONG pTimeout;
+        UINT64 FenceValue;
+        ULONG pPrivateRuntimeData;
+        UINT PrivateRuntimeDataSize;
+    } *desc32 = get_ptr( &args );
+    D3DKMT_ACQUIREKEYEDMUTEX2 desc;
+    NTSTATUS status;
+
+    if (!desc32) return STATUS_INVALID_PARAMETER;
+    desc.hKeyedMutex = desc32->hKeyedMutex;
+    desc.Key = desc32->Key;
+    desc.pTimeout = UlongToHandle( desc32->pTimeout );
+    desc.FenceValue = desc32->FenceValue;
+    desc.pPrivateRuntimeData = UlongToHandle( desc32->pPrivateRuntimeData );
+    desc.PrivateRuntimeDataSize = desc32->PrivateRuntimeDataSize;
+    status = NtGdiDdDDIAcquireKeyedMutex2( &desc );
+    desc32->FenceValue = desc.FenceValue;
+
+    return status;
+}
+
 NTSTATUS WINAPI wow64_NtGdiDdDDICheckOcclusion( UINT *args )
 {
     struct
@@ -1430,6 +1480,36 @@ NTSTATUS WINAPI wow64_NtGdiDdDDIQueryVideoMemoryInfo( UINT *args )
     return status;
 }
 
+NTSTATUS WINAPI wow64_NtGdiDdDDIReleaseKeyedMutex( UINT *args )
+{
+    D3DKMT_RELEASEKEYEDMUTEX *desc = get_ptr( &args );
+    return NtGdiDdDDIReleaseKeyedMutex( desc );
+}
+
+NTSTATUS WINAPI wow64_NtGdiDdDDIReleaseKeyedMutex2( UINT *args )
+{
+    struct
+    {
+        D3DKMT_HANDLE hKeyedMutex;
+        UINT64 Key;
+        UINT64 FenceValue;
+        ULONG pPrivateRuntimeData;
+        UINT PrivateRuntimeDataSize;
+    } *desc32 = get_ptr( &args );
+    D3DKMT_RELEASEKEYEDMUTEX2 desc;
+    NTSTATUS status;
+
+    if (!desc32) return STATUS_INVALID_PARAMETER;
+    desc.hKeyedMutex = desc32->hKeyedMutex;
+    desc.Key = desc32->Key;
+    desc.FenceValue = desc32->FenceValue;
+    desc.pPrivateRuntimeData = UlongToHandle( desc32->pPrivateRuntimeData );
+    desc.PrivateRuntimeDataSize = desc32->PrivateRuntimeDataSize;
+    status = NtGdiDdDDIReleaseKeyedMutex2( &desc );
+
+    return status;
+}
+
 NTSTATUS WINAPI wow64_NtGdiDdDDISetQueuedLimit( UINT *args )
 {
     D3DKMT_SETQUEUEDLIMIT *desc = get_ptr( &args );
