@@ -356,7 +356,7 @@ static void WINAPI getaddrinfo_callback(TP_CALLBACK_INSTANCE *instance, void *co
     if (res)
     {
         *args->result = addrinfo_list_AtoW(res);
-        overlapped->u.Pointer = args->result;
+        overlapped->Pointer = args->result;
         freeaddrinfo(res);
     }
 
@@ -836,7 +836,7 @@ static struct hostent *get_local_ips( char *hostname )
         /* Check if this is a default route (there may be more than one) */
         if (!routes->table[n].dwForwardDest)
             ifdefault = ++default_routes;
-        else if (routes->table[n].u1.ForwardType != MIB_IPROUTE_TYPE_DIRECT)
+        else if (routes->table[n].ForwardType != MIB_IPROUTE_TYPE_DIRECT)
             continue;
         ifindex = routes->table[n].dwForwardIfIndex;
         ifmetric = routes->table[n].dwForwardMetric1;
@@ -1794,7 +1794,20 @@ int WINAPI WSAAddressToStringA( struct sockaddr *addr, DWORD addr_len,
             sprintf( buffer + strlen( buffer ), "]:%u", ntohs( addr6->sin6_port ) );
         break;
     }
+    case AF_BTH:
+    {
+        const SOCKADDR_BTH *sockaddr_bth = (const SOCKADDR_BTH *)addr;
+        BLUETOOTH_ADDRESS addr_bth;
 
+        if (addr_len < sizeof(SOCKADDR_BTH)) return -1;
+
+        addr_bth.ullLong = sockaddr_bth->btAddr;
+        sprintf( buffer, "(%02X:%02X:%02X:%02X:%02X:%02X)", addr_bth.rgBytes[5], addr_bth.rgBytes[4],
+                 addr_bth.rgBytes[3], addr_bth.rgBytes[2], addr_bth.rgBytes[1], addr_bth.rgBytes[0] );
+        if (sockaddr_bth->port)
+            sprintf( buffer + 19, ":%lu", sockaddr_bth->port );
+        break;
+    }
     default:
         SetLastError( WSAEINVAL );
         return -1;
@@ -1895,18 +1908,18 @@ u_long WINAPI inet_addr( const char *str )
 /***********************************************************************
  *      htonl   (ws2_32.8)
  */
-u_long WINAPI WS_htonl( u_long hostlong )
+u_long WINAPI htonl( u_long hostlong )
 {
-    return htonl( hostlong );
+    return RtlUlongByteSwap( hostlong );
 }
 
 
 /***********************************************************************
  *      htons   (ws2_32.9)
  */
-u_short WINAPI WS_htons( u_short hostshort )
+u_short WINAPI htons( u_short hostshort )
 {
-    return htons( hostshort );
+    return RtlUshortByteSwap( hostshort );
 }
 
 
@@ -1943,18 +1956,18 @@ int WINAPI WSAHtons( SOCKET s, u_short hostshort, u_short *netshort )
 /***********************************************************************
  *      ntohl   (ws2_32.14)
  */
-u_long WINAPI WS_ntohl( u_long netlong )
+u_long WINAPI ntohl( u_long netlong )
 {
-    return ntohl( netlong );
+    return RtlUlongByteSwap( netlong );
 }
 
 
 /***********************************************************************
  *      ntohs   (ws2_32.15)
  */
-u_short WINAPI WS_ntohs( u_short netshort )
+u_short WINAPI ntohs( u_short netshort )
 {
-    return ntohs( netshort );
+    return RtlUshortByteSwap( netshort );
 }
 
 
@@ -2161,8 +2174,26 @@ int WINAPI WSAEnumNameSpaceProvidersW( DWORD *len, WSANAMESPACE_INFOW *buffer )
 int WINAPI WSAProviderConfigChange( HANDLE *handle, OVERLAPPED *overlapped,
                                     LPWSAOVERLAPPED_COMPLETION_ROUTINE completion )
 {
+    SOCKET s;
+
     FIXME( "(%p %p %p) Stub!\n", handle, overlapped, completion );
-    return -1;
+
+    if (!handle)
+    {
+        SetLastError( WSAEFAULT );
+        return SOCKET_ERROR;
+    }
+
+    if (*handle != NULL) return 0;
+
+    s = WSASocketW( AF_INET, SOCK_STREAM, 0, NULL, 0, WSA_FLAG_OVERLAPPED );
+    if (s != INVALID_SOCKET)
+    {
+        *handle = (HANDLE)s;
+        return 0;
+    }
+
+    return SOCKET_ERROR;
 }
 
 
@@ -2190,6 +2221,26 @@ int WINAPI WSCEnableNSProvider( GUID *provider, BOOL enable )
 }
 
 
+/***********************************************************************
+ *      WSCGetApplicationCategory   (ws2_32.@)
+ */
+int WINAPI WSCGetApplicationCategory( const WCHAR *path, DWORD path_len, const WCHAR *extra,
+		                      DWORD extra_len, DWORD *category, int *errcode )
+{
+    FIXME( "(%s %lu %s %lu %p %p) Stub!\n",
+           debugstr_w(path), path_len, debugstr_w(extra), extra_len, category, errcode );
+
+    if (!path)
+    {
+        *errcode = WSAEINVAL;
+        return -1;
+    }
+
+    *errcode = WSANO_RECOVERY;
+    return -1;
+}
+
+
 /***********************************************************************
  *      WSCGetProviderInfo   (ws2_32.@)
  */
