@@ -878,6 +878,40 @@ typedef struct _D3DKMT_OPENKEYEDMUTEXFROMNTHANDLE
     UINT PrivateRuntimeDataSize;
 } D3DKMT_OPENKEYEDMUTEXFROMNTHANDLE;
 
+typedef struct _D3DKMT_ACQUIREKEYEDMUTEX
+{
+    D3DKMT_HANDLE hKeyedMutex;
+    UINT64 Key;
+    LARGE_INTEGER *pTimeout;
+    UINT64 FenceValue;
+} D3DKMT_ACQUIREKEYEDMUTEX;
+
+typedef struct _D3DKMT_RELEASEKEYEDMUTEX
+{
+    D3DKMT_HANDLE hKeyedMutex;
+    UINT64 Key;
+    UINT64 FenceValue;
+} D3DKMT_RELEASEKEYEDMUTEX;
+
+typedef struct _D3DKMT_ACQUIREKEYEDMUTEX2
+{
+    D3DKMT_HANDLE hKeyedMutex;
+    UINT64 Key;
+    LARGE_INTEGER *pTimeout;
+    UINT64 FenceValue;
+    VOID *pPrivateRuntimeData;
+    UINT PrivateRuntimeDataSize;
+} D3DKMT_ACQUIREKEYEDMUTEX2;
+
+typedef struct _D3DKMT_RELEASEKEYEDMUTEX2
+{
+    D3DKMT_HANDLE hKeyedMutex;
+    UINT64 Key;
+    UINT64 FenceValue;
+    VOID *pPrivateRuntimeData;
+    UINT PrivateRuntimeDataSize;
+} D3DKMT_RELEASEKEYEDMUTEX2;
+
 typedef struct _D3DKMT_OPENNTHANDLEFROMNAME
 {
     DWORD dwDesiredAccess;
@@ -1295,6 +1329,8 @@ extern "C"
 {
 #endif /* __cplusplus */
 
+NTSTATUS WINAPI D3DKMTAcquireKeyedMutex( D3DKMT_ACQUIREKEYEDMUTEX *params );
+NTSTATUS WINAPI D3DKMTAcquireKeyedMutex2( D3DKMT_ACQUIREKEYEDMUTEX2 *params );
 NTSTATUS WINAPI D3DKMTCheckOcclusion( const D3DKMT_CHECKOCCLUSION *desc );
 NTSTATUS WINAPI D3DKMTCheckVidPnExclusiveOwnership(const D3DKMT_CHECKVIDPNEXCLUSIVEOWNERSHIP *desc);
 NTSTATUS WINAPI D3DKMTCloseAdapter(const D3DKMT_CLOSEADAPTER *desc);
@@ -1334,6 +1370,8 @@ NTSTATUS WINAPI D3DKMTQueryResourceInfo( D3DKMT_QUERYRESOURCEINFO *params );
 NTSTATUS WINAPI D3DKMTQueryResourceInfoFromNtHandle( D3DKMT_QUERYRESOURCEINFOFROMNTHANDLE *params );
 NTSTATUS WINAPI D3DKMTQueryStatistics(D3DKMT_QUERYSTATISTICS *stats);
 NTSTATUS WINAPI D3DKMTQueryVideoMemoryInfo(D3DKMT_QUERYVIDEOMEMORYINFO *desc);
+NTSTATUS WINAPI D3DKMTReleaseKeyedMutex( D3DKMT_RELEASEKEYEDMUTEX *params );
+NTSTATUS WINAPI D3DKMTReleaseKeyedMutex2( D3DKMT_RELEASEKEYEDMUTEX2 *params );
 NTSTATUS WINAPI D3DKMTSetQueuedLimit(D3DKMT_SETQUEUEDLIMIT *desc);
 NTSTATUS WINAPI D3DKMTSetVidPnSourceOwner(const D3DKMT_SETVIDPNSOURCEOWNER *desc);
 NTSTATUS WINAPI D3DKMTShareObjects( UINT count, const D3DKMT_HANDLE *handles, OBJECT_ATTRIBUTES *attr, UINT access, HANDLE *handle );
