@@ -19,13 +19,14 @@
  */
 
 #define WIN32_LEAN_AND_MEAN
-#define NONAMELESSSTRUCT
-#define NONAMELESSUNION
 
 #include <stdarg.h>
+#include <ntstatus.h>
+#define WIN32_NO_STATUS
 #include <windows.h>
 #include <winternl.h>
 #include <winsvc.h>
+#include <ddk/ntddk.h>
 #include <ntsecapi.h>
 #include <rpc.h>
 
@@ -114,7 +115,6 @@ struct sc_lock
     struct scmdatabase *db;
 };
 
-static const WCHAR emptyW[] = {0};
 static PTP_CLEANUP_GROUP cleanup_group;
 HANDLE exit_event;
 
@@ -163,7 +163,7 @@ static void CALLBACK shutdown_callback(TP_CALLBACK_INSTANCE *instance, void *con
     result = WaitForSingleObject(process->control_mutex, 30000);
     if (result == WAIT_OBJECT_0)
     {
-        process_send_control(process, FALSE, emptyW, SERVICE_CONTROL_STOP, NULL, 0, &result);
+        process_send_control(process, FALSE, L"", SERVICE_CONTROL_STOP, NULL, 0, &result);
         ReleaseMutex(process->control_mutex);
     }
 
@@ -868,11 +868,32 @@ static void fill_notify(struct sc_notify_handle *notify, struct service_entry *s
     SetEvent(notify->event);
 }
 
+void notify_service_state(struct service_entry *service)
+{
+    struct sc_service_handle *service_handle;
+    DWORD mask;
+
+    mask = 1 << (service->status.dwCurrentState - SERVICE_STOPPED);
+    LIST_FOR_EACH_ENTRY(service_handle, &service->handles, struct sc_service_handle, entry)
+    {
+        struct sc_notify_handle *notify = service_handle->notify;
+        if (notify && (notify->notify_mask & mask))
+        {
+            fill_notify(notify, service);
+            sc_notify_release(notify);
+            service_handle->notify = NULL;
+            service_handle->status_notified = TRUE;
+        }
+        else
+            service_handle->status_notified = FALSE;
+    }
+}
+
 DWORD __cdecl svcctl_SetServiceStatus(SC_RPC_HANDLE handle, SERVICE_STATUS *status)
 {
-    struct sc_service_handle *service, *service_handle;
+    struct sc_service_handle *service;
     struct process_entry *process;
-    DWORD err, mask;
+    DWORD err;
 
     WINE_TRACE("(%p, %p)\n", handle, status);
 
@@ -902,21 +923,7 @@ DWORD __cdecl svcctl_SetServiceStatus(SC_RPC_HANDLE handle, SERVICE_STATUS *stat
         release_process(process);
     }
 
-    mask = 1 << (service->service_entry->status.dwCurrentState - SERVICE_STOPPED);
-    LIST_FOR_EACH_ENTRY(service_handle, &service->service_entry->handles, struct sc_service_handle, entry)
-    {
-        struct sc_notify_handle *notify = service_handle->notify;
-        if (notify && (notify->notify_mask & mask))
-        {
-            fill_notify(notify, service->service_entry);
-            sc_notify_release(notify);
-            service_handle->notify = NULL;
-            service_handle->status_notified = TRUE;
-        }
-        else
-            service_handle->status_notified = FALSE;
-    }
-
+    notify_service_state(service->service_entry);
     service_unlock(service->service_entry);
 
     return ERROR_SUCCESS;
@@ -967,6 +974,18 @@ DWORD __cdecl svcctl_ChangeServiceConfig2W( SC_RPC_HANDLE hService, SC_RPC_CONFI
         save_service_config( service->service_entry );
         service_unlock( service->service_entry );
         break;
+    case SERVICE_CONFIG_DELAYED_AUTO_START_INFO:
+        WINE_TRACE( "SERVICE_CONFIG_DELAYED_AUTO_START_INFO: fDelayedAutostart %d\n",
+                    config.delayedstart->fDelayedAutostart);
+
+        if (config.delayedstart->fDelayedAutostart && config.delayedstart->fDelayedAutostart != 1)
+            return ERROR_INVALID_PARAMETER;
+
+        service_lock( service->service_entry );
+        service->service_entry->delayed_autostart = config.delayedstart->fDelayedAutostart;
+        save_service_config( service->service_entry );
+        service_unlock( service->service_entry );
+        break;
     default:
         WINE_FIXME("level %lu not implemented\n", config.dwInfoLevel);
         err = ERROR_INVALID_LEVEL;
@@ -1028,6 +1047,18 @@ DWORD __cdecl svcctl_QueryServiceConfig2W( SC_RPC_HANDLE hService, DWORD level,
         service_unlock(service->service_entry);
         break;
 
+    case SERVICE_CONFIG_DELAYED_AUTO_START_INFO:
+        service_lock(service->service_entry);
+
+        *needed = sizeof(SERVICE_DELAYED_AUTO_START_INFO);
+        if (size >= *needed)
+            ((SERVICE_DELAYED_AUTO_START_INFO *)buffer)->fDelayedAutostart =
+                service->service_entry->delayed_autostart;
+        else err = ERROR_INSUFFICIENT_BUFFER;
+
+        service_unlock(service->service_entry);
+        break;
+
     default:
         WINE_FIXME("level %lu not implemented\n", level);
         err = ERROR_INVALID_LEVEL;
@@ -1136,8 +1167,8 @@ static BOOL process_send_command(struct process_entry *process, const void *data
     DWORD count, ret;
     BOOL r;
 
-    overlapped.u.s.Offset = 0;
-    overlapped.u.s.OffsetHigh = 0;
+    overlapped.Offset = 0;
+    overlapped.OffsetHigh = 0;
     overlapped.hEvent = process->overlapped_event;
     r = WriteFile(process->control_pipe, data, size, &count, &overlapped);
     if (!r && GetLastError() == ERROR_IO_PENDING)
@@ -1195,7 +1226,7 @@ BOOL process_send_control(struct process_entry *process, BOOL shared_process, co
         control |= SERVICE_CONTROL_FORWARD_FLAG;
         data = (BYTE *)name;
         data_size = (lstrlenW(name) + 1) * sizeof(WCHAR);
-        name = emptyW;
+        name = L"";
     }
 
     /* calculate how much space we need to send the startup info */
