@@ -37,24 +37,7 @@
 
 #include "wordpad.h"
 
-#ifdef NONAMELESSUNION
-# define U(x)  (x).u
-# define U2(x) (x).u2
-# define U3(x) (x).u3
-#else
-# define U(x)  (x)
-# define U2(x) (x)
-# define U3(x) (x)
-#endif
-
 /* use LoadString */
-static const WCHAR wszAppTitle[] = {'W','i','n','e',' ','W','o','r','d','p','a','d',0};
-
-static const WCHAR wszMainWndClass[] = {'W','O','R','D','P','A','D','T','O','P',0};
-
-static const WCHAR stringFormat[] = {'%','2','d','\0'};
-
-const WCHAR wszPreviewWndClass[] = {'P','r','t','P','r','e','v','i','e','w',0};
 LRESULT CALLBACK preview_proc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);
 
 static HWND hMainWnd;
@@ -99,23 +82,20 @@ typedef struct
 static void DoLoadStrings(void)
 {
     LPWSTR p = wszFilter;
-    static const WCHAR files_rtf[] = {'*','.','r','t','f','\0'};
-    static const WCHAR files_txt[] = {'*','.','t','x','t','\0'};
-    static const WCHAR files_all[] = {'*','.','*','\0'};
 
     HINSTANCE hInstance = GetModuleHandleW(0);
 
     p += 1 + LoadStringW(hInstance, STRING_RICHTEXT_FILES_RTF, p, MAX_STRING_LEN);
-    lstrcpyW(p, files_rtf);
+    lstrcpyW(p, L"*.rtf");
     p += lstrlenW(p) + 1;
     p += 1 + LoadStringW(hInstance, STRING_TEXT_FILES_TXT, p, MAX_STRING_LEN);
-    lstrcpyW(p, files_txt);
+    lstrcpyW(p, L"*.txt");
     p += lstrlenW(p) + 1;
     p += 1 + LoadStringW(hInstance, STRING_TEXT_FILES_UNICODE_TXT, p, MAX_STRING_LEN);
-    lstrcpyW(p, files_txt);
+    lstrcpyW(p, L"*.txt");
     p += lstrlenW(p) + 1;
     p += 1 + LoadStringW(hInstance, STRING_ALL_FILES, p, MAX_STRING_LEN);
-    lstrcpyW(p, files_all);
+    lstrcpyW(p, L"*.*");
     p += lstrlenW(p) + 1;
     *p = '\0';
 
@@ -230,25 +210,20 @@ LPWSTR file_basename(LPWSTR path)
 
 static void set_caption(LPCWSTR wszNewFileName)
 {
-    static const WCHAR wszSeparator[] = {' ','-',' '};
     WCHAR *wszCaption;
-    SIZE_T length = 0;
 
     if(!wszNewFileName)
         wszNewFileName = wszDefaultFileName;
     else
         wszNewFileName = file_basename((LPWSTR)wszNewFileName);
 
-    wszCaption = calloc(1, lstrlenW(wszNewFileName)*sizeof(WCHAR)+sizeof(wszSeparator)+sizeof(wszAppTitle));
+    wszCaption = calloc(1, lstrlenW(wszNewFileName)*sizeof(WCHAR)+sizeof(L" - Wine Wordpad"));
 
     if(!wszCaption)
         return;
 
     memcpy(wszCaption, wszNewFileName, lstrlenW(wszNewFileName)*sizeof(WCHAR));
-    length += lstrlenW(wszNewFileName);
-    memcpy(wszCaption + length, wszSeparator, sizeof(wszSeparator));
-    length += ARRAY_SIZE(wszSeparator);
-    memcpy(wszCaption + length, wszAppTitle, sizeof(wszAppTitle));
+    wcscat(wszCaption, L" - Wine Wordpad");
 
     SetWindowTextW(hMainWnd, wszCaption);
 
@@ -335,7 +310,7 @@ static void on_sizelist_modified(HWND hwndSizeList, LPWSTR wszNewFontSize)
     format.cbSize = sizeof(format);
     SendMessageW(hEditorWnd, EM_GETCHARFORMAT, SCF_SELECTION, (LPARAM)&format);
 
-    wsprintfW(sizeBuffer, stringFormat, format.yHeight / 20);
+    wsprintfW(sizeBuffer, L"%2d", format.yHeight / 20);
     if(lstrcmpW(sizeBuffer, wszNewFontSize))
     {
         float size = 0;
@@ -347,7 +322,7 @@ static void on_sizelist_modified(HWND hwndSizeList, LPWSTR wszNewFontSize)
         {
             SetWindowTextW(hwndSizeList, sizeBuffer);
             MessageBoxWithResStringW(hMainWnd, MAKEINTRESOURCEW(STRING_INVALID_NUMBER),
-                        wszAppTitle, MB_OK | MB_ICONINFORMATION);
+                        L"Wine Wordpad", MB_OK | MB_ICONINFORMATION);
         }
     }
 }
@@ -359,7 +334,7 @@ static void add_size(HWND hSizeListWnd, unsigned size)
     cbItem.mask = CBEIF_TEXT;
     cbItem.iItem = -1;
 
-    wsprintfW(buffer, stringFormat, size);
+    wsprintfW(buffer, L"%2d", size);
     cbItem.pszText = buffer;
     SendMessageW(hSizeListWnd, CBEM_INSERTITEMW, 0, (LPARAM)&cbItem);
 }
@@ -399,7 +374,7 @@ static void populate_size_list(HWND hSizeListWnd)
             add_size(hSizeListWnd, choices[i]);
     }
 
-    wsprintfW(buffer, stringFormat, fmt.yHeight / 20);
+    wsprintfW(buffer, L"%2d", fmt.yHeight / 20);
     SendMessageW(hListEditWnd, WM_SETTEXT, 0, (LPARAM)buffer);
 }
 
@@ -417,7 +392,7 @@ static void update_size_list(void)
     SendMessageW(hEditorWnd, EM_GETCHARFORMAT, SCF_SELECTION, (LPARAM)&fmt);
 
     SendMessageW(hwndSizeListEdit, WM_GETTEXT, MAX_PATH, (LPARAM)fontSize);
-    wsprintfW(sizeBuffer, stringFormat, fmt.yHeight / 20);
+    wsprintfW(sizeBuffer, L"%2d", fmt.yHeight / 20);
 
     if(lstrcmpW(fontSize, sizeBuffer))
         SendMessageW(hwndSizeListEdit, WM_SETTEXT, 0, (LPARAM)sizeBuffer);
@@ -502,9 +477,6 @@ static void set_font(LPCWSTR wszFaceName)
 
 static void set_default_font(void)
 {
-    static const WCHAR richTextFont[] = {'T','i','m','e','s',' ','N','e','w',' ',
-                                         'R','o','m','a','n',0};
-    static const WCHAR plainTextFont[] = {'C','o','u','r','i','e','r',' ','N','e','w',0};
     CHARFORMAT2W fmt;
     LPCWSTR font;
 
@@ -515,9 +487,9 @@ static void set_default_font(void)
     fmt.dwEffects = 0;
 
     if(fileFormat & SF_RTF)
-        font = richTextFont;
+        font = L"Times New Roman";
     else
-        font = plainTextFont;
+        font = L"Courier New";
 
     lstrcpyW(fmt.szFaceName, font);
 
@@ -637,8 +609,7 @@ static void add_font(struct font_array *fonts, LPCWSTR fontName, DWORD fontType,
         fontHeight = ntmc->ntmTm.tmHeight - ntmc->ntmTm.tmInternalLeading;
 
     idx = fonts->count;
-    fonts->fonts[idx].name = malloc((lstrlenW(fontName) + 1)*sizeof(WCHAR) );
-    lstrcpyW( fonts->fonts[idx].name, fontName );
+    fonts->fonts[idx].name = wcsdup(fontName);
     fonts->fonts[idx].lParam = MAKELONG(fontType, fontHeight);
 
     fonts->count++;
@@ -794,7 +765,7 @@ static void preview_exit(HWND hMainWnd)
     HWND hEditorWnd = GetDlgItem(hMainWnd, IDC_EDITOR);
 
     set_bar_states();
-    ShowWindow(hEditorWnd, TRUE);
+    ShowWindow(hEditorWnd, SW_SHOW);
 
     close_preview(hMainWnd);
 
@@ -826,7 +797,7 @@ static void ShowOpenError(DWORD Code)
         default:
             Message = MAKEINTRESOURCEW(STRING_OPEN_FAILED);
     }
-    MessageBoxW(hMainWnd, Message, wszAppTitle, MB_ICONEXCLAMATION | MB_OK);
+    MessageBoxW(hMainWnd, Message, L"Wine Wordpad", MB_ICONEXCLAMATION | MB_OK);
 }
 
 static void DoOpenFile(LPCWSTR szOpenFileName)
@@ -863,7 +834,7 @@ static void DoOpenFile(LPCWSTR szOpenFileName)
         {
             CloseHandle(hFile);
             MessageBoxWithResStringW(hMainWnd, MAKEINTRESOURCEW(STRING_OLE_STORAGE_NOT_SUPPORTED),
-                    wszAppTitle, MB_OK | MB_ICONEXCLAMATION);
+                    L"Wine Wordpad", MB_OK | MB_ICONEXCLAMATION);
             return;
         }
     }
@@ -900,7 +871,7 @@ static void ShowWriteError(DWORD Code)
         default:
             Message = MAKEINTRESOURCEW(STRING_WRITE_FAILED);
     }
-    MessageBoxW(hMainWnd, Message, wszAppTitle, MB_ICONEXCLAMATION | MB_OK);
+    MessageBoxW(hMainWnd, Message, L"Wine Wordpad", MB_ICONEXCLAMATION | MB_OK);
 }
 
 static BOOL DoSaveFile(LPCWSTR wszSaveFileName, WPARAM format)
@@ -963,7 +934,6 @@ static BOOL DialogSaveFile(void)
     OPENFILENAMEW sfn;
 
     WCHAR wszFile[MAX_PATH] = {'\0'};
-    static const WCHAR wszDefExt[] = {'r','t','f','\0'};
 
     ZeroMemory(&sfn, sizeof(sfn));
 
@@ -973,7 +943,7 @@ static BOOL DialogSaveFile(void)
     sfn.lpstrFilter = wszFilter;
     sfn.lpstrFile = wszFile;
     sfn.nMaxFile = MAX_PATH;
-    sfn.lpstrDefExt = wszDefExt;
+    sfn.lpstrDefExt = L"rtf";
     sfn.nFilterIndex = fileformat_number(fileFormat)+1;
 
     while(GetSaveFileNameW(&sfn))
@@ -981,7 +951,7 @@ static BOOL DialogSaveFile(void)
         if(fileformat_flags(sfn.nFilterIndex-1) != SF_RTF)
         {
             if(MessageBoxWithResStringW(hMainWnd, MAKEINTRESOURCEW(STRING_SAVE_LOSEFORMATTING),
-                           wszAppTitle, MB_YESNO | MB_ICONEXCLAMATION) != IDYES)
+                           L"Wine Wordpad", MB_YESNO | MB_ICONEXCLAMATION) != IDYES)
                 continue;
         }
         return DoSaveFile(sfn.lpstrFile, fileformat_flags(sfn.nFilterIndex-1));
@@ -1021,7 +991,7 @@ static BOOL prompt_save_changes(void)
 
         wsprintfW(text, wszSaveChanges, displayFileName);
 
-        ret = MessageBoxW(hMainWnd, text, wszAppTitle, MB_YESNOCANCEL | MB_ICONEXCLAMATION);
+        ret = MessageBoxW(hMainWnd, text, L"Wine Wordpad", MB_YESNOCANCEL | MB_ICONEXCLAMATION);
 
         free(text);
 
@@ -1046,7 +1016,6 @@ static void DialogOpenFile(void)
     OPENFILENAMEW ofn;
 
     WCHAR wszFile[MAX_PATH] = {'\0'};
-    static const WCHAR wszDefExt[] = {'r','t','f','\0'};
 
     ZeroMemory(&ofn, sizeof(ofn));
 
@@ -1056,7 +1025,7 @@ static void DialogOpenFile(void)
     ofn.lpstrFilter = wszFilter;
     ofn.lpstrFile = wszFile;
     ofn.nMaxFile = MAX_PATH;
-    ofn.lpstrDefExt = wszDefExt;
+    ofn.lpstrDefExt = L"rtf";
     ofn.nFilterIndex = fileformat_number(fileFormat)+1;
 
     if(GetOpenFileNameW(&ofn))
@@ -1069,7 +1038,7 @@ static void DialogOpenFile(void)
 static void dialog_about(void)
 {
     HICON icon = LoadImageW(GetModuleHandleW(0), MAKEINTRESOURCEW(IDI_WORDPAD), IMAGE_ICON, 48, 48, LR_SHARED);
-    ShellAboutW(hMainWnd, wszAppTitle, 0, icon);
+    ShellAboutW(hMainWnd, L"Wine Wordpad", 0, icon);
 }
 
 static INT_PTR CALLBACK formatopts_proc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
@@ -1180,7 +1149,7 @@ static void dialog_viewproperties(void)
 
     psp[0].dwSize = sizeof(PROPSHEETPAGEW);
     psp[0].dwFlags = PSP_USETITLE;
-    U(psp[0]).pszTemplate = MAKEINTRESOURCEW(IDD_FORMATOPTS);
+    psp[0].pszTemplate = MAKEINTRESOURCEW(IDD_FORMATOPTS);
     psp[0].pfnDlgProc = formatopts_proc;
     psp[0].hInstance = hInstance;
     psp[0].lParam = reg_formatindex(SF_TEXT);
@@ -1190,7 +1159,7 @@ static void dialog_viewproperties(void)
     {
         psp[i].dwSize = psp[0].dwSize;
         psp[i].dwFlags = psp[0].dwFlags;
-        U(psp[i]).pszTemplate = U(psp[0]).pszTemplate;
+        psp[i].pszTemplate = psp[0].pszTemplate;
         psp[i].pfnDlgProc = psp[0].pfnDlgProc;
         psp[i].hInstance = psp[0].hInstance;
         psp[i].lParam = reg_formatindex(SF_RTF);
@@ -1204,13 +1173,13 @@ static void dialog_viewproperties(void)
     psh.hInstance = hInstance;
     psh.pszCaption = MAKEINTRESOURCEW(STRING_VIEWPROPS_TITLE);
     psh.nPages = ARRAY_SIZE(psp);
-    U3(psh).ppsp = ppsp;
-    U(psh).pszIcon = MAKEINTRESOURCEW(IDI_WORDPAD);
+    psh.ppsp = ppsp;
+    psh.pszIcon = MAKEINTRESOURCEW(IDI_WORDPAD);
 
     if(fileFormat & SF_RTF)
-        U2(psh).nStartPage = 1;
+        psh.nStartPage = 1;
     else
-        U2(psh).nStartPage = 0;
+        psh.nStartPage = 0;
     PropertySheetW(&psh);
     set_bar_states();
     target_device(hMainWnd, wordWrap[reg_formatindex(fileFormat)]);
@@ -1266,7 +1235,7 @@ static void HandleCommandLine(LPWSTR cmdline)
     }
 
     if (opt_print)
-        MessageBoxWithResStringW(hMainWnd, MAKEINTRESOURCEW(STRING_PRINTING_NOT_IMPLEMENTED), wszAppTitle, MB_OK);
+        MessageBoxWithResStringW(hMainWnd, MAKEINTRESOURCEW(STRING_PRINTING_NOT_IMPLEMENTED), L"Wine Wordpad", MB_OK);
 }
 
 static LRESULT handle_findmsg(LPFINDREPLACEW pFr)
@@ -1348,7 +1317,7 @@ static LRESULT handle_findmsg(LPFINDREPLACEW pFr)
             custom_data->endPos = -1;
             EnableWindow(hMainWnd, FALSE);
             MessageBoxWithResStringW(hFindWnd, MAKEINTRESOURCEW(STRING_SEARCH_FINISHED),
-                                     wszAppTitle, MB_OK | MB_ICONASTERISK | MB_TASKMODAL);
+                                     L"Wine Wordpad", MB_OK | MB_ICONASTERISK | MB_TASKMODAL);
             EnableWindow(hMainWnd, TRUE);
         } else {
             SendMessageW(hEditorWnd, EM_SETSEL, ft.chrgText.cpMin, ft.chrgText.cpMax);
@@ -1367,7 +1336,6 @@ static void dialog_find(LPFINDREPLACEW fr, BOOL replace)
     static WCHAR selBuffer[128];
     static WCHAR replaceBuffer[128];
     static FINDREPLACE_custom custom_data;
-    static const WCHAR endl = '\r';
     FINDTEXTW ft;
 
     /* Allow only one search/replace dialog to open */
@@ -1386,7 +1354,7 @@ static void dialog_find(LPFINDREPLACEW fr, BOOL replace)
      * find field is used. */
     SendMessageW(hEditorWnd, EM_GETSEL, (WPARAM)&ft.chrg.cpMin,
                  (LPARAM)&ft.chrg.cpMax);
-    ft.lpstrText = &endl;
+    ft.lpstrText = L"\r";
     if (ft.chrg.cpMin != ft.chrg.cpMax &&
         SendMessageW(hEditorWnd, EM_FINDTEXTW, FR_DOWN, (LPARAM)&ft) == -1)
     {
@@ -1435,17 +1403,15 @@ static int units_to_twips(UNIT unit, float number)
 
 static void append_current_units(LPWSTR buffer)
 {
-    static const WCHAR space[] = {' ', 0};
-    lstrcatW(buffer, space);
+    lstrcatW(buffer, L" ");
     lstrcatW(buffer, units_cmW);
 }
 
 static void number_with_units(LPWSTR buffer, int number)
 {
-    static const WCHAR fmt[] = {'%','.','2','f',' ','%','s','\0'};
     float converted = (float)number / (float)TWIPS_PER_INCH *(float)CENTMM_PER_INCH / 1000.0;
 
-    swprintf(buffer, MAX_STRING_LEN, fmt, converted, units_cmW);
+    swprintf(buffer, MAX_STRING_LEN, L"%.2f %s", converted, units_cmW);
 }
 
 static BOOL get_comboexlist_selection(HWND hComboEx, LPWSTR wszBuffer, UINT bufferLength)
@@ -1666,7 +1632,7 @@ static INT_PTR CALLBACK paraformat_proc(HWND hWnd, UINT message, WPARAM wParam,
                         if(ret != 3)
                         {
                             MessageBoxWithResStringW(hMainWnd, MAKEINTRESOURCEW(STRING_INVALID_NUMBER),
-                                        wszAppTitle, MB_OK | MB_ICONASTERISK);
+                                        L"Wine Wordpad", MB_OK | MB_ICONASTERISK);
                             return FALSE;
                         } else
                         {
@@ -1786,10 +1752,10 @@ static INT_PTR CALLBACK tabstops_proc(HWND hWnd, UINT message, WPARAM wParam, LP
                             if(!number_from_string(buffer, &number, &unit))
                             {
                                 MessageBoxWithResStringW(hWnd, MAKEINTRESOURCEW(STRING_INVALID_NUMBER),
-                                             wszAppTitle, MB_OK | MB_ICONINFORMATION);
+                                             L"Wine Wordpad", MB_OK | MB_ICONINFORMATION);
                             } else if (item_count >= MAX_TAB_STOPS) {
                                 MessageBoxWithResStringW(hWnd, MAKEINTRESOURCEW(STRING_MAX_TAB_STOPS),
-                                             wszAppTitle, MB_OK | MB_ICONINFORMATION);
+                                             L"Wine Wordpad", MB_OK | MB_ICONINFORMATION);
                             } else {
                                 int i;
                                 float next_number = -1;
@@ -1880,12 +1846,8 @@ static LRESULT OnCreate( HWND hWnd )
     HDC hdc;
     SIZE name_sz, size_sz;
     int height, dpi;
-    static const WCHAR wszRichEditDll[] = {'R','I','C','H','E','D','2','0','.','D','L','L','\0'};
-    static const WCHAR wszRichEditText[] = {'R','i','c','h','E','d','i','t',' ','t','e','x','t','\0'};
-    static const WCHAR font_text[] = {'T','i','m','e','s',' ','N','e','w',' ','R','o','m','a','n',0}; /* a long font name */
-    static const WCHAR size_text[] = {' ','0','0',0}; /* enough for two digits */
 
-    CreateStatusWindowW(CCS_NODIVIDER|WS_CHILD|WS_VISIBLE, wszRichEditText, hWnd, IDC_STATUSBAR);
+    CreateStatusWindowW(CCS_NODIVIDER|WS_CHILD|WS_VISIBLE, L"RichEdit text", hWnd, IDC_STATUSBAR);
 
     hReBarWnd = CreateWindowExW(WS_EX_TOOLWINDOW, REBARCLASSNAMEW, NULL,
       CCS_NODIVIDER|WS_CHILD|WS_VISIBLE|WS_CLIPSIBLINGS|WS_CLIPCHILDREN|RBS_VARHEIGHT|CCS_TOP,
@@ -1953,8 +1915,8 @@ static LRESULT OnCreate( HWND hWnd )
     font = (HFONT)SendMessageW(hFontListWnd, WM_GETFONT, 0, 0);
     hdc = GetDC(hFontListWnd);
     font = SelectObject(hdc, font);
-    GetTextExtentPointW(hdc, font_text, ARRAY_SIZE(font_text) - 1, &name_sz);
-    GetTextExtentPointW(hdc, size_text, ARRAY_SIZE(size_text) - 1, &size_sz);
+    GetTextExtentPointW(hdc, L"Times New Roman", ARRAY_SIZE(L"Times New Roman") - 1, &name_sz);
+    GetTextExtentPointW(hdc, L" 00", ARRAY_SIZE(L" 00") - 1, &size_sz);
     font = SelectObject(hdc, font);
     ReleaseDC(hFontListWnd, hdc);
     rbb.hwndChild = hFontListWnd;
@@ -2009,10 +1971,10 @@ static LRESULT OnCreate( HWND hWnd )
 
     SendMessageW(hReBarWnd, RB_INSERTBANDW, -1, (LPARAM)&rbb);
 
-    hDLL = LoadLibraryW(wszRichEditDll);
+    hDLL = LoadLibraryW(L"riched20.dll");
     if(!hDLL)
     {
-        MessageBoxWithResStringW(hWnd, MAKEINTRESOURCEW(STRING_LOAD_RICHED_FAILED), wszAppTitle,
+        MessageBoxWithResStringW(hWnd, MAKEINTRESOURCEW(STRING_LOAD_RICHED_FAILED), L"Wine Wordpad",
                     MB_OK | MB_ICONEXCLAMATION);
         PostQuitMessage(1);
     }
@@ -2022,12 +1984,7 @@ static LRESULT OnCreate( HWND hWnd )
       |ES_WANTRETURN|WS_VSCROLL|ES_NOHIDESEL|WS_HSCROLL,
       0, 0, 1000, 100, hWnd, (HMENU)IDC_EDITOR, hInstance, NULL);
 
-    if (!hEditorWnd)
-    {
-        fprintf(stderr, "Error code %lu\n", GetLastError());
-        return -1;
-    }
-    assert(hEditorWnd);
+    if (!hEditorWnd) return -1;
 
     setup_richedit_olecallback(hEditorWnd);
     SetFocus(hEditorWnd);
@@ -2292,7 +2249,7 @@ static LRESULT OnCommand( HWND hWnd, WPARAM wParam, LPARAM lParam)
             barState[index] = 1 << BANDID_STATUSBAR;
             set_bar_states();
             barState[index] = tmp;
-            ShowWindow(hEditorWnd, FALSE);
+            ShowWindow(hEditorWnd, SW_HIDE);
 
             init_preview(hWnd, wszFileName);
 
@@ -2406,7 +2363,7 @@ static LRESULT OnCommand( HWND hWnd, WPARAM wParam, LPARAM lParam)
         TEXTRANGEW tr;
 
         GetWindowTextW(hwndEditor, data, nLen+1);
-        MessageBoxW(NULL, data, wszAppTitle, MB_OK);
+        MessageBoxW(NULL, data, L"Wine Wordpad", MB_OK);
 
         free(data);
         data = malloc((nLen+1)*sizeof(WCHAR));
@@ -2414,7 +2371,7 @@ static LRESULT OnCommand( HWND hWnd, WPARAM wParam, LPARAM lParam)
         tr.chrg.cpMax = nLen;
         tr.lpstrText = data;
         SendMessageW(hwndEditor, EM_GETTEXTRANGE, 0, (LPARAM)&tr);
-        MessageBoxW(NULL, data, wszAppTitle, MB_OK);
+        MessageBoxW(NULL, data, L"Wine Wordpad", MB_OK);
         free(data);
 
         /* SendMessage(hwndEditor, EM_SETSEL, 0, -1); */
@@ -2454,7 +2411,7 @@ static LRESULT OnCommand( HWND hWnd, WPARAM wParam, LPARAM lParam)
         SendMessageW(hwndEditor, EM_GETSELTEXT, 0, (LPARAM)data);
         sprintf(buf, "Start = %ld, End = %ld", range.cpMin, range.cpMax);
         MessageBoxA(hWnd, buf, "Editor", MB_OK);
-        MessageBoxW(hWnd, data, wszAppTitle, MB_OK);
+        MessageBoxW(hWnd, data, L"Wine Wordpad", MB_OK);
         free(data);
         /* SendMessage(hwndEditor, EM_SETSEL, 0, -1); */
         return 0;
@@ -2820,12 +2777,10 @@ int CALLBACK WinMain(HINSTANCE hInstance, HINSTANCE hOldInstance, LPSTR szCmdPar
     MONITORINFO info;
     HMONITOR monitor;
     int x, y;
-    static const WCHAR wszAccelTable[] = {'M','A','I','N','A','C','C','E','L',
-                                          'T','A','B','L','E','\0'};
 
     InitCommonControlsEx(&classes);
 
-    hAccel = LoadAcceleratorsW(hInstance, wszAccelTable);
+    hAccel = LoadAcceleratorsW(hInstance, L"MAINACCELTABLE");
 
     wc.cbSize = sizeof(wc);
     wc.style = 0;
@@ -2839,7 +2794,7 @@ int CALLBACK WinMain(HINSTANCE hInstance, HINSTANCE hOldInstance, LPSTR szCmdPar
     wc.hCursor = LoadCursorW(NULL, (LPWSTR)IDC_IBEAM);
     wc.hbrBackground = GetSysColorBrush(COLOR_WINDOW);
     wc.lpszMenuName = MAKEINTRESOURCEW(IDM_MAINMENU);
-    wc.lpszClassName = wszMainWndClass;
+    wc.lpszClassName = L"WORDPADTOP";
     RegisterClassExW(&wc);
 
     wc.style = 0;
@@ -2852,7 +2807,7 @@ int CALLBACK WinMain(HINSTANCE hInstance, HINSTANCE hOldInstance, LPSTR szCmdPar
     wc.hCursor = LoadCursorW(NULL, (LPWSTR)IDC_IBEAM);
     wc.hbrBackground = NULL;
     wc.lpszMenuName = NULL;
-    wc.lpszClassName = wszPreviewWndClass;
+    wc.lpszClassName = L"PrtPreview";
     RegisterClassExW(&wc);
 
     registry_read_winrect(&rc);
@@ -2866,7 +2821,7 @@ int CALLBACK WinMain(HINSTANCE hInstance, HINSTANCE hOldInstance, LPSTR szCmdPar
     if (IsRectEmpty(&info.rcWork))
         x = y = CW_USEDEFAULT;
 
-    hMainWnd = CreateWindowExW(0, wszMainWndClass, wszAppTitle, WS_CLIPCHILDREN|WS_OVERLAPPEDWINDOW,
+    hMainWnd = CreateWindowExW(0, L"WORDPADTOP", L"Wine Wordpad", WS_CLIPCHILDREN|WS_OVERLAPPEDWINDOW,
             x, y, rc.right - rc.left, rc.bottom - rc.top, NULL, NULL, hInstance, NULL);
     registry_read_maximized(&bMaximized);
     if ((nCmdShow == SW_SHOWNORMAL || nCmdShow == SW_SHOWDEFAULT)
