@@ -19,7 +19,6 @@
  */
 
 #include "config.h"
-#include "wine/port.h"
 
 #include <assert.h>
 #include <errno.h>
@@ -35,12 +34,8 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <sys/types.h>
-#ifdef HAVE_SYS_SOCKET_H
-# include <sys/socket.h>
-#endif
-#ifdef HAVE_SYS_WAIT_H
-# include <sys/wait.h>
-#endif
+#include <sys/socket.h>
+#include <sys/wait.h>
 #ifdef HAVE_SYS_UIO_H
 #include <sys/uio.h>
 #endif
@@ -48,9 +43,7 @@
 #include <sys/un.h>
 #endif
 #include <unistd.h>
-#ifdef HAVE_POLL_H
 #include <poll.h>
-#endif
 #ifdef __APPLE__
 # include <mach/mach_time.h>
 #endif
@@ -61,14 +54,13 @@
 #include "winbase.h"
 #include "wincon.h"
 #include "winternl.h"
-#include "wine/library.h"
 
 #include "file.h"
 #include "process.h"
 #include "thread.h"
 #include "security.h"
-#define WANT_REQUEST_HANDLERS
-#include "request.h"
+#include "handle.h"
+#include "request_handlers.h"
 
 /* Some versions of glibc don't define this */
 #ifndef SCM_RIGHTS
@@ -92,17 +84,19 @@ static void master_socket_poll_event( struct fd *fd, int event );
 static const struct object_ops master_socket_ops =
 {
     sizeof(struct master_socket),  /* size */
+    &no_type,                      /* type */
     master_socket_dump,            /* dump */
-    no_get_type,                   /* get_type */
     no_add_queue,                  /* add_queue */
     NULL,                          /* remove_queue */
     NULL,                          /* signaled */
     NULL,                          /* satisfied */
     no_signal,                     /* signal */
     no_get_fd,                     /* get_fd */
-    no_map_access,                 /* map_access */
+    default_get_sync,              /* get_sync */
+    default_map_access,            /* map_access */
     default_get_sd,                /* get_sd */
     default_set_sd,                /* set_sd */
+    no_get_full_name,              /* get_full_name */
     no_lookup_name,                /* lookup_name */
     no_link_name,                  /* link_name */
     NULL,                          /* unlink_name */
@@ -127,6 +121,7 @@ static const struct fd_ops master_socket_fd_ops =
 struct thread *current = NULL;  /* thread handling the current request */
 unsigned int global_error = 0;  /* global error code for when no thread is current */
 timeout_t server_start_time = 0;  /* server startup time */
+char *server_dir = NULL;   /* server directory */
 int server_dir_fd = -1;    /* file descriptor for the server dir */
 int config_dir_fd = -1;    /* file descriptor for the config dir */
 
@@ -204,7 +199,7 @@ const struct object_attributes *get_req_object_attributes( const struct security
     }
     if (root && attr->rootdir && attr->name_len)
     {
-        if (!(*root = get_directory_obj( current->process, attr->rootdir ))) return NULL;
+        if (!(*root = get_handle_obj( current->process, attr->rootdir, 0, NULL ))) return NULL;
     }
     *sd = attr->sd_len ? (const struct security_descriptor *)(attr + 1) : NULL;
     name->len = attr->name_len;
@@ -384,28 +379,22 @@ int receive_fd( struct process *process )
     struct iovec vec;
     struct send_fd data;
     struct msghdr msghdr;
-    int fd = -1, ret;
-
-#ifdef HAVE_STRUCT_MSGHDR_MSG_ACCRIGHTS
-    msghdr.msg_accrightslen = sizeof(int);
-    msghdr.msg_accrights = (void *)&fd;
-#else  /* HAVE_STRUCT_MSGHDR_MSG_ACCRIGHTS */
     char cmsg_buffer[256];
-    msghdr.msg_control    = cmsg_buffer;
-    msghdr.msg_controllen = sizeof(cmsg_buffer);
-    msghdr.msg_flags      = 0;
-#endif  /* HAVE_STRUCT_MSGHDR_MSG_ACCRIGHTS */
+    int fd = -1, ret;
 
     msghdr.msg_name    = NULL;
     msghdr.msg_namelen = 0;
     msghdr.msg_iov     = &vec;
     msghdr.msg_iovlen  = 1;
+    msghdr.msg_control = cmsg_buffer;
+    msghdr.msg_controllen = sizeof(cmsg_buffer);
+    msghdr.msg_flags   = 0;
+
     vec.iov_base = (void *)&data;
     vec.iov_len  = sizeof(data);
 
     ret = recvmsg( get_unix_fd( process->msg_fd ), &msghdr, 0 );
 
-#ifndef HAVE_STRUCT_MSGHDR_MSG_ACCRIGHTS
     if (ret > 0)
     {
         struct cmsghdr *cmsg;
@@ -415,7 +404,6 @@ int receive_fd( struct process *process )
             if (cmsg->cmsg_type == SCM_RIGHTS) fd = *(int *)CMSG_DATA(cmsg);
         }
     }
-#endif  /* HAVE_STRUCT_MSGHDR_MSG_ACCRIGHTS */
 
     if (ret == sizeof(data))
     {
@@ -470,32 +458,27 @@ int send_client_fd( struct process *process, int fd, obj_handle_t handle )
 {
     struct iovec vec;
     struct msghdr msghdr;
-    int ret;
-
-#ifdef HAVE_STRUCT_MSGHDR_MSG_ACCRIGHTS
-    msghdr.msg_accrightslen = sizeof(fd);
-    msghdr.msg_accrights = (void *)&fd;
-#else  /* HAVE_STRUCT_MSGHDR_MSG_ACCRIGHTS */
     char cmsg_buffer[256];
     struct cmsghdr *cmsg;
-    msghdr.msg_control    = cmsg_buffer;
+    int ret;
+
+    msghdr.msg_name    = NULL;
+    msghdr.msg_namelen = 0;
+    msghdr.msg_iov     = &vec;
+    msghdr.msg_iovlen  = 1;
+    msghdr.msg_control = cmsg_buffer;
     msghdr.msg_controllen = sizeof(cmsg_buffer);
-    msghdr.msg_flags      = 0;
+    msghdr.msg_flags   = 0;
+
+    vec.iov_base = (void *)&handle;
+    vec.iov_len  = sizeof(handle);
+
     cmsg = CMSG_FIRSTHDR( &msghdr );
     cmsg->cmsg_len   = CMSG_LEN( sizeof(fd) );
     cmsg->cmsg_level = SOL_SOCKET;
     cmsg->cmsg_type  = SCM_RIGHTS;
     *(int *)CMSG_DATA(cmsg) = fd;
     msghdr.msg_controllen = cmsg->cmsg_len;
-#endif  /* HAVE_STRUCT_MSGHDR_MSG_ACCRIGHTS */
-
-    msghdr.msg_name    = NULL;
-    msghdr.msg_namelen = 0;
-    msghdr.msg_iov     = &vec;
-    msghdr.msg_iovlen  = 1;
-
-    vec.iov_base = (void *)&handle;
-    vec.iov_len  = sizeof(handle);
 
     if (debug_level)
         fprintf( stderr, "%04x: *fd* %04x -> %d\n", current ? current->id : process->id, handle, fd );
@@ -522,24 +505,24 @@ int send_client_fd( struct process *process, int fd, obj_handle_t handle )
     return -1;
 }
 
-/* get current tick count to return to client */
-unsigned int get_tick_count(void)
+/* return a monotonic time counter */
+timeout_t monotonic_counter(void)
 {
 #ifdef __APPLE__
     static mach_timebase_info_data_t timebase;
 
     if (!timebase.denom) mach_timebase_info( &timebase );
-    return mach_absolute_time() * timebase.numer / timebase.denom / 1000000;
+    return mach_continuous_time() * timebase.numer / timebase.denom / 100;
 #elif defined(HAVE_CLOCK_GETTIME)
     struct timespec ts;
-#ifdef CLOCK_MONOTONIC_RAW
-    if (!clock_gettime( CLOCK_MONOTONIC_RAW, &ts ))
-        return ts.tv_sec * 1000 + ts.tv_nsec / 1000000;
+#ifdef CLOCK_BOOTTIME
+    if (!clock_gettime( CLOCK_BOOTTIME, &ts ))
+        return (timeout_t)ts.tv_sec * TICKS_PER_SEC + ts.tv_nsec / 100;
 #endif
     if (!clock_gettime( CLOCK_MONOTONIC, &ts ))
-        return ts.tv_sec * 1000 + ts.tv_nsec / 1000000;
+        return (timeout_t)ts.tv_sec * TICKS_PER_SEC + ts.tv_nsec / 100;
 #endif
-    return (current_time - server_start_time) / 10000;
+    return current_time - server_start_time;
 }
 
 static void master_socket_dump( struct object *obj, int verbose )
@@ -578,7 +561,7 @@ static void master_socket_poll_event( struct fd *fd, int event )
         int client = accept( get_unix_fd( master_socket->fd ), (struct sockaddr *) &dummy, &len );
         if (client == -1) return;
         fcntl( client, F_SETFL, O_NONBLOCK );
-        if ((process = create_process( client, NULL, 0, NULL )))
+        if ((process = create_process( client, NULL, 0, NULL, NULL, NULL, 0, NULL )))
         {
             create_thread( -1, process, NULL );
             release_object( process );
@@ -599,7 +582,7 @@ static void create_dir( const char *name, struct stat *st )
     if (lstat( name, st ) == -1)
     {
         if (errno != ENOENT)
-            fatal_error( "lstat %s: %s", name, strerror( errno ));
+            fatal_error( "lstat %s: %s\n", name, strerror( errno ));
         if (mkdir( name, 0700 ) == -1 && errno != EEXIST)
             fatal_error( "mkdir %s: %s\n", name, strerror( errno ));
         if (lstat( name, st ) == -1)
@@ -611,22 +594,67 @@ static void create_dir( const char *name, struct stat *st )
 }
 
 /* create the server directory and chdir to it */
-static void create_server_dir( const char *dir )
+static char *create_server_dir( int force )
 {
-    char *p, *server_dir;
+    const char *prefix = getenv( "WINEPREFIX" );
+    char *p, *config_dir, *base_dir;
     struct stat st, st2;
 
-    if (!(server_dir = strdup( dir ))) fatal_error( "out of memory\n" );
+    /* open the configuration directory */
 
-    /* first create the base directory if needed */
+    if (prefix)
+    {
+        if (!(config_dir = strdup( prefix ))) fatal_error( "out of memory\n" );
+        for (p = config_dir + strlen(config_dir); p > config_dir; p--) if (p[-1] != '/') break;
+        if (p > config_dir) *p = 0;
+        if (config_dir[0] != '/')
+            fatal_error( "invalid directory %s in WINEPREFIX: not an absolute path\n", prefix );
+    }
+    else
+    {
+        const char *home = getenv( "HOME" );
+        if (!home)
+        {
+            struct passwd *pwd = getpwuid( getuid() );
+            if (pwd) home = pwd->pw_dir;
+        }
+        if (!home) fatal_error( "could not determine your home directory\n" );
+        if (home[0] != '/') fatal_error( "your home directory %s is not an absolute path\n", home );
+        if (!(config_dir = malloc( strlen(home) + sizeof("/.wine") ))) fatal_error( "out of memory\n" );
+        strcpy( config_dir, home );
+        for (p = config_dir + strlen(config_dir); p > config_dir; p--) if (p[-1] != '/') break;
+        strcpy( p, "/.wine" );
+    }
 
-    p = strrchr( server_dir, '/' );
-    *p = 0;
-    create_dir( server_dir, &st );
+    if (chdir( config_dir ) == -1)
+    {
+        if (errno != ENOENT || force) fatal_error( "chdir to %s: %s\n", config_dir, strerror( errno ));
+        return NULL;
+    }
+    if ((config_dir_fd = open( ".", O_RDONLY )) == -1)
+        fatal_error( "open %s: %s\n", config_dir, strerror( errno ));
+    if (fstat( config_dir_fd, &st ) == -1)
+        fatal_error( "stat %s: %s\n", config_dir, strerror( errno ));
+    if (st.st_uid != getuid())
+        fatal_error( "%s is not owned by you\n", config_dir );
+
+    /* create the base directory if needed */
+
+#ifdef __ANDROID__  /* there's no /tmp dir on Android */
+    if (asprintf( &base_dir, "%s/.wineserver", config_dir ) == -1)
+        fatal_error( "out of memory\n" );
+#else
+    if (asprintf( &base_dir, "/tmp/.wine-%u", getuid() ) == -1)
+        fatal_error( "out of memory\n" );
+#endif
+    create_dir( base_dir, &st2 );
 
     /* now create the server directory */
 
-    *p = '/';
+    if (asprintf( &server_dir, "%s/server-%llx-%llx", base_dir,
+                  (unsigned long long)st.st_dev, (unsigned long long)st.st_ino ) == -1)
+        fatal_error( "out of memory\n" );
+
     create_dir( server_dir, &st );
 
     if (chdir( server_dir ) == -1)
@@ -638,7 +666,9 @@ static void create_server_dir( const char *dir )
     if (st.st_dev != st2.st_dev || st.st_ino != st2.st_ino)
         fatal_error( "chdir did not end up in %s\n", server_dir );
 
-    free( server_dir );
+    free( base_dir );
+    free( config_dir );
+    return server_dir;
 }
 
 /* create the lock file and return its file descriptor */
@@ -650,29 +680,28 @@ static int create_server_lock(void)
     if (lstat( server_lock_name, &st ) == -1)
     {
         if (errno != ENOENT)
-            fatal_error( "lstat %s/%s: %s", wine_get_server_dir(), server_lock_name, strerror( errno ));
+            fatal_error( "lstat %s/%s: %s\n", server_dir, server_lock_name, strerror( errno ));
     }
     else
     {
         if (!S_ISREG(st.st_mode))
-            fatal_error( "%s/%s is not a regular file\n", wine_get_server_dir(), server_lock_name );
+            fatal_error( "%s/%s is not a regular file\n", server_dir, server_lock_name );
     }
 
     if ((fd = open( server_lock_name, O_CREAT|O_TRUNC|O_WRONLY, 0600 )) == -1)
-        fatal_error( "error creating %s/%s: %s", wine_get_server_dir(), server_lock_name, strerror( errno ));
+        fatal_error( "error creating %s/%s: %s\n", server_dir, server_lock_name, strerror( errno ));
     return fd;
 }
 
 /* wait for the server lock */
 int wait_for_lock(void)
 {
-    const char *server_dir = wine_get_server_dir();
     int fd, r;
     struct flock fl;
 
+    server_dir = create_server_dir( 0 );
     if (!server_dir) return 0;  /* no server dir, so no lock to wait on */
 
-    create_server_dir( server_dir );
     fd = create_server_lock();
 
     fl.l_type   = F_WRLCK;
@@ -688,14 +717,13 @@ int wait_for_lock(void)
 /* kill the wine server holding the lock */
 int kill_lock_owner( int sig )
 {
-    const char *server_dir = wine_get_server_dir();
     int fd, i, ret = 0;
     pid_t pid = 0;
     struct flock fl;
 
+    server_dir = create_server_dir( 0 );
     if (!server_dir) return 0;  /* no server dir, nothing to do */
 
-    create_server_dir( server_dir );
     fd = create_server_lock();
 
     for (i = 1; i <= 20; i++)
@@ -756,7 +784,7 @@ static void acquire_lock(void)
                      "Warning: a previous instance of the wine server seems to have crashed.\n"
                      "Please run 'gdb %s %s/core',\n"
                      "type 'backtrace' at the gdb prompt and report the results. Thanks.\n\n",
-                     server_argv0, wine_get_server_dir() );
+                     server_argv0, server_dir );
         }
         unlink( server_socket_name ); /* we got the lock, we can safely remove the socket */
         got_lock = 1;
@@ -776,7 +804,7 @@ static void acquire_lock(void)
         case EAGAIN:
             exit(2); /* we didn't get the lock, exit with special status */
         default:
-            fatal_error( "fcntl %s/%s: %s", wine_get_server_dir(), server_lock_name, strerror( errno ));
+            fatal_error( "fcntl %s/%s: %s\n", server_dir, server_lock_name, strerror( errno ));
         }
         /* it seems we can't use locks on this fs, so we will use the socket existence as lock */
         close( fd );
@@ -807,14 +835,12 @@ static void acquire_lock(void)
         !(master_socket->fd = create_anonymous_fd( &master_socket_fd_ops, fd, &master_socket->obj, 0 )))
         fatal_error( "out of memory\n" );
     set_fd_events( master_socket->fd, POLLIN );
-    make_object_static( &master_socket->obj );
+    make_object_permanent( &master_socket->obj );
 }
 
 /* open the master server socket and start waiting for new clients */
 void open_master_socket(void)
 {
-    const char *server_dir = wine_get_server_dir();
-    const char *config_dir = wine_get_config_dir();
     int fd, pid, status, sync_pipe[2];
     char dummy;
 
@@ -826,14 +852,7 @@ void open_master_socket(void)
     fd = open( "/dev/null", O_RDWR );
     while (fd >= 0 && fd <= 2) fd = dup( fd );
 
-    if (!server_dir)
-        fatal_error( "directory %s cannot be accessed\n", config_dir );
-    if (chdir( config_dir ) == -1)
-        fatal_error( "chdir to %s: %s\n", config_dir, strerror( errno ));
-    if ((config_dir_fd = open( ".", O_RDONLY )) == -1)
-        fatal_error( "open %s: %s\n", config_dir, strerror( errno ));
-
-    create_server_dir( server_dir );
+    server_dir = create_server_dir( 1 );
 
     if (!foreground)
     {
